##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

source bwidgets/titleframe.tcl
source bwidgets/scrollw.tcl
source bwidgets/dynhelp.tcl


namespace eval Alt {
    variable alist
    variable application
    variable idx
    variable openTool
    
    set alist ""
    set application(nbr) 0
    set idx 0
    set openTool 0
}


proc Alt::Tool {} {
    variable openTool

    if {$openTool} {
	Alt::OpenTool
    } else {
	Alt::CloseTool
    }
} 


proc Alt::OpenTool { } {
    ALT::OpenRTPI
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Alt::CloseTool { } {
    ALT::CloseRTPI
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Alt::Create { parent } {
    variable alist

    frame $parent.top
    label $parent.top.l -text "Application Launch Tool" -anchor w

    set sw [ScrolledWindow $parent.applsw]
    set alist [listbox $sw.l -selectmode single -bg white -height 3 -width 8 -relief sunken]
    $sw setwidget $alist

    bind $alist <Double-ButtonPress-1> Alt::Edit

    frame $parent.bottom
    
    Button $parent.bottom.n -image new -anchor w -relief flat -bd 1 -command {Alt::New} 
    DynamicHelp::register $parent.bottom.n balloon "Add new application"
    HighlightWidget $parent.bottom.n

    Button $parent.bottom.e -image edit -anchor w -relief flat -bd 1 -command {Alt::Edit}
    DynamicHelp::register $parent.bottom.e balloon "Edit selected application"
    HighlightWidget $parent.bottom.e

    Button $parent.bottom.d -image delete -anchor w -relief flat -bd 1 -command {Alt::Delete}
    DynamicHelp::register $parent.bottom.d balloon "Delete selected application"
    HighlightWidget $parent.bottom.d

    Button $parent.bottom.s -image start -anchor w -relief flat -bd 1 -command {Alt::Start}
    DynamicHelp::register $parent.bottom.s balloon "Start selected application"
    HighlightWidget $parent.bottom.s

    pack $parent.top.l -anchor w -side left
    pack $parent.top -fill x -anchor w		       

    pack $sw  -side top  -expand yes -fill both -anchor w

    pack $parent.bottom.n -side left -padx 4
    pack $parent.bottom.e -side left -padx 4
    pack $parent.bottom.d -side left -padx 4
    pack $parent.bottom.s -side left -padx 4
    pack $parent.bottom -fill x -anchor w		       

    return $alist
}


proc Alt::New { } {
    toplevel .alt
    wm withdraw .alt
    focus .alt
    grab .alt
    wm title .alt "Application Launch"
    catch {wm iconbitmap .alt "../bin/mlb.ico"}
    wm resizable .alt 0 0 
    wm geometry .alt +[winfo pointerx .]+[winfo pointery .]

    frame .alt.v

    frame .alt.v.v1
    label .alt.v.v1.l -text "Name:" -width 12 -justify left -anchor w
    entry .alt.v.v1.e -background white -justify left -width 30
    .alt.v.v1.e xview moveto 1.0
    frame .alt.v.v2
    label .alt.v.v2.l -text "Executable:" -width 12 -justify left -anchor w
    entry .alt.v.v2.e -background white -justify left -width 30
    .alt.v.v2.e xview moveto 1.0
    frame .alt.v.v3
    label .alt.v.v3.l -text "Parameters:" -width 12 -justify left -anchor w
    entry .alt.v.v3.e -background white -justify left -width 30
    .alt.v.v3.e xview moveto 1.0
    frame .alt.v.v4
    label .alt.v.v4.l -text "Local Path:" -width 12 -justify left -anchor w
    entry .alt.v.v4.e -background white -justify left -width 30
    .alt.v.v4.e xview moveto 1.0
    
    pack .alt.v.v1.l .alt.v.v1.e -side left -padx 2 -pady 2
    pack .alt.v.v2.l .alt.v.v2.e -side left -padx 2 -pady 2
    pack .alt.v.v3.l .alt.v.v3.e -side left -padx 2 -pady 2
    pack .alt.v.v4.l .alt.v.v4.e -side left -padx 2 -pady 2

    pack .alt.v.v1 .alt.v.v2 .alt.v.v3 .alt.v.v4 -side top

    frame .alt.b
    button .alt.b.b1 -text "Ok" -command "Alt::NewOK"
    button .alt.b.b2 -text "Cancel" -command "destroy .alt"

    pack .alt.b.b1 .alt.b.b2 -side left -padx 10

    pack .alt.v -side top -pady 5 -padx 2
    pack .alt.b -ipadx 10 -ipady 5
    wm deiconify .alt
}


proc Alt::NewOK { } {
    ALT::AddApplication [.alt.v.v1.e get] [.alt.v.v2.e get] [.alt.v.v3.e get] [.alt.v.v4.e get]
    destroy .alt
}


proc Alt::Edit { } {
    variable application
    variable alist
    variable idx

    set selected [$alist curselection]
    if {[llength $selected] == 0} {
	return
    }

    set idx [lindex [$alist curselection] 0]

    toplevel .alt
    wm withdraw .alt
    focus .alt
    grab .alt
    wm title .alt "Application Launch"
    catch {wm iconbitmap .alt "../bin/mlb.ico"}
    wm resizable .alt 0 0 
    wm geometry .alt +[winfo pointerx .]+[winfo pointery .]

    frame .alt.v

    frame .alt.v.v1
    label .alt.v.v1.l -text "Name:" -width 12 -justify left -anchor w
    entry .alt.v.v1.e -background white -justify left -width 30
    .alt.v.v1.e insert end $application($idx,name)
    .alt.v.v1.e xview moveto 1.0
    frame .alt.v.v2
    label .alt.v.v2.l -text "Executable:" -width 12 -justify left -anchor w
    entry .alt.v.v2.e -background white -justify left -width 30
    .alt.v.v2.e insert end $application($idx,exec)
    .alt.v.v2.e xview moveto 1.0
    frame .alt.v.v3
    label .alt.v.v3.l -text "Parameters:" -width 12 -justify left -anchor w
    entry .alt.v.v3.e -background white -justify left -width 30
    .alt.v.v3.e insert end $application($idx,params)
    .alt.v.v3.e xview moveto 1.0
    frame .alt.v.v4
    label .alt.v.v4.l -text "Local Path:" -width 12 -justify left -anchor w
    entry .alt.v.v4.e -background white -justify left -width 30
    .alt.v.v4.e insert end $application($idx,lpath)
    .alt.v.v4.e xview moveto 1.0
    
    pack .alt.v.v1.l .alt.v.v1.e -side left -padx 2 -pady 2
    pack .alt.v.v2.l .alt.v.v2.e -side left -padx 2 -pady 2
    pack .alt.v.v3.l .alt.v.v3.e -side left -padx 2 -pady 2
    pack .alt.v.v4.l .alt.v.v4.e -side left -padx 2 -pady 2

    pack .alt.v.v1 .alt.v.v2 .alt.v.v3 .alt.v.v4 -side top

    frame .alt.b
    button .alt.b.b1 -text "Ok" -command "Alt::EditOK"
    button .alt.b.b2 -text "Cancel" -command "destroy .alt"

    pack .alt.b.b1 .alt.b.b2 -side left -padx 10

    pack .alt.v -side top -pady 5 -padx 2
    pack .alt.b -ipadx 10 -ipady 5
    wm deiconify .alt
}


proc Alt::EditOK { } {
    variable application
    variable idx

    ALT::EditApplication $application($idx,id) [.alt.v.v1.e get] [.alt.v.v2.e get] [.alt.v.v3.e get] [.alt.v.v4.e get]
    destroy .alt
}


proc Alt::Delete { } {
    variable application
    variable alist

    set selected [$alist curselection]
    if {[llength $selected] == 0} {
	return
    }

    set idx [lindex [$alist curselection] 0]

    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "Delete Confirmation" -default 1 -cancel 1\
	    -message "Do you really want to delete $application($idx,name)?" -icon question -justify center -type yesno] {
	return
    }

    ALT::DeleteApplication $application($idx,id)
}


proc Alt::Start { } {
    variable application
    variable alist

    set selected [$alist curselection]
    if {[llength $selected] == 0} {
	return
    }

    set idx [lindex [$alist curselection] 0]

    ALT::StartApplication $application($idx,id)

    StartApplication $application($idx,id)
}


proc Alt::StartApplication {id} {
    variable application

    set idx -1
    for {set i 0} {$i < $application(nbr)} {incr i} {
	if {$application($i,id) == $id} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} {
	return
    }

    set cmd $application($idx,lpath)
    set cmd [string map {\\ /} $cmd]
    append cmd $application($idx,exec)

    if {[file exists $cmd] == 0} {
	DebugInfo::errorPopup "Could not launch application, file $cmd does not exist"
	return
    }

    if {[file executable $cmd] == 0} {
	DebugInfo::errorPopup "Could not launch application, file $cmd is not executable"
	return
    }

    if {[string length $application($idx,params)] != 0} {
	append cmd " "
	append cmd $application($idx,params)
    }

    if {[catch {exec $cmd} result]} {
	destroy .msgdlg
	MessageDlg .msgdlg -parent . -title "Error"\
		-message "Error: execution of $cmd failed" -icon error -justify center -type ok
    }
}


proc Alt::AddApplication {id name exec params lpath} {
    variable application
    variable alist

    set idx $application(nbr)
    incr application(nbr)

    set application($idx,id) $id
    set application($idx,name) $name
    set application($idx,exec) $exec
    set application($idx,params) $params
    set application($idx,lpath) $lpath

    if [winfo exists $alist] {
	$alist insert $idx $name
    }
}


proc Alt::EditApplication {id name exec params lpath} {
    variable application
    variable alist
    
    set idx -1
    for {set i 0} {$i < $application(nbr)} {incr i} {
	if {$application($i,id) == $id} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} {
	return
    }

    set application($idx,name) $name
    set application($idx,exec) $exec
    set application($idx,params) $params
    set application($idx,lpath) $lpath

    if [winfo exists $alist] {
	$alist delete $idx
	$alist insert $idx $name
    }
}


proc Alt::DeleteApplication {id} {
    variable application
    variable alist

    set idx -1
    for {set i 0} {$i < $application(nbr)} {incr i} {
	if {$application($i,id) == $id} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} {
	return
    }

    if [winfo exists .alt] {
	if {$idx == $Alt::idx} {
	    destroy .alt
	}
    }
    
    $alist delete $idx

    set last [expr $application(nbr) - 1]

    for {set i $idx} {$i < $last} {incr i} {
	set next [expr $i + 1]
	set application($i,id) $application($next,id)
	set application($i,name) $application($next,name)
	set application($i,exec) $application($next,exec)
	set application($i,params) $application($next,params)
	set application($i,lpath) $application($next,lpath)
    }

    unset application($last,id)
    unset application($last,name)
    unset application($last,exec)
    unset application($last,params)
    unset application($last,lpath)

    incr application(nbr) -1
}


proc Alt::RebuildApplicationList { } {
    variable application
    variable alist

    if [winfo exists $alist] {
	$alist delete 0 end
	for {set i 0} {$i < $application(nbr)} {incr i} {
	    $alist insert $i $application($i,name)
	}
    }
}