##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

source bwidgets/titleframe.tcl
source bwidgets/scrollw.tcl
source bwidgets/tree.tcl
source bwidgets/bitmap.tcl
source bwidgets/dynhelp.tcl

##=========================================================================##

namespace eval Animations {
    variable tree
    variable newStep
    variable addShowSelection
    variable addHideSelection
    variable stepForward
    variable stepBackward
    variable selectedNode
    variable editMode
    variable editModeValue
    variable menuAction

    # initial values
    set newStep(state)          "normal"
    set editMode(state)         "normal"
    set addShowSelection(state) "normal"
    set addHideSelection(state) "normal"
    set stepForward(state)      "normal"
    set stepBackward(state)     "normal"
    set editModeValue           1
}

##-------------------------------------------------------------------------##

proc Animations::create { parent } {
    variable tree
    variable newStep
    variable editMode
    variable addShowSelection
    variable addHideSelection
    variable stepForward
    variable stepBackward
    variable editModeValue

    frame $parent.top
    label $parent.top.label -text "Animations"

    # new step button
    set newStep(button) [Button $parent.top.newp   \
        -image      [Bitmap::get ../gifs/new.gif]   \
		-relief     flat                            \
		-bd         1                               \
		-state      $newStep(state)                \
		-command    { Animations::insertNewStep }  ]
    DynamicHelp::register $newStep(button) balloon "New Step"
    HighlightWidget $newStep(button)

    # add selected button
    set addShowSelection(button) [Button $parent.top.addShow    \
        -image      [Bitmap::get ../gifs/group_show.gif] \
        -relief     flat                            \
        -bd         1                               \
        -state      $addShowSelection(state)            \
        -command    { Animations::AddSelectionToAnimationStep 1 } ]

    DynamicHelp::register $addShowSelection(button) balloon "Show selected object"
    HighlightWidget $addShowSelection(button)

    # add selected button
    set addHideSelection(button) [Button $parent.top.addHide    \
        -image      [Bitmap::get ../gifs/group_hide.gif] \
        -relief     flat                            \
        -bd         1                               \
        -state      $addHideSelection(state)            \
        -command    { Animations::AddSelectionToAnimationStep 0 } ]

    DynamicHelp::register $addHideSelection(button) balloon "Hide selected object"
    HighlightWidget $addHideSelection(button)

    pack $parent.top.label   -side left  -anchor w
    pack $parent.top.addShow -side right -anchor e
    pack $parent.top.addHide -side right -anchor e
    pack $parent.top.newp    -side right -anchor e
    pack $parent.top         -fill x     -anchor center

    set sw [ScrolledWindow $parent.pagesw]
    set tree  [Tree $sw.tree    \
        -background     white   \
        -linesfill      grey    \
        -relief         sunken  \
        -borderwidth    2       \
        -width          8       \
        -highlightthickness 0   \
		-redraw         0       \
		-dropenabled    1       \
		-dragenabled    1       \
        -dragevent      1       \
        -droptypes {
                       TREE_NODE {move {}}
                   } \
	    -dropcmd     "Animations::moveNode" \
        -opencmd     "MLB::SetAnimationStepOpen 1" \
        -closecmd    "MLB::SetAnimationStepOpen 0"]
    $sw setwidget $tree

    pack $sw  -side top  -expand yes -fill both -anchor w

    # bind mouse actions
    $tree bindText  <ButtonPress-1>        "Animations::select 1 $tree"
    $tree bindText  <ButtonPress-3>        "Animations::selectNodeAction $tree %X %Y"
    $tree bindText  <Double-ButtonPress-1> "Animations::select 2 $tree"
    $tree bindImage <ButtonPress-1>        "Animations::select 1 $tree"
    $tree bindImage <ButtonPress-3>        "Animations::selectNodeAction $tree %X %Y"
    $tree bindImage <Double-ButtonPress-1> "Animations::select 2 $tree"

    bind $tree <ButtonPress-3> {Animations::ShowPopup %X %Y}

    # step backward button
    frame $parent.bottom
    set stepBackward(button) [Button $parent.bottom.pp      \
        -image      [Bitmap::get ../gifs/previouspage.gif]  \
        -relief     flat                                    \
        -bd         1                                       \
        -state      $stepBackward(state)                    \
        -command    { Animations::doStepBackward } ]

    DynamicHelp::register $stepBackward(button) balloon "Previous"
    HighlightWidget $stepBackward(button)

    # step forward button
    set stepForward(button) [Button $parent.bottom.np       \
        -image      [Bitmap::get ../gifs/nextpage.gif]      \
        -relief     flat                                    \
        -bd         1                                       \
        -state      $stepBackward(state)                    \
        -command    { Animations::doStepForward } ]

    DynamicHelp::register $stepForward(button) balloon "Next"
    HighlightWidget $stepForward(button)

    # edit/presentation mode button
    set editMode(button) [Button $parent.bottom.edit   \
        -image      [Bitmap::get ../gifs/presentation.gif]  \
        -relief     flat                            \
        -bd         1                               \
        -state      $editMode(state)                \
        -command    { Animations::setEditMode }     ]
    DynamicHelp::register $editMode(button) balloon "Presentation mode"
    HighlightWidget $editMode(button)

    # pack buttons
    pack $parent.bottom.pp   -side left
    pack $parent.bottom.np   -side left
    pack $parent.bottom.edit -side left
    pack $parent.bottom -anchor center

    update
    return $tree
}

##-------------------------------------------------------------------------##

proc Animations::setEditMode {} {
    variable tree
    variable editModeValue

    if { "NO"==[MLB::HasActivePage] } { return }

    if { $editModeValue == 1 } { set editModeValue 0
    } else                     { set editModeValue 1 }

    MLB::SetAnimationEditMode $editModeValue
    update
}
# setEditMode

##-------------------------------------------------------------------------##

proc Animations::updateTree {} {
    variable tree
    variable editModeValue

    # clean old nodes
    $tree delete [$tree nodes root]

    # if no page - return
    if { "NO"==[MLB::HasActivePage] } { return }

    # add current steps
    set steps [MLB::GetAnimationStepNames]
    foreach step $steps {

        set title [MLB::GetAnimationStepTitle $step]
        insertStep $step $title

        # add current objects
        if { 1==$editModeValue } {
            # show objects
            set objects [MLB::GetAnimationObjectNames $step 1]
            foreach object $objects { insertObject $step $object [MLB::GetAnimationObjectTitle $step $object] 1 }

            # hide objects
            set objects [MLB::GetAnimationObjectNames $step 0]
            foreach object $objects { insertObject $step $object [MLB::GetAnimationObjectTitle $step $object] 0 }

            # update open status
            moddir [MLB::IsAnimationStepOpen $step] $step
        }
    }

    # select steps if any
    if { 0==$editModeValue } {
        foreach sel [MLB::GetAnimationRunStepNames] { $tree selection add $sel }
    }

    # enable/disable drag/drop and redraw
    $tree configure -redraw 1       \
        -dropenabled $editModeValue \
		-dragenabled $editModeValue
}
# updateTree

##-------------------------------------------------------------------------##

proc Animations::updateEditMode {} {
    variable editMode
    variable editModeValue
    variable menuAction

    # read edit mode value
    if { "YES"==[MLB::HasActivePage] } {
        set editModeValue [MLB::GetAnimationEditMode]
    }

    # enabled/disabled
    if { "NO"==[MLB::HasActivePage] } {
        $editMode(button) configure -state disabled
        #.animationsMenu entryconfigure $menuAction(presentation) -state disabled
    } else {
        $editMode(button) configure -state normal
        #.animationsMenu entryconfigure $menuAction(presentation) -state normal
    }

    # sunken/flat
    if { $editModeValue == 1 } { $editMode(button) configure -relief flat   -bg $Config::bgColor
    } else                     { $editMode(button) configure -relief sunken -bg LightGrey }
}
# updateEditMode

##-------------------------------------------------------------------------##

proc Animations::updateNewStepAction {} {
    variable newStep
    variable editModeValue
    variable menuAction

    # enabled/disabled
    if { "NO"==[MLB::HasActivePage] || 0==$editModeValue
    } { $newStep(button) configure -state disabled
        .animationsMenu entryconfigure $menuAction(newStep) -state disabled
    } else {
        $newStep(button) configure -state normal
        .animationsMenu entryconfigure $menuAction(newStep) -state normal
    }
}
# updateNewStepAction

##-------------------------------------------------------------------------##

proc Animations::updateAddSelectedAction {} {
    variable tree
    variable addShowSelection
    variable addHideSelection
    variable editModeValue
    variable menuAction

    # enabled/disabled - we do not check if there is anything to add (selected)
    if { "NO" == [MLB::HasActivePage]       || \
#          0   == [MLB::HasSelectedObjects]  || \
          0   == $editModeValue             || \
          0   != [IsAnimationStep [$tree selection get]] \
    } { $addShowSelection(button)   configure -state disabled
        $addHideSelection(button)   configure -state disabled
        .animationsMenu entryconfigure $menuAction(addShowSelected) -state disabled
        .animationsMenu entryconfigure $menuAction(addHideSelected) -state disabled

    } else {
        $addShowSelection(button)   configure -state normal
        $addHideSelection(button)   configure -state normal
        .animationsMenu entryconfigure $menuAction(addShowSelected) -state normal
        .animationsMenu entryconfigure $menuAction(addHideSelected) -state normal
    }
}
# updateAddSelectedAction

##-------------------------------------------------------------------------##

proc Animations::updateStepForwardAction {} {
    variable stepForward
    variable editModeValue
    variable menuAction

    # enabled/disabled
    if { "NO" == [MLB::HasActivePage] || 1 == $editModeValue
    } {
        $stepForward(button) configure -state disabled
        .animationsMenu entryconfigure $menuAction(stepForward) -state disabled
    } else {
        $stepForward(button) configure -state normal
        .animationsMenu entryconfigure $menuAction(stepForward) -state normal
    }
}
# updateStepForwardAction

##-------------------------------------------------------------------------##

proc Animations::updateStepBackwardAction {} {
    variable stepBackward
    variable editModeValue
    variable menuAction

    # enabled/disabled
    if { "NO" == [MLB::HasActivePage] || 1 == $editModeValue
    } {
        $stepBackward(button) configure -state disabled
        .animationsMenu entryconfigure $menuAction(stepBackward) -state disabled
    } else {
        $stepBackward(button) configure -state normal
        .animationsMenu entryconfigure $menuAction(stepBackward) -state normal
    }
}
# updateStepBackwardAction

##-------------------------------------------------------------------------##

proc Animations::updateRenameAction {} {
    variable tree
    variable editModeValue
    variable menuAction

    # enabled/disabled
    if { "NO"==[MLB::HasActivePage] || 0==$editModeValue || 0!=[IsAnimationStep [$tree selection get]]
    } {
        .animationsMenu entryconfigure $menuAction(rename) -state disabled
    } else {
        .animationsMenu entryconfigure $menuAction(rename) -state normal
    }
}
# updateRenameAction

##-------------------------------------------------------------------------##

proc Animations::updateDeleteAction {} {
    variable tree
    variable editModeValue
    variable menuAction

    # enabled/disabled
    if { "NO"==[MLB::HasActivePage] || 0==$editModeValue || 0==[llength [$tree selection get]]
    } {
        .animationsMenu entryconfigure $menuAction(delete) -state disabled
    } else {
        .animationsMenu entryconfigure $menuAction(delete) -state normal
    }
}
# updateDeleteAction

##-------------------------------------------------------------------------##

proc Animations::updateActions {} {
    updateRenameAction
    updateDeleteAction
    updateNewStepAction
    updateAddSelectedAction
    updateStepForwardAction
    updateStepBackwardAction
}
# updateActions

##-------------------------------------------------------------------------##

proc Animations::update {} {
    variable tree
    if { $Config::showAnimations } {
        updateEditMode
        updateTree
        updateActions
    }
}
# update

##-------------------------------------------------------------------------##

proc Animations::insertStep { step title } {
    variable tree
    variable selectedNode
    variable editModeValue

    #DebugInfo::info "Inserting: $step $title \n"

    $tree insert end root $step                                    \
        -text    $title                                             \
        -image     [Bitmap::get ../gifs/folder.gif]                 \
        -drawcross auto                                             \
	    -open 1
}

##-------------------------------------------------------------------------##

proc Animations::insertNewStep {} {
    variable tree
    variable editModeValue

    if { "NO"==[MLB::HasActivePage] } return
    if { $editModeValue == 0        } return
    set step [MLB::InsertNewAnimationStep]
    set title [MLB::GetAnimationStepTitle $step]
    insertStep $step $title

    if { $editModeValue == 1 } {
        $tree selection set $step
        set selectedNode $step
    }
    updateActions
}
# insertNewStep

##-------------------------------------------------------------------------##

proc Animations::insertObject { step object title show } {
    variable tree

    if { 1==$show } { set gifName "../gifs/objects_show.gif"
    } else          { set gifName "../gifs/objects_hide.gif" }

    $tree insert end $step $object                  \
      -text      $title                             \
      -image     [Bitmap::get $gifName]             \
      -drawcross never
}
# insertObject

##-------------------------------------------------------------------------##

proc Animations::AddSelectionToAnimationStep  { show } {
    variable tree
    variable editModeValue

    if { "NO"==[MLB::HasActivePage] } return
    if { $editModeValue == 0        } return
    set step    [$tree selection get]
    set objects [MLB::AddSelectedToAnimationStep $step $show]
    #foreach obj $objects { insertObject $step $obj $show }
    MLB::SetAnimationStepOpen 1 $step
    update
}
# AddSelectionToAnimationStep

##-------------------------------------------------------------------------##

proc Animations::moddir { idx node } {
    variable tree

    $tree itemconfigure $node -open $idx -image [Bitmap::get [lindex {../gifs/folder.gif ../gifs/openfold.gif} $idx]]
}

##-------------------------------------------------------------------------##

proc Animations::selectNodeAction { tree x y node} {
    variable selectedNode

    select_node $tree $node
    ShowPopup $x $y
}

##-------------------------------------------------------------------------##

proc Animations::select { num tree node } {
    variable selectedNode
    variable editModeValue

    if {$editModeValue != 1} return

    select_node $tree $node
    if {$num == 2} {
        if { 0==[IsAnimationStep $node] } {
            Animations::edit $node
    	}
    }
    updateActions
}

##-------------------------------------------------------------------------##

proc Animations::select_node { tree node } {
    variable selectedNode
    variable editModeValue

    if {$editModeValue != 1} return

    if { 0==[IsAnimationStep $node] } { MLB::SelectAnimationStep    $node
    } else                            { MLB::SelectAnimationObject  [$tree itemcget $node -text] }

    $tree selection set $node
    set selectedNode $node
}

##-------------------------------------------------------------------------##

proc Animations::IsAnimationStep { node } {
    return [ string compare -length 4 "Step" $node ]
}
# IsAnimationStep

##-------------------------------------------------------------------------##

proc Animations::edit { node } {
    variable tree

    set res [$tree edit $node [$tree itemcget $node -text]]

    #DebugInfo::info $res

    if { $res != "" } {
        $tree itemconfigure $node -text $res
 	    MLB::SetAnimationStepTitle $node $res
    }
}

##-------------------------------------------------------------------------##

proc Animations::moveNode {tree source target op type data} {
    variable editModeValue

    if { [string compare $tree $source] != 0 || \
         $editModeValue != 1 } {
        return
    }

    set mode [lindex $target 0]
    set dest [lindex $target 1]
    if {[string compare $mode "position"] == 0} {

        set position    [lindex $target 2     ]
        set index       [$tree  index   $data ]
        set nextIndex   [expr   $index + 1    ]

        # move animation steps
        if { [IsAnimationStep $data       ] == 0        &&  \
             [string compare  $dest "root"] == 0        &&  \
             $position                      != $index   &&  \
             $position                      != $nextIndex   \
        } {
            #DebugInfo::info "Moving animation step - $data to position $position\n"
            MLB::MoveAnimationStep $data $position
            #DebugInfo::info "After moving animation step - $data to position $position\n"
            update
            #DebugInfo::info "After update\n"
        }

    }
}
# moveNode

##-------------------------------------------------------------------------##

proc Animations::Rename {} {
    variable tree
    variable selectedNode

    set res [$tree edit $selectedNode [$tree itemcget $selectedNode -text]]
    if { $res != "" } {
	$tree itemconfigure $selectedNode -text $res
	MLB::SetAnimationStepTitle $selectedNode $res
    }
}

##-------------------------------------------------------------------------##

proc Animations::Delete {} {
    variable tree
    variable selectedNode

    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "Delete Confirmation" -default 1 -cancel 1\
	    -message "Do you really want to delete \"[$tree itemcget $Animations::selectedNode -text]\"?" -icon question -justify center -type yesno] {
	  return
    }

    # get parent of the node and perform remove
    set parent [$tree parent $selectedNode]
    MLB::RemoveAnimationItem $parent $Animations::selectedNode
    $tree delete $selectedNode
}

##-------------------------------------------------------------------------##

proc Animations::BuildPopup { } {
    variable tree
    variable menuAction

    set menuAction(rename)          0
    set menuAction(delete)          1
    set menuAction(separator1)      2
    set menuAction(newStep)         3
    set menuAction(addShowSelected) 4
    set menuAction(addHideSelected) 5
    set menuAction(separator2)      6
    set menuAction(stepForward)     7
    set menuAction(stepBackward)    8
    set menuAction(separator3)      9
    #set menuAction(presentation)    10

    menu .animationsMenu -tearoff 0

    .animationsMenu insert $menuAction(rename)          command -label "Rename" -command { Animations::Rename }
    .animationsMenu insert $menuAction(delete)          command -label "Delete" -command { Animations::Delete }
    .animationsMenu insert $menuAction(separator1)      separator
    .animationsMenu insert $menuAction(newStep)         command -label "New Animation Step"     -command { Animations::insertNewStep }
    .animationsMenu insert $menuAction(addShowSelected) command -label "Add objects to show"   -command { Animations::AddSelectionToAnimationStep 1 }
    .animationsMenu insert $menuAction(addHideSelected) command -label "Add objects to hide"   -command { Animations::AddSelectionToAnimationStep 0 }
    .animationsMenu insert $menuAction(separator2)      separator
    .animationsMenu insert $menuAction(stepForward)     command -label "Step forward"  -command { Animations::doStepForward    }
    .animationsMenu insert $menuAction(stepBackward)    command -label "Step backward" -command { Animations::doStepBackward   }
    #.animationsMenu insert $menuAction(separator3)      separator
    #.animationsMenu insert $menuAction(presentation)    checkbutton -label "Presentation mode"  -command { Animations::setEditMode }
}
# BuildPopup

##-------------------------------------------------------------------------##

proc Animations::ShowPopup {x y} {
    variable tree
    variable menuAction

    updateActions
    tk_popup .animationsMenu $x $y
}

##-------------------------------------------------------------------------##

proc Animations::doStepForward {} {
    variable tree
    variable editModeValue
    global   Et_EvalTrace

    if { "NO"==[MLB::HasActivePage] } return
    if { $editModeValue == 1        } return

    set selected [$tree selection get]
    set steps   [$tree nodes root]
    set num      [llength $selected]
    set new      [lrange $steps $num $num ]

    # set Et_EvalTrace 1
    MLB::RunAnimationStep $new 1
    # set Et_EvalTrace 0

    $tree selection add $new
}
# StepForward

##-------------------------------------------------------------------------##

proc Animations::doStepBackward {} {
    variable tree
    variable editModeValue

    if { "NO"==[MLB::HasActivePage] } return
    if { $editModeValue == 1        } return

    set selected [$tree selection get]
    set steps   [$tree nodes root]
    set num      [llength $selected]
    if {$num == 0} return
    set num      [expr $num-1]
    set new      [lrange $steps $num $num ]

    MLB::RunAnimationStep  $new 0
    $tree selection remove $new
}
# StepBackward

##-------------------------------------------------------------------------##

proc Animations::OnSpace {} {
    variable tree
    variable editModeValue

    #DebugInfo::info "Space key pressed.\n"
    #DebugInfo::info "Config::showAnimations: $Config::showAnimations\n"
    #DebugInfo::info "editModeValue: $editModeValue\n"

    if { !$Config::showAnimations || $editModeValue != 0 } return
    #if { [llength [$tree selection get]] != [llength [$tree nodes root]] } { doStepForward }
    doStepForward
}
# onSpace

##-------------------------------------------------------------------------##

proc Animations::OnShiftSpace {} {
    variable tree
    variable editModeValue

    if { !$Config::showAnimations || $editModeValue != 0 } return
    if { [llength [$tree selection get]] != [llength [$tree nodes root]] } { doStepForward }
}
# onSpace

##-------------------------------------------------------------------------##

Animations::BuildPopup

##=========================================================================##
