##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################


namespace eval ArrowShape {
    variable arrowInfo
    variable widget
    variable canvas
    variable attribute
    variable created

    set created 0
}

proc ArrowShape::create {} {
    variable arrowInfo
    variable value
    variable widget
    variable canvas
    variable created

    set created 1

    set widget [toplevel .arrowshape]
    wm withdraw $widget
    wm title $widget "Arrowhead Editor"
    grab set $widget
    raise $widget
    focus $widget
    
    frame $widget.buttons
    pack $widget.buttons -side bottom -fill x -pady 2m
    button $widget.buttons.apply -text "Ok" \
	-command ArrowShape::Ok
    button $widget.buttons.cancel -text Cancel \
         -command ArrowShape::Cancel

    pack $widget.buttons.apply -side left -expand 1
    pack $widget.buttons.cancel -side left -expand 1
  
    set canvas [canvas $widget.canvas -width 350 -height 210 -relief sunken -borderwidth 2]
    pack $widget.canvas -expand yes -fill both
  
    set arrowInfo(a) [lindex $Attributes::arrowShape 0]
    set arrowInfo(b) [lindex $Attributes::arrowShape 1]
    set arrowInfo(c) [lindex $Attributes::arrowShape 2]
    set arrowInfo(width) $Attributes::lineWidth
    set arrowInfo(x1) 40
    set arrowInfo(x2) 230
    set arrowInfo(y) 90
    set arrowInfo(smallTips) {5 5 2}
    set arrowInfo(count) 0
    set arrowInfo(bigLineStyle) "-fill blue"
    set arrowInfo(boxStyle) "-fill {} -outline black -width 1"
    set arrowInfo(activeStyle) "-fill red -outline black -width 1"

    $canvas bind box <Enter> "$canvas itemconfigure current $arrowInfo(activeStyle)"
    $canvas bind box <Leave> "$canvas itemconfigure current $arrowInfo(boxStyle)"
    $canvas bind box <B1-Enter> " "
    $canvas bind box <B1-Leave> " "
    $canvas bind box1 <1> {set arrowInfo(motionProc) ArrowShape::arrowMove1}
    $canvas bind box2 <1> {set arrowInfo(motionProc) ArrowShape::arrowMove2}
    $canvas bind box <B1-Motion> "\$arrowInfo(motionProc) %x %y"
    bind $canvas <Any-ButtonRelease-1> "ArrowShape::Draw"

    wm deiconify $widget
}

proc ArrowShape::Draw {} {
    variable widget
    variable canvas
    variable arrowInfo
    variable value
    variable created

    if {!$created} {
	ArrowShape::create
    } else {
	wm deiconify $widget
	raise $widget
	grab current $widget
    }


    set tags [$canvas gettags current]
    if {$tags != ""} {
	set cur [lindex $tags [lsearch -glob $tags box?]]
    } else {
	set cur ""
    }

    # Create the arrow and outline.

    $canvas delete all
    eval "$canvas create line $arrowInfo(x1) $arrowInfo(y) $arrowInfo(x2) $arrowInfo(y) -width [expr 6*$arrowInfo(width)] \
	    -arrowshape {[expr 6*$arrowInfo(a)] [expr 6*$arrowInfo(b)] [expr 6*$arrowInfo(c)]} \
	    -arrow last $arrowInfo(bigLineStyle)"

    set xtip [expr $arrowInfo(x2)-6*$arrowInfo(b)]
    set deltaY [expr 6*$arrowInfo(c)+6*$arrowInfo(width)/2]

    $canvas create line $arrowInfo(x2) $arrowInfo(y) $xtip [expr $arrowInfo(y)+$deltaY] \
	    [expr $arrowInfo(x2)-6*$arrowInfo(a)] $arrowInfo(y) $xtip [expr $arrowInfo(y)-$deltaY] \
	    $arrowInfo(x2) $arrowInfo(y) -fill black -width 2 -capstyle round -joinstyle round

    # Create the boxes for reshaping the line and arrowhead.

    eval "$canvas create rect [expr $arrowInfo(x2)-6*$arrowInfo(a)-6] [expr $arrowInfo(y)-6] \
	    [expr $arrowInfo(x2)-6*$arrowInfo(a)+6] [expr $arrowInfo(y)+6] $arrowInfo(boxStyle) \
	    -tags {box1 box}"
    eval "$canvas create rect [expr $xtip-6] [expr $arrowInfo(y)-$deltaY-6] \
	    [expr $xtip+5] [expr $arrowInfo(y)-$deltaY+5] $arrowInfo(boxStyle) \
	    -tags {box2 box}"

    if {$cur != ""} {
	eval $canvas itemconfigure $cur $arrowInfo(activeStyle)
    }

    # Create three arrows in actual size with the same parameters

    $canvas create line [expr $arrowInfo(x2)+40] 0 [expr $arrowInfo(x2)+40] 220 \
	    -width 2

    set tmp [expr $arrowInfo(x2)+80]

    $canvas create line $tmp [expr $arrowInfo(y)-75] $tmp [expr $arrowInfo(y)-25] \
	-width $arrowInfo(width) \
	-arrow both -arrowshape "$arrowInfo(a) $arrowInfo(b) $arrowInfo(c)"
    $canvas create line [expr $tmp-25] $arrowInfo(y) [expr $tmp+25] $arrowInfo(y) \
	    -width $arrowInfo(width) \
	    -arrow both -arrowshape "$arrowInfo(a) $arrowInfo(b) $arrowInfo(c)"
    $canvas create line [expr $tmp-25] [expr $arrowInfo(y)+25] [expr $tmp+25] \
	    [expr $arrowInfo(y)+75] -width $arrowInfo(width) \
	    -arrow both -arrowshape "$arrowInfo(a) $arrowInfo(b) $arrowInfo(c)"

    # Create a bunch of other arrows and text items showing the
    # current dimensions.

    set tmp [expr $arrowInfo(x2)+6]
    $canvas create line $tmp [expr $arrowInfo(y)-$arrowInfo(width)] \
	    $tmp [expr $arrowInfo(y)-$deltaY] \
	    -arrow both -arrowshape $arrowInfo(smallTips)
    $canvas create text [expr $arrowInfo(x2)+12] [expr $arrowInfo(y)-$deltaY+6*$arrowInfo(c)] \
	    -text $arrowInfo(c) -anchor w

    set tmp [expr $arrowInfo(x1)-6]
    $canvas create line $tmp [expr $arrowInfo(y)-6*$arrowInfo(width)] \
	    $tmp [expr $arrowInfo(y)+6*$arrowInfo(width)] \
	    -arrow both -arrowshape $arrowInfo(smallTips)
    $canvas create text [expr $arrowInfo(x1)-15] $arrowInfo(y) -text $arrowInfo(width) -anchor e

    set tmp [expr $arrowInfo(y)+6*$arrowInfo(width)+6*$arrowInfo(c)+6]
    $canvas create line [expr $arrowInfo(x2)-6*$arrowInfo(a)] $tmp $arrowInfo(x2) $tmp \
	    -arrow both -arrowshape $arrowInfo(smallTips)
    $canvas create text [expr $arrowInfo(x2)-6*$arrowInfo(a)] [expr $tmp+6] \
	    -text $arrowInfo(a) -anchor n

    set tmp [expr $tmp+25]
    $canvas create line [expr $arrowInfo(x2)-6*$arrowInfo(b)] $tmp $arrowInfo(x2) $tmp \
	    -arrow both -arrowshape $arrowInfo(smallTips)
    $canvas create text [expr $arrowInfo(x2)-6*$arrowInfo(b)] [expr $tmp+6] \
	    -text $arrowInfo(b) -anchor n

    $canvas create text [expr $arrowInfo(x1)-20] 180 -text "-width  $arrowInfo(width)" \
	    -anchor w -font {Helvetica 12}
    $canvas create text [expr $arrowInfo(x1)-20] 195 -text "-arrowshape  {$arrowInfo(a)  $arrowInfo(b)  $arrowInfo(c)}" \
	    -anchor w -font {Helvetica 12}

    incr arrowInfo(count)
}

proc ArrowShape::arrowMove1 {x y} {
    variable canvas
    variable arrowInfo
    
    set newA [expr ($arrowInfo(x2)+6-round([$canvas canvasx $x]))/6]
    if {$newA < 0} {
	set newA 0
    }
    if {$newA > 25} {
	set newA 25
    }
    if {$newA != $arrowInfo(a)} {
	$canvas move box1 [expr 6*($arrowInfo(a)-$newA)] 0
	set arrowInfo(a) $newA
    }
}

proc ArrowShape::arrowMove2 {x y} {
    variable canvas
    variable arrowInfo

    set newB [expr ($arrowInfo(x2)+6-round([$canvas canvasx $x]))/6]
    if {$newB < 0} {
	set newB 0
    }
    if {$newB > 25} {
	set newB 25
    }
    set newC [expr ($arrowInfo(y)+6-round([$canvas canvasy $y])-6*$arrowInfo(width))/6]
    if {$newC < 0} {
	set newC 0
    }
    if {$newC > 20} {
	set newC 20
    }
    if {($newB != $arrowInfo(b)) || ($newC != $arrowInfo(c))} {
	$canvas move box2 [expr 6*($arrowInfo(b)-$newB)] [expr 6*($arrowInfo(c)-$newC)]
	set arrowInfo(b) $newB
	set arrowInfo(c) $newC
    }
}

proc ArrowShape::Ok {} {
    variable widget
    variable arrowInfo
    
    set Attributes::arrowShape \
	[list $arrowInfo(a) $arrowInfo(b) $arrowInfo(c)]
    MLB::ChangeArrowShape "$Attributes::arrowShape"
    
    wm withdraw $widget
    grab release $widget
}

proc ArrowShape::Cancel {} {
    variable widget

    wm withdraw $widget
    grab release $widget
}
