##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################


namespace eval Attributes {
    variable toolBar
    variable position
    variable fontSelection
    variable fontFamily
    variable fontSize
    variable fontWeight
    variable fontSlant
    variable fontPosition
    variable colorEntry
    variable color
    variable lineAttribute
    variable lineWidth
    variable lineStyle
    variable joinStyle
    variable capStyle
    variable arrowStyle
    variable arrowShape
    variable pointerType
    variable lineStyleButton ""
    
    #MB
    variable boxWidth
    variable boxStyle
    #        boxColor = pointerColor
    
    set position top
    set fontFamily "arial"
    set fontSize "20"
    set fontWeight "normal"
    set fontSlant "roman"
    set fontPosition "normal"
    set pointerType 0
    set pointerColor "#FF0000"

    #MB
    set boxWidth 2
    set boxStyle " "
    
    set color(0) "#000000" 
    set color(1) "#FFFFFF" 

    set lineWidth 2
    set lineStyle " "
    set joinStyle "bevel"
    set capStyle "butt"
    set arrowStyle "none"
    set arrowShape {8 10 4}

    set colorEntry(0) {
	{Attributes::ChooseOutlineColor}
	{Color }
    } 
    set colorEntry(1) {
	{Attributes::ChooseFillColor}
	{Fill Color }
    } 

    set lineAttribute(0) {
	{Attributes::ChooseLineWidth}
    }

    set lineAttribute(1) {
	{{../gifs/style0.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} { }}
	{{../gifs/style1.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} {.}}
	{{../gifs/style2.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} {,}}
	{{../gifs/style3.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} {-}}
	{{../gifs/style4.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} {_}}
	{{../gifs/style5.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} {.-}}
	{{../gifs/style6.gif} {Attributes::lineStyle} {Attributes::ChooseLineStyle} {..-}}
    }
}

proc Attributes::create parent {
    variable toolBar
    variable fontSelection
    variable colorEntry
    variable lineAttribute
    variable lineStyleButton
    
    set toolBar [frame $parent.toolBar2 -borderwidth 1 -relief flat]
    
    set fontSelection [SelectFont $toolBar.fontsel -command Attributes::ChooseToolFont \
			   -family $Attributes::fontFamily -size $Attributes::fontSize \
			   -weight $Attributes::fontWeight -slant $Attributes::fontSlant \
			   -position $Attributes::fontPosition -sampletext "Sample Text" \
			   -title "Font selection"]
    pack $fontSelection -side left
    
    set sep(2) [Separator $toolBar.sep12 -bg "grey" -orient vertical -relief ridge]
    pack $sep(2) -fill both -padx 10 -side left
	     
    for {set i 0} {$i < 2} {incr i} {
	set colorLabel($i) [LabelFrame $toolBar.lab$i -text [lindex $colorEntry($i) 1]]
	set colorButton($i) [SelectColor [$colorLabel($i) getframe].col -variable Attributes::color($i) -command [lindex $colorEntry($i) 0] -type menubutton]
	pack $colorButton($i)
	pack $colorLabel($i) -side left -padx 5
    } 

    set lineWidthButton [SpinBox $toolBar.lw -label "Line Width" \
			     -underline 0 -labelwidth 10 -labelanchor w \
			     -range {1 10 1} -textvariable Attributes::lineWidth \
			     -modifycmd [lindex $Attributes::lineAttribute(0) 0]]
    $lineWidthButton setvalue @[expr $Attributes::lineWidth-1]
    $lineWidthButton configure -width 2
    pack $lineWidthButton -side left -padx 10

    
    set lineStyleButton [ImageBox::create $toolBar.ls "Line Style" 0 -values $lineAttribute(1)]
    pack $lineStyleButton -side left -padx 10
    

}

proc Attributes::ChooseOutlineColor {} {
    variable color
    
    MLB::ChangeOutlineColor $color(0)
}

proc Attributes::ChooseFillColor {} {
    variable color
    
    MLB::ChangeFillColor $color(1)
}

proc Attributes::ChooseLineWidth {} {
    variable lineWidth
       
    MLB::ChangeLineWidth $lineWidth
}

proc Attributes::ChooseLineStyle {} {
    variable lineStyle

    MLB::ChangeLineStyle $lineStyle
}

proc Attributes::SetLineStyleButton { } {
    variable lineStyle
    variable lineStyleButton

    if {[winfo exists $lineStyleButton]} {
	if {[string compare $lineStyle "none"] == 0} {
	    ImageBox::_select $lineStyleButton 0 {NOP}
	} elseif {[string compare $lineStyle "."] == 0} {
	    ImageBox::_select $lineStyleButton 1 {NOP}
	} elseif {[string compare $lineStyle ","] == 0} {
	    ImageBox::_select $lineStyleButton 2 {NOP}
	} elseif {[string compare $lineStyle "-"] == 0} {
	    ImageBox::_select $lineStyleButton 3 {NOP}
	} elseif {[string compare $lineStyle "_"] == 0} {
	    ImageBox::_select $lineStyleButton 4 {NOP}
	} elseif {[string compare $lineStyle ".-"] == 0} {
	    ImageBox::_select $lineStyleButton 5 {NOP}
	} elseif {[string compare $lineStyle "..-"] == 0} {
	    ImageBox::_select $lineStyleButton 6 {NOP}
	}
    }
}


proc Attributes::ChooseJoinStyle {} {
    variable joinStyle

    MLB::ChangeJoinStyle $joinStyle
}

proc Attributes::ChooseCapStyle {} {
    variable capStyle

    MLB::ChangeCapStyle $capStyle
}

proc Attributes::ChooseArrowStyle {} {
    variable arrowStyle

    MLB::ChangeArrowStyle $arrowStyle
}
proc Attributes::Top {} {

    if {$ToolBar::position == "top"} {
        grid $Attributes::toolBar -row 2 -column 1
        grid $Toplevel::frame.sep -row 3 -column 1
        grid $WorkingArea::frame -row 4 -column 1
	if {$Config::showToolBar} {
	    grid $ToolBar::toolBar -row 1 -column 1
	    pack $ToolBar::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
	}
        pack $Attributes::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
        pack $Toplevel::frame.sep -fill x -pady 2
        pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2
    } elseif {$ToolBar::position == "bottom"} {
        grid $Attributes::toolBar -row 1 -column 1
        grid $WorkingArea::frame -row 3 -column 1
        grid $Toplevel::frame.sep -row 2 -column 1
	if {$Config::showToolBar} {
	    grid $ToolBar::toolBar -row 4 -column 1
	    pack $ToolBar::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
	}
        pack $Attributes::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
        pack $Toplevel::frame.sep -fill x -pady 2
        pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2
    } else {
        grid $Attributes::toolBar -row 1 -column 1
        grid $Toplevel::frame.sep -row 2 -column 1
        grid $WorkingArea::frame -row 3 -column 1
        pack $Attributes::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
        pack $Toplevel::frame.sep -fill x -pady 2
        pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2
    }

}

proc Attributes::Bottom {} {

    if {$ToolBar::position == "top"} {
        grid $Toplevel::frame.sep -row 2 -column 1
        grid $Attributes::toolBar -row 4 -column 1
        grid $WorkingArea::frame -row 3 -column 1
	if {$Config::showToolBar} {
	    grid $ToolBar::toolBar -row 1 -column 1
	    pack $ToolBar::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
	}
        pack $Toplevel::frame.sep -fill x -pady 2
        pack $Attributes::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
        pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2
    } elseif {$ToolBar::position == "bottom"} {
        grid $Toplevel::frame.sep -row 2 -column 1
        grid $Attributes::toolBar -row 4 -column 1
        grid $WorkingArea::frame -row 1 -column 1
        pack $Attributes::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
	if {$Config::showToolBar} {
	    grid $ToolBar::toolBar -row 3 -column 1
	    pack $ToolBar::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
	}
        pack $Toplevel::frame.sep -fill x -pady 2
        pack $WorkingArea::frame -side top -fill both -expand 1 -padx 2 -pady 2
    } else {
        grid $Toplevel::frame.sep -row 2 -column 1
        grid $Attributes::toolBar -row 3 -column 1
        grid $WorkingArea::frame -row 1 -column 1
        pack $Attributes::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
        pack $Toplevel::frame.sep -fill x -pady 2 -side bottom
        pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2
    }

    pack $Attributes::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
}


proc Attributes::ChooseFont {} {
#     set Attributes::fontFamily [SelectFont $Toplevel::frame.fontsel -family $Attributes::fontFamily -sampletext "Sample Text" -title "Font selection" -type dialog]
#     $Attributes::fontSelection configure -family $Attributes::fontFamily
}


proc Attributes::ChooseToolFont {} {
    global textw

    set Attributes::fontFamily [$Attributes::fontSelection cget -family] 
    set Attributes::fontSize [$Attributes::fontSelection cget -size] 
    set Attributes::fontWeight [$Attributes::fontSelection cget -weight] 
    set Attributes::fontSlant [$Attributes::fontSelection cget -slant] 
    set Attributes::fontPosition [$Attributes::fontSelection cget -position] 


    MLB::ChangeFont $Attributes::fontFamily $Attributes::fontSize \
	$Attributes::fontWeight $Attributes::fontSlant $Attributes::fontPosition
}


proc Attributes::UpdateFont {family size weight slant position} {
    set Attributes::fontFamily $family
    $Attributes::fontSelection configure -family $Attributes::fontFamily
    set Attributes::fontSize $size
    $Attributes::fontSelection configure -size $Attributes::fontSize
    set Attributes::fontWeight $weight
    $Attributes::fontSelection configure -weight $Attributes::fontWeight
    set Attributes::fontSlant $slant
    $Attributes::fontSelection configure -slant $Attributes::fontSlant
    set Attributes::fontPosition $position
    $Attributes::fontSelection configure -position $Attributes::fontPosition
}


proc Attributes::SetOutlineColor {color} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::color(0) $color
}

proc Attributes::SetFillColor {color} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::color(1) $color
}

proc Attributes::SetLineWidth {lineWidth} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::lineWidth $lineWidth
}

proc Attributes::SetLineStyle {lineStyle} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::lineStyle $lineStyle
    ImageBox::select $lineStyle
}

proc Attributes::SetJoinStyle {joinStyle} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::joinStyle $joinStyle
}

proc Attributes::SetCapStyle {capStyle} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::capStyle $capStyle
}

proc Attributes::SetArrowStyle {arrowStyle} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::arrowStyle $arrowStyle
}

proc Attributes::SetArrowShape {arrowShape} {

    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::arrowShape $arrowShape
}

proc Attributes::SetFont {family size weight slant position} {
 
    if {$ToolBar::currentTool == $ToolBar::Move} {return}
    if {$ToolBar::currentTool == $ToolBar::Copy} {return}
    if {$ToolBar::currentTool == $ToolBar::Select} {return}
    if {$ToolBar::currentTool == $ToolBar::Delete} {return}

    set Attributes::fontFamily $family
    $Attributes::fontSelection configure -family $Attributes::fontFamily
    set Attributes::fontSize $size
    $Attributes::fontSelection configure -size $Attributes::fontSize
    set Attributes::fontWeight $weight
    $Attributes::fontSelection configure -weight $Attributes::fontWeight
    set Attributes::fontSlant $slant
    $Attributes::fontSelection configure -slant $Attributes::fontSlant
    set Attributes::fontPosition $position
    $Attributes::fontSelection configure -position $Attributes::fontPosition
}


namespace eval AttributesBox {
    variable attributesBox
    variable fontSelection
    variable colorLabel
    variable colorButton
    variable lineWidthButton

}

proc AttributesBox::Create {} {
    variable attributesBox
    variable fontSelection
    variable colorLabel
    variable colorButton
    variable lineWidthButton
    variable lineStyle

    Dialog .attributesBox -modal none
    wm protocol  .attributesBox WM_DELETE_WINDOW AttributesBox::Hide
    set attributesBox [.attributesBox getframe]
    
    set fontSelection [SelectFont $attributesBox.fontsel -command Attributes::ChooseToolFont \
			   -family $Attributes::fontFamily -size $Attributes::fontSize \
			   -weight $Attributes::fontWeight -slant $Attributes::fontSlant \
			   -position $Attributes::fontPosition -sampletext "Sample Text" \
			   -title "Font selection"]
    
    for {set i 0} {$i < 2} {incr i} {
	set colorLabel($i) [LabelFrame $attributesBox.lab$i -text [lindex $Attributes::colorEntry($i) 1]]
	set colorButton($i) [SelectColor [$colorLabel($i) getframe].col -variable Attributes::color($i) -command [lindex $Attributes::colorEntry($i) 0] -type menubutton]
    } 
    
    set lineWidthButton [SpinBox $attributesBox.lw -label "Line Width" \
			     -underline 0 -labelwidth 10 -labelanchor w \
			     -range {1 10 1} -textvariable Attributes::lineWidth \
			     -modifycmd [lindex $Attributes::lineAttribute(0) 0]]
    $lineWidthButton setvalue @[expr $Attributes::lineWidth-1]
    $lineWidthButton configure -width 2

    set lineStyle [ImageBox::create $attributesBox.ls "Line Style" 0 -values $Attributes::lineAttribute(1)]

    .attributesBox add -text "Hide" -command AttributesBox::Hide

    AttributesBox::Update
}

proc AttributesBox::Update {} {
    variable attributesBox
    variable fontSelection
    variable colorLabel
    variable colorButton
    variable lineWidthButton
    variable lineStyle
    
    if {![winfo exists .attributesBox]} {
	AttributesBox::Create
    } else {
	pack $fontSelection -side top -anchor nw
	for {set i 0} {$i < 2} {incr i} {
	    pack $colorLabel($i) $colorButton($i) -side top -anchor nw
	} 
	pack $lineWidthButton -side top  -anchor nw
	pack $lineStyle -side top -anchor nw
    }
    
    .attributesBox draw none
}

proc AttributesBox::Show {} {
    
    if {![winfo exists .attributesBox]} {
	AttributesBox::Create
	.attributesBox draw none
    } else {
	wm deiconify .attributesBox
    }
    
}

proc AttributesBox::Hide {} {
    
    .attributesBox withdraw
}
