# ------------------------------------------------------------------------------
#  combobox.tcl
#  This file is part of Unifix BWidget Toolkit
#  $Id: imagebox.tcl,v 1.1 2001/07/27 08:01:30 jvogel Exp $
# ------------------------------------------------------------------------------
#  Index of commands:
#     - ImageBox::create
#     - ImageBox::configure
#     - ImageBox::cget
#     - ImageBox::setvalue
#     - ImageBox::getvalue
#     - ImageBox::_create_popup
#     - ImageBox::_mapliste
#     - ImageBox::_unmapliste
#     - ImageBox::_select
# ------------------------------------------------------------------------------

source bwidgets/arrow.tcl
source bwidgets/entry.tcl
#source bwidgets/attributes.tcl

namespace eval ImageBox {   
    ArrowButton::use
    Entry::use
    LabelFrame::use

    Widget::bwinclude ImageBox LabelFrame .labf \
        rename     {-text -label} \
        remove     {-focus} \
        prefix     {label -justify -width -anchor -height -font} \
        initialize {-relief sunken -borderwidth 2}

    Widget::bwinclude ImageBox Entry .e \
        remove {-relief -bd -borderwidth -bg -fg} \
        rename {-foreground -entryfg -background -entrybg}

    Widget::declare ImageBox {
        {-height      TkResource 0  0 listbox}
        {-values      String     "" 0}
        {-modifycmd   String     "" 0}
        {-postcommand String     "" 0}
    }

    Widget::addmap ImageBox "" :cmd {-background {}}
    Widget::addmap ImageBox ArrowButton .a \
        {-foreground {} -background {} -disabledforeground {} -state {}}

    Widget::syncoptions ImageBox Entry .e {-text {}}
    Widget::syncoptions ImageBox LabelFrame .labf {-label -text -underline {}}

    ::bind BwImageBox <FocusIn> {focus %W.labf}
    ::bind BwImageBox <Destroy> {Widget::destroy %W; rename %W {}}

    proc ::ImageBox { path args } { return [eval ImageBox::create $path $args] }
    proc use {} {}
}


# ------------------------------------------------------------------------------
#  Command ImageBox::create
# ------------------------------------------------------------------------------
proc ImageBox::create { path title selected args} {
    variable image

    Widget::init ImageBox $path $args

    frame $path -background [Widget::getoption $path -background] \
        -highlightthickness 0 -bd 0 -relief flat -takefocus 0

    bindtags $path [list $path BwImageBox [winfo toplevel $path] all]

    set entries [Widget::getoption $path -values]
    set i 0
    foreach val $entries {
	set image($i) [Bitmap::get [lindex $val 0]]
	incr i
    }

    set labf  [LabelFrame $path.labf -relief flat -text $title -bd 0]
    set entry [Button $path.e -image $image($selected) -relief flat -highlightthickness 0 -bd 1 -command "ImageBox::_unmapliste"] 

    set width  11
    set arrow [ArrowButton $path.a \
                   -width $width -bd 0\
                   -highlightthickness 0 -borderwidth 1 -takefocus 0 \
                   -dir   bottom \
                   -type  button \
                   -command "ImageBox::_mapliste $path"]

    set labelFrame [LabelFrame::getframe $labf]
    $labelFrame configure -relief sunken -bd 1 
 
    pack $arrow -in $labelFrame -side right -fill y 
    pack $entry -in $labelFrame -side left -fill x -expand yes
    pack $labf -side left


    bind $path <ButtonPress-1> "ImageBox::_unmapliste" 

    return $path

}

# ------------------------------------------------------------------------------
#  Command ImageBox::create_list
# ------------------------------------------------------------------------------
proc ImageBox::create_shell {path} {
    variable image

    set shell [menu $path.shell -tearoff 0 -relief flat -bd 0]
    wm overrideredirect $shell 1
    wm transient $shell [winfo toplevel $path]
    wm group $shell [winfo toplevel $path]

    set frame [frame $shell.frame -relief sunken -bd 0 ]
 
    set listb [listbox $frame.listb -relief flat -bd 0 -highlightthickness 0 \
		   -exportselection false]
    
    set entries [Widget::getoption $path -values]
    set i 0
    foreach val $entries {
	set width [image width $image($i)]
	set buttons($i) [radiobutton $listb.b$i -width $width -image $image($i) \
			     -indicatoron false -bd 1 -relief raised \
			     -variable [lindex $val 1] -value [lindex $val 3] \
			     -command "ImageBox::_select $path $i [lindex $val 2]"]
	pack $buttons($i) -in $listb -side top -fill both
	incr i
    }

    pack $listb -in $frame -side left -fill both -expand yes
    pack $frame -side left -fill both -expand yes
    pack $listb

}

# ------------------------------------------------------------------------------
#  Command ImageBox::_mapliste
# ------------------------------------------------------------------------------
proc ImageBox::_mapliste {path } {

    set listb $path.shell.listb
    if { [winfo exists $path.shell] } {
	_unmapliste $path
        return
    }

    if { [Widget::getoption $path -state] == "disabled" } {
        return
    }
    if { [set cmd [Widget::getoption $path -postcommand]] != "" } {
        uplevel \#0 $cmd
    }
    if { ![llength [Widget::getoption $path -values]] } {
        return
    }

    ArrowButton::configure $path.a -dir top
    create_shell $path


    set frame [LabelFrame::getframe $path.labf]
    BWidget::place $path.shell [winfo x $frame] [winfo y $frame] below $frame    
    wm deiconify $path.shell
    raise $path.shell
    BWidget::grab global $path
}


# ------------------------------------------------------------------------------
#  Command ImageBox::_unmapliste
# ------------------------------------------------------------------------------
proc ImageBox::_unmapliste {path } {
    BWidget::grab release $path
    destroy $path.shell
    ArrowButton::configure $path.a -dir bottom
}


# ------------------------------------------------------------------------------
#  Command ImageBox::_select
# ------------------------------------------------------------------------------
proc ImageBox::_select { path index command} {
    variable buttons 
    variable entry
    variable image


    $path.e configure -image $image($index)
    $command
     _unmapliste $path

}

# ------------------------------------------------------------------------------
#  Command ImageBox::select
# ------------------------------------------------------------------------------
proc ImageBox::select { style } {
    variable entries 
    variable entry
    variable image


    set i 0
    foreach val $entries {
	if {[lindex $val 3] == $style} {
	    $entry configure -image $image($i)
	    return;
	}
	incr i
    }

}

