##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval Chat {
    variable nickname
    variable message
    variable openTool

    set nickname ""
    set message ""
    set openTool 0

}


proc Chat::Tool { } {
    variable openTool
   
    if {$openTool} {
	Create
    } else {
	Close
    }
}


proc Chat::Create {} {
    variable nickname
    variable message
    variable openTool

    if [winfo exists .chat] {
	wm iconify .chat
	wm deiconify .chat
	return
    }

    toplevel .chat
    wm withdraw .chat
    focus .chat
    wm title .chat "Chat Tool"
    catch {wm iconbitmap .chat "../bin/mlb.ico"}
    wm protocol .chat WM_DELETE_WINDOW Chat::Close
    wm resizable .chat 0 0 
 
    frame .chat.n
    label .chat.n.l -text "Your Nickname:" -justify left -anchor w -width 15
    entry .chat.n.e -background white -justify left -textvariable Chat::nickname -width 30
    pack  .chat.n.l .chat.n.e -side left -anchor w -fill x -side left -padx 2 -pady 2
    
    frame .chat.e
    label .chat.e.l -text "Your Message" -justify left -anchor w -width 15
    entry .chat.e.e -background grey -justify left -textvariable Chat::message -width 30 -state disabled
    bind  .chat.e.e <Return> {Chat::SendMessage}
    pack  .chat.e.l .chat.e.e -side left -anchor w -fill x -side left -padx 2 -pady 2

    frame .chat.m
    set sw [ScrolledWindow .chat.m.sw]
    text $sw.t -relief sunken -bd 2 -setgrid 1 -height 15 -width 50 -state disabled -bg white
    $sw setwidget $sw.t
    pack $sw -side top  -fill both -expand 1

    frame .chat.b
    button .chat.b.b1 -text "Close" -command "Chat::Close"
    pack .chat.b.b1 -side left -padx 10

    pack .chat.n -pady 2 -fill x
    pack .chat.e -pady 2 -fill x
    pack .chat.m -pady 2 -fill x -fill y -expand yes
    pack .chat.b -ipadx 10 -ipady 5 

    set openTool 1

    CHAT::OpenRTPI

    wm deiconify .chat
}


proc Chat::SendMessage {} {
    variable nickname
    variable message
    
    if {[string length $nickname] == 0} {
	destroy .msgdlg
	MessageDlg .msgdlg -parent .chat -title "Chat Tool"  -cancel 1\
	    -message "Please enter nickname first!" -icon error -justify center -type ok
	set message ""
	return
    }

    ShowMessage "$nickname" "$message"
    CHAT::SendMessage "$nickname" "$message"

    set message ""
}


proc Chat::ShowMessage {name text} {
    variable openTool

    if {!$openTool} {
	DebugInfo::info "ChatShowMessage - closed"
	return
    }

    .chat.m.sw.t configure -state normal
    .chat.m.sw.t insert end "$name > $text\n"
    .chat.m.sw.t see end
    .chat.m.sw.t configure -state disabled

    DebugInfo::info "ChatShowMessage - $name > $text\n"
}


proc Chat::ClearMessages { } {
    .chat.m.sw.t delete 0 end
}


proc Chat::EnableMessageEntry { } {
    .chat.e.e configure -state normal -background white
}

proc Chat::Close { } {
    variable openTool

    if {!$openTool} {
	return
    }

    destroy .chat
    CHAT::CloseRTPI

    set openTool 0
}


