##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval Config {
    variable openShared       0
    variable zoom             100
    variable width            800
    variable height           600
    variable showPagetree     1
    variable showParticipants 1
    variable showToolBar      1
    variable showAttributes   1
    variable showStatusBar    1
    variable showDebuginfo    0
    variable freezeImages     1
    variable freezeTree       0
    variable participantHints 0
    variable bgColor
    
    variable profileParameter 
    variable count            1
    variable profiles         {}
    
    variable configVersion    "1.0"
}



proc Config::SetIpAddr { ipAddr } {
    variable openShared
    set openShared 1
    set Session::ipAddr $ipAddr
}



proc Config::SetPort { port } {
    set Session::port $port
}



proc Config::SetTtl { ttl } {
    set Session::ttl $ttl
}



proc Config::GetProfileNames { } {
    set names {}
    foreach id $Config::profiles {
         set names [linsert $names end $Config::profileParameter($id,name)]
    }
    return $names
}


proc Config::GetProfile { id } {
    set Session::id                      $Config::profileParameter($id,id)
    set Session::name                    $Config::profileParameter($id,name)
    set Session::ipAddr                  $Config::profileParameter($id,ipAddr) 
    set Session::port                    $Config::profileParameter($id,port)  
    set Session::ttl                     $Config::profileParameter($id,ttl)  
    set Session::bandwidth               $Config::profileParameter($id,bandwidth)  
    set Session::cueRate                 $Config::profileParameter($id,cueRate)  
    set Session::localLag                $Config::profileParameter($id,localLag)  
    set Session::storageTime             $Config::profileParameter($id,storageTime)  
    set Session::checkIntentionViolation $Config::profileParameter($id,checkIntentionViolation)  
    set Session::logPageChanges          $Config::profileParameter($id,logPageChanges)  
}


proc Config::SetProfile { id } {
    set Config::profileParameter($id,id)                      $Session::id 
    set Config::profileParameter($id,name)                    $Session::name 
    set Config::profileParameter($id,ipAddr)                  $Session::ipAddr
    set Config::profileParameter($id,port)                    $Session::port  
    set Config::profileParameter($id,ttl)                     $Session::ttl  
    set Config::profileParameter($id,bandwidth)               $Session::bandwidth      
    set Config::profileParameter($id,cueRate)                 $Session::cueRate     
    set Config::profileParameter($id,localLag)                $Session::localLag       
    set Config::profileParameter($id,storageTime)             $Session::storageTime   
    set Config::profileParameter($id,checkIntentionViolation) $Session::checkIntentionViolation   
    set Config::profileParameter($id,logPageChanges)          $Session::logPageChanges
}


proc Config::NewProfile { } {
    # default values
    set Session::id                      $Config::count
    set Session::ipAddr                  "224.100.100.120"
    set Session::port                    "8000"
    set Session::ttl                     "4"
    set Session::bandwidth               "100000"
    set Session::cueRate                 "10"
    set Session::localLag                "100"
    set Session::storageTime             "180"
    set Session::checkIntentionViolation "1"
    set Session::logPageChanges          "1"
    set name "default"
    for { set i 2 } { [lsearch [Config::GetProfileNames] $name] != -1 } { incr i } {
         set name "default $i"
    }
    set Session::name $name
    
    set Config::profiles [linsert $Config::profiles end $Config::count]
    Config::SetProfile $Config::count
    incr Config::count
}


proc Config::OpenProfile { } {
    foreach id $Config::profiles {
         if { $Config::profileParameter($id,name) == $Session::name } {
              Config::GetProfile $id
              break
         }
    }          
}


proc Config::DeleteProfile { } {
    unset Config::profileParameter($Session::id,id)                     
    unset Config::profileParameter($Session::id,name)                    
    unset Config::profileParameter($Session::id,ipAddr)               
    unset Config::profileParameter($Session::id,port)                 
    unset Config::profileParameter($Session::id,ttl)                  
    unset Config::profileParameter($Session::id,bandwidth)            
    unset Config::profileParameter($Session::id,cueRate)              
    unset Config::profileParameter($Session::id,localLag)              
    unset Config::profileParameter($Session::id,storageTime)            
    unset Config::profileParameter($Session::id,checkIntentionViolation) 
    unset Config::profileParameter($Session::id,logPageChanges) 
    
    set k [lsearch -exact $Config::profiles $Session::id]
    set Config::profiles [lreplace $Config::profiles $k $k]
    if { [llength $Config::profiles] > 0 } {
         Config::GetProfile [lindex $Config::profiles 0]
    } else {
         Config::NewProfile 
    }
}


proc Config::ReadConfigFile {} {
    if [file exists ".mlb_config"] {
    	set config [open ".mlb_config" r]
    	
    	gets $config version
     	if { $version != $Config::configVersion } {
     	  close $config
    	  Config::NewProfile
     	  return
     	}
     	
    	gets $config Session::local
    	gets $config Session::idServerAddr
    	gets $config Session::idServerPort
    	gets $config Session::uceServerAddr
    	gets $config Session::uceServerPort
    
    	gets $config Config::zoom
    	gets $config Config::width
    	gets $config Config::height
    	gets $config Config::showPagetree
    	gets $config Config::showParticipants
    	gets $config Config::showToolBar
    	gets $config Config::showAttributes
    	gets $config Config::showStatusBar
    	gets $config Config::showDebuginfo
    	gets $config Record::openTool
    	gets $config Config::freezeImages
    	gets $config Config::freezeTree
    	gets $config Config::participantHints
    
    	gets $config Attributes::color(0)
    	gets $config Attributes::color(1)
    	gets $config Attributes::lineWidth
    	gets $config ToolBar::currentTool
      gets $config
    
     	set Config::count 1
     	while { [eof $config] == 0 } {
        gets $config Config::profileParameter($Config::count,name)
        gets $config Config::profileParameter($Config::count,ipAddr) 
        gets $config Config::profileParameter($Config::count,port)  
        gets $config Config::profileParameter($Config::count,ttl)  
        gets $config Config::profileParameter($Config::count,bandwidth)  
        gets $config Config::profileParameter($Config::count,cueRate)  
        gets $config Config::profileParameter($Config::count,localLag)  
        gets $config Config::profileParameter($Config::count,storageTime)  
        gets $config Config::profileParameter($Config::count,checkIntentionViolation)  
        gets $config Config::profileParameter($Config::count,logPageChanges)  

        set Config::profileParameter($Config::count,id) $Config::count
        set Config::profiles [linsert $Config::profiles end $Config::count]
        incr Config::count

        gets $config
   	}
    
    	close $config
    	
    	Config::GetProfile 1	
	
    } else {
    	Config::NewProfile
    }
}


proc Config::WriteConfigFile {} {
    set config [open ".mlb_config" w 0600]

    set w [wm geometry .]
    set idx1 [string first "x" $w]
    set idx2 [string first "+" $w]
    set Config::width  [string range $w 0 [expr $idx1 - 1]]
    set Config::height [string range $w [expr $idx1 + 1] [expr $idx2 - 1]]

    puts $config "$Config::configVersion"
    puts $config "$Session::local"
    puts $config "$Session::idServerAddr"
    puts $config "$Session::idServerPort"
    puts $config "$Session::uceServerAddr"
    puts $config "$Session::uceServerPort"

    puts $config "$Config::zoom"
    puts $config "$Config::width"
    puts $config "$Config::height"
    puts $config "$Config::showPagetree"
    puts $config "$Config::showParticipants"
    puts $config "$Config::showToolBar"
    puts $config "$Config::showAttributes"
    puts $config "$Config::showStatusBar"
    puts $config "$Config::showDebuginfo"
    puts $config "$Record::openTool"
    puts $config "$Config::freezeImages"
    puts $config "$Config::freezeTree"
    puts $config "$Config::participantHints"

    puts $config "$Attributes::color(0)"
    puts $config "$Attributes::color(1)"
    puts $config "$Attributes::lineWidth"
    puts $config "$ToolBar::currentTool"

    foreach id $Config::profiles {
         puts $config ""
         puts $config "$Config::profileParameter($id,name)"
         puts $config "$Config::profileParameter($id,ipAddr)" 
         puts $config "$Config::profileParameter($id,port)"  
         puts $config "$Config::profileParameter($id,ttl)"  
         puts $config "$Config::profileParameter($id,bandwidth)"  
         puts $config "$Config::profileParameter($id,cueRate)"  
         puts $config "$Config::profileParameter($id,localLag)"  
         puts $config "$Config::profileParameter($id,storageTime)"  
         puts $config "$Config::profileParameter($id,checkIntentionViolation)"  
         puts $config "$Config::profileParameter($id,logPageChanges)"  
    }     
              
    close $config
}


proc Config::Zoom { } {
    toplevel .zoom
    wm withdraw .zoom
    focus .zoom
    grab .zoom

    wm title .zoom "Zoom"
    catch {wm iconbitmap .zoom "../bin/mlb.ico"}
    wm resizable .zoom 0 0 
    wm protocol  .zoom WM_DELETE_WINDOW {destroy .zoom}
    wm geometry  .zoom +[winfo pointerx .]+[winfo pointery .]

    frame .zoom.f0
    label   .zoom.f0.l -text "Zoom:" -width 7 -justify left -anchor w
    SpinBox .zoom.f0.s -dragenabled false -dropenabled false -entrybg white -textvariable Config::zoom \
	    -width 5 -range {0 250 5} -editable 0 -justify left
    label   .zoom.f0.p -text "%" -width 2 -justify left -anchor w
    
    pack .zoom.f0.l .zoom.f0.s .zoom.f0.p -side left -padx 2 -pady 6
    pack .zoom.f0 -fill both

    frame .zoom.f1
    button .zoom.f1.b -text "Ok" -command "Config::SetZoom"  -width 8 -justify center -default active
    pack .zoom.f1.b -padx 2 -pady 2
    pack .zoom.f1 -fill both
    bind .zoom.f1.b <Return> {destroy .zoom}

    wm deiconify .zoom
}


proc Config::SetZoom { } {
    destroy .zoom
    MLB::SetZoom $Config::zoom
}


proc Config::SetInitialValues { } {
    MLB::SetZoom $Config::zoom
    MLB::FreezeImages $Config::freezeImages
    MLB::SetLogPageChanges $Session::logPageChanges
}


proc Config::OpenShared { } {
    variable openShared
    if {$openShared} {
	Session::OpenRTPI
    }
}


button .test
set Config::bgColor [.test cget -bg]
destroy .test

