##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval DebugInfo {
    variable textwindow " "
}

proc DebugInfo::create {} {
    variable textwindow
    set textwindow .debugInfo
    catch {destroy $textwindow}

    toplevel $textwindow
    wm title $textwindow "DebugInfos"
    catch {wm iconbitmap $textwindow "../bin/mlb.ico"}
    wm iconname $textwindow "DebugInfos"

    frame $textwindow.buttons
    pack $textwindow.buttons -side bottom -fill x -pady 2
    button $textwindow.buttons.memInfo -text "Print memory info" -command DebugInfo::PrintMemoryInfo
    pack $textwindow.buttons.memInfo -side left -expand 1
    button $textwindow.buttons.dismiss -text Dismiss -command DebugInfo::Hide
    pack $textwindow.buttons.dismiss  -side left -expand 1


    text $textwindow.text -relief sunken -bd 2 -yscrollcommand "$textwindow.scroll set" -setgrid 1 \
	    -height 30
    scrollbar $textwindow.scroll -command "$textwindow.text yview"
    pack $textwindow.scroll -side right -fill y
    pack $textwindow.text -expand yes -fill both

    $textwindow.text tag configure err -foreground red
    $textwindow.text tag configure blueinfo -foreground blue
    $textwindow.text tag configure debug -foreground green

    lower .debugInfo .
}

proc DebugInfo::Hide {} {
    set Config::showDebuginfo 0
    Toplevel::ShowHideDebuginfo
}

proc DebugInfo::info {info} {
    variable textwindow
    
    if {![winfo exists $textwindow.text]} {return}

    $textwindow.text insert end $info
    $textwindow.text see end
}


proc DebugInfo::infoBlue {info} {
    variable textwindow

    if {![winfo exists $textwindow.text]} {return}

    $textwindow.text insert end $info blueinfo
    $textwindow.text see end
}


proc DebugInfo::debug {info} {
    variable textwindow

     if {![winfo exists $textwindow.text]} {return}

   $textwindow.text insert end $info debug
    $textwindow.text see end
}


proc DebugInfo::error {err} {
    variable textwindow

    if {![winfo exists $textwindow.text]} {return}

    $textwindow.text insert end $err err
    $textwindow.text see end

}

proc DebugInfo::errorPopup {err} {
    tk_messageBox -default ok -message "$err" -title "Error" -icon error -type ok
}

proc DebugInfo::PrintMemoryInfo {} {

    # variables that are allways defined
    set globs "tkPriv.:0 tcl_version argv argv0 tcl_interactive tcl_traceCompile tk_library tk_version errorCode auto_path tclDefaultLibrary et_version tk_strictMotif errorInfo Et_EvalTrace tkPriv auto_index env tcl_patchLevel progPath argc tk_patchLevel tcl_libPath tcl_traceExec tcl_platform tcl_library"
    set cur_globs [::info globals]
    set our_globs [lDeleteList $cur_globs $globs]

    # variables
    DebugInfo::info "Number of non Tcl global variables: [llength $our_globs] \n"
    foreach val [lsort $our_globs] {
        DebugInfo::info "$val \n"
    }

    # images
    DebugInfo::info "Number of images: [llength [image names]] \n"
    foreach val [lsort [image names]] {
        if { [string match "imgTag*" $val] } {
            DebugInfo::info "$val \n"
        }
    }
    catch {
        DebugInfo::info [memory info]
    }
}

proc DebugInfo::lDeleteList { list value_list } {

    set res $list;
    foreach val $value_list {
        set res [lDelete $res $val]
    }
    return $res
}


proc DebugInfo::lDelete { list value } {

    set ix [lsearch -exact $list $value]
    if { $ix >=0 } {
        return [lreplace $list $ix $ix]
    } else {
        return $list
    }
}
