##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval Feedback {
    variable openTool
    variable question
    variable parentWindow
    variable qlist
    variable ui_font_tiny   "-*-helvetica-medium-r-normal-*-8-*-*-*-p-*-iso8859-1"
    variable ui_font_small  "-*-helvetica-medium-r-normal-*-10-*-*-*-p-*-iso8859-1" 
    variable tmp

    set openTool 0
    set question(nbr) 0
    set qlist " "
}


proc Feedback::Tool {} {
    variable openTool

    if {$openTool} {
	Feedback::OpenTool
    } else {
	Feedback::CloseTool
    }
} 


proc Feedback::OpenTool { } {
    FT::OpenRTPI
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Feedback::CloseTool { } {
    FT::CloseRTPI
    set openTool 0
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Feedback::Create {parent} {
    variable question
    variable parentWindow

    set sw [ScrolledWindow $parent.sw]
    set sf [ScrollableFrame $parent.sw.f]
    set parentWindow [$sf getframe]
    $sw setwidget $sf

    frame $parentWindow.top
    label $parentWindow.top.l -text "Feedback Tool" -anchor w

    Button $parentWindow.top.admin -image adminquestion -anchor w -relief flat -bd 1 -command {Feedback::QuestionDialog}
    DynamicHelp::register $parentWindow.top.admin balloon "Manage Feedback Question"
    HighlightWidget $parentWindow.top.admin
   
    canvas $parentWindow.canvas -height 120 -width 160 -bg white -relief sunken

    frame $parentWindow.s
    for {set idx 1} {$idx <= $question(nbr)} {incr idx} {
	if {($question($idx,visible)) && (!$question($idx,deleted))} {
	    CreateScale $idx
	}
    }

    pack $parentWindow.top.l -anchor w -side left
    pack $parentWindow.top.admin -anchor e -side right
    pack $parentWindow.top -fill x -anchor w		

    pack $parentWindow.canvas
    pack $sw  -side top  -expand yes -fill both -anchor w

    pack $parentWindow.s  -fill x -anchor w	
    
    set openTool 1

    return $parentWindow
}


proc Feedback::CreateScale {idx} {
    variable parentWindow
    variable question
    variable ui_font_small

    frame $parentWindow.s.sf($idx) 
	
    scale $parentWindow.s.sf($idx).s\
	    -orient horizontal\
	    -from 1\
	    -to $question($idx,nbrOfResps)\
	    -width 8\
	    -length 90\
	    -sliderlength 10\
	    -activebackground blue\
	    -showvalue 0\
	    -bigincrement 1\
	    -tickinterval 1 \
	    -font $ui_font_small

    $parentWindow.s.sf($idx).s set $question($idx,myAnswer)

    button $parentWindow.s.sf($idx).b \
	    -relief flat\
	    -width 8\
	    -command "Feedback::ShowScaleMeaning $idx"\
	    -text $question($idx,text) \
	    -font $ui_font_small \
	    -activebackground lightgrey

    DynamicHelp::register $parentWindow.s.sf($idx).b balloon "Question Details"

    pack $parentWindow.s.sf($idx) -padx 2
    pack $parentWindow.s.sf($idx).s -side right
    bind $parentWindow.s.sf($idx).s <ButtonRelease-1> "Feedback::ChangeAnswer $idx"
	    
    pack $parentWindow.s.sf($idx).b -side left
}


proc Feedback::DestroyScale {idx} {
    variable parentWindow
    pack forget $parentWindow.s.sf($idx)
}


proc Feedback::ShowScaleMeaning {idx} {
    variable question

    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Question Details"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +[winfo pointerx .]+[winfo pointery .]
    
    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background white -justify left -width 20 
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    .question.t.e insert 0 $question($idx,text)
    pack   .question.t -pady 4 

    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	frame  .question.a$i 
	label  .question.a$i.l -text "$i:" -width 3 -justify left -anchor w
	entry  .question.a$i.e -background white -justify left -width 15 
	pack   .question.a$i.l .question.a$i.e -side left -padx 2 -pady 2
	.question.a$i.e insert 0 $question($idx,resp,$i)
	pack   .question.a$i -fill both -expand yes
    }
   
    frame  .question.b
    button .question.b.b1 -text "Dismiss" -command "destroy .question"
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Feedback::ShowResults {} {
    variable question
    variable parentWindow
    variable ui_font_tiny

    if {![winfo exists $parentWindow]} {return}

    $parentWindow.canvas delete obj

    $parentWindow.canvas create line 17 10 17 105 -width 2 -tags obj
    $parentWindow.canvas create line 17 105 160 105 -width 2 -tags obj

    for {set i 0} {$i < 7} {incr i} {
	set y [expr 105 -($i*15)]
	$parentWindow.canvas create line 15 $y 17 $y -width 1 -tags obj
	$parentWindow.canvas create text 12 $y \
		-text [expr $i + 1] -anchor e\
		-font $ui_font_tiny -tags obj
    }
	    
    set nbrOfQuestionsToShow 0
    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {($question($i,visible)) && (!$question($i,deleted))} {
	    incr nbrOfQuestionsToShow
	}
    }
    if {$nbrOfQuestionsToShow == 0} {return}

    set slice [expr 143 / $nbrOfQuestionsToShow]
    set bar [expr $slice * 0.67]
    set space [expr $slice - $bar]
    set width [expr $bar / 2]

    set j 1
    set pos 0
    for {set i 0} {$i < $nbrOfQuestionsToShow} {incr i 1} {
	# 1. find next visible question
	while {$question($j,deleted) || ($question($j,visible) == 0)} {incr j 1}

	# 2. create description of x-axis
	set x [expr 17 + $space + $width + $i*$slice]
	set txt $question($j,text)
	$parentWindow.canvas create text $x 109 \
		-text $txt\
		-anchor n\
		-width $slice\
		-font $ui_font_tiny\
		-justify left -tags obj

	$parentWindow.canvas create line $x 105 $x 107 -width 1 -tags obj

	# 3. create result rectangle
	set xpos [expr 17 + $space + $width + $pos*$slice]
	set ypos 105
	set myy [expr $ypos - (($question($j,nbrOfResps) - 1) * 15)]
	set topy [expr $ypos - (($question($j,result) - 1) * 15)]
	set leftx [expr $xpos - $width]
	set rightx [expr $xpos + $width]

	$parentWindow.canvas create rectangle\
		$leftx $ypos $rightx $myy \
		-outline black -tags obj
	
	if {$question($j,result) > 1} {
	    $parentWindow.canvas create rectangle \
		    $leftx $ypos $rightx $topy \
		    -fill blue -tags obj
	}
	
	incr pos 1
	incr j 1
    }
}


proc Feedback::QuestionDialog { } {
    variable qlist
    variable question

    toplevel .feedback
    wm withdraw .feedback
    focus .feedback
    wm title .feedback "Feedback"
    catch {wm iconbitmap .feedback "../bin/mlb.ico"}
    wm resizable .feedback 0 0 
    wm geometry .feedback +[winfo pointerx .]+[winfo pointery .]

    label .feedback.l -text "Feedback Questions:" -anchor w
    pack  .feedback.l -pady 2

    frame .feedback.m
    set sw [ScrolledWindow .feedback.m.sw]
    set qlist [ListBox $sw.lb -height 5 -width 15 -highlightthickness 0]

    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {!$question($i,deleted)} {
	    if {$question($i,visible)} {
		$qlist insert $i $question($i,id) -text "$question($i,text)" -image checked
	    } else {
		$qlist insert $i $question($i,id) -text "$question($i,text)" -image unchecked
	    }
	}
    }

    $qlist bindImage <ButtonPress-1> "Feedback::ChangeQuestionVisibility"
    $qlist bindText  <ButtonPress-1> "Feedback::Select"

    $sw setwidget $qlist
    pack $sw -fill both -expand yes -side left -padx 2
    pack   .feedback.m

    frame  .feedback.b
    Button .feedback.b.b1 -image new  -anchor w -relief flat -bd 1 -command "Feedback::ResponseNbrDialog"
    DynamicHelp::register .feedback.b.b1 balloon "Create New Question"
    HighlightWidget .feedback.b.b1

    Button .feedback.b.b2 -image edit  -anchor w -relief flat -bd 1 -command "Feedback::EditQuestion"
    DynamicHelp::register .feedback.b.b2 balloon "Edit Selected Question"
    HighlightWidget .feedback.b.b2

    Button .feedback.b.b3 -image delete  -anchor w -relief flat -bd 1 -command "Feedback::DeleteQuestion"
    DynamicHelp::register .feedback.b.b3 balloon "Delete Selected Question"
    HighlightWidget .feedback.b.b3

    pack   .feedback.b.b1 .feedback.b.b2 .feedback.b.b3  -side left -padx 4
    pack   .feedback.b  -fill x -anchor w
    
    wm deiconify .feedback
}


proc Feedback::Select {id} {
    variable qlist
    
    $qlist selection set $id
}


proc Feedback::ChangeQuestionVisibility {id} {
    variable parentWindow
    variable question
    variable qlist

    set selected -1
    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {(!$question($i,deleted)) && ($question($i,id) == $id)} {
	    set selected $i
	    break
	}
    }
    if {$selected == -1} {return}

    if {$question($selected,visible)} {
	set question($selected,visible) 0
	$qlist itemconfigure $id -image unchecked
    } else {
	set question($selected,visible) 1
	$qlist itemconfigure $id -image checked
    }

    FT::ChangeQuestionVisibility $id $question($selected,visible)

    pack forget $parentWindow.s
    destroy $parentWindow.s

    frame $parentWindow.s
    for {set idx 1} {$idx <= $question(nbr)} {incr idx} {
	if {($question($idx,visible)) && (!$question($idx,deleted))} {
	    CreateScale $idx
	}
    }
    pack $parentWindow.s

    ShowResults
}


proc Feedback::ResponseNbrDialog {} {
    variable tmp
    
    toplevel .rspdlg
    wm withdraw .rspdlg
    focus .rspdlg
    grab .rspdlg
    wm title .rspdlg "Feedback"
    catch {wm iconbitmap .rspdlg "../bin/mlb.ico"}
    wm resizable .rspdlg 0 0 
    wm geometry .rspdlg +[winfo pointerx .]+[winfo pointery .]

    frame   .rspdlg.top
    label   .rspdlg.top.l -text "Number of responses:" -width 17 -justify left -anchor w
    SpinBox .rspdlg.top.s -dragenabled false -dropenabled false -entrybg white -textvariable Feedback::tmp \
	    -width 5 -values {2 3 4 5 6 7} -editable 0 -justify left\
	    -helptext "Specify the number of responses\npossible for this question"
    pack .rspdlg.top.l .rspdlg.top.s -side left -padx 2 -pady 2
    pack .rspdlg.top
    
    frame  .rspdlg.b
    button .rspdlg.b.b1 -text "OK" -command "Feedback::CreateQuestion" -width 5
    button .rspdlg.b.b2 -text "Cancel" -command "destroy .responsenbr" -width 5
    pack   .rspdlg.b.b1 .rspdlg.b.b2 -side left -pady 2 -padx 2
    pack   .rspdlg.b -pady 10

    wm deiconify .rspdlg
}


proc Feedback::CreateQuestion {} {
    variable tmp

    regexp {\+([0-9]+)\+([0-9]+)} [wm geometry .rspdlg] m x y
    destroy .rspdlg

    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Create Question"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +$x+$y
    
    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background white -justify left -width 20
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    pack   .question.t -pady 4 

    for {set i 1} {$i <= $tmp} {incr i} {
	frame  .question.a$i 
	label  .question.a$i.l -text "$i:" -width 3 -justify left -anchor w
	entry  .question.a$i.e -background white -justify left -width 15
	pack   .question.a$i.l .question.a$i.e -side left -padx 2 -pady 2
	pack   .question.a$i -fill both -expand yes
    }
   
    frame  .question.b
    button .question.b.b1 -text "OK" -command "Feedback::CreateQuestionOK" -width 5
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Feedback::CreateQuestionOK {} {
    variable tmp

    for {set i 1} {$i <= $tmp} {incr i} {
	set t [.question.a$i.e get] 
	append t " "
	lappend q $t
    }

    FT::CreateQuestion [.question.t.e get] $tmp $q

    destroy .question
}


proc Feedback::EditQuestion {} {
    variable qlist
    variable question

    set id [lindex [$qlist selection get] 0]
    set selected -1
    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {(!$question($i,deleted)) && ($question($i,id) == $id)} {
	    set selected $i
	    break
	}
    }
    if {$selected == -1} {return}

    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Edit Question"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +[winfo pointerx .]+[winfo pointery .]
    
    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background white -justify left -width 20
    .question.t.e insert 0 $question($selected,text)
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    pack   .question.t -pady 4 

    for {set i 1} {$i <= $question($selected,nbrOfResps)} {incr i} {
	frame  .question.a$i 
	label  .question.a$i.l -text "$i:" -width 3 -justify left -anchor w
	entry  .question.a$i.e -background white -justify left -width 15
	.question.a$i.e insert 0 $question($selected,resp,$i)
	pack   .question.a$i.l .question.a$i.e -side left -padx 2 -pady 2
	pack   .question.a$i -fill both -expand yes
    }
   
    frame  .question.b
    button .question.b.b1 -text "OK" -command "Feedback::EditQuestionOK $selected" -width 5
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Feedback::EditQuestionOK {idx} {
    variable qlist
    variable parentWindow
    variable question

    # 1. update variables
    set question($idx,text) [.question.t.e get]
    
    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	set question($idx,resp,$i) [.question.a$i.e get]
    }

    # 2. update gui
    $qlist itemconfigure $question($idx,id) -text $question($idx,text)

    pack forget $parentWindow.s
    destroy $parentWindow.s
    
    frame $parentWindow.s
    for {set i 1} {$i <= $question(nbr)} {incr i 1} {
	if {($question($i,visible)) && (!$question($i,deleted))} {
	    CreateScale $i
	}
    }
    pack $parentWindow.s -after $parentWindow.canvas -fill x -anchor w	
    
    ShowResults

    # 3. update c++
    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	set t $question($idx,resp,$i)
	append t " "
	lappend q $t
    }

    FT::EditQuestion $question($idx,id) $question($idx,text) $question($idx,nbrOfResps) $q
    
    destroy .question
}


proc Feedback::DeleteQuestion {} {
    variable qlist
    variable parentWindow
    variable question

    set id [lindex [$qlist selection get] 0]
    set idx -1
    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {!($question($i,deleted)) && ($id == $question($i,id))} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} return

    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "Delete" -default 1 -cancel 1\
	    -message "Delete question $question($idx,text)?" -icon question -justify center -type yesno] {
	return
    }

    FT::DeleteQuestion $question($idx,id)

    set question($idx,deleted) 1
    unset question($idx,text)
    unset question($idx,result)
    unset question($idx,myAnswer)
    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	unset question($idx,resp,$i) 
    }
    unset question($idx,nbrOfResps)

    if [winfo exists $qlist] {
	$qlist delete $question($idx,id)
    }

    if [winfo exists $parentWindow] {
	pack forget $parentWindow.s
	destroy $parentWindow.s
	
	frame $parentWindow.s
	for {set i 1} {$i <= $question(nbr)} {incr i} {
	    if {!($question($i,deleted)) && $question($i,visible)} {
		CreateScale $i
	    }
	}
	pack $parentWindow.s -after $parentWindow.canvas -fill x -anchor w	
	
	ShowResults
    }
}


proc Feedback::RemoteDeleteQuestion {id} {
    variable question
    variable qlist
    variable parentWindow

    set idx -1
    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {!($question($i,deleted)) && ($id == $question($i,id))} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} return

    set question($idx,deleted) 1
    unset question($idx,text)
    unset question($idx,result)
    unset question($idx,myAnswer)
    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	unset question($idx,resp,$i) 
    }
    unset question($idx,nbrOfResps)

    if [winfo exists $qlist] {
	$qlist delete $question($idx,id)
    }

    if [winfo exists $parentWindow] {
	pack forget $parentWindow.s
	destroy $parentWindow.s
	
	frame $parentWindow.s
	for {set i 1} {$i <= $question(nbr)} {incr i} {
	    if {!($question($i,deleted)) && $question($i,visible)} {
		CreateScale $i
	    }
	}
	pack $parentWindow.s -after $parentWindow.canvas -fill x -anchor w	
	
	ShowResults
    }
}


proc Feedback::AddQuestion {id text visible nbrOfResps resplist} {
    variable question
    variable qlist
    variable parentWindow

    incr question(nbr)
    set idx $question(nbr)

    set question($idx,id) $id
    set question($idx,deleted) 0
    set question($idx,text) $text
    set question($idx,visible) $visible
    set question($idx,nbrOfResps) $nbrOfResps
    set question($idx,result) 1
    set question($idx,myAnswer) 1

    set i 1
    foreach resp $resplist {
	set question($idx,resp,$i) $resp
	incr i
    }

    if {$question($idx,visible)} {
	if [winfo exists $qlist] {
	    $qlist insert $idx $question($idx,id) -text "$question($idx,text)" -image checked
	}
	
	if [winfo exists $parentWindow] {
	    pack forget $parentWindow.s
	    destroy $parentWindow.s
	    
	    frame $parentWindow.s
	    for {set idx 1} {$idx <= $question(nbr)} {incr idx} {
		if {!($question($idx,deleted)) && $question($idx,visible)} {
		    CreateScale $idx
		}
	    }
	    pack $parentWindow.s -after $parentWindow.canvas -fill x -anchor w	

	    ShowResults
	}

    } else {
	if [winfo exists $qlist] {
	    $qlist insert $idx $question($idx,id) -text "$question($idx,text)" -image unchecked
	}
    }
}


proc Feedback::UpdateQuestion {id text visible nbrOfResps resplist} {
    variable question
    variable qlist
    variable parentWindow

    set idx -1
    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {!($question($i,deleted)) && ($id == $question($i,id))} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} return

    # 1. update variables
    set question($idx,text) $text
    set question($idx,visible) $visible
    set question($idx,nbrOfResps) $nbrOfResps

    set i 1
    foreach resp $resplist {
	set question($idx,resp,$i) $resp
	incr i
    }

    # 2. update gui
    if [winfo exists $qlist] {
	$qlist itemconfigure $question($idx,id) -text $question($idx,text)
    }

    if [winfo exists $parentWindow] {
	pack forget $parentWindow.s
	destroy $parentWindow.s
	
	frame $parentWindow.s
	for {set idx 1} {$idx <= $question(nbr)} {incr idx} {
	    if {!($question($idx,deleted)) && $question($idx,visible)} {
		CreateScale $idx
	    }
	}
	pack $parentWindow.s -after $parentWindow.canvas -fill x -anchor w	
	
	ShowResults
    }
}


proc Feedback::ChangeAnswer {idx} {
    variable parentWindow
    variable question

    set question($idx,myAnswer) [$parentWindow.s.sf($idx).s get]

    FT::ChangeAnswer $question($idx,id) $question($idx,myAnswer)
}


proc Feedback::SetResult {id result} {
    variable parentWindow
    variable question

    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {!($question($i,deleted)) && ($id == $question($i,id))} {
	    set idx $i
	    break
	}
    }

    set question($idx,result) $result

    if {$question($idx,visible)} {
	if [winfo exists $parentWindow] {
	    ShowResults
	}
    }
}

proc Feedback::SetVisibility {id visible} {
    variable question
    variable parentWindow
    variable qlist

    for {set i 1} {$i <= $question(nbr)} {incr i} {
	if {!($question($i,deleted)) && ($id == $question($i,id))} {
	    set idx $i
	    break
	}
    }

    set question($idx,visible) $visible

    if {$question($idx,visible)} {
	if [winfo exists $qlist] {
	    $qlist itemconfigure $id -image checked
	}

    } else {
	if [winfo exists $qlist] {
	    $qlist itemconfigure $id -image unchecked
	}
    }

    if [winfo exists $parentWindow] {
	pack forget $parentWindow.s
	destroy $parentWindow.s
	
	frame $parentWindow.s
	for {set i 1} {$i <= $question(nbr)} {incr i} {
	    if {!($question($i,deleted)) && $question($i,visible)} {
		CreateScale $i
	    }
	}
	pack $parentWindow.s
	
	ShowResults
    }
}


proc Feedback::ClearAllQuestions { } {
    variable question

    for {set i 1} {$i <= $question(nbr)} {incr i} {
	unset question($i,deleted)
	unset question($i,id)
	unset question($i,visible)
	unset question($i,text)
	unset question($i,result)
	unset question($i,myAnswer)
	for {set j 1} {$j <= $question($i,nbrOfResps)} {incr j} {
	    unset question($i,resp,$j) 
	}
	unset question($i,nbrOfResps)
    }

    set question(nbr) 0
}