##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################


source bwidgets/scrollw.tcl

namespace eval Filename {
    variable lastname
    variable lastdir

    set lastname "noname.mlb"
    if {[info exists env(PWD)]} {
	set lastdir $env(PWD)
    } else {
	set lastdir "./"
    }
}

proc Filename::read {} {
    variable lastdir
    variable lastname

    set filename [tk_getOpenFile \
		      -title "Open document" \
		      -filetypes  {
			  {{MLB file} {.mlb} }
			  {{All files} * }
		      } \
		      -initialdir $lastdir \
		      -defaultextension ".mlb" ]

    if {$filename != ""} {
	set lastname [file tail $filename]
	set lastdir [file dirname $filename]
    }

    return $filename
}

proc Filename::open {lastname filetypes defaultextension} {
    variable lastdir

    set filename [tk_getOpenFile \
		      -title "Open document" \
		      -filetypes  $filetypes \
		      -initialdir $lastdir \
		      -defaultextension $defaultextension ]

    set lastdir [file dirname $filename]

    return $filename
}

proc Filename::save {lastname filetypes defaultextension} {
    variable lastdir

    set filename [tk_getSaveFile \
		      -title "Save document" \
		      -filetypes $filetypes \
		      -initialfile [file tail $lastname] \
		      -initialdir $lastdir \
		      -defaultextension $defaultextension ]

    set lastdir [file dirname $filename]

    return $filename
}


