##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval Grid {
    variable gridDialog .gridsel
    variable gridFrame 


    variable gridStatus 0
    variable snapStatus 0
    variable tickStatus 0

    variable gridSize 20
    variable snapSize 20
}

proc Grid::Set {} { 

    MLB::ReDraw 
}

proc Grid::Create {} {
    variable gridDialog
    variable gridFrame
    variable gridStatus
    variable snapStatus
    variable tickStatus
    variable gridSize
    variable snapSize
    
    Dialog $gridDialog -modal none
    set gridFrame [$gridDialog getframe]

    label $gridFrame._testing
    set gap [expr {[font metrics [$gridFrame._testing cget -font] -linespace]/2+1}]
    destroy $Grid::gridFrame._testing

    frame $gridFrame.border1 -bd 2 -relief ridge
    frame $gridFrame.border2 -bd 2 -relief ridge
    grid $gridFrame.border1 -row 2 -column 0 -rowspan 3 -columnspan 12 \
        -padx $gap -pady $gap -sticky nsew
    grid $gridFrame.border2 -row 5 -column 0 -rowspan 3 -columnspan 12 \
        -padx $gap -pady $gap -sticky nsew
    foreach col {0 11} {
	grid columnconfig  $gridFrame $col -minsize $gap
    }
    incr gap $gap
    foreach row {0 2 4 5 7} {
	grid rowconfig  $gridFrame $row -minsize $gap
    }
    
    label $gridFrame.grid_label -text GRID -padx 1m -pady 1
    label $gridFrame.snap_label -text SNAP -padx 1m -pady 1
    grid $gridFrame.grid_label -row 2 -column 2 -sticky w
    grid $gridFrame.snap_label -row 5 -column 2 -sticky w
    
    frame $gridFrame.gridsize
    frame $gridFrame.snapsize
    grid $gridFrame.gridsize -row 3 -column 1 -rowspan 1 -columnspan 10 -sticky nsew \
	-padx $gap
    grid $gridFrame.snapsize -row 6 -column 1 -rowspan 1 -columnspan 10 -sticky nsew \
	-padx $gap
    
    foreach {size row col} {
	5    0 0
	10   1 0
	15   2 0
	20   0 1
	25   1 1
	30   2 1
	35   0 2
	40   1 2
	45   2 2
	50   0 3
	55   1 3
	60   2 3
    } {
	set b $gridFrame.gridsize.but_${row}_${col}
	radiobutton $b -variable Grid::gridSize -value $size \
	    -text $size -anchor w
	grid $b -row $row -column $col -sticky ew
	set b $gridFrame.snapsize.but_${row}_${col}
	radiobutton $b -variable Grid::snapSize -value $size \
	    -text $size -anchor w
	grid $b -row $row -column $col -sticky ew
    }
    
    entry $gridFrame.gridsize.entry -textvariable gridSize -width 6 -bg white
    grid $gridFrame.gridsize.entry -row 0 -column 6 -rowspan 3 -sticky ew
    entry $gridFrame.snapsize.entry -textvariable snapSize -width 6 -bg white
    grid $gridFrame.snapsize.entry -row 0 -column 6 -rowspan 3 -sticky ew
    set colwidth [expr 1.25*[winfo reqwidth $gridFrame.gridsize.but_1_3]]
    foreach c {0 1 2 3} {
	grid columnconfig $gridFrame.gridsize $c -minsize $colwidth
	grid columnconfig $gridFrame.snapsize $c -minsize $colwidth
    }
    grid column $gridFrame.gridsize 6 -weight 1
    grid column $gridFrame.snapsize 6 -weight 1
    bind $gridFrame.gridsize.entry <Return> {Grid}
    
    frame $gridFrame.butnframe1
    grid $gridFrame.butnframe1 -row 0 -column 3 -rowspan 2 -columnspan 2 \
   	-sticky nsew -pady $gap
    checkbutton $gridFrame.butnframe1.gridon -text "Grid on" \
        -variable Grid::gridStatus -anchor w
    pack $gridFrame.butnframe1.gridon -side top -fill x -padx [expr {$gap/2}]
    checkbutton $gridFrame.butnframe1.snapon -text "Snap on" \
        -variable Grid::snapStatus -anchor w
    pack $gridFrame.butnframe1.snapon -side top -fill x -padx [expr {$gap/2}]
    checkbutton $gridFrame.butnframe1.tickson -text "Ticks on" \
        -variable Grid::tickStatus -anchor w
    pack $gridFrame.butnframe1.tickson -side top -fill x -padx [expr {$gap/2}]
    
    frame $gridFrame.butnframe2
    grid $gridFrame.butnframe2 -row 0 -column 5 -rowspan 2 -columnspan 2 \
   	-sticky ew -pady $gap
    button $gridFrame.butnframe2.ok -text Set -command {
        Grid::Set
    }
    pack $gridFrame.butnframe2.ok -side top -fill x -padx [expr {$gap/2}]
    button $gridFrame.butnframe2.cancel -text Close -command {
	$Grid::gridDialog withdraw
    }
    pack $gridFrame.butnframe2.cancel -side top -fill x -padx [expr {$gap/2}]
    bind $gridFrame <FocusOut> "raise $gridFrame; focus -force $Grid::gridFrame"
}

proc Grid::Selector {} {
    variable gridDialog

    if {![winfo exists $gridDialog]} {
	Grid::Create
	$gridDialog draw none
    } else {
	wm deiconify $gridDialog
    }
    
    
}

proc Grid::Draw {canvas width height} {
    variable grid 
    
    $canvas delete gridObject
    $canvas delete gridLine

    set g [expr $Grid::gridSize * $Config::zoom / 100]
    set size [winfo fpixels $canvas $g]
    set n [expr int($height/$size)]
    set m [expr int($width/$size)]

    for {set i 1} {$i<=$m} {incr i} {
	set Xi [expr $i*$size]
	$canvas create line $Xi 0 $Xi $height \
	    -width 1.5 \
	    -stipple gray12 \
	    -tags {gridLine gridObject}
    }

    for {set i 1} {$i<=$n} {incr i} {
	set Yi [expr $i*$size]
	$canvas create line 0 $Yi $width $Yi \
	    -stipple gray12 \
	    -width 1.5 \
	    -tags {gridLine gridObject}
    }
}

proc Grid::Ticks {canvas width height} { 
    variable gridSize 

    $canvas delete gridText gridTick
    
    set size [winfo fpixels $canvas $Grid::gridSize]
    
    if {$size<2} {
	return
    }
    
    set m [expr int($width/$size)]
    set n [expr int($height/$size)]
    
    for {set i 1} {$i<=$m} {incr i} {
	set Xnum [expr $i*$size+3]
	set Xtick [expr $i*$size]
	$canvas create text $Xnum 3 \
	    -text $i \
	    -fill red \
	    -anchor nw \
	    -tags {gridText gridObject}
	$canvas create line $Xtick 0 $Xtick 4 \
	    -width 2 \
	    -fill red \
	    -tags {gridTick gridObject}
    }
    
    for {set i 1} {$i<=$n} {incr i} {
	set Ynum [expr $i*$size+3]
	set Ytick [expr $i*$size]
	$canvas create text 3 $Ynum\
	    -text $i \
	    -fill red \
	    -anchor nw \
	    -tags {gridText gridObject}
	$canvas create line 0 $Ytick 4 $Ytick \
	    -width 2 \
	    -fill red \
	    -tags {gridTick gridObject}
    }
    MLB::ShowTicks 0 0 0
}

