##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

source bwidgets/titleframe.tcl
source bwidgets/scrollw.tcl
source bwidgets/dynhelp.tcl


namespace eval Hrt {
    variable raisehand
    variable lowerhand
    variable call
    variable indicate
    variable participantList
    variable openTool

    set raisehand(state) "normal"
    set lowerhand(state) "disabled"
    set call(state) "disabled"
    set indicate(mode) 1
    
    set openTool 0
}


proc Hrt::Tool {} {
    variable openTool

    if {$openTool} {
	Hrt::OpenTool
    } else {
	Hrt::CloseTool
    }
} 


proc Hrt::OpenTool { } {
    variable raisehand

    HRT::OpenRTPI
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
    
    set $raisehand(state) disabled
    $raisehand(button) configure -state disabled
}


proc Hrt::CloseTool { } {
    variable openTool
    variable raisehand

    if {[string compare $raisehand(state) "disabled"] == 0} {
	Hrt::LowerHand
    }
    
    HRT::CloseRTPI
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Hrt::Create { parent } {
    variable raisehand
    variable lowerhand
    variable call
    variable indicate
    variable participantList

    frame $parent.top
    label $parent.top.l -text "Hand-Raising Tool" -anchor w

    set indicate(button) [Button $parent.top.ind -image indicate -anchor w -relief sunken -bg LightGrey -bd 1 -command {Hrt::ChangeIndicate}]
    DynamicHelp::register $indicate(button) balloon "Open a message window if \na participant raises his hand"
    HighlightWidget $indicate(button)

    set sw [ScrolledWindow $parent.participantsw]
    set participantList [listbox $sw.participantList -selectmode single -bg white -height 3 -width 8 -relief sunken]
    $sw setwidget $participantList

    frame $parent.bottom
    
    set raisehand(button) [Button $parent.bottom.raise -image raisehand -anchor w -relief flat -bd 1 -state $raisehand(state) -command {Hrt::RaiseHand}]
    DynamicHelp::register $raisehand(button) balloon "Raise Hand"
    HighlightWidget $raisehand(button)

    set lowerhand(button) [Button $parent.bottom.lower -image lowerhand -anchor w -relief flat -bd 1 -state $lowerhand(state) -command {Hrt::LowerHand}]
    DynamicHelp::register $lowerhand(button) balloon "Lower Hand"
    HighlightWidget $lowerhand(button)

    set call(button) [Button $parent.bottom.call -image call -anchor w -relief flat -bd 1 -state $call(state) -command {Hrt::Call}]
    DynamicHelp::register $call(button) balloon "Call selected participant"
    HighlightWidget $call(button)

    pack $parent.top.l -anchor w -side left
    pack $parent.top.ind -anchor e -side right
    pack $parent.top -fill x -anchor w		       

    pack $sw  -side top  -expand yes -fill both -anchor w

    pack $parent.bottom.raise -anchor w -side left
    pack $parent.bottom.lower -anchor w -side left
    pack $parent.bottom.call  -anchor e -side right
    pack $parent.bottom -fill x -anchor w		       

    return $participantList
}


proc Hrt::ChangeIndicate { } {
    variable indicate
 
    if {!$indicate(mode)} {
	HRT::SetIndicate 1
	set indicate(mode) 1
	$indicate(button) configure -relief "sunken" -bg LightGrey
    } else {
	HRT::SetIndicate 0
	set indicate(mode) 0
	$indicate(button) configure -relief "flat" -bg $Config::bgColor
    }

}


proc Hrt::RaiseHand { } {
    variable raisehand
    variable lowerhand

    set raisehand(state) "disabled"
    set lowerhand(state) "normal"

    $raisehand(button) configure -state disabled
    $lowerhand(button) configure -state normal

    HRT::RaiseHand
}


proc Hrt::LowerHand { } {
    variable raisehand
    variable lowerhand

    set raisehand(state) "normal"
    set lowerhand(state) "disabled"

    $raisehand(button) configure -state normal
    $lowerhand(button) configure -state disabled

    HRT::LowerHand
}


proc Hrt::Call { } {
    variable participantList

    set selected [$participantList curselection]
    if {[llength $selected] == 0} {
	return
    }

    HRT::CallParticipant [lindex [$participantList curselection] 0]
}


proc Hrt::ClearParticipantList { } {
    variable participantList
    #variable raisehand
    #variable lowerhand
    #variable call

    #set raisehand(state) "normal"
    #set lowerhand(state) "disabled"
    #set call(state) "disabled"

    #$raisehand(button) configure -state normal
    #$lowerhand(button) configure -state disabled
    #$call(button) configure -state disabled

    $participantList delete 0 end
}


proc Hrt::DisplayParticipantList { participants } {
    variable participantList
    variable call

    if {[llength $participants] == 0} {
	$call(button) configure -state disabled
	set call(state) "disabled"
    } else {
	$call(button) configure -state normal
	set call(state) "normal"
    }

    $participantList delete 0 end

    foreach participant $participants {
	$participantList insert end $participant
    }
}


proc Hrt::CallDialog { participant } {
    variable raisehand
    variable lowerhand

    set raisehand(state) "normal"
    set lowerhand(state) "disabled"

    $raisehand(button) configure -state normal
    $lowerhand(button) configure -state disabled

    destroy .msgdlg
    MessageDlg .msgdlg -parent . -title "Hand-Raising Tool"  -cancel 1\
	    -message "You have been called by $participant" -icon info -justify center -type ok
}


proc Hrt::RaiseHandDialog { participant } {
    destroy .msgdlg
    MessageDlg .msgdlg -parent . -title "Hand-Raising Tool"  -cancel 1\
	    -message "Participant $participant has raised his hand" -icon info -justify center -type ok
}


proc Hrt::EnableGUI { } {
    variable raisehand
    set $raisehand(state) normal
    $raisehand(button) configure -state normal
}