/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb
 authors : Gabriela Maass, Juergen Vogel
 date    : 18.02.2000
 desc    : interface-functions between tcl and c

 **********************************************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#ifdef WIN32
#pragma warning(disable: 4786)
#include <sys/types.h>
#include <winsock.h>
#include <windows.h>
#endif

#include <iostream>
#include <sys/stat.h>
#include <fcntl.h>

#include "tcl.h"
#include "tk.h"

#include "estring.h"

#include "alt_manager.h"
#include "chat_manager.h"
#include "hrt_manager.h"
#include "ft_manager.h"
#include "vt_manager.h"

#include "UCE_Interface.h"
#include "mlb_uce.h"

#include "mlb.h"
#include "mlb_adu.h"
#include "mlb_communication.h"
#include "mlb_gui.h"
#include "mlb_objects.h"
#include "mlb_participants.h"
#include "mlb_session.h"
//#include "mlb_tcpip.h"
#include "mlb_timermanager.h"
#include "mlb_undomanager.h"

static char* docName = NULL;


#define xIDSERVER


//-------------------------------------------------------------------------//

/**   Macro checking the active page existence
  **/
#define CHECK_PAGE( page )                            \
  if( NULL==(page) ) {                                \
    Et_ResultF( interp, "No active page found" );     \
    return TCL_ERROR;                                 \
  }

//-------------------------------------------------------------------------//

/**   Macro checking the active session existence
  **/
#define CHECK_SESSION( session )                      \
  if( NULL==(session) ) {                             \
    Et_ResultF( interp, "No active session found" );  \
    return TCL_ERROR;                                 \
  }

//-------------------------------------------------------------------------//

/**   Macro checking parameter count
  **/
#define CHECK_PARAM_COUNT( argc, num )                    \
  if( (argc)<(num) ) {                                    \
    Et_ResultF( interp, ("To few parameters. Expected: "  \
    #num                                                  \
    ". Got: "+                                            \
    estring(argc)).c_str() );                             \
    return TCL_ERROR;                                     \
  }

//-------------------------------------------------------------------------//

/*** CREATE DRAW_OBJECTS ***/

int ET_COMMAND_MLB__CreateFreehand(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::FREEHAND);

  return TCL_OK;
}

int ET_COMMAND_MLB__CreatePolyline(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::POLYLINE);

  return TCL_OK;
}

int ET_COMMAND_MLB__CreatePolygon(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::POLYGON);

  return TCL_OK;
}

int ET_COMMAND_MLB__CreateRectangle(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::RECTANGLE);

  return TCL_OK;
}

int ET_COMMAND_MLB__CreateOval(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::OVAL);

  return TCL_OK;
}

int ET_COMMAND_MLB__CreateLine(ET_TCLARGS)
{
  MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::LINE);

  return TCL_OK;
}

int ET_COMMAND_MLB__CreateText(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->AppendNewObject(argc,argv,MLB::GenericObject::ASCIITEXT);

  return TCL_OK;
}

int ET_COMMAND_MLB__LoadImage(ET_TCLARGS) {
  bool ret = false;
  if (MLB::SessionManager::sm->GetActiveSession())
    ret = MLB::SessionManager::sm->GetActiveSession()->LoadImage(argc,argv);

  if( ret ) {
    Et_ResultF( interp, "OK" );
    return TCL_OK;
  }
  else {
    Et_ResultF( interp, "ERROR" );
    return TCL_OK;
  }
}

int ET_COMMAND_MLB__PlaceImage(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->PlaceImage(argc,argv);

  return TCL_OK;
}


int ET_COMMAND_MLB__CreateThumbnails(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->CreateThumbnails(argc,argv);

  return TCL_OK;
}


/*** CHANGE OBJECTS ***/

int ET_COMMAND_MLB__Backspace(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::BACKSPACE);

  return TCL_OK;
}

int ET_COMMAND_MLB__NewLine(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->NewTextLine(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__GoLeft(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::LEFT);

  return TCL_OK;
}

int ET_COMMAND_MLB__GoRight(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::RIGHT);

  return TCL_OK;
}

int ET_COMMAND_MLB__GoUp(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::UP);

  return TCL_OK;
}

int ET_COMMAND_MLB__GoDown(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::DOWN);

  return TCL_OK;
}

int ET_COMMAND_MLB__GoEnd(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::END);

  return TCL_OK;
}

int ET_COMMAND_MLB__GoHome(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::HOME);

  return TCL_OK;
}

int ET_COMMAND_MLB__Delete(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->DeleteObject(argc,argv);

//    // update animations
//    MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
//    if( NULL==page ) return TCL_ERROR;
//    page->GetAnimations().Update();
//    if( Et_GlobalEvalF( Et_Interp,"Animations::update" ) != TCL_OK )
//    GUI::MessageWindow::Error( "Animations::update - tcl error %q\n", Et_Interp->result );

  return TCL_OK;
}

int ET_COMMAND_MLB__OnlinePointer(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ActivateOnlinePointer(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__OnlineBox(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ActivateOnlineBox(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__UpdateText(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeObject(argc,argv,MLB::Text::INSERT);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeFont(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeFont(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeOutlineColor(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeLineColor(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeFillColor(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeFillColor(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeLineWidth(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeLineWidth(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeLineStyle(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeLineStyle(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeCapStyle(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeCapStyle(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeJoinStyle(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeJoinStyle(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeArrowStyle(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeArrowStyle(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeArrowShape(ET_TCLARGS)
{

  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ChangeArrowShape(argc,argv);

  return TCL_OK;
}

/*** SELECT OBJECTS ***/

int ET_COMMAND_MLB__UnSelectObjects(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->UnSelectObjects(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__SelectObjects(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SelectObjects(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__SelectThumbnail(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SelectThumbnail(argc,argv);
  return TCL_OK;
}


/*** MOVE OBJECTS ***/

int ET_COMMAND_MLB__Move(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->MoveObjects(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__Copy(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->CopyObjects(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__RaiseObjects(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->RaiseObjects();

  return TCL_OK;
}

int ET_COMMAND_MLB__LowerObjects(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->LowerObjects();

  return TCL_OK;
}

/*** CONTAINER ***/

int ET_COMMAND_MLB__SaveContainer(ET_TCLARGS)
{
  MLB::SessionManager::sm->SaveContainer(argv[1]);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeContainerName(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession()) {
    Tcl_DString name;
    Tcl_UtfToExternalDString(NULL, argv[2], -1, &name);
    MLB::SessionManager::sm->GetActiveSession()->ChangeContainerName(argv[1], Tcl_DStringValue(&name));
    MLB::SessionManager::sm->DrawPageTree();
    Tcl_DStringFree(&name);
  }

  return TCL_OK;
}

int ET_COMMAND_MLB__SelectContainer(ET_TCLARGS)
{
  MLB::SessionManager::sm->SelectContainer(argv[1]);

  return TCL_OK;
}

int ET_COMMAND_MLB__InsertNewPage(ET_TCLARGS)
{
  MLB::SessionManager::sm->InsertNewPage(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__InsertNewFolder(ET_TCLARGS)
{
  MLB::SessionManager::sm->InsertNewFolder(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__DuplicatePage(ET_TCLARGS)
{
  MLB::SessionManager::sm->DuplicatePage();

  return TCL_OK;
}

int ET_COMMAND_MLB__DeletePages(ET_TCLARGS)
{
  MLB::SessionManager::sm->DeletePages(atoi(argv[1]), atoi(argv[2]));

  return TCL_OK;
}

int ET_COMMAND_MLB__SetPageDimension(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SetPageDimension(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__SetPageColor(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SetPageColor(argc, argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ReDraw(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->Draw();

  return TCL_OK;
}

int ET_COMMAND_MLB__ReDrawPage(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActivePage())
    MLB::SessionManager::sm->GetActivePage()->ReDraw();

  return TCL_OK;
}

int ET_COMMAND_MLB__ShowTicks(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->ShowTicks(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeHierachy(ET_TCLARGS)
{
  MLB::SessionManager::sm->ChangeHierachy(atoi(argv[1]), atoi(argv[2]), argv[3]);

  return TCL_OK;
}

int ET_COMMAND_MLB__DeleteContainer(ET_TCLARGS)
{
  MLB::SessionManager::sm->DeleteContainer(argv[1]);

  return TCL_OK;
}

int ET_COMMAND_MLB__CutObjects(ET_TCLARGS)
{

  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->CutObjects();

  return TCL_OK;
}

int ET_COMMAND_MLB__CopyObjects(ET_TCLARGS)
{

  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->CopyObjects();

  return TCL_OK;
}

int ET_COMMAND_MLB__PasteObjects(ET_TCLARGS)
{

  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->PasteObjects();

  return TCL_OK;
}

/*** CORRECTION ***/

int ET_COMMAND_MLB__Undo(ET_TCLARGS)
{
  MLB::UndoManager::um->Undo();
  return TCL_OK;
}

int ET_COMMAND_MLB__Redo(ET_TCLARGS)
{
  MLB::UndoManager::um->Redo();
  return TCL_OK;
}


/*** Save ***/

int ET_COMMAND_MLB__SaveDocument(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession()) {
    if (MLB::SessionManager::sm->GetActiveSession()->Save(argc,argv) != -1) {
      return TCL_OK;
    } else {
      return TCL_ERROR;
    }
  }
}

int ET_COMMAND_MLB__SaveDocumentAsPS(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SaveDocumentAsPS(argv[1]);

  return TCL_OK;
}

int ET_COMMAND_MLB__SaveDocumentAsHtml(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SaveDocumentAsHtml(argv[1]);

  return TCL_OK;
}

int ET_COMMAND_MLB__SaveDocumentAsImage(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SaveDocumentAsImage(argv[1], argv[2]);

  return TCL_OK;
}

/*** OPEN ***/

int ET_COMMAND_MLB__OpenDocument(ET_TCLARGS) {
  MLB::Session* session = new MLB::Session(MLB::CommunicationManager::cm->GetObjectID(), MLB::Session::PRIVATE, MLB::Session::INACTIVE);
  MLB::SessionManager::sm->AddSession(session);
  MLB::SessionManager::sm->SetActiveSession(session->GetID());
  session->Open(argc,argv);
  MLB::SessionManager::sm->DrawPageTree();
  return TCL_OK;
}

int ET_COMMAND_MLB__InsertDocument(ET_TCLARGS) {
  MLB::Session* session = new MLB::Session(MLB::CommunicationManager::cm->GetObjectID(), MLB::Session::PRIVATE, MLB::Session::INACTIVE);
  MLB::SessionManager::sm->AddSession(session);
  MLB::SessionManager::sm->SetActiveSession(session->GetID());
  session->Open(argc,argv);
  MLB::SessionManager::sm->DrawPageTree();
  return TCL_OK;
}

int ET_COMMAND_MLB__ImportDocument(ET_TCLARGS) {
  MLB::SessionManager::sm->ImportDocument(argv[1]);
  return TCL_OK;
}


//-------------------------------------------------------------------------//
/*** ANIMATIONS BEGIN ***/
//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__HasActivePage(ET_TCLARGS)
{
  MLB::Page*  page = MLB::SessionManager::sm->GetActivePage();
  char*       yes  = "YES";
  char*       no   = "NO";
  if( NULL==page ) Et_ResultF( interp, no    );
  else             Et_ResultF( interp, yes   );

  return TCL_OK;
} // HasActivePage()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__HasSelectedObjects(ET_TCLARGS)
{
  MLB::Page*  page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );

  Et_ResultF( interp, estring(page->HasSelectedObjects()).c_str() );

  return TCL_OK;
} // HasActivePage()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__AddSelectedToAnimationStep(ET_TCLARGS)
{
  MLB::Page*  page    = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  string objects = page->GetAnimations().AddSelectedToAnimationStep( argv[1], atoi(argv[2]) );

  Et_ResultF( interp, objects.c_str() );
  return TCL_OK;

} // AddSelectedToAnimationStep()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__RunAnimationStep(ET_TCLARGS)
{
  MLB::Page*  page    = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  if (!page->GetAnimations().RunStep( argv[1], atoi(argv[2]))) {
    // this page is done; activate next page
    MLB::SessionManager::sm->ActivateNextPage(); 
  }

  return TCL_OK;

} // RunAnimationStep()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__SetAnimationEditMode(ET_TCLARGS)
{
  //MLB::Session* session = MLB::SessionManager::sm->GetActiveSession();
  //CHECK_SESSION    ( session );
  CHECK_PARAM_COUNT( argc, 2 );

  if( string(argv[1])=="0" ) MLB::SessionManager::sm->SetAnimationEditMode( false );
  else MLB::SessionManager::sm->SetAnimationEditMode( true );

  return TCL_OK;

} // SetAnimationEditMode()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__GetAnimationEditMode(ET_TCLARGS)
{
  MLB::Session* session = MLB::SessionManager::sm->GetActiveSession();
  CHECK_SESSION(session);

  Et_ResultF( interp, estring( MLB::SessionManager::sm->GetAnimationEditMode() ).c_str() );
  return TCL_OK;

} // GetAnimationEditMode()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__SelectAnimationObject(ET_TCLARGS)
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 2 );

  page->GetAnimations().SelectAnimationObject( argv[1] );

  return TCL_OK;

} // SelectAnimationObject()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__SelectAnimationStep(ET_TCLARGS)
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 2 );

  page->GetAnimations().SelectAnimationStepObjects( argv[1] );

  return TCL_OK;

} // SelectAnimationStep()

//-------------------------------------------------------------------------//

/**   Read info about animations of the current page.
  *   @param  argv[1] step name
  *   @return         title for step with the given name
  **/
int ET_COMMAND_MLB__GetAnimationStepTitle( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE(page);
  CHECK_PARAM_COUNT( argc, 2 );

  Et_ResultF( interp, page->GetAnimations().GetStepTitle( argv[1] ).c_str() );

  return TCL_OK;
} // GetAnimationStepTitles()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__SetAnimationStepTitle(ET_TCLARGS)
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  page->GetAnimations().SetStepTitle( argv[1], argv[2] );

  return TCL_OK;

} // SetAnimationStepTitle()

//-------------------------------------------------------------------------//

/**   Read info about animations of the current page.
  *   @param  argv[1] step name
  *   @return         true if step is displayed as open, false - otherwise
  **/
int ET_COMMAND_MLB__IsAnimationStepOpen( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE(page);
  CHECK_PARAM_COUNT( argc, 2 );

  Et_ResultF( interp, estring( page->GetAnimations().IsStepOpen( argv[1] ) ).c_str() );

  return TCL_OK;
} // IsAnimationStepOpen()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__SetAnimationStepOpen(ET_TCLARGS)
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  page->GetAnimations().SetStepOpen( argv[2], argv[1] );

  return TCL_OK;

} // SetAnimationStepOpen()

//-------------------------------------------------------------------------//

/**   Deletes either an animation step or an animation object from the step
  *   @param argv[1]  - node's parent
  *   @param argv[2]  - node itself
  **/
int ET_COMMAND_MLB__RemoveAnimationItem( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  if( "Step" == string(argv[2]).substr(0,4) )
    page->GetAnimations().RemoveStep( argv[2] );
  else
    page->GetAnimations().RemoveObject( argv[1], argv[2] );

  return TCL_OK;
} // DeleteAnimationItem()

//-------------------------------------------------------------------------//

/**   Read info about animations of the current page. Without params.
  *   @return space-delimited list of names of existing animation steps
  **/
int ET_COMMAND_MLB__GetAnimationStepNames( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE(page);

  Et_ResultF( interp, page->GetAnimations().GetStepNames().c_str() );

  return TCL_OK;
} // GetAnimationStepNames()

//-------------------------------------------------------------------------//

/**   Read info about animations of the current page. Without params.
  *   @return space-delimited list of names of currently run animation steps
  **/
int ET_COMMAND_MLB__GetAnimationRunStepNames( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE(page);

  Et_ResultF( interp, page->GetAnimations().GetRunStepNames().c_str() );

  return TCL_OK;
} // GetAnimationRunStepNames()

//-------------------------------------------------------------------------//

/**   Read info about animated objects of the current page and given step.
  *   @param argv[1]  animation step
  *   @param argv[2]  bool group: 1 - show objects, 0 - hide objects
  *   @return         space-delimited list of object names of given animation step
  **/
int ET_COMMAND_MLB__GetAnimationObjectNames( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  Et_ResultF( interp, page->GetAnimations().GetObjectNames( argv[1], atoi(argv[2]) ).c_str() );

  return TCL_OK;
} // GetAnimationObjectNames()

//-------------------------------------------------------------------------//

/**   Read info about animated objects of the current page and given step.
  *   @param argv[1]  animation step
  *   @param argv[2]  object name
  *   @return         name of the object with the given name and animation step
  **/
int ET_COMMAND_MLB__GetAnimationObjectTitle( ET_TCLARGS )
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  Et_ResultF( interp, page->GetAnimations().GetObjectTitle( argv[1], argv[2] ).c_str() );

  return TCL_OK;
} // GetAnimationObjectTitles()

//-------------------------------------------------------------------------//

int ET_COMMAND_MLB__InsertNewAnimationStep(ET_TCLARGS)
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE(page);

  MLB::Animations&                animations  = page->GetAnimations();
  MLB::Animations::step_iterator step       = animations.AddNewStep();

  Et_ResultF( interp, step->GetName().c_str() );
  return TCL_OK;
} // InsertNewAnimationStep()

//-------------------------------------------------------------------------//

/**   Moves the animation step to another position.
  *   @param argv[1]  animation step name
  *   @param argv[2]  new position (0-first 1-second in the list of animation steps of current page)
  *   @return         space-delimited list of object names of given animation step
  **/
int ET_COMMAND_MLB__MoveAnimationStep(ET_TCLARGS)
{
  MLB::Page* page = MLB::SessionManager::sm->GetActivePage();
  CHECK_PAGE       ( page    );
  CHECK_PARAM_COUNT( argc, 3 );

  std::cout << "Before moving step " << argv[1] << " to position " << atoi(argv[2]) << std::endl;
  page->GetAnimations().MoveStep( argv[1], atoi(argv[2]) );
  std::cout << "After moving step " << argv[1] << " to position " << atoi(argv[2]) << std::endl;
  return TCL_OK;

} // MoveAnimationStep()

//-------------------------------------------------------------------------//
/*** ANIMATIONS END ***/
//-------------------------------------------------------------------------//


/***  ***/
int ET_COMMAND_MLB__SetWindowPointer(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->SetWindowPointer(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__UnSetWindowPointer(ET_TCLARGS)
{
  if (MLB::SessionManager::sm->GetActiveSession())
    MLB::SessionManager::sm->GetActiveSession()->UnSetWindowPointer(argc,argv);

  return TCL_OK;
}

int ET_COMMAND_MLB__HandleRecording(ET_TCLARGS)
{
  MLB::SessionManager::sm->HandleRecording(argc,argv);
  return TCL_OK;
}

int ET_COMMAND_MLB__ChangeFrameRate(ET_TCLARGS)
{
  MLB::SessionManager::sm->ChangeFrameRate(argc,argv);
  return TCL_OK;
}


int ET_COMMAND_MLB__SetZoom(ET_TCLARGS) {
  MLB::SessionManager::sm->SetZoom(atoi(argv[1]));
  return TCL_OK;
}


int ET_COMMAND_MLB__Init(ET_TCLARGS) {

  Tcl_InitMemory( Et_Interp );

  MLB::CommunicationManager::cm = new MLB::CommunicationManager();
  MLB::SessionManager::sm       = new MLB::SessionManager(argv[1]);
  MLB::AduManager::am           = new MLB::AduManager();
  MLB::TelepointerManager::tm   = new MLB::TelepointerManager();
  MLB::UndoManager::um          = new MLB::UndoManager();

  return TCL_OK;
}


void usage() {
  std::cout << "mlb 0.9.1 - University of Mannheim" << std::endl;
  std::cout << "mlb [maddr/mport] [-t mttl] [-f file]\n";
  exit(-1);
}

void parse(const char* s, string* group, int* port) {
  char* p;
  char spec[128];
  strcpy(spec, s);
  p = strchr(spec, '/');
  if (p == NULL) usage();
  *port = atoi(p+1);
  *p = '\0';
  *group = spec;
}


int ET_COMMAND_MLB__Start(ET_TCLARGS) {
  int numArgs = atoi(Tcl_GetVar(Et_Interp, "argc", 0));
  if (numArgs < 1) return TCL_OK;

  const char* Args = Tcl_GetVar(Et_Interp, "argv", 0);
  int listArgc;
  const char** listArgv;
  Tcl_SplitList(Et_Interp, Args, &listArgc, &listArgv);

  for (int i = 0; i < listArgc; i++) {
    if ((strcmp(listArgv[i], "-t") == 0) || (strcmp(listArgv[i], "-ttl") == 0)) {
      if (++i >= listArgc) {
	usage();
	return TCL_OK;
      }
      
      if (Et_GlobalEvalF(Et_Interp, "Config::SetTtl %s", listArgv[i]) != TCL_OK) {
	return TCL_ERROR;
      }
    } else if (strcmp(listArgv[i], "-f") == 0) {
      if (++i >= listArgc) {
	usage();
	return TCL_OK;
      }
      
      docName = strdup(listArgv[i]);
    } else {
      string mgroup;
      int mport;
      parse(listArgv[i], &mgroup, &mport);
      if ((Et_GlobalEvalF(Et_Interp, "Config::SetIpAddr %s", mgroup.c_str()) != TCL_OK) || 
	  (Et_GlobalEvalF(Et_Interp, "Config::SetPort %d", mport) != TCL_OK)) {
	return TCL_ERROR;
      }
    }
  }
  
  return TCL_OK;
}

int ET_COMMAND_MLB__Online(ET_TCLARGS) {
  MLB::TimerManager::tm         = new MLB::TimerManager();
  MLB::TimeWarp::tw             = new MLB::TimeWarp();
  HRT::HrtManager::hrtm         = new HRT::HrtManager();
  CHAT::ChatManager::cm         = new CHAT::ChatManager();
  FT::FtManager::ftm            = new FT::FtManager();
  VT::VtManager::vtm            = new VT::VtManager();
  ALT::AltManager::altm         = new ALT::AltManager();
  return TCL_OK;
}

int ET_COMMAND_MLB__Exit(ET_TCLARGS) {
  if (MLB::CommunicationManager::cm) MLB::CommunicationManager::cm->Shutdown();

  if (MLB::TimerManager::tm)       delete MLB::TimerManager::tm;        MLB::TimerManager::tm = NULL;
  if (MLB::SessionManager::sm)     delete MLB::SessionManager::sm;      MLB::SessionManager::sm = NULL;
  if (HRT::HrtManager::hrtm)       delete HRT::HrtManager::hrtm;        HRT::HrtManager::hrtm = NULL;
  if (CHAT::ChatManager::cm)       delete CHAT::ChatManager::cm;        CHAT::ChatManager::cm = NULL;
  if (FT::FtManager::ftm)          delete FT::FtManager::ftm;           FT::FtManager::ftm = NULL;
  if (VT::VtManager::vtm)          delete VT::VtManager::vtm;           VT::VtManager::vtm = NULL;
  if (ALT::AltManager::altm)       delete ALT::AltManager::altm;        ALT::AltManager::altm = NULL;
  if (MLB::TelepointerManager::tm) delete MLB::TelepointerManager::tm;  MLB::TelepointerManager::tm = NULL;
  if (MLB::AduManager::am)         delete MLB::AduManager::am;          MLB::AduManager::am = NULL;
  if (MLB::TimeWarp::tw)           delete MLB::TimeWarp::tw;            MLB::TimeWarp::tw = NULL;
  if (UCE::UCE_Interface::ucei)    delete UCE::UCE_Interface::ucei;     UCE::UCE_Interface::ucei = NULL;
  if (MLB::UceManager::ucem)       delete MLB::UceManager::ucem;        MLB::UceManager::ucem = NULL;
  // MB 
  if (MLB::UndoManager::um)        delete MLB::UndoManager::um;         MLB::UndoManager::um = NULL;

  return TCL_OK;
}

int ET_COMMAND_MLB__CloseNetwork(ET_TCLARGS) {
  if (MLB::CommunicationManager::cm) delete MLB::CommunicationManager::cm;
  return TCL_OK;
}


int ET_COMMAND_MLB__SetParticipantName(ET_TCLARGS) {
  Tcl_DString str;
  Tcl_UtfToExternalDString(NULL, argv[1], -1, &str);
  MLB::CommunicationManager::cm->SetParticipantName(Tcl_DStringValue(&str));
  Tcl_DStringFree(&str);
  return TCL_OK;
}

int ET_COMMAND_MLB__SetParticipantEmail(ET_TCLARGS) {
  Tcl_DString str;
  Tcl_UtfToExternalDString(NULL, argv[1], -1, &str);
  MLB::CommunicationManager::cm->SetParticipantEmail(Tcl_DStringValue(&str));
  Tcl_DStringFree(&str);
  return TCL_OK;
}

int ET_COMMAND_MLB__SetParticipantPhone(ET_TCLARGS) {
  Tcl_DString str;
  Tcl_UtfToExternalDString(NULL, argv[1], -1, &str);
  MLB::CommunicationManager::cm->SetParticipantPhone(Tcl_DStringValue(&str));
  Tcl_DStringFree(&str);
  return TCL_OK;
}

int ET_COMMAND_MLB__SetParticipantLocation(ET_TCLARGS) {
  Tcl_DString str;
  Tcl_UtfToExternalDString(NULL, argv[1], -1, &str);
  MLB::CommunicationManager::cm->SetParticipantLocation(Tcl_DStringValue(&str));
  Tcl_DStringFree(&str);
  return TCL_OK;
}

int ET_COMMAND_MLB__DisplayParticipant(ET_TCLARGS) {
  MLB::ParticipantManager::pm->DisplayParticipant(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__RemoveExitedParticipants(ET_TCLARGS) {
  MLB::ParticipantManager::pm->RemoveAllExitedParticipants();
  return TCL_OK;
}

int ET_COMMAND_MLB__OpenSMP(ET_TCLARGS) {
  if (MLB::CommunicationManager::cm->OpenSMP())return TCL_OK;
  else return TCL_ERROR;
}

int ET_COMMAND_MLB__CloseSMP(ET_TCLARGS) {
  if (MLB::CommunicationManager::cm->CloseSMP()) return TCL_OK;
  else return TCL_ERROR;
}

#ifdef IDSERVER
int ET_COMMAND_MLB__OpenIDServer(ET_TCLARGS)
{
  if (argc == 3) {
    if (MLB::CommunicationManager::cm->OpenIDServer(argv[1], atoi(argv[2]))) {
      Et_ResultF(interp, "OK");
      return TCL_OK;
    } else {
      Et_ResultF(interp, "ERROR");
      return TCL_ERROR;
    }
  }

  // start local id server
  if (MLB::CommunicationManager::cm->OpenIDServer()) return TCL_OK;
  else return TCL_ERROR;
}

int ET_COMMAND_MLB__CloseIDServer(ET_TCLARGS)
{
  if (MLB::CommunicationManager::cm->CloseIDServer()) return TCL_OK;
  else return TCL_ERROR;
}
#endif

int ET_COMMAND_MLB__OpenUCEServer(ET_TCLARGS)
{
  if (argc == 3) {
    MLB::UceManager::ucem = new MLB::UceManager();
    UCE::UCE_Interface::ucei = new UCE::UCE_Interface(MLB::UceManager::ucem, argv[1], atoi(argv[2]), "mlb", "mlb");

    if (UCE::UCE_Interface::ucei->cmd_open_connection() == 0) {
      MLB::UceManager::connected = true;
      Et_ResultF(interp, "OK");
      return TCL_OK;
    } else {
      MLB::UceManager::connected = false;
      Et_ResultF(interp, "ERROR");
      return TCL_ERROR;
    }
  }
}

int ET_COMMAND_MLB__OpenSharedSession(ET_TCLARGS) {
  if (!(MLB::CommunicationManager::cm->OpenRTPI(argv[1], atoi(argv[2]), atoi(argv[3]), atoi(argv[4]))))
    return TCL_ERROR;

  MLB::TelepointerManager::tm->OpenRTPI();
  MLB::Session* session = new MLB::Session(1, MLB::Session::SHARED, MLB::Session::INACTIVE);
  if (MLB::SessionManager::sm->GetActivePage()) MLB::SessionManager::sm->GetActivePage()->DeActivate();
  MLB::SessionManager::sm->SetActivePage(NULL);
  MLB::SessionManager::sm->AddSession(session);
  MLB::SessionManager::sm->SetActiveSession(session->GetID());
  MLB::SessionManager::sm->DrawPageTree();

  return TCL_OK;
}

int ET_COMMAND_MLB__CloseSharedSession(ET_TCLARGS) {
  MLB::TelepointerManager::tm->CloseRTPI();
  if (!MLB::CommunicationManager::cm->CloseRTPI())
    return TCL_ERROR;

  MLB::SessionManager::sm->CloseSharedSession();
  return TCL_OK;
}

int ET_COMMAND_MLB__SetCueRate(ET_TCLARGS) {
  MLB::CommunicationManager::cm->SetCueRate((double)atoi(argv[1])/100.0);
  return TCL_OK;
}

int ET_COMMAND_MLB__SetLocalLag(ET_TCLARGS) {
  MLB::AduManager::am->SetLocalLag(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__SetStorageTime(ET_TCLARGS)
{
  MLB::TimeWarp::tw->SetStorageTime(1000 * atoi(argv[1])); // in ms

  return TCL_OK;
}

int ET_COMMAND_MLB__SetCheckIntention(ET_TCLARGS)
{
  MLB::CommunicationManager::cm->SetCheckIntentionViolation(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__SetLogPageChanges(ET_TCLARGS)
{
  MLB::SessionManager::sm->SetLogPageChanges(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__NewPrivateSession(ET_TCLARGS) {
  MLB::Session* session = new MLB::Session(MLB::CommunicationManager::cm->GetObjectID(), MLB::Session::PRIVATE, MLB::Session::INACTIVE);
  if (MLB::SessionManager::sm->GetActivePage()) {
    if (MLB::SessionManager::sm->GetParentSession(MLB::SessionManager::sm->GetActivePage())->GetSessionType() == MLB::Session::SHARED) {
      MLB::SessionManager::sm->GetActivePage()->ChangeSubcomponentsActiveness(false);
    }
    MLB::SessionManager::sm->GetActivePage()->DeActivate();
  }
  MLB::SessionManager::sm->AddSession(session);
  MLB::SessionManager::sm->SetActiveSession(session->GetID());
  MLB::SessionManager::sm->InsertNewPage(new MLB::Page(MLB::CommunicationManager::cm->GetObjectID(), MLB::Session::PRIVATE));
  MLB::SessionManager::sm->DrawPageTree();

  return TCL_OK;
}


int ET_COMMAND_MLB__LoadInitialDocument(ET_TCLARGS) {
  if (!docName) return TCL_OK;

  MLB::Session* session = new MLB::Session(MLB::CommunicationManager::cm->GetObjectID(), MLB::Session::PRIVATE, MLB::Session::INACTIVE);
  if (MLB::SessionManager::sm->GetActivePage()) {
    if (MLB::SessionManager::sm->GetParentSession(MLB::SessionManager::sm->GetActivePage())->GetSessionType() == MLB::Session::SHARED) {
      MLB::SessionManager::sm->GetActivePage()->ChangeSubcomponentsActiveness(false);
    }
    MLB::SessionManager::sm->GetActivePage()->DeActivate();
  }
  MLB::SessionManager::sm->AddSession(session);
  MLB::SessionManager::sm->SetActiveSession(session->GetID());
  session->Open(docName);
  delete docName; docName = NULL;
  MLB::SessionManager::sm->DrawPageTree();

  return TCL_OK;
}


int ET_COMMAND_MLB__DeleteSession(ET_TCLARGS)
{
  MLB::SessionManager::sm->DeleteActiveSession();

  return TCL_OK;
}

int ET_COMMAND_MLB__SetAnnotationMode(ET_TCLARGS)
{
   MLB::SessionManager::sm->SetAnnotationMode(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__SetSynchMode(ET_TCLARGS) {
  MLB::SessionManager::sm->SetSynchMode(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__FirstPage(ET_TCLARGS) {
  MLB::SessionManager::sm->ActivateFirstPage();
  return TCL_OK;
}

int ET_COMMAND_MLB__PreviousPage(ET_TCLARGS) {
  MLB::SessionManager::sm->ActivatePreviousPage();
  return TCL_OK;
}

int ET_COMMAND_MLB__NextPage(ET_TCLARGS)
{
  MLB::SessionManager::sm->ActivateNextPage();
  return TCL_OK;
}

int ET_COMMAND_MLB__ImportPage(ET_TCLARGS)
{
  MLB::SessionManager::sm->ImportPage();
  return TCL_OK;
}

int ET_COMMAND_MLB__RebuildPageTree(ET_TCLARGS)
{
  MLB::SessionManager::sm->ClearNodeNames();
  MLB::SessionManager::sm->DrawPageTree();

  return TCL_OK;
}

int ET_COMMAND_MLB__SetMute(ET_TCLARGS) {
        MLB::SessionManager::sm->SetAudioMute(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__RebuildParticipantList(ET_TCLARGS) {
  MLB::ParticipantManager::pm->DisplayParticipantList();
  return TCL_OK;
}

int ET_COMMAND_MLB__FreezeImages(ET_TCLARGS) {
  MLB::SessionManager::sm->FreezeImages(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_MLB__ParticipantHints(ET_TCLARGS) {
  MLB::AduManager::am->SetParticipantHints(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_HRT__OpenRTPI(ET_TCLARGS) {
  HRT::HrtManager::hrtm->OpenRTPI();
  return TCL_OK;
}

int ET_COMMAND_HRT__CloseRTPI(ET_TCLARGS) {
  HRT::HrtManager::hrtm->CloseRTPI();
  return TCL_OK;
}

int ET_COMMAND_HRT__SetIndicate(ET_TCLARGS) {
  HRT::HrtManager::hrtm->SetIndicate(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_HRT__RaiseHand(ET_TCLARGS) {
  HRT::HrtManager::hrtm->RaiseHand();
  return TCL_OK;
}

int ET_COMMAND_HRT__LowerHand(ET_TCLARGS) {
  HRT::HrtManager::hrtm->LowerHand();
  return TCL_OK;
}

int ET_COMMAND_HRT__CallParticipant(ET_TCLARGS) {
  HRT::HrtManager::hrtm->CallParticipant(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_HRT__RebuildParticipantList(ET_TCLARGS) {
  HRT::HrtManager::hrtm->DisplayParticipantList();
  return TCL_OK;
}

int ET_COMMAND_CHAT__OpenRTPI(ET_TCLARGS) {
  CHAT::ChatManager::cm->OpenRTPI();
  return TCL_OK;
}

int ET_COMMAND_CHAT__CloseRTPI(ET_TCLARGS) {
  CHAT::ChatManager::cm->CloseRTPI();
  return TCL_OK;
}

int ET_COMMAND_CHAT__SendMessage(ET_TCLARGS) {
  Tcl_DString name;
  Tcl_DString message;
  Tcl_UtfToExternalDString(NULL, argv[1], -1, &name);
  Tcl_UtfToExternalDString(NULL, argv[2], -1, &message);
  CHAT::ChatManager::cm->SendMessage(Tcl_DStringValue(&name), Tcl_DStringValue(&message));
  Tcl_DStringFree(&name);
  Tcl_DStringFree(&message);
  return TCL_OK;
}

int ET_COMMAND_FT__OpenRTPI(ET_TCLARGS) {
  FT::FtManager::ftm->OpenRTPI();
  return TCL_OK;
}

int ET_COMMAND_FT__CloseRTPI(ET_TCLARGS) {
  FT::FtManager::ftm->CloseRTPI();
  return TCL_OK;
}

int ET_COMMAND_FT__RebuildScores(ET_TCLARGS) {
  FT::FtManager::ftm->RebuildScores();
  return TCL_OK;
}

int ET_COMMAND_FT__ChangeQuestionVisibility(ET_TCLARGS) {
  FT::FtManager::ftm->ChangeVisibility(atoi(argv[1]),atoi(argv[2]));
  return TCL_OK;
}

int ET_COMMAND_FT__CreateQuestion(ET_TCLARGS) {
  FT::FtManager::ftm->CreateQuestion(argc, argv);
  return TCL_OK;
}

int ET_COMMAND_FT__DeleteQuestion(ET_TCLARGS) {
  FT::FtManager::ftm->DeleteQuestion(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_FT__EditQuestion(ET_TCLARGS) {
  FT::FtManager::ftm->EditQuestion(argc, argv);
  return TCL_OK;
}

int ET_COMMAND_FT__ChangeAnswer(ET_TCLARGS) {
  FT::FtManager::ftm->ChangeAnswer(atoi(argv[1]),atoi(argv[2]));
  return TCL_OK;
}

int ET_COMMAND_VT__OpenRTPI(ET_TCLARGS) {
  VT::VtManager::vtm->OpenRTPI();
  return TCL_OK;
}

int ET_COMMAND_VT__CloseRTPI(ET_TCLARGS) {
  VT::VtManager::vtm->CloseRTPI();
  return TCL_OK;
}

int ET_COMMAND_VT__CreateMCQuestion(ET_TCLARGS) {
  VT::VtManager::vtm->CreateMCQuestion(argc, argv);
  return TCL_OK;
}

int ET_COMMAND_VT__CreateTextQuestion(ET_TCLARGS) {
  VT::VtManager::vtm->CreateTextQuestion(argc, argv);
  return TCL_OK;
}

int ET_COMMAND_VT__ChangeMCAnswer(ET_TCLARGS) {
  VT::VtManager::vtm->ChangeMCAnswer(atoi(argv[1]),atoi(argv[2]));
  return TCL_OK;
}

int ET_COMMAND_VT__ChangeTextAnswer(ET_TCLARGS) {
  VT::VtManager::vtm->ChangeTextAnswer(atoi(argv[1]),argv[2]);
  return TCL_OK;
}

int ET_COMMAND_VT__DeleteQuestion(ET_TCLARGS) {
  VT::VtManager::vtm->DeleteQuestion(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_ALT__OpenRTPI(ET_TCLARGS) {
  ALT::AltManager::altm->OpenRTPI();
  return TCL_OK;
}

int ET_COMMAND_ALT__CloseRTPI(ET_TCLARGS) {
  ALT::AltManager::altm->CloseRTPI();
  return TCL_OK;
}

int ET_COMMAND_ALT__AddApplication(ET_TCLARGS) {
  Tcl_DString name;
  Tcl_DString exec;
  Tcl_DString params;
  Tcl_DString lpath;
  Tcl_UtfToExternalDString(NULL, argv[1], -1, &name);
  Tcl_UtfToExternalDString(NULL, argv[2], -1, &exec);
  Tcl_UtfToExternalDString(NULL, argv[3], -1, &params);
  Tcl_UtfToExternalDString(NULL, argv[4], -1, &lpath);
  ALT::AltManager::altm->AddApplication(Tcl_DStringValue(&name), Tcl_DStringValue(&exec), Tcl_DStringValue(&params), Tcl_DStringValue(&lpath));
  Tcl_DStringFree(&name);
  Tcl_DStringFree(&exec);
  Tcl_DStringFree(&params);
  Tcl_DStringFree(&lpath);
  return TCL_OK;
}

int ET_COMMAND_ALT__EditApplication(ET_TCLARGS) {
  Tcl_DString name;
  Tcl_DString exec;
  Tcl_DString params;
  Tcl_DString lpath;
  Tcl_UtfToExternalDString(NULL, argv[2], -1, &name);
  Tcl_UtfToExternalDString(NULL, argv[3], -1, &exec);
  Tcl_UtfToExternalDString(NULL, argv[4], -1, &params);
  Tcl_UtfToExternalDString(NULL, argv[5], -1, &lpath);
  ALT::AltManager::altm->EditApplication(atoi(argv[1]), Tcl_DStringValue(&name), Tcl_DStringValue(&exec),
					 Tcl_DStringValue(&params), Tcl_DStringValue(&lpath));
  Tcl_DStringFree(&name);
  Tcl_DStringFree(&exec);
  Tcl_DStringFree(&params);
  Tcl_DStringFree(&lpath);
  return TCL_OK;
}

int ET_COMMAND_ALT__DeleteApplication(ET_TCLARGS) {
  ALT::AltManager::altm->DeleteApplication(atoi(argv[1]));
  return TCL_OK;
}

int ET_COMMAND_ALT__StartApplication(ET_TCLARGS) {
  ALT::AltManager::altm->StartApplication(atoi(argv[1]));
  return TCL_OK;
}




#ifdef WIN32


int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
                     LPSTR lpszCmdLine,int nCmdShow) {
  extern int main(int argc, char** argv);
  main(__argc, __argv);
  return 0;
}

#endif
