##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

set mlbVersion "0.9.1"

source attributes.tcl
source arrowshape.tcl
source page.tcl
source telepointer.tcl
source toolbar.tcl
source workingarea.tcl
source animations.tcl
source grid.tcl
source debuginfo.tcl
source session.tcl
source participants.tcl
source pagetree.tcl
source chat.tcl
source feedback.tcl
source voting.tcl
source alt.tcl
source file.tcl
source record.tcl
source time.tcl
source config.tcl

source bwidgets/mainframe.tcl
source bwidgets/spinbox.tcl
source bwidgets/panedw.tcl
source bwidgets/font.tcl
source bwidgets/color.tcl
source bwidgets/bitmap.tcl
source bwidgets/imagebox.tcl
source bwidgets/scrollw.tcl
source bwidgets/scrollframe.tcl
source bwidgets/messagedlg.tcl
source bwidgets/progressdlg.tcl
source bwidgets/listbox.tcl


set Blue1 "#3D6DAF"
set Blue2 "#6A94CC"
set Blue3 "#AEC4E3"
set Blue4 "#CBD9ED"
set Blue5 "#DBE4F2"

#option add *background $Blue4


proc NOP { } {
}


proc HighlightWidget {widget} {
    bind $widget <Enter> {+
	if {[string compare [%W cget -relief] "sunken"] == 0} {
	    %W configure -bg grey
	} else {
	    %W configure -relief raised
	}
    }
    bind $widget <Leave> {+
	if {[string compare [%W cget -relief] "sunken"] == 0} {
	    %W configure -bg LightGrey
	} else {
	    %W configure -relief flat
	}
    }
}


namespace eval Toplevel {
    variable toplevel
    variable frame
    variable status
    variable prgBar
    variable prgIndic -1
    variable prgLabel
    variable prgType
    variable verLabel
}


proc Toplevel::CreateImages { } {
    image create photo new -file "../gifs/new.gif"
    image create photo delete -file "../gifs/delete.gif"
    image create photo edit -file "../gifs/edit.gif"
    image create photo answer -file "../gifs/answer.gif"
    image create photo results -file "../gifs/results.gif"
    image create photo start -file "../gifs/start.gif"
    image create photo adminquestion -file "../gifs/feedback.gif"
    image create photo unchecked -file "../gifs/unchecked.gif"
    image create photo checked -file "../gifs/checked.gif"
    image create photo indicate -file "../gifs/indicate.gif"
    image create photo raisehand -file "../gifs/raisehand.gif"
    image create photo lowerhand -file "../gifs/lowerhand.gif"
    image create photo call -file "../gifs/call.gif"
    image create photo logo -file "../gifs/mlb.gif" 
    image create photo hrOn -file "../gifs/hr_on.gif"
    image create photo hrOff -file "../gifs/hr_off.gif" 
    image create photo exited -file "../gifs/exited.gif"
}


proc Toplevel::create {} {
    global   Blue1
    global   mlbVersion
    variable toplevel
    variable frame
    variable status
    variable verLabel

    set toplevel [MainFrame .mainframe -menu $Menu::entries -textvariable Toplevel::status\
		      -progressfg $Blue1 -progressvar Toplevel::prgIndic]
    $toplevel setmenustate openfile disabled
    $toplevel setmenustate sharedsession disabled
    $toplevel setmenustate sessionedit disabled
    $toplevel setmenustate userprofile disabled
    $toplevel setmenustate privatesession disabled
    $toplevel setmenustate tool disabled
    $toplevel setmenustate services disabled

    if {!$Config::showStatusBar} {
 	$toplevel showstatusbar none
    }

    wm title . "mlb $mlbVersion"
    wm protocol . WM_DELETE_WINDOW ExitMLB
    catch {wm iconbitmap . "../bin/mlb.ico"}

    set dx [expr $Config::width/2]
    set dy [expr $Config::height/2]
    append g "$Config::width"  "x"  "$Config::height"  \
	    "+[expr [winfo screenwidth .] / 2 - $dx]"  \
	    "+[expr [winfo screenheight .] / 2 - $dy]"
    wm geometry . "$g"

    set frame [$toplevel getframe]
    $frame configure -borderwidth 0 -relief flat

    pack $toplevel -fill both -expand 1
    pack $frame -fill both -expand 1 -padx 2 -pady 2

#    bind $toplevel <Key-Tab> {NOP}
#    bind $toplevel <Shift-Key-Tab> {NOP}
}

proc Toplevel::SetStatus {_status} {
    variable status

    set Toplevel::status $_status
    after 3000 [list set Toplevel::status ""]
}

proc Toplevel::ShowProgress {text max} {
    variable prgIndic
    variable prgLabel
    variable prgType

    if {$Config::showStatusBar} {
	$Toplevel::toplevel showstatusbar progression
	set prgLabel [$Toplevel::toplevel addindicator -text $text]
	set prgIndic 0
	if {$max <= 0} {
	    $Toplevel::toplevel configure -progressmax 10 -progresstype infinite
	    set prgType "infinite"
	} else {
	    $Toplevel::toplevel configure -progressmax $max -progresstype normal
	    set prgType "normal"
	}
    }
}

proc Toplevel::IncrementProgress {} {
    variable prgIndic
    variable prgType

    if {$Config::showStatusBar} {
	if {$Toplevel::prgType == "normal"} {
	    incr prgIndic
	} else {
	    set prgIndic 2
	}
    }
}

proc Toplevel::HideProgress {} {
    variable prgIndic
    variable prgLabel
    variable prgType

    if {$Config::showStatusBar} {
	$Toplevel::toplevel showstatusbar status
	destroy $prgLabel
	set prgIndic -1
	unset prgType
    }
}

proc Toplevel::ShowHideStatusBar {} {

    if {$Config::showStatusBar} {
	$Toplevel::toplevel showstatusbar progression
    } else {
	$Toplevel::toplevel showstatusbar none
    }
}

proc Toplevel::ShowHideDebuginfo {} {
    global   mlbVersion

    if {$Config::showDebuginfo} {
	DebugInfo::create
	DebugInfo::info "mlb $mlbVersion\n"
    } else {
	if [winfo exists $DebugInfo::textwindow] {
	    destroy $DebugInfo::textwindow
	}
    }
}

namespace eval Menu {
    variable entries
    variable ipaddr
    variable port
    variable ttl

    set ipaddr ""
    set port 0
    set ttl 0
}

proc Menu::create {} {
    variable entries

    foreach entry [array names OnlinePointer::ptype] {
	set name [lindex $OnlinePointer::ptype($entry) 1]
	image create photo pImg$entry -file $name
    }

    image create photo style0 -file "../gifs/style0.gif"
    image create photo style1 -file "../gifs/style1.gif"
    image create photo style2 -file "../gifs/style2.gif"
    image create photo style3 -file "../gifs/style3.gif"
    image create photo style4 -file "../gifs/style4.gif"
    image create photo style5 -file "../gifs/style5.gif"
    image create photo style6 -file "../gifs/style6.gif"

    set entries {
        "&File" {} {} 0 {
	    {command "&New Private Document" privatesession "Create new private document" {} \
		    -command {Session::NewPrivateSession}}
            {command "&Open Document..." openfile "Open document" {} \
		 -command {ToolBar::OpenFile}}
	    {command "&Shared Document..." sharedsession "Manage shared document" {} \
		    -command {Session::SharedSessionDialog}}
 	    {separator}
            {command "&Close Document" sessionedit "Close active document" {} \
		 -command {Session::DeleteSession}}
 	    {separator}
            {command "S&ave Document..." sessionedit "Save document" {} \
		 -command {ToolBar::SaveFile}}
	    {separator}
            {command "&Import Document..." {} "Import document" {} \
		 -command {ToolBar::ImportDocument}}
            {command "&Export Document..." sessionedit "Export document" {} \
		 -command {ToolBar::ExportDocument}}
	    {separator}
            {command "E&xit" {} "Exit mlb" {} -command ExitMLB}
        }
        "&Edit" {} {} 0 {
	    {command "&Cut" cutobject "Cut selected objects" {} \
		 -command {MLB::CutObjects}}
	    {command "C&opy" copyobject "Copy selected objects" {} \
		 -command {MLB::CopyObjects}}
	    {command "&Paste" pasteobject "Paste selected objects" {} \
		 -command {MLB::PasteObjects}}
	    {separator}
	    {command "&Delete Page / Chapter" sessionedit "Delete selected page / chapter" {} \
		 -command {PageTree::Delete}}
	    {command "Delete Pages..." sessionedit "Delete multiple pages of selected private session" {} \
		 -command {PageTree::DeletePages}}
            {command "Delete Document" sessionedit "Delete selected document" {} \
		 -command {Session::DeleteSession}}
	    {separator}
	    {checkbutton "Freeze &Images" {} "Disable / enable moving and resizing of images" {} \
		 -variable Config::freezeImages \
		 -command {WorkingArea::FreezeImages}}
	    {checkbutton "Freeze Document &Tree" {} "Disable / enable drag&drop in document tree" {} \
		    -variable Config::freezeTree}
	}
        "&Insert" {} {} 0 {
            {command "New &Chapter" sessionedit "Insert a new chapter" {} \
		 -command {MLB::InsertNewFolder}}
            {command "Empty &Page" sessionedit "Insert an empty page" {} \
		 -command {PageOptions::InsertNewPage}}
	    {separator}
            {command "&Duplicate Page" sessionedit "Duplicate active page" {} \
		 -command {MLB::DuplicatePage}}
	    {separator}
            {command "&Image" sessionedit "Insert image" {} \
		 -command {ToolBar::LoadImage $ToolBar::Image}}
        }
	"&View" {} {} 0 {
	    {checkbutton "Participant &Hints" {} "Show / do not show name of editing participant " {} \
		 -variable Config::participantHints \
		 -command {WorkingArea::ParticipantHints}}
            {separator}
	    {checkbutton "&Document Tree" {} "Open / close document tree" {} \
		 -variable Config::showPagetree \
		 -command {WorkingArea::create $Toplevel::frame; MLB::ReDrawPage}}
	    {checkbutton "&Participants" {} "Open / close participants" {} \
		 -variable Config::showParticipants \
		 -command {WorkingArea::create $Toplevel::frame; MLB::ReDrawPage}}
	    {checkbutton "&Toolbar" {} "Open / close toolbar" {} \
		 -variable Config::showToolBar \
		 -command {WorkingArea::ShowHideToolBar $ToolBar::toolBar}}
	    {checkbutton "&Attributes" {} "Open / close attributes bar" {} \
		 -variable Config::showAttributes \
		 -command {WorkingArea::ShowHideAttributes $Attributes::toolBar}}
	    {checkbutton "&Status Bar" {} "Show / hide status bar" {} \
		 -variable Config::showStatusBar \
		 -command {Toplevel::ShowHideStatusBar}}
	    {checkbutton "Debug &Info" {} "Show / hide debug infos" {} \
		 -variable Config::showDebuginfo \
		 -command {Toplevel::ShowHideDebuginfo}}
            {separator}
            {cascad "Toolbar Appearance" {} {} 0 {
		{radiobutton "Top" {} "Toolbar appears top" {} \
		     -value top -variable ToolBar::position -command ToolBar::Top}
		{radiobutton "Left" {} "Toolbar appears left" {} \
		     -value left -variable ToolBar::position -command ToolBar::Left}
		{radiobutton "Right" {} "Toolbar appears right" {} \
		     -value right -variable ToolBar::position -command ToolBar::Right}
		{radiobutton "Bottom" {} "Toolbar appears bottom" {} \
		     -value bottom -variable ToolBar::position -command ToolBar::Bottom}
	    }}
            {cascad "Attribute Bar Appearance" {} {} 0 {
		{radiobutton "Top" {} "Attributes bar appears top" {} \
		     -value top -variable Attributes::position -command Attributes::Top}
		{radiobutton "Bottom" {} "Attributes bar appears bottom" {} \
		     -value bottom -variable Attributes::position -command Attributes::Bottom}
	    }}
            {separator}
	    {command "Toolbox" {} "Activate separate toolbox" {} \
		 -command {ToolBox::Show}}
	    {command "Attributes Box" {} "Activate separate attributes box" {} \
		 -command {AttributesBox::Show}}
      	    {separator}
	    {command "Thumb&nails" sessionedit "Create thumbnails of pages" {} \
		 -command CreateThumbnails}
            {separator}
	    {command "&Full Screen" {} "Enable full screen mode" {} \
		 -command {WorkingArea::FullScreen 0}}
	    {command "Full Screen &+ Toolbar" {} "Enable full screen mode with additional toolbar" {} \
		    -command {WorkingArea::FullScreen 1}}
            {separator}
	    {command "Fit Page to &Window" {} "Fit Page to Window" {} \
		    -command {WorkingArea::FitPageToWindow}}
	    {command "&Zoom..." {} "Set zoom factor" {} \
		 -command {Config::Zoom}}
	}
	"&Tools" {} {} 0 {
	    {checkbutton "&AOF Recording Tool" {} "Open / close AOF recording tool" {} \
		    -variable Record::openTool \
	            -command {WorkingArea::create $Toplevel::frame; MLB::ReDrawPage}}
            {separator}
	    {checkbutton "A&nimations" {} "Open / close presentation animations" {} \
		 -variable Config::showAnimations \
		 -command {WorkingArea::create $Toplevel::frame; MLB::ReDrawPage}}
            {separator}
	    {checkbutton "&Feedback Tool" tool "Open / close feedback tool and join / leave feedback tool session" {} \
		    -variable Feedback::openTool
		    -command {Feedback::Tool}}
	    {checkbutton "&Voting Tool" tool "Open / close voting tool and join / leave voting tool session" {} \
		    -variable Voting::openTool
		    -command {Voting::Tool}}
            {separator}
	    {checkbutton "&Chat" tool "Open chat tool and join chat tool session" {} \
		    -variable Chat::openTool
		    -command {Chat::Tool}}
	    {checkbutton "Application &Launch" tool "Open application launch tool and join application launch tool session" {} \
		    -variable Alt::openTool
		    -command {Alt::Tool}}
	}
        "&Options" {} {} 0 {
	    {command "&Services..." services "Manage services" {} \
		    -command {Session::ServicesDialog}}
	    {command "&User Profile..." userprofile "Mantain user profile" {} \
		    -command {Participant::DisplayLocalParticipant}}
	    {separator}
	    {command "Page Setup..." sessionedit "Change page setup" {} -command \
		    {PageOptions::Run}}
	    {command "Toolbox Setup..." toolopt "Select tools for separate toolbox" {} -command \
		    {ToolOptions::Run}}
	    {separator}
	    {cascad "Line Attributes" {} {} 0 {
		{cascad "Width" {} {} 0 {
		    {radiobutton "1" {} "width = 1" {} \
			 -value 1 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "2" {} "width = 2" {} \
			 -value 2 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "3" {} "width = 3" {} \
			 -value 3 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "4" {} "width = 4" {} \
			 -value 4 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "5" {} "width = 5" {} \
			 -value 5 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "6" {} "width = 6" {} \
			 -value 6 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "7" {} "width = 7" {} \
			 -value 7 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "8" {} "width = 8" {} \
			 -value 8 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "9" {} "width = 9" {} \
			 -value 9 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		    {radiobutton "10" {} "width = 10" {} \
			 -value 10 -variable Attributes::lineWidth \
			 -command Attributes::ChooseLineWidth}
		}}
		{cascad "Style" {} {} 0 {
		    {radiobutton "none" {} "solid line" {} \
			 -value "none" -variable Attributes::lineStyle \
			 -image style0 \
			 -command {Attributes::SetLineStyleButton}}
		    {radiobutton "." {} "dashed line" {} \
			 -value "." -variable Attributes::lineStyle \
			 -image style1 \
			 -command {Attributes::SetLineStyleButton}}
		    {radiobutton "," {} "dashed line" {} \
			 -value "," -variable Attributes::lineStyle \
			 -image style2 \
			 -command {Attributes::SetLineStyleButton}}
		    {radiobutton "-" {} "dashed line" {} \
			 -value "-" -variable Attributes::lineStyle \
			 -image style3 \
			 -command {Attributes::SetLineStyleButton}}
		    {radiobutton "_" {} "dashed line" {} \
			 -value "_" -variable Attributes::lineStyle \
			 -image style4 \
			 -command {Attributes::SetLineStyleButton}}
		    {radiobutton ".-" {} "dashed line" {} \
			 -value ".-" -variable Attributes::lineStyle \
			 -image style5 \
			 -command {Attributes::SetLineStyleButton}}
		    {radiobutton "..-" {} "dashed line" {} \
			 -value "..-" -variable Attributes::lineStyle \
			 -image style6 \
			 -command {Attributes::SetLineStyleButton}}
		}}
		{cascad "Arrow Style" {} {} 0 {
		    {radiobutton "None" {} "Arrow style" {} \
			 -value "none" -variable Attributes::arrowStyle \
			 -command {Attributes::ChooseArrowStyle}}
		    {radiobutton "First" {} "Arrow style" {} \
			 -value "first" -variable Attributes::arrowStyle \
			 -command {Attributes::ChooseArrowStyle}}
		    {radiobutton "Last" {} "Arrow style" {} \
			 -value "last" -variable Attributes::arrowStyle \
			 -command {Attributes::ChooseArrowStyle}}
		    {radiobutton "Both" {} "Arrow style" {} \
			 -value "both" -variable Attributes::arrowStyle \
			 -command {Attributes::ChooseArrowStyle}}
		}}
		{command "Arrow Shape..." {} "Arrow shape" {} \
		     -command ArrowShape::Draw}
		{cascad "Join Style" {} {} 0 {
		    {radiobutton "Bevel" {} "Join style" {} \
			 -value "bevel" -variable Attributes::joinStyle \
			 -command {Attributes::ChooseJoinStyle}}
		    {radiobutton "Miter" {} "Join style" {} \
			 -value "miter" -variable Attributes::joinStyle \
			 -command {Attributes::ChooseJoinStyle}}
		    {radiobutton "Round" {} "Join style" {} \
			 -value "round" -variable Attributes::joinStyle \
			 -command {Attributes::ChooseJoinStyle}}
		}}
		{cascad "Cap Style" {} {} 0 {
		    {radiobutton "Butt" {} "Cap style" {} \
			 -value "butt" -variable Attributes::capStyle \
			 -command {Attributes::ChooseCapStyle}}
		    {radiobutton "Projecting" {} "Cap style" {} \
			 -value "projecting" -variable Attributes::capStyle \
			 -command {Attributes::ChooseCapStyle}}
		    {radiobutton "Round" {} "Cap style" {} \
			 -value "round" -variable Attributes::capStyle \
			 -command {Attributes::ChooseCapStyle}}
		}}
	    }}
	    {separator}
	    {command "&TelePointer..." {} "Change telepointer options" {} -command \
		    {PointerOptions::Run}}
	    {separator}
	    {command "&Grid..." {} "Select grid" {} -command Grid::Selector}
	    {separator}
	    {cascad "Frame Rate (Audio)" {} {} 0 {
		{radiobutton "8000" {} "" {} \
		     -value 8000 -variable Record::frameRate \
		     -command Record::ChangeFrameRate}
		{radiobutton "11025" {} "" {} \
		     -value 11025 -variable Record::frameRate \
		     -command Record::ChangeFrameRate}
		{radiobutton "16000" {} "" {} \
		     -value 16000 -variable Record::frameRate \
		     -command Record::ChangeFrameRate}
		{radiobutton "22050" {} "" {} \
		     -value 22050 -variable Record::frameRate \
		     -command Record::ChangeFrameRate}
	    }}
            {checkbutton "Audio &Mute" muting "Mute / Unmute Audio" {} \
                    -variable Record::mute \
                    -command {MLB::SetMute $Record::mute}}
        }
	"&Help" {} {} 0 {
	    {command "&About..." {} "About mlb" {} \
		    -command {AboutDialog}}
	}
    }

}


proc AboutDialog { } {
    global mlbVersion
    global Blue1

    toplevel .about
    wm withdraw .about
    wm title .about "About mlb"
    wm resizable .about 0 0 
    wm protocol  .about WM_DELETE_WINDOW "destroy .about"
    catch {wm iconbitmap .about "../bin/mlb.ico"}
    wm geometry .about +[winfo pointerx .]+[winfo pointery .]
    focus .about
    grab .about

    frame  .about.f
    pack   .about.f -side top -fill both

    canvas .about.f.canvas -bg $Config::bgColor -width 60 -height 60
    .about.f.canvas create image 10 10 -image logo -anchor nw
    pack   .about.f.canvas -side left -fill y

    text   .about.f.t -bg $Config::bgColor -width 105 -height 32 -relief flat
    .about.f.t insert end "mlb - multimedia lecture board $mlbVersion\n\n"
    .about.f.t insert end "Copyright 1999 - 2002\n"
    .about.f.t insert end "University of Mannheim, Germany\n"
    .about.f.t insert end "University of Freiburg, Germany\n\n"
    .about.f.t insert end "Development:\n"
    .about.f.t insert end "Leader of the pack: Juergen Vogel\n"
    .about.f.t insert end "Team Mannheim: Juergen Vogel, Andrius Kurtinaitis, Marcel Busse, and Werner Geyer.\n"
    .about.f.t insert end "Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.\n\n"
    .about.f.t insert end "Development was sponsored by DFN (ANETTE project).\n\n"
    .about.f.t insert end "License Information:\n"
    .about.f.t insert end "The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP.\n\n"
    .about.f.t insert end "This program is free software; you can redistribute it and/or modify it under the terms of the GNU\n"
    .about.f.t insert end "General Public License as published by the Free Software Foundation; either version 2 of the License,\n"
    .about.f.t insert end "or (at your option) any later version.\n\n"
    .about.f.t insert end "This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without\n"
    .about.f.t insert end "even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
    .about.f.t insert end "See the GNU General Public License for more details.\n"
    .about.f.t insert end "The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html\n\n"
    .about.f.t insert end "The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and modifications\n"
    .about.f.t insert end "of this software made by others.\n\n"
    .about.f.t insert end "Contact:\n"
    .about.f.t insert end "Juergen Vogel - vogel@informatik.uni-mannheim.de\n"
    .about.f.t insert end "www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb\n\n"
    .about.f.t tag add bold 1.0 1.end
    .about.f.t tag add bold 3.0 3.9
    .about.f.t tag add bold 7.0 7.end
    .about.f.t tag add bold 14.0 14.end
    .about.f.t tag add bold 29.0 29.end 
    .about.f.t tag configure bold -foreground $Blue1
    .about.f.t configure -state disable
    pack .about.f.t -side right

    frame .about.f2
    pack  .about.f2 -fill both -side top

    button .about.f2.b -text "Dismiss" -command "destroy .about"
    pack   .about.f2.b -padx 2 -pady 2
    bind   .about.f2.b <Return> {destroy .about}
    focus  .about.f2.b

    wm deiconify .about
}


proc CreateThumbnails {} {

    destroy .msgdlg
    set ret [MessageDlg .msgdlg -parent . -title "Thumbnail Confirmation" -default 0 \
  	         -message "Flat or original slide hierarchy?" -buttons {"Flat" "Original" "Cancel"} \
  	         -icon question -justify center -type user]
  	if {$ret == 2} {
	    return
    }

    MLB::CreateThumbnails $ret
}


proc ExitMLB {} {
    global x

    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "mlb Exit Confirmation" -default 1 -cancel 1\
	    -message "Leave session and exit mlb?" -icon question -justify center -type yesno] {
	return
    }

    toplevel .exitdlg
    wm withdraw .exitdlg
    wm title .exitdlg "Exit"
    catch {wm iconbitmap .exitdlg "../bin/mlb.ico"}
    wm geometry .exitdlg +[winfo pointerx .]+[winfo pointery .]
    focus .exitdlg
    grab .exitdlg

    message .exitdlg.msg -text "Please wait while exiting..." -aspect 2000
    pack .exitdlg.msg
    wm deiconify .exitdlg

    if {$Chat::openTool} {
	Chat::Close
    }

    if {$Feedback::openTool} {
	Feedback::CloseTool
    }

    MLB::Exit

    set x 0
    after 2000 {set x 1}
    vwait x

    MLB::CloseNetwork
    Config::WriteConfigFile

    exit
}

wm withdraw .

Config::ReadConfigFile

Toplevel::CreateImages
Menu::create
Toplevel::create
WorkingArea::create $Toplevel::frame
ToolBar::create $Toplevel::frame
Attributes::create $Toplevel::frame

Separator $Toplevel::frame.sep -orient horizontal
ToolBar::Top

set progPath [file dirname [info nameofexecutable]]

MLB::Init $progPath

Toplevel::ShowHideDebuginfo 
WorkingArea::ShowHideToolBar $ToolBar::toolBar
WorkingArea::ShowHideAttributes $Attributes::toolBar

MLB::Start

wm deiconify .

#Session::SetupDialog
Session::StartupSharedMode

Config::SetInitialValues
MLB::LoadInitialDocument
Config::OpenShared
Participant::HrDisableGUI


set WorkingArea::weights(0) 20
set WorkingArea::weights(1) 80

