##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval PageOptions {

    variable preview
    variable pageBackground "#ffffff"
    variable pageDimension
    variable pageSize
    variable pageOrientation
    variable resolution 28 ;# pixel/cm
    variable maxSize 1600
    variable w

    set pageDimension(x) 842
    set pageDimension(y) 594
    set pageSize "A4"
    set pageOrientation "landscape"

    variable orient
    variable size
    variable bgColor
    variable x
    variable y
    variable tmp

    variable cmSize "0.00cm x 0.00cm"
    variable ccx 60;# Canvas Center X coord
    variable ccy 60;# Canvas Center Y coord
    variable minccxy [expr {$ccx < $ccy ? $ccx : $ccy}]
    variable cfact [expr 0.9 * $minccxy / $PageOptions::maxSize]    
}


proc PageOptions::UpdateXY {args} {
    variable x
    variable y
    if {[string compare $PageOptions::size "A4"] == 0} {
	set x 594
	set y 842
    } elseif {[string compare $PageOptions::size "letter"] == 0} {
	set x 612
	set y 792
    }

    UpdatePreview
} 

proc PageOptions::ChangeOrientation { } {
    variable x
    variable y
    variable orient

    set t $x
    set x $y
    set y $t

    UpdatePreview
}

proc PageOptions::Run { } { 

    variable w
    variable preview
    variable resolution
    variable pageDimension
    variable pageSize
    variable pageOrientation
    variable orient
    variable size
    variable bgColor
    variable x
    variable y
    variable tmp
    variable maxSize

    set bgColor $PageOptions::pageBackground
    set size $pageSize
    set orient $pageOrientation
    
    set x $pageDimension(x)
    set y $pageDimension(y)

    set w .pageOptions
    toplevel $w
    wm title $w "Page Setup"
    catch {wm iconbitmap $w "../bin/mlb.ico"}
    wm group $w .
    wm geometry .pageOptions +[winfo pointerx .]+[winfo pointery .]
    grab $w

    
    set b [frame $w.buttons]
    button $b.ok -text "Ok" -width 10 -default active -command {PageOptions::Exit}
    button $b.cancel -text "Cancel" -width 10 -command {destroy $PageOptions::w} 
    foreach i {ok cancel} {
      pack $b.$i -side left -padx 10
    }
    
    frame $w.size
    TitleFrame $w.size.r -text "Format" -relief groove -bd 2
    set r [$w.size.r getframe]
    pack $w.size.r -side left -fill both -padx 5 -pady 5 -expand 1


    set edit(x) $r.fx.editx
    set edit(y) $r.fy.edity

    frame $r.fx
    frame $r.fy
    pack [label $r.fx.lbl -text "Width:"] -side left 
    pack [label $r.fy.lbl -text "Height:"] -side left
    pack $r.fy -side bottom -fill x -padx 5 -pady 5
    pack $r.fx -side bottom -fill x -padx 5 -pady 5
    pack [label $r.lbl -text "Page size (pixel)"] -side bottom -padx 5 -pady 5 -anchor w

    foreach i {y x} {
 	entry $edit($i) -textvariable PageOptions::$i -relief sunken -width 5 -validate focusout \
		-invcmd {bell; focus -force %W}\
		-vcmd {PageOptions::Validate %P} \
	-state [expr {($PageOptions::size == "other") ? "normal" : "disabled"}] 
	pack $edit($i) -side right
    }
   
    ComboBox $r.combo -values {A4 letter other} -editable 0 -helptext "Page format" -textvariable PageOptions::size\
	    -helptype balloon \
	    -modifycmd "if {\"\$PageOptions::size\" != \"other\"} {$edit(x) configure -state disabled;$edit(y) configure -state disabled} else {$edit(x) configure -state normal;$edit(y) configure -state normal}"
    pack $r.combo -side top -padx 5 -pady 5

    variable ccx
    variable ccy
    variable cfact

    TitleFrame $w.size.preview -relief groove -text "Preview" -bd 2
    set prevfr [$w.size.preview getframe]
    pack [label $prevfr.xcm -textvariable PageOptions::cmSize] -side top -anchor w -padx 5 -pady 5
    set preview [canvas $prevfr.canvas -state disabled -width [expr $ccx * 2] -height [expr $ccy * 2]]
    pack $preview
    pack $w.size.preview -side right -fill y -padx 5 -pady 5

    $preview create rectangle $ccx $ccy $ccx $ccy -fill black -tag shadow
    $preview create rectangle $ccx $ccy $ccx $ccy -outline black -fill $bgColor -tag page
    PageOptions::UpdatePreview

    frame $w.low
    TitleFrame $w.low.orient -text "Orientation" -relief groove -bd 2
    set orfr [$w.low.orient getframe]
    pack [label $orfr.lbl -text "Orientation"] -side left -padx 5
    pack [frame $orfr.r -relief groove -bd 2] -side right
    foreach i {"landscape" "portrait"} {
	radiobutton $orfr.$i -text $i -value $i -variable PageOptions::orient -command PageOptions::ChangeOrientation
	pack $orfr.$i -side right -padx 5
    }
    pack $w.low.orient -side left -padx 5 -pady 5 -fill both -expand 1 
    
    TitleFrame $w.low.clrfr -text "Background color" -relief groove -bd 2
    set clr [$w.low.clrfr getframe]
    SelectColor $clr.clr -variable PageOptions::bgColor -type menubutton \
	      -command "raise $w; focus -force $w; PageOptions::UpdatePreview" -width [expr 2 * $ccy - 5]
    pack $clr.clr -fill both -padx 5 -pady 5 -expand 1
    pack $w.low.clrfr -side right -fill both -padx 5 -pady 5 -expand 1

    pack $w.size -side top -padx 5 -pady 5 -fill x
    pack $w.low -side top -padx 5 -pady 5
    pack $b -side bottom -pady 10

    wm resizable $w 0 0
    focus $b.ok
}  

# Updates the page preview in canvas canv
proc PageOptions::UpdatePreview {args} {
    variable preview
    variable ccx
    variable ccy
    variable cfact
    variable x
    variable y
    variable resolution
    if ![PageOptions::Validate $x] {return} elseif ![PageOptions::Validate $y] {return} 
    $preview coords shadow [expr 5 + $ccx - round($cfact * $x)] [expr 5 + $ccy - round($cfact * $y)]\
	    [expr 5 + $ccx + round($cfact * $x)] [expr 5 + $ccy + round($cfact * $y)]
    $preview coords page [expr $ccx - round($cfact * $x)] [expr $ccy - round($cfact * $y)]\
	    [expr $ccx + round($cfact * $x)] [expr $ccy + round($cfact * $y)] 
    $preview itemconfigure page -fill $PageOptions::bgColor
    set PageOptions::cmSize [format "%3.2fcm x %3.2fcm" [expr 1.0 * $x / $resolution] [expr 1.0 * $y / $resolution]]
}

# Validetes if s1 is a non-negative int
proc PageOptions::Integer {i} {
    if [expr ![regexp {([^0-9])} $i]] {expr ![regexp {^0+([1-9])} $i]} else {return 0}
}


# Validates if s1 is corect width/heigth
proc PageOptions::Validate {s1} {
    if {"$s1" == ""} {return 0}
    if [PageOptions::Integer $s1] {set s [expr int($s1)]} else {return 0} 
    expr [PageOptions::Integer $s]?[expr ($s>=0)&&($s<$PageOptions::maxSize)]:{0}
}

proc PageOptions::Exit { } {
# Called by OK button
    if [expr [PageOptions::Validate $PageOptions::x] && [PageOptions::Validate $PageOptions::y]] { 
	destroy $PageOptions::w
	set PageOptions::pageSize $PageOptions::size

	if {[string compare $PageOptions::pageBackground $PageOptions::bgColor] != 0} {
	    set PageOptions::pageBackground $PageOptions::bgColor
	    MLB::SetPageColor $PageOptions::pageBackground
	}

	if {($PageOptions::pageDimension(x) != $PageOptions::x) || ($PageOptions::pageDimension(y) != $PageOptions::y) \
		|| ([string compare $PageOptions::pageOrientation $PageOptions::orient] != 0)} {
	    set PageOptions::pageDimension(x) $PageOptions::x
	    set PageOptions::pageDimension(y) $PageOptions::y
	    MLB::SetPageDimension $PageOptions::pageDimension(x) $PageOptions::pageDimension(y)
	}

	set PageOptions::pageOrientation $PageOptions::orient
    }

}


proc PageOptions::InsertNewPage { } {
    variable pageBackground 
    variable pageDimension
    WorkingArea::AutoEndPolyObjects
    MLB::InsertNewPage $pageDimension(x) $pageDimension(y) $pageBackground
}
