##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

source bwidgets/titleframe.tcl
source bwidgets/scrollw.tcl
source bwidgets/tree.tcl
source bwidgets/bitmap.tcl
source bwidgets/dynhelp.tcl


namespace eval PageTree {
    variable count
    variable tree
    variable newPage
    variable annotation
    variable synch
    variable import
    variable selectedNode
	variable thumbnail

    set newPage(state) "disabled"
    set annotation(state) "disabled"
    set annotation(relief) "flat"
    set annotation(mode) 0
    set synch(state) "disabled"
    set synch(relief) "sunken"
    set synch(mode) 1
}


proc PageTree::SetNewPageButtonState {state} {
    variable newPage

    if {$state == 0} {
	set newPage(state) "disabled"
    } else {
	set newPage(state) "normal"
    }

    $newPage(button) configure -state $newPage(state)
}


proc PageTree::SetAnnotationButtonState {state} {
    variable annotation

    if {$state == 0} {
	set annotation(state) "disabled"
    } else {
	set annotation(state) "normal"
    }

    $annotation(button) configure -state $annotation(state)
}


proc PageTree::SetAnnotationButtonRelief {relief} {
    variable annotation

    if {$relief == 0} {
	set annotation(relief) "flat"
	$annotation(button) configure -relief flat -bg $Config::bgColor
    } else {
	set annotation(relief) "sunken"
	$annotation(button) configure -relief sunken -bg LightGrey
     }
}


proc PageTree::SetSynchButtonState {state} {
    variable synch

    if {$state == 0} {
	set synch(state) "disabled"
    } else {
	set synch(state) "normal"
    }

    $synch(button) configure -state $synch(state)
}


proc PageTree::SetSynchButtonRelief {relief} {
    variable synch

    if {$relief == 0} {
	set synch(relief) "flat"
	$synch(button) configure -relief flat -bg $Config::bgColor
    } else {
	set synch(relief) "sunken"
	$synch(button) configure -relief sunken -bg LightGrey
    }

}


proc PageTree::create { parent } {
    variable tree
    variable newPage
    variable annotation
    variable synch
    variable import

    frame $parent.top

    label $parent.top.l -text "Documents" -anchor w

    set newPage(button) [Button $parent.top.newp -image [Bitmap::get ../gifs/newpage.gif] -anchor w -bg $Config::bgColor\
			     -relief flat -bd 1 -state $newPage(state) -command {PageOptions::InsertNewPage}]
    DynamicHelp::register $newPage(button) balloon "New Page"
    HighlightWidget $newPage(button)

    if {$Session::globalVisible} {
	set annotation(button) [Button $parent.top.ann -image [Bitmap::get ../gifs/annotate.gif] -anchor w \
				-relief $annotation(relief) -bd 1 -state $annotation(state) -command {PageTree::ChangeAnnotationMode}]
	DynamicHelp::register $annotation(button) balloon "Annotation Mode"
	HighlightWidget $annotation(button)
	
	set synch(button) [Button $parent.top.synch -image  [Bitmap::get ../gifs/synchronize.gif] -anchor w \
		-relief $synch(relief) -bd 1 -state $synch(state) -command {PageTree::ChangeSynchMode}]
	DynamicHelp::register $synch(button) balloon "Live Mode"
	SetSynchButtonRelief $synch(relief)
	HighlightWidget $synch(button)
    }

    set sw [ScrolledWindow $parent.pagesw]
    set tree  [Tree $sw.tree \
	           -background white -linesfill grey\
                   -relief sunken -borderwidth 2 -width 8 -highlightthickness 0\
		   -redraw 0 -dropenabled 1 -dragenabled 1 \
                   -dragevent 1 \
                   -droptypes {
                       TREE_NODE {move {}}
                   } \
	           -dropcmd     "PageTree::moveNode" \
                   -opencmd     "PageTree::moddir 1" \
                   -closecmd    "PageTree::moddir 0"]
    $sw setwidget $tree

    pack $parent.top.l -anchor w -side left
    pack $parent.top.newp -anchor e -side right
    if {$Session::globalVisible} {      
	pack $parent.top.synch -anchor e -side right
	pack $parent.top.ann -anchor e -side right
    }
    pack $parent.top -fill x -anchor w

    pack $sw  -side top  -expand yes -fill both -anchor w

    $tree bindText  <ButtonPress-1>        "PageTree::select 1 $tree"
    $tree bindText  <Double-ButtonPress-1> "PageTree::select 2 $tree"
    $tree bindText  <ButtonPress-3>        "PageTree::selectNodeAction $tree %X %Y"
    $tree bindImage <ButtonPress-1>        "PageTree::select 1 $tree"
    $tree bindImage <Double-ButtonPress-1> "PageTree::select 2 $tree"
    $tree bindImage  <ButtonPress-3>       "PageTree::selectNodeAction $tree %X %Y"

    bind $tree <ButtonPress-3> {PageTree::ShowPopup %X %Y}

    frame $parent.bottom

    Button $parent.bottom.pp -image [Bitmap::get ../gifs/previouspage.gif] -relief flat -bd 1 -command {MLB::PreviousPage}
    DynamicHelp::register $parent.bottom.pp balloon "Present previous page"
    HighlightWidget $parent.bottom.pp

    Button $parent.bottom.np -image [Bitmap::get ../gifs/nextpage.gif] -relief flat -bd 1 -command {MLB::NextPage}
    DynamicHelp::register $parent.bottom.np balloon "Present next page"
    HighlightWidget $parent.bottom.np
   
    if {$Session::globalVisible} {      
	set import(button) [Button $parent.bottom.ip -image [Bitmap::get ../gifs/import.gif] -relief flat -bd 1 -command {MLB::ImportPage}]
	DynamicHelp::register $parent.bottom.ip balloon "Move first page of private document to shared document"
	HighlightWidget $parent.bottom.ip

	pack $parent.bottom.ip -anchor e -side right -padx 8
    }

    pack $parent.bottom.np -anchor e -side right
    pack $parent.bottom.pp -anchor w -side right
    pack $parent.bottom -anchor center

    return $tree
}

proc PageTree::init { home title } {
    variable count
    variable tree

    global   tcl_platform

    set count 0
    if {[$tree exists $home]} {
	$tree itemconfigure $home -text $title
    } else {
	$tree insert end root $home -text $title -open 1 -image [Bitmap::get ../gifs/openfold.gif]
    }

    $tree configure -redraw 1
}


proc PageTree::redraw { } {
    variable tree

    $PageTree::tree configure -redraw 1
}

proc PageTree::insertFolder {number name parent node data index} {
    variable tree
    variable selectedNode

    $tree insert $index $parent $node \
	-text      $name \
	-number    $number \
	-image     [Bitmap::get ../gifs/openfold.gif] \
	-data      $data \
	-drawcross allways \
	-open 1

    if {$parent == [$tree selection get]} {
	PageTree::moddir 1 $parent
    }
    set selectedNode $node
}

proc PageTree::insertSession {name parent node data index} {
    variable tree
    variable selectedNode

    $tree insert $index $parent $node \
	-text      $name \
	-image     [Bitmap::get ../gifs/openfold.gif] \
	-data      $data \
	-drawcross allways \
	-open 1

    if {$parent == [$tree selection get]} {
	PageTree::moddir 1 $parent
    }
    set selectedNode $node
}

proc PageTree::insertThumbnails {number name parent node data index} {
    variable tree
    variable selectedNode
	variable thumbnail

	set thumbnail($node) 1

    $tree insert $index $parent $node \
	-text      $name \
	-number    $number \
	-image     [Bitmap::get ../gifs/thumbnails.gif] \
	-data      $data \
	-drawcross allways \
	-open 1

    if {$parent == [$tree selection get]} {
	PageTree::moddir 1 $parent
    }
    set selectedNode $node
}

proc PageTree::updateFolder {number name node data } {
    variable tree

    $tree itemconfigure $node -text $name -data $data -number $number
}


proc PageTree::insertPage {number name parent node data index} {
    variable selectedNode
    variable count
    variable tree

    $tree insert $index $parent $node \
	-text      $name \
	-number    $number \
	-image     [Bitmap::get ../gifs/file.gif] \
	-data      $data \
	-drawcross never
    incr count

    if {$parent == [$tree selection get]} {
	PageTree::moddir 1 $parent
    }
    set selectedNode $node
}

proc PageTree::updatePage { number name node data} {
    variable tree

    $tree itemconfigure $node -text $name -data $data -number $number
}


proc PageTree::moddir { idx node } {
    variable tree
	variable thumbnail

    if { [info exists thumbnail($node)] == 0 || $thumbnail($node) != 1 } {
    $tree itemconfigure $node -open $idx -image [Bitmap::get [lindex {../gifs/folder.gif ../gifs/openfold.gif} $idx]]
    }
}


proc PageTree::selectNodeAction { tree x y node} {
    variable selectedNode

    set selectedNode $node
    ShowPopup $x $y
}

proc PageTree::select { num tree node } {
    variable selectedNode

    if { $num == 1 } {
	select_node $tree $node
    } elseif {$num == 2} {
	if { [lsearch [$tree selection get] $node] != -1 } {
	    PageTree::edit tree $tree $node
	    return
	}
    }
    set selectedNode $node
}

proc PageTree::select_node { tree node} {
    variable selectedNode

    set data [$tree itemcget $node -data]

    WorkingArea::AutoEndPolyObjects

    MLB::SelectContainer $data
    set selectedNode $node
}

proc PageTree::openFolder {parent} {

    PageTree::moddir 1 $parent
}


proc PageTree::edit { where tree node } {
    if { $where == "tree" && [lsearch [$tree selection get] $node] != -1 } {
	if [IsSession $node] {
	    # do not edit session names
	    return
	}
        set res [$tree edit $node [$tree itemcget $node -text]]
        if { $res != "" } {
            $tree itemconfigure $node -text $res
	    set data [$tree itemcget $node -data]
	    MLB::ChangeContainerName $data $res
        }
    }
}


proc PageTree::IsSession { node } {
    variable tree

    set data [$tree itemcget $node -data]

    set points 0
    set start 0
    while {1} {
	set start [string first . $data $start]
	if {$start == -1} {
	    break
	}
	incr start
	incr points
    }

    if {$points <= 2} {
	return 1
    }

    return 0
}


proc PageTree::moveNode {tree source target op type data} {
    if {$Config::freezeTree} {
	return
    }
    if {[string compare $tree $source] != 0} {
	return
    }

    WorkingArea::AutoEndPolyObjects

    set mode [lindex $target 0]
    if {[string compare $mode "position"] == 0} {
	MLB::ChangeHierachy [PageTree::extractID $data] [PageTree::extractID [lindex $target 1]] [lindex $target 2]
    } elseif {[string compare $mode "node"] == 0} {
	MLB::ChangeHierachy [PageTree::extractID $data] [PageTree::extractID [lindex $target 1]] end
    } else {
	DebugInfo::error "PageTree::moveNode - wrong mode: $mode"
	return
    }
}


proc PageTree::extractID {node} {
    return [string range $node 2 end]
}


proc PageTree::ChangeAnnotationMode {} {
    variable annotation

    if {!$annotation(mode)} {
	MLB::SetAnnotationMode 1
	set annotation(mode) 1
	SetAnnotationButtonRelief 1
        set Record::mute 1
        MLB::SetMute 1
    } else {
	MLB::SetAnnotationMode 0
	set annotation(mode) 0
	SetAnnotationButtonRelief 0
    }
}


proc PageTree::ChangeSynchMode {} {
    variable synch

    if {!$synch(mode)} {
	MLB::SetSynchMode 1
	set synch(mode) 1
	SetSynchButtonRelief 1
    } else {
	MLB::SetSynchMode 0
	set synch(mode) 0
	SetSynchButtonRelief 0
    }
}


proc PageTree::Rename {} {
    variable tree
    variable selectedNode

    set res [$tree edit $selectedNode [$tree itemcget $selectedNode -text]]
    if { $res != "" } {
	$tree itemconfigure $selectedNode -text $res
	set data [$tree itemcget $selectedNode -data]
	MLB::ChangeContainerName $data $res
    }
}


proc PageTree::Delete {} {
    variable tree
    variable selectedNode

    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "Delete Confirmation" -default 1 -cancel 1\
	    -message "Do you really want to delete \"[$tree itemcget $PageTree::selectedNode -text]\"?" -icon question -justify center -type yesno] {
	return
    }

    WorkingArea::AutoEndPolyObjects

    MLB::DeleteContainer [$tree itemcget $PageTree::selectedNode -data]
}


proc PageTree::Save {} {
    variable tree
    variable selectedNode

    MLB::SaveContainer [$tree itemcget $PageTree::selectedNode -data]
}

proc PageTree::BuildPopup { } {
    menu .pagetreeMenu -tearoff 0
    .pagetreeMenu add command -label "Rename" -command {PageTree::Rename}
    .pagetreeMenu add command -label "Delete" -command {PageTree::Delete}
    .pagetreeMenu add command -label "Save" -command {PageTree::Save}
    .pagetreeMenu add separator
    .pagetreeMenu add command -label "New Page" -command  {PageOptions::InsertNewPage}
    .pagetreeMenu add command -label "New Chapter" -command  {MLB::InsertNewFolder}
}


proc PageTree::ShowPopup {x y} {
    variable tree

    .pagetreeMenu entryconfigure 0 -state normal
    .pagetreeMenu entryconfigure 1 -state normal
    .pagetreeMenu entryconfigure 2 -state normal
    .pagetreeMenu entryconfigure 4 -state normal
    .pagetreeMenu entryconfigure 5 -state normal

    if {[llength [$tree selection get]] == 0} {
	.pagetreeMenu entryconfigure 0 -state disabled
	.pagetreeMenu entryconfigure 1 -state disabled
	.pagetreeMenu entryconfigure 2 -state disabled
    }

    if {!$Session::sessionConnected && !$Session::privateSession} {
	.pagetreeMenu entryconfigure 4 -state disabled
	.pagetreeMenu entryconfigure 5 -state disabled
    }

    tk_popup .pagetreeMenu $x $y
}

PageTree::BuildPopup


proc PageTree::DeletePages { } {
    toplevel .deletepages
    wm withdraw .deletepages
    focus .deletepages
    grab .deletepages

    wm title .deletepages "Delete Pages"
    catch {wm iconbitmap .deletepages "../bin/mlb.ico"}
    wm resizable .deletepages 0 0 
    wm geometry .deletepages +[winfo pointerx .]+[winfo pointery .]

    set f1 [frame .deletepages.f1]

    label  $f1.l1 -text "From page" -justify left -anchor w
    entry  $f1.e1 -background white -justify left -width 3
    $f1.e1 insert 0 "1"
    label  $f1.l2 -text "to page (incl.)" -justify left -anchor w
    entry  $f1.e2 -background white -justify left -width 3
    $f1.e2 insert 0 "2"

    pack $f1.l1 -side left -padx 2 -pady 2
    pack $f1.e1 -side left -padx 6 -pady 2
    pack $f1.l2 -side left -padx 2 -pady 2
    pack $f1.e2 -side left -padx 6 -pady 2
    pack $f1 -fill both -pady 4 -padx 4

    #Buttons
    frame  .deletepages.f3 
    button .deletepages.f3.b1 -text "Ok" -command PageTree::DeletePagesOK
    button .deletepages.f3.b2 -text "Cancel" -command "destroy .deletepages"
    pack   .deletepages.f3.b1 .deletepages.f3.b2 -side left -pady 2 -padx 10
    pack   .deletepages.f3 -pady 10 -padx 4

    wm deiconify .deletepages
}


proc PageTree::DeletePagesOK { } {
    WorkingArea::AutoEndPolyObjects

    set from [.deletepages.f1.e1 get]
    set to [.deletepages.f1.e2 get]

    if {([string length $from] == 0) || ([string length $to] == 0)} {return}
    if {$from >= $to} {return}

    MLB::DeletePages $from [expr $to - $from + 1]

    destroy .deletepages
}
