##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################


namespace eval Participant {
    variable show
    variable cname
    variable name
    variable email
    variable phone
    variable location
    variable color
    variable participantList
    variable raisehand
    variable lowerhand
    variable call
    variable indicate

    set show 1
    set raisehand(state) "normal"
    set lowerhand(state) "disabled"
    set call(state) "disabled"
    set indicate(mode) 1
}


proc Participant::DisplayLocalParticipant {} {
    if [winfo exists .lparticipant] {
	return -1
    }

    variable cname
    variable name
    variable email
    variable phone
    variable location

    toplevel .lparticipant
    wm withdraw .lparticipant
    focus .lparticipant
    grab .lparticipant

    wm title .lparticipant "Local Participant"
    catch {wm iconbitmap .lparticipant "../bin/mlb.ico"}
    wm resizable .lparticipant 0 0 
    wm geometry .lparticipant +[winfo pointerx .]+[winfo pointery .]

    frame .lparticipant.values

    frame .lparticipant.values.v1
    label .lparticipant.values.v1.l -text "CName:" -width 9 -justify left -anchor w
    entry .lparticipant.values.v1.e -background white -justify left -textvariable Participant::cname -width 50 -state disabled
    .lparticipant.values.v1.e xview moveto 1.0
    frame .lparticipant.values.v2
    label .lparticipant.values.v2.l -text "Name:" -width 9 -justify left -anchor w
    entry .lparticipant.values.v2.e -background white -justify left -textvariable Participant::name -width 50
    .lparticipant.values.v2.e xview moveto 1.0
    frame .lparticipant.values.v3
    label .lparticipant.values.v3.l -text "EMail:" -width 9 -justify left -anchor w
    entry .lparticipant.values.v3.e -background white -justify left -textvariable Participant::email -width 50
    .lparticipant.values.v3.e xview moveto 1.0
    frame .lparticipant.values.v4
    label .lparticipant.values.v4.l -text "Phone:" -width 9 -justify left -anchor w
    entry .lparticipant.values.v4.e -background white -justify left -textvariable Participant::phone -width 50
    .lparticipant.values.v4.e xview moveto 1.0
    frame .lparticipant.values.v5
    label .lparticipant.values.v5.l -text "Location:" -width 9 -justify left -anchor w
    entry .lparticipant.values.v5.e -background white -justify left -textvariable Participant::location -width 50
    .lparticipant.values.v5.e xview moveto 1.0
    
    pack .lparticipant.values.v1.l .lparticipant.values.v1.e -side left -padx 2 -pady 2
    pack .lparticipant.values.v2.l .lparticipant.values.v2.e -side left -padx 2 -pady 2
    pack .lparticipant.values.v3.l .lparticipant.values.v3.e -side left -padx 2 -pady 2
    pack .lparticipant.values.v4.l .lparticipant.values.v4.e -side left -padx 2 -pady 2
    pack .lparticipant.values.v5.l .lparticipant.values.v5.e -side left -padx 2 -pady 2

    pack .lparticipant.values.v1 .lparticipant.values.v2 .lparticipant.values.v3 .lparticipant.values.v4 .lparticipant.values.v5 -side top

    frame  .lparticipant.buttons
    button .lparticipant.buttons.b1  -text "Save" -command "Participant::SaveLocalParticipant; destroy .lparticipant"
    button .lparticipant.buttons.b2  -text "Cancel" -command "destroy .lparticipant"

    pack .lparticipant.buttons.b1 .lparticipant.buttons.b2 -side left -padx 10

    pack .lparticipant.values -side top -pady 5 -padx 2
    pack .lparticipant.buttons -ipadx 10 -ipady 5

    wm deiconify .lparticipant
}


proc Participant::SaveLocalParticipant {} {
    variable cname
    variable name
    variable email
    variable phone
    variable location

    set profile [open ".mlb_userprofile" w 0600]
    puts $profile "$Participant::name"
    puts $profile "$Participant::email"
    puts $profile "$Participant::phone" 
    puts $profile "$Participant::location"

    close $profile

    MLB::SetParticipantName "$Participant::name" 
    MLB::SetParticipantEmail "$Participant::email" 
    MLB::SetParticipantPhone "$Participant::phone" 
    MLB::SetParticipantLocation "$Participant::location"
}


proc Participant::ReadLocalParticipant {} {
    variable cname
    variable name
    variable email
    variable phone
    variable location
    
    if [file exists ".mlb_userprofile"] {
	set profile [open ".mlb_userprofile" r]
	gets $profile Participant::name
	gets $profile Participant::email
	gets $profile Participant::phone
	gets $profile Participant::location
	
	close $profile
    } else {
	set Participant::name  ""
	set Participant::email ""
	set Participant::phone ""
	set Participant::location ""
    }

    MLB::SetParticipantName "$Participant::name" 
    MLB::SetParticipantEmail "$Participant::email" 
    MLB::SetParticipantPhone "$Participant::phone" 
    MLB::SetParticipantLocation "$Participant::location"
}


proc Participant::SetCName {_cname} {
    variable cname
    set Participant::cname $_cname
}


proc Participant::SetColor {pid fg sfg} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    $participantList itemconfigure $pid -fill $fg
}


proc Participant::CreateParticipantList {parent} {
    variable raisehand
    variable lowerhand
    variable call
    variable indicate
    variable participantList

    frame $parent.top
    label $parent.top.l -text "Participants" -anchor w

    set indicate(button) [Button $parent.top.ind -image indicate -anchor w -relief sunken -bg LightGrey -bd 1 -command {Participant::ChangeIndicate}]
    DynamicHelp::register $indicate(button) balloon "Open a message window if \na participant raises his hand"
    HighlightWidget $indicate(button)

    set sw [ScrolledWindow $parent.participantsw]
    set participantList [ListBox $sw.participantList -bg white -height 6 -width 8 -relief sunken -selectforeground "#787878" -padx 26]
    $sw setwidget $participantList

    $participantList bindText <Double-ButtonPress-1> Participant::SelectParticipantAndShowInfo
    $participantList bindText <ButtonPress-1> Participant::SelectParticipant

    bind $participantList <ButtonPress-3> {Participant::ShowPopup %X %Y}

    frame $parent.bottom
    
    set raisehand(button) [Button $parent.bottom.raise -image raisehand -anchor w -relief flat -bd 1 \
	    -state $raisehand(state) -command {Participant::RaiseHand}]
    DynamicHelp::register $raisehand(button) balloon "Raise Hand"
    HighlightWidget $raisehand(button)

    set lowerhand(button) [Button $parent.bottom.lower -image lowerhand -anchor w -relief flat -bd 1 \
	    -state $lowerhand(state) -command {Participant::LowerHand}]
    DynamicHelp::register $lowerhand(button) balloon "Lower Hand"
    HighlightWidget $lowerhand(button)

    set call(button) [Button $parent.bottom.call -image call -anchor w -relief flat -bd 1 \
	    -state $call(state) -command {Participant::Call}]
    DynamicHelp::register $call(button) balloon "Call selected participant"
    HighlightWidget $call(button)

    pack $parent.top.l -anchor w -side left
    pack $parent.top.ind -anchor e -side right
    pack $parent.top -fill x -anchor w		       

    pack $sw  -side top  -expand yes -fill both -anchor w

    pack $parent.bottom.raise -anchor w -side left
    pack $parent.bottom.lower -anchor w -side left
    pack $parent.bottom.call  -anchor e -side right
    pack $parent.bottom -fill x -anchor w		       

    return $participantList
}


proc Participant::DisplayParticipantList {participants} {
    variable participantList
    
    if {![winfo exists $participantList]} {return}

    foreach pid [$participantList items] {
	.plistcanvas$pid delete plist1image$pid
	.plistcanvas$pid delete plist2image$pid
	destroy .plistcanvas$pid
    }
    $participantList delete [$participantList items]

    foreach {pid pName tp exited}  $participants {
	canvas .plistcanvas$pid -bg white -highlightbackground white -highlightcolor white -bd 0 -relief flat -state disabled -height 11 -width 21
	if {$exited == 1} {
	    set image [.plistcanvas$pid create image 6 6 -image exited]
	    .plistcanvas$pid addtag plist1image$pid withtag $image
	} else {
	    set image [.plistcanvas$pid create image 6 6 -image hrOff]
	    .plistcanvas$pid addtag plist1image$pid withtag $image
	}
	if {[string compare $tp "none"] != 0} {
	    image create bitmap tpsmall$pid -file "../gifs/tp_small.bmp" -foreground $tp -background white
	    set image [.plistcanvas$pid create image 16 6 -image tpsmall$pid]
	    .plistcanvas$pid addtag plist2image$pid withtag $image
	}
	$participantList insert end "$pid" -text "$pName" -fill "#787878" -window .plistcanvas$pid
    }
} 


proc Participant::ClearParticipantList { } {
    variable participantList

    if {![winfo exists $participantList]} {return}

    foreach pid [$participantList items] {
	.plistcanvas$pid delete plist1image$pid
	.plistcanvas$pid delete plist2image$pid
	destroy .plistcanvas$pid
    }
    $participantList delete [$participantList items]
}


proc Participant::DeleteSelection {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    $participantList selection clear
}


proc Participant::SelectParticipant {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}
    
    $participantList selection set $pid
}


proc Participant::SelectParticipantAndShowInfo {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}
    
    $participantList selection set $pid

    MLB::DisplayParticipant $pid
}

proc Participant::ShowInfo { } {
    variable participantList

    set selected [$participantList selection get]
    if {[llength $selected] == 0} {
	return
    }

    MLB::DisplayParticipant [lindex $selected 0]
}

proc Participant::DisplayParticipant {cname name email phone location tool} {
    if [winfo exists .participant] {
	return -1
    }

    toplevel .participant
    wm withdraw .participant
    focus .participant
    #grab .participant

    wm title .participant "Participant Info"
    catch {wm iconbitmap .participant "../bin/mlb.ico"}
    wm resizable .participant 0 0 
    wm geometry .participant +[winfo pointerx .]+[winfo pointery .]

    frame .participant.values

    frame .participant.values.v1
    label .participant.values.v1.l -text "CName:" -width 9 -justify left -anchor w
    label .participant.values.v1.e -background white -justify left -text $cname -width 50 -relief sunken -anchor w
    frame .participant.values.v2
    label .participant.values.v2.l -text "Name:" -width 9 -justify left -anchor w
    label .participant.values.v2.e -background white -justify left -text $name -width 50 -relief sunken -anchor w
    frame .participant.values.v3
    label .participant.values.v3.l -text "EMail:" -width 9 -justify left -anchor w
    label .participant.values.v3.e -background white -justify left -text $email -width 50 -relief sunken -anchor w
    frame .participant.values.v4
    label .participant.values.v4.l -text "Phone:" -width 9 -justify left -anchor w
    label .participant.values.v4.e -background white -justify left -text $phone -width 50 -relief sunken -anchor w
    frame .participant.values.v5
    label .participant.values.v5.l -text "Location:" -width 9 -justify left -anchor w
    label .participant.values.v5.e -background white -justify left -text $location -width 50 -relief sunken -anchor w
    frame .participant.values.v6
    label .participant.values.v6.l -text "Tool:" -width 9 -justify left -anchor w
    label .participant.values.v6.e -background white -justify left -text $tool -width 50 -relief sunken -anchor w
    
    pack .participant.values.v1.l .participant.values.v1.e -side left -padx 2 -pady 2
    pack .participant.values.v2.l .participant.values.v2.e -side left -padx 2 -pady 2
    pack .participant.values.v3.l .participant.values.v3.e -side left -padx 2 -pady 2
    pack .participant.values.v4.l .participant.values.v4.e -side left -padx 2 -pady 2
    pack .participant.values.v5.l .participant.values.v5.e -side left -padx 2 -pady 2
    pack .participant.values.v6.l .participant.values.v6.e -side left -padx 2 -pady 2

    pack .participant.values.v1 .participant.values.v2 .participant.values.v3 .participant.values.v4 .participant.values.v5 .participant.values.v6 -side top

    frame .participant.buttons
    button .participant.buttons.b1  -text "Ok" -command Participant::KillParticipantInfo

    pack .participant.buttons.b1 -side left

    pack .participant.values -side top -pady 5 -padx 2
    pack .participant.buttons -ipadx 10 -ipady 5

    wm deiconify .participant
}

proc Participant::KillParticipantInfo { } {
     variable participantList
    $participantList selection clear
    destroy .participant
}

proc Participant::ExitIndicatorOn {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    if {![$participantList exists $pid]} {return}

    .plistcanvas$pid itemconfigure plist1image$pid -image exited
}

proc Participant::ExitIndicatorOff {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    if {![$participantList exists $pid]} {return}

    .plistcanvas$pid itemconfigure plist1image$pid -image hrOff
}

proc Participant::HrIndicatorOn {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    if {![$participantList exists $pid]} {return}

    .plistcanvas$pid itemconfigure plist1image$pid -image hrOn
}

proc Participant::HrIndicatorOff {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    if {![$participantList exists $pid]} {return}

    .plistcanvas$pid itemconfigure plist1image$pid -image hrOff
}

proc Participant::TpIndicatorOn {pid color} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    if {![$participantList exists $pid]} {return}

    image create bitmap tpsmall$pid -file "../gifs/tp_small.bmp" -foreground $color -background white
    set image [.plistcanvas$pid create image 16 6 -image tpsmall$pid]
    .plistcanvas$pid addtag plist2image$pid withtag $image
}

proc Participant::TpIndicatorOff {pid} {
    variable participantList
    if {![winfo exists $participantList]} {return}

    if {![$participantList exists $pid]} {return}

    image create bitmap tpsmall$pid -file "../gifs/tp_small.bmp" -foreground white -background white
    set image [.plistcanvas$pid create image 16 6 -image tpsmall$pid]
    .plistcanvas$pid addtag plist2image$pid withtag $image
}


proc Participant::HrChangeIndicate { } {
    variable indicate
 
    if {!$indicate(mode)} {
	HRT::SetIndicate 1
	set indicate(mode) 1
	$indicate(button) configure -relief "sunken" -bg LightGrey
    } else {
	HRT::SetIndicate 0
	set indicate(mode) 0
	$indicate(button) configure -relief "flat" -bg $Config::bgColor
    }

}

proc Participant::RaiseHand { } {
    variable raisehand
    variable lowerhand

    set raisehand(state) "disabled"
    set lowerhand(state) "normal"

    $raisehand(button) configure -state disabled
    $lowerhand(button) configure -state normal

    HRT::RaiseHand
}


proc Participant::LowerHand { } {
    variable raisehand
    variable lowerhand

    set raisehand(state) "normal"
    set lowerhand(state) "disabled"

    $raisehand(button) configure -state normal
    $lowerhand(button) configure -state disabled

    HRT::LowerHand
}


proc Participant::Call { } {
    variable participantList

    set selected [$participantList selection get]
    if {[llength $selected] == 0} {
	return
    }

    $participantList selection clear
    HRT::CallParticipant [lindex $selected 0]
}

proc Participant::CallDialog { participant } {
    variable raisehand
    variable lowerhand

    set raisehand(state) "normal"
    set lowerhand(state) "disabled"

    $raisehand(button) configure -state normal
    $lowerhand(button) configure -state disabled

    destroy .msgdlg
    MessageDlg .msgdlg -parent . -title "Hand-Raising Tool"  -cancel 1\
	    -message "You have been called by $participant" -icon info -justify center -type ok
}


proc Participant::SetCallButtonState {state} {
    variable call
    $call(button) configure -state $state
    set call(state) $state
}


proc Participant::RaiseHandDialog { participant } {
    destroy .msgdlg
    MessageDlg .msgdlg -parent . -title "Hand-Raising Tool"  -cancel 1\
	    -message "Participant $participant has raised his hand" -icon info -justify center -type ok
}


proc Participant::HrEnableGUI { } {
    variable raisehand
    set $raisehand(state) normal

    variable participantList
    if {![winfo exists $participantList]} {return}

    $raisehand(button) configure -state normal
}


proc Participant::HrDisableGUI {} {
    variable raisehand
    set $raisehand(state) disabled

   variable participantList
    if {![winfo exists $participantList]} {return}

    $raisehand(button) configure -state disabled
}

proc Participant::BuildPopup { } {
    menu .participantMenu -tearoff 0
    .participantMenu add command -label "Show Info..." -command {Participant::ShowInfo}
    .participantMenu add command -label "Call" -command {Participant::Call}
    .participantMenu add separator
    .participantMenu add command -label "Clear List" -command {MLB::RemoveExitedParticipants}
}

Participant::BuildPopup

proc Participant::ShowPopup {x y} {
    if {!$Session::sessionConnected} {
	.participantMenu entryconfigure 0 -state disabled
	.participantMenu entryconfigure 1 -state disabled
	.participantMenu entryconfigure 3 -state disabled
    } else {
	.participantMenu entryconfigure 0 -state normal
	.participantMenu entryconfigure 1 -state normal
	.participantMenu entryconfigure 3 -state normal
    }
    tk_popup .participantMenu $x $y
}
