##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval Record {
 
    variable parent
    variable openTool 0 
    variable Record 0
    variable Pause  1
    variable Stop   2
    variable recordEntry
    variable frameRate 16000
    variable status $Stop
    

    set recordEntry($Record) {
	{Record::StartRecording}
	{Bitmap::get "../gifs/record.gif"}
	{Bitmap::get "../gifs/record_i.gif"}
	{Record Session (AOF)}
    }
    set recordEntry($Pause) {
	{Record::PauseRecording}
	{Bitmap::get "../gifs/pause.gif"}
	{Bitmap::get "../gifs/pause_i.gif"}
	{Pause Recording}
    }
    set recordEntry($Stop) {
	{Record::StopRecording}
	{Bitmap::get "../gifs/stop.gif"}
	{Bitmap::get "../gifs/stop_i.gif"}
	{Stop Recording}
    }
}

proc Record::Create {_parent} {
    variable parent
    variable Record
    variable Pause
    variable Stop
    variable recordEntry
    
    set parent $_parent
    foreach entry [array names recordEntry] {
	Button $parent.rec$entry -image [eval [lindex $recordEntry($entry) 1]] \
	    -highlightthickness 0 -takefocus 0 -borderwidth 0 \
	    -command [lindex $recordEntry($entry) 0] 
	pack $parent.rec$entry -ipadx 2 -side left -anchor n
    }	
}

proc Record::StartRecording {} {
    variable recordEntry
    variable parent
    variable recLabel

    $parent.rec$Record::Stop configure -image [eval [lindex $recordEntry($Record::Stop) 1]]
    $parent.rec$Record::Pause configure -image [eval [lindex $recordEntry($Record::Pause) 1]]
    $parent.rec$Record::Record configure -image [eval [lindex $recordEntry($Record::Record) 2]]

    if {[info exists recLabel]} {
	destroy $recLabel
    }
    set recLabel [$Toplevel::toplevel addindicator -text "RECORDING" -fg red]
    
    set Record::status $Record::Record
    MLB::HandleRecording $Record::Record

    $Toplevel::toplevel setmenustate muting disable

    set Config::freezeTree 1
}

proc Record::PauseRecording {} {
    variable recordEntry
    variable parent
    variable recLabel

    $parent.rec$Record::Record configure -image [eval [lindex $recordEntry($Record::Stop) 1]]
    $parent.rec$Record::Pause configure -image [eval [lindex $recordEntry($Record::Pause) 2]]
    $parent.rec$Record::Record configure -image [eval [lindex $recordEntry($Record::Record) 1]]

    if {[info exists recLabel]} {
	destroy $recLabel
    }
    set recLabel [$Toplevel::toplevel addindicator -text "PAUSE RECORDING" -fg red]
    set Record::status $Record::Pause
    MLB::HandleRecording $Record::Pause

    set Config::freezeTree 0
}

proc Record::StopRecording {} {
    variable recordEntry
    variable parent
    variable recLabel

    if {[info exists recLabel]} {
	destroy $recLabel
    }

    $parent.rec$Record::Stop configure -image [eval [lindex $recordEntry($Record::Stop) 1]]
    $parent.rec$Record::Pause configure -image [eval [lindex $recordEntry($Record::Pause) 1]]
    $parent.rec$Record::Record configure -image [eval [lindex $recordEntry($Record::Record) 1]]

    set Record::status $Record::Stop
    MLB::HandleRecording $Record::Stop

    $Toplevel::toplevel setmenustate muting normal

    set Config::freezeTree 0
}

proc Record::ChangeFrameRate {} {
    variable frameRate

    MLB::ChangeFrameRate $frameRate
}
