##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

source debuginfo.tcl

namespace eval Session {
    variable local
    variable globalVisible 1

    variable smpConnected

    variable idServerConnected
    variable idServerAddr
    variable idServerPort
    variable idConnectButton

    variable uceServerConnected
    variable uceServerAddr
    variable uceServerPort
    variable uceConnectButton

    variable sessionConnected
    variable name	
    variable ipAddr
    variable port
    variable ttl
    variable bandwidth
    variable cueRate
    variable localLag
    variable storageTime
    variable checkIntentionViolation
    variable logPageChanges

    variable privateSession

    set local 1

    set smpConnected 0

    set idServerConnected 0
    set idServerAddr "134.155.48.96"
    set idServerPort "6000"

    set uceServerConnected 0
    set uceServerAddr "134.155.48.96"
    set uceServerPort "8000"

    set sessionConnected 0
    set id "0"
    set name "default"
    set ipAddr "224.100.100.120"
    set port "8000"
    set ttl "4"
    set bandwidth "100000"
    set cueRate 10
    set localLag 100
    set storageTime 180
    set checkIntentionViolation 1
    set logPageChanges 1

    set privateSession 0
}


proc Session::SetupDialog { } {
    variable local

    toplevel .networkSetup
    wm withdraw .networkSetup
    focus .networkSetup
    grab .networkSetup

    wm title .networkSetup "Setup mlb"
    catch {wm iconbitmap .networkSetup "../bin/mlb.ico"}
    wm resizable .networkSetup 0 0 
    wm protocol  .networkSetup WM_DELETE_WINDOW Session::CloseSetupDialog
    #wm geometry .networkSetup +[winfo pointerx .]+[winfo pointery .]
    wm geometry .networkSetup +[expr [winfo screenwidth .networkSetup] / 2 - 100]+[expr [winfo screenheight .networkSetup] / 2 - 50]

    frame .networkSetup.f0
    radiobutton .networkSetup.f0.r1 -variable Session::local -text "offline" -value "1"
    DynamicHelp::register .networkSetup.f0.r1 balloon \
	    "Start mlb stand alone.\nOnly private documents are available.\nGoing online later is not possible"
    pack .networkSetup.f0.r1 -pady 2 -padx 2 -side left
    pack .networkSetup.f0 -fill both -expand yes 

    frame .networkSetup.f2
    radiobutton .networkSetup.f2.r2 -variable Session::local -text "online"  -value "0"
    DynamicHelp::register .networkSetup.f0.r2 balloon \
	    "Start mlb with network ability.\nPrivate and shared documents are available"

    button .networkSetup.f2.a -text "Advanced >>" -anchor w -command "Session::OpenSetupAdvanced" -relief groove

    pack .networkSetup.f2.r2 -side left -padx 2 -pady 2 -anchor w
    pack .networkSetup.f2.a -side right -padx 2 -pady 2 -anchor e
    pack .networkSetup.f2 -fill both -expand yes 

    frame .networkSetup.f3
    pack .networkSetup.f3 -fill both

    frame .networkSetup.f1
    button .networkSetup.f1.b -text "Ok" -command "Session::CloseSetupDialog"  -width 8 -justify center -default active
    pack .networkSetup.f1.b -padx 2 -pady 2
    pack .networkSetup.f1 -fill both
    bind .networkSetup.f1.b <Return> {Session::CloseSetupDialog}
    focus .networkSetup.f1.b

    wm deiconify .networkSetup
    raise .networkSetup .
}


proc Session::OpenSetupAdvanced { } {
    .networkSetup.f2.a configure -text "Advanced <<" -command "Session::CloseSetupAdvanced"

    # ID Server
    frame  .networkSetup.f3.f1
    label  .networkSetup.f3.f1.l -text "ID Server Adress:" -width 17 -justify left -anchor w
    entry  .networkSetup.f3.f1.e -background white -justify left -textvariable Session::idServerAddr -width 15
    pack .networkSetup.f3.f1.l  -side left -padx 2 -pady 2
    pack .networkSetup.f3.f1.e  -side left -padx 12 -pady 2
    pack .networkSetup.f3.f1 -fill both -pady 4 -padx 4

    frame  .networkSetup.f3.f4
    label  .networkSetup.f3.f4.l -text "ID Server Port:" -width 17 -justify left -anchor w
    entry  .networkSetup.f3.f4.e -background white -justify left -textvariable Session::idServerPort -width 5 
    pack .networkSetup.f3.f4.l -side left -padx 2 -pady 2
    pack .networkSetup.f3.f4.e  -side left -padx 12 -pady 2
    pack .networkSetup.f3.f4 -fill both -pady 4 -padx 4

    #Separator .networkSetup.f3.s1 -orient horizontal 
    #pack .networkSetup.f3.s1 -fill x -pady 4 -padx 4

    # SMP
    #frame  .networkSetup.f3.f5
    #label  .networkSetup.f3.f5.l -text "SMP port:" -width 17 -justify left -anchor w
    #entry  .networkSetup.f3.f5.e -background white -justify left -textvariable Session::smpPort -width 5
    #pack .networkSetup.f3.f5.l  -side left -padx 2 -pady 2
    #pack .networkSetup.f3.f5.e  -side left -padx 12 -pady 2
    #pack .networkSetup.f3.f5 -fill both -pady 4 -padx 4
}


proc Session::CloseSetupAdvanced { } {
    .networkSetup.f2.a configure -text "Advanced >>" -command "Session::OpenSetupAdvanced"

    pack forget .networkSetup.f3.f1
    pack forget .networkSetup.f3.f4
    #pack forget .networkSetup.f3.f5
    #pack forget .networkSetup.f3.s1
    destroy .networkSetup.f3.f1
    destroy .networkSetup.f3.f4
    #destroy .networkSetup.f3.f5
    #destroy .networkSetup.f3.s1
}


proc Session::CloseSetupDialog { } {
    variable local
    variable globalVisible
    
    if {$local == 1} {
	StartLocalIDServer 
	if {$Config::showPagetree} {
	    pack forget $PageTree::synch(button)
	    pack forget $PageTree::annotation(button)
	    pack forget $PageTree::import(button)
	}
	set globalVisible 0
    } else {
	if [OpenIDServer] {
	    MLB::Online
	    OpenSMP
	    $Toplevel::toplevel setmenustate services normal
	    $Toplevel::toplevel setmenustate userprofile normal
	    Participant::ReadLocalParticipant
	} else { ;# failed, start locally instead
	    set local 1
	    set globalVisible 0
	    StartLocalIDServer 
	    if {$Config::showPagetree} {
		pack forget $PageTree::synch(button)
		pack forget $PageTree::annotation(button)
		pack forget $PageTree::import(button)
	    }
	}
    }
    
    MLB::LoadInitialDocument

    destroy .networkSetup
}


proc Session::StartupSharedMode { } {
    variable local
    variable globalVisible
    variable idServerConnected

    set local 0
    set globalVisible 1
    set idServerConnected 1

    MLB::Online
    OpenSMP

    $Toplevel::toplevel setmenustate services normal
    $Toplevel::toplevel setmenustate userprofile normal
    $Toplevel::toplevel setmenustate openfile normal
    $Toplevel::toplevel setmenustate privatesession normal
    $Toplevel::toplevel setmenustate sharedsession normal

    Participant::ReadLocalParticipant
    MLB::LoadInitialDocument
}


proc Session::StartLocalIDServer {} {
    variable local

    if {[catch {MLB::OpenIDServer} result]} {
	DebugInfo::error "Error: Could not connect to ID Server\n"
	Toplevel::SetStatus "Error: Could not connect to ID Server"
	return
    } 

    $Toplevel::toplevel setmenustate privatesession normal
    $Toplevel::toplevel setmenustate openfile normal

    set local 1
}


proc Session::ServicesDialog { } {
    variable local

    variable smpConnected

    variable idServerConnected
    variable idServerAddr
    variable idServerPort
    variable idConnectButton

    variable uceServerConnected
    variable uceServerAddr
    variable uceServerPort
    variable uceConnectButton

    variable sessionConnected

    toplevel .network
    wm withdraw .network
    focus .network
    grab .network

    wm title .network "Services"
    catch {wm iconbitmap .network "../bin/mlb.ico"}
    wm resizable .network 0 0 
    wm geometry .network +[winfo pointerx .]+[winfo pointery .]

    # ID Server
#    TitleFrame .network.f1 -text "ID Server"
#    set f1 [.network.f1 getframe]

#    frame  $f1.v1
#    label  $f1.v1.l -text "Adress:" -width 7 -justify left -anchor w
#    entry  $f1.v1.e -background white -justify left -textvariable Session::idServerAddr -width 15
#    set idConnectButton [button $f1.v1.b -text "Connect" -command "Session::OpenIDServer" -anchor e -width 8]
#    if {$idServerConnected || $local} {
#	$f1.v1.b configure -state disabled
#    }

#    pack $f1.v1.l  -side left -padx 2 -pady 2
#    pack $f1.v1.e  -side left -padx 6 -pady 2
#    pack $f1.v1.b -side right -pady 2 -padx 2
#    pack $f1.v1 -fill both

#    frame  $f1.v2
#    label  $f1.v2.l -text "Port:" -width 7 -justify left -anchor w
#    entry  $f1.v2.e -background white -justify left -textvariable Session::idServerPort -width 5 
    
#    pack $f1.v2.l -side left -padx 2 -pady 2
#    pack $f1.v2.e  -side left -padx 6 -pady 2
#    pack $f1.v2 -fill both

#    pack $f1 -fill both
#    pack .network.f1 -fill both -pady 4 -padx 4

    # UCE Server
    TitleFrame .network.f2 -text "UCE Server"
    set f2 [.network.f2 getframe]

    frame  $f2.v1
    label  $f2.v1.l -text "Adress:" -width 7 -justify left -anchor w
    entry  $f2.v1.e -background white -justify left -textvariable Session::uceServerAddr -width 15
    set uceConnectButton [button $f2.v1.b -text "Connect" -command "Session::OpenUCEServer" -anchor e -width 8]
    if {$uceServerConnected || $local} {
	$f2.v1.b configure -state disabled
    }

    pack $f2.v1.l  -side left -padx 2 -pady 2
    pack $f2.v1.e  -side left -padx 6 -pady 2
    pack $f2.v1.b -side right -pady 2 -padx 2
    pack $f2.v1 -fill both

    frame  $f2.v2
    label  $f2.v2.l -text "Port:" -width 7 -justify left -anchor w
    entry  $f2.v2.e -background white -justify left -textvariable Session::uceServerPort -width 5 
    
    pack $f2.v2.l -side left -padx 2 -pady 2
    pack $f2.v2.e  -side left -padx 6 -pady 2
    pack $f2.v2 -fill both

    pack $f2 -fill both
    pack .network.f2 -fill both -pady 4 -padx 4
    

    #Buttons
    frame  .network.f3 
    button .network.f3.b2 -text "Dismiss" -command "destroy .network"
    pack   .network.f3.b2 -pady 2 -padx 2
    pack   .network.f3 -pady 10 -padx 4

    wm deiconify .network
}


proc Session::OpenSMP {} {
    variable idServerConnected
    variable sessionConnected
    variable smpConnected

    if {[catch {MLB::OpenSMP} result]} {
	DebugInfo::error "Error: Could not connect to SMP-Daemon\n"
	Toplevel::SetStatus "Error: Could not connect to SMP-Daemon"
	return
    } 

    if [winfo exists .network] {
	.network.f0.v1.b configure -state disabled
    }
    set smpConnected 1

    if {($sessionConnected == 0) && $idServerConnected} {
	$Toplevel::toplevel setmenustate sharedsession normal
    }
}


proc Session::CloseSMP {} {
    variable sessionConnected
    variable smpConnected

    if {$sessionConnected} {
	return
    }

    if {[catch {MLB::CloseSMP} result]} {
	DebugInfo::error "Error: Could not close Connection to SMP-Daemon\n"
	Toplevel::SetStatus "Error: Could not close Connection to SMP-Daemon"
	return
    } 
    
    $Toplevel::toplevel setmenustate sharedsession disabled

    .network.f0.v1.b configure -state normal
    .network.f0.v2.b configure -state disabled
    set smpConnected 0
}


proc Session::OpenIDServer {} {
    variable sessionConnected
    variable smpConnected
    variable idServerConnected
    variable idServerAddr
    variable idServerPort
    variable idConnectButton
    
    if {[catch {MLB::OpenIDServer $Session::idServerAddr $Session::idServerPort} result]} {
	DebugInfo::errorPopup "Error: Could not connect to ID Server,\nstarting mlb offline instead"
	Toplevel::SetStatus "Error: Could not connect to ID Server, starting mlb offline instead"
	return 0
    } 

    if [winfo exists .network] {
	$idConnectButton configure -state disabled
    }
    set idServerConnected 1

    $Toplevel::toplevel setmenustate openfile normal
    $Toplevel::toplevel setmenustate privatesession normal

    if {($sessionConnected == 0) && $smpConnected} {
	$Toplevel::toplevel setmenustate sharedsession normal
    }

    return 1
}


proc Session::CloseIDServer {} {
    variable sessionConnected
    variable idServerConnected

    if {$sessionConnected} {
	return
    }

    if {[catch {MLB::CloseIDServer} result]} {
	DebugInfo::error "Error: Could not close Connection to ID Server\n"
	Toplevel::SetStatus "Error: Could not close Connection to ID Server"
	return
    } 

    $Toplevel::toplevel setmenustate openfile disabled
    $Toplevel::toplevel setmenustate sharedsession disabled
    $Toplevel::toplevel setmenustate privatesession disabled

    .network.f1.v1.b configure -state normal
    set idServerConnected 1
}


proc Session::OpenUCEServer {} {
    variable uceServerConnected
    variable uceServerAddr
    variable uceServerPort
    variable uceConnectButton
    
    if {[catch {MLB::OpenUCEServer $Session::uceServerAddr $Session::uceServerPort} result]} {
	DebugInfo::errorPopup "Error: Could not connect to UCE Server"
	Toplevel::SetStatus "Error: Could not connect to UCE Server"
	return 0
    } 

    if [winfo exists .network] {
	$uceConnectButton configure -state disabled
    }
    set uceServerConnected 1

    return 1
}


proc Session::ValidateSharedName { status text } {
    if { $status == 0 || $status == 1 } {
         if { [lsearch [Config::GetProfileNames] $text] == -1 } {
              set Config::profileParameter($Session::id,name) $text
              .session.f1.l.v1.c configure -values [Config::GetProfileNames]
         } else {
              return 0
         }     
    }
    return 1
}


proc Session::SharedSessionDialog { } {
    variable smpConnected
    variable idServerConnected
    variable sessionConnected
    variable name
    variable ipAddr
    variable port
    variable ttl
    variable bandwidth

    toplevel .session
    wm withdraw .session
    focus .session
    grab .session

    wm title .session "Shared Document"
    catch {wm iconbitmap .session "../bin/mlb.ico"}
    wm resizable .session 0 0 
    wm geometry .session +[winfo pointerx .]+[winfo pointery .]

    frame  .session.f1
    frame  .session.f1.l
    frame  .session.f1.l.v1
    label  .session.f1.l.v1.l -text "Session Name:" -width 18 -justify left -anchor w
    ComboBox .session.f1.l.v1.c -values [Config::GetProfileNames] -editable 1 -textvariable Session::name\
        	    -modifycmd "Config::OpenProfile" \
        	    -width 15 -justify left\
              -validatecommand "after idle {%W config -validate %v}; Session::ValidateSharedName %d %P"\
              -validate key
    pack   .session.f1.l.v1.l .session.f1.l.v1.c -side left -padx 2 -pady 2
    pack   .session.f1.l.v1 -padx 9 -fill both

    # RTP/I Session
    frame  .session.f1.l.v2
    label  .session.f1.l.v2.l -text "Session IP Address:" -width 18 -justify left -anchor w
    entry  .session.f1.l.v2.e -background white -textvariable Session::ipAddr -width 17 -justify left\
           -validatecommand "after idle {%W config -validate %v}; if { %d == 0 || %d == 1 } { Session::SetIpAddr %P }; return 1"\
           -validate key
    
    pack   .session.f1.l.v2.l .session.f1.l.v2.e -side left -padx 2 -pady 2
    pack   .session.f1.l.v2 -padx 9 -fill both

    frame  .session.f1.l.v3
    label  .session.f1.l.v3.l -text "Session Port:" -width 18 -justify left -anchor w
    entry  .session.f1.l.v3.e -background white -justify left -textvariable Session::port -width 5\
           -validatecommand "after idle {%W config -validate %v}; if { %d == 0 || %d == 1 } { Session::SetPort %P }; return 1"\
           -validate key

    pack   .session.f1.l.v3.l .session.f1.l.v3.e -side left -padx 2 -pady 2
    pack   .session.f1.l.v3 -padx 9 -fill both
    
    frame  .session.f1.r
    button .session.f1.r.b1 -text "Join" -command "Session::OpenRTPI" -width 8
    if {$sessionConnected || ($smpConnected == 0) || ($idServerConnected == 0)} {
	    .session.f1.r.b1 configure -state disabled
    }
    pack   .session.f1.r.b1 -padx 11 -pady 2
    button .session.f1.r.b2 -text "Leave" -command "Session::CloseRTPI" -width 8 
    if {$sessionConnected == 0} {
	    .session.f1.r.b2 configure -state disabled
    }
    pack   .session.f1.r.b2 -padx 11 -pady 2
        
    pack   .session.f1.l .session.f1.r -side left -pady 2 -anchor n
    pack   .session.f1 -pady 2

    frame  .session.f2
    frame  .session.f2.v1
    frame  .session.f2.v1.l
    button .session.f2.v1.l.b -text "Advanced >>" -command "Session::OpenAdvanced" -relief groove 
    pack   .session.f2.v1.l.b -padx 11 
    pack   .session.f2.v1.l -anchor w -side left
    pack   .session.f2.v1 -fill both
    pack   .session.f2 -pady 2 -anchor w -fill both

    frame  .session.f3 
    button .session.f3.b -text "Dismiss" -command "destroy .session"
    pack   .session.f3.b 
    pack   .session.f3 -pady 4 

    wm deiconify .session
}


proc Session::OpenAdvanced { } {
    variable cueRate
    variable sessionConnected
 
    .session.f2.v1.l.b configure -text "Advanced <<" -command "Session::CloseAdvanced"
    
    frame  .session.f2.v1.r
    button .session.f2.v1.r.b1 -text "New" -command "Session::NewProfile" -width 8 
    button .session.f2.v1.r.b2 -text "Delete" -command "Session::DeleteProfile" -width 8 
    pack   .session.f2.v1.r.b1 .session.f2.v1.r.b2 -padx 4 -side left 
    pack   .session.f2.v1.r -padx 9 -anchor e
        
    if { $sessionConnected } {
      .session.f2.v1.r.b1 configure -state disabled
      .session.f2.v1.r.b2 configure -state disabled
    }
        
    frame  .session.f2.v2
    frame  .session.f2.v2.v1
    label  .session.f2.v2.v1.l -text "TTL:" -width 18 -justify left -anchor w
    entry  .session.f2.v2.v1.e -background white -justify left -textvariable Session::ttl -width 5\
           -validatecommand "after idle {%W config -validate %v}; if { %d == 0 || %d == 1 } { Session::SetTTL %P }; return 1"\
           -validate key

    pack .session.f2.v2.v1.l .session.f2.v2.v1.e -side left -padx 2 -pady 2 
    pack .session.f2.v2.v1 -padx 9 -fill both

    frame   .session.f2.v2.v2
    label  .session.f2.v2.v2.l -text "Bandwidth:" -width 18 -justify left -anchor w
    entry  .session.f2.v2.v2.e -background white -justify left -textvariable Session::bandwidth -width 7\
           -validatecommand "after idle {%W config -validate %v}; if { %d == 0 || %d == 1 } { Session::SetBandwidth %P }; return 1"\
           -validate key
    label  .session.f2.v2.v2.m -text "bytes/sec." -justify left -anchor w

    pack .session.f2.v2.v2.l .session.f2.v2.v2.e .session.f2.v2.v2.m -side left -padx 2 -pady 2
    pack .session.f2.v2.v2 -padx 9 -fill both

    frame   .session.f2.v2.v3
    label   .session.f2.v2.v3.l -text "Cue Rate:" -width 18 -justify left -anchor w
  
    SpinBox .session.f2.v2.v3.s -dragenabled false -dropenabled false -entrybg white -modifycmd "Session::SetCueRate" -textvariable Session::cueRate \
	    -width 5 -values {10 20 30 40 50 60 70 80 90 100} -editable 0 -justify left\
	    -helptext "Specify the percentage of mouse events\nbeing transmitted during move and\nchange size operations"

    label   .session.f2.v2.v3.m -text "%" -justify left -anchor w

    pack .session.f2.v2.v3.l .session.f2.v2.v3.s .session.f2.v2.v3.m -side left -padx 2 -pady 2
    pack .session.f2.v2.v3 -padx 9 -fill both

    frame   .session.f2.v2.v4
    label   .session.f2.v2.v4.l -text "Local Lag:" -width 18 -justify left -anchor w
    SpinBox .session.f2.v2.v4.s -dragenabled false -dropenabled false -entrybg white -modifycmd "Session::SetLocalLag" -textvariable Session::localLag \
	    -width 5 -range {0 300 1} -editable 0 -justify left\
	    -helptext "Specify the local lag in milliseconds. The local lag\ndelays local user operations and is used to prevent short\nterm inconsistencies. A higher value will provide\nbetter consistency but will cause lower responsiveness\nof the medium."
    label   .session.f2.v2.v4.m -text "ms" -justify left -anchor w

    pack .session.f2.v2.v4.l .session.f2.v2.v4.s .session.f2.v2.v4.m -side left -padx 2 -pady 2
    pack .session.f2.v2.v4 -padx 9 -fill both

    frame   .session.f2.v2.v5
    label   .session.f2.v2.v5.l -text "State Storage:" -width 18 -justify left -anchor w
    SpinBox .session.f2.v2.v5.s -dragenabled false -dropenabled false -entrybg white -modifycmd "Session::SetStorageTime" -textvariable Session::storageTime \
	    -width 5 -range {30 600 1} -editable 0 -justify left\
	    -helptext "Specify the maximum time (in seconds) received and locally\nmade operations are stored. Inconsistencies are repaired by\nTimeWarp during this time. Inconsistencies relating to\na time further back in history cannot be repaired locally\nbut by State Request only. A higher value for the storage\n time therefore increases the probability that inconsistencies\ncan be repaired by TimeWarp but causes a higher demand\non memory."
    label   .session.f2.v2.v5.m -text "s" -justify left -anchor w

    pack .session.f2.v2.v5.l .session.f2.v2.v5.s .session.f2.v2.v5.m -side left -padx 2 -pady 2
    pack .session.f2.v2.v5 -padx 9 -fill both

    frame   .session.f2.v2.v6
    label   .session.f2.v2.v6.l -text "Indicate Conflicts:" -width 18 -justify left -anchor w
    checkbutton .session.f2.v2.v6.s -command "Session::SetCheckIntention" -variable Session::checkIntentionViolation 
    DynamicHelp::register .session.f2.v2.v5.s balloon "If enabled, all local and remote operations are checked\nif they conflict with each other (e.g., if the color of an\nobject is changed by two participants simultanously).\nA detected conflict will be displayed to all affected\nparticipants."

    pack .session.f2.v2.v6.l .session.f2.v2.v6.s -side left -padx 2
    pack .session.f2.v2.v6 -padx 9 -fill both

    frame   .session.f2.v2.v7
    label   .session.f2.v2.v7.l -text "Log Page Changes:" -width 18 -justify left -anchor w
    checkbutton .session.f2.v2.v7.s -command "Session::SetLogPageChanges" -variable Session::logPageChanges
    DynamicHelp::register .session.f2.v2.v7.s balloon "If enabled, all changes in the active page are logged in\n\"pages.log\". Use this file to generate a CBT."
    pack .session.f2.v2.v7.l .session.f2.v2.v7.s -side left -padx 2
    pack .session.f2.v2.v7 -padx 9 -fill both
    
    pack   .session.f2.v2 -side left -pady 2 -anchor n -fill both
}


proc Session::NewProfile { } {
    Config::NewProfile
    .session.f1.l.v1.c configure -values [Config::GetProfileNames]
}


proc Session::DeleteProfile { } {
    Config::DeleteProfile
    .session.f1.l.v1.c configure -values [Config::GetProfileNames]
}


proc Session::SetIpAddr { value } {
    set Config::profileParameter($Session::id,ipAddr) $value
}


proc Session::SetPort { value } {
    set Config::profileParameter($Session::id,port) $value
}


proc Session::SetTTL { value } {
    set Config::profileParameter($Session::id,ttl) $value
}


proc Session::SetBandwidth { value } {
    set Config::profileParameter($Session::id,bandwidth) $value
}


proc Session::SetCueRate { } {
    set Config::profileParameter($Session::id,cueRate) $Session::cueRate
    MLB::SetCueRate $Session::cueRate
}


proc Session::SetLocalLag { } {
    set Config::profileParameter($Session::id,localLag) $Session::localLag
    MLB::SetLocalLag $Session::localLag
}


proc Session::SetStorageTime { } {
    set Config::profileParameter($Session::id,storageTime) $Session::storageTime
    MLB::SetStorageTime $Session::storageTime
}


proc Session::SetCheckIntention { } {
    set Config::profileParameter($Session::id,checkIntentionViolation) $Session::checkIntentionViolation
    MLB::SetCheckIntention $Session::checkIntentionViolation
}


proc Session::SetLogPageChanges { } {
    set Config::profileParameter($Session::id,logPageChanges) $Session::logPageChanges
    MLB::SetLogPageChanges $Session::logPageChanges
}


proc Session::CloseAdvanced { } {
    .session.f2.v1.l.b configure -text "Advanced >>" -command "Session::OpenAdvanced"

    pack forget .session.f2.v2
    destroy .session.f2.v2

    pack forget .session.f2.v1.r
    destroy .session.f2.v1.r
}


proc Session::OpenRTPI {} {
    variable sessionConnected
    variable ipAddr
    variable port
    variable ttl
    variable bandwidth

    if {[catch {MLB::OpenSharedSession $Session::ipAddr $Session::port $Session::ttl $bandwidth} result]} {
	DebugInfo::error "Error: Could not open RTP/I Session\n"
	Toplevel::SetStatus "Error: Could not open RTP/I Session"
	return
    } 

    MLB::SetLocalLag $Session::localLag
    MLB::SetCueRate $Session::cueRate
    MLB::SetStorageTime $Session::storageTime
    MLB::SetCheckIntention $Session::checkIntentionViolation
    MLB::ParticipantHints $Config::participantHints
    MLB::SetLogPageChanges $Session::logPageChanges

    if {[winfo exists .session]} {
      .session.f1.r.b1 configure -state disabled
      .session.f1.r.b2 configure -state normal
    }

    $Toplevel::toplevel setmenustate sessionedit normal
    $Toplevel::toplevel setmenustate tool normal

    PageTree::SetNewPageButtonState 1
    PageTree::SetSynchButtonState 1
    PageTree::SetAnnotationButtonState 1

    set sessionConnected 1

    HRT::OpenRTPI

    if {[winfo exists .session]} {destroy .session}
}


proc Session::CloseRTPI {} {
    variable sessionConnected

    if {[catch {MLB::CloseSharedSession} result]} {
	DebugInfo::error "Error: Could not close RTP/I Session\n"
	Toplevel::SetStatus "Error: Could not close RTP/I Session"
	return
    } 

    .session.f1.r.b1 configure -state normal
    .session.f1.r.b2 configure -state disabled
    
    HRT::CloseRTPI
    Participant::LowerHand
    Participant::HrDisableGUI

    Chat::Close

    $Toplevel::toplevel setmenustate tool disabled
    PageTree::SetSynchButtonState 0

    set sessionConnected 0

    destroy .session
}


proc Session::NewPrivateSession { } {
    variable privateSession

    MLB::NewPrivateSession
    $Toplevel::toplevel setmenustate sessionedit normal
    
    set privateSession 1
}


proc Session::DeleteSession { } {
    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "Close Confirmation" -default 1 -cancel 1\
	    -message "Do you really want to close the active document?" -icon question -justify center -type yesno] {
	return
    }

    MLB::DeleteSession
}

