##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

source attributes.tcl

source bwidgets/bitmap.tcl

namespace eval ToolBar {

    variable toolBar
    variable toolBarFS
    variable currentAction
    variable currentTool
    variable position top
    variable fileEntry
    variable toolEntry
    variable actionEntry
    variable recLabel
    variable pointerEntry
    variable length
    variable FileTree
    variable sep
    variable undo
    variable redo
    variable mlbName "document.mlb"
    variable exportName "document.ps"
    variable imageName "image.gif"

    variable showTool

    variable Freehand       01
    variable OutRect        02
    variable OutOval        03
    variable FillRect       04
    variable FillOval       05
    variable Polyline       06
    variable OutlinePolygon 07
    variable FilledPolygon  08
    variable Line           09
    variable Arrow          10
    variable Text           11
    variable OnlinePointer  12

    variable Move   20
    variable Copy   21
    variable Delete 22
    variable Select 23

    variable OpenFile  30
    variable SaveFile  31
    variable Image     32

    set showTool($Freehand)       1
    set showTool($OutRect)        1
    set showTool($OutOval)        0
    set showTool($FillRect)       0
    set showTool($FillOval)       0
    set showTool($Polyline)       0
    set showTool($OutlinePolygon) 0
    set showTool($FilledPolygon)  0
    set showTool($Line)           1
    set showTool($Arrow)          1
    set showTool($Text)           0
    set showTool($OnlinePointer)  1

    set showTool($Move)   0
    set showTool($Copy)   0
    set showTool($Delete) 0
    set showTool($Select) 0

    set showTool($SaveFile)  0
    set showTool($OpenFile)  0

    set currentAction 0
    set currentTool $Freehand
    
    set undo(state) "disable"
    set redo(state) "disable"
    
    set fileEntry($OpenFile) {
	{ToolBar::OpenFile}
	{Bitmap::get "../gifs/open.gif"}
	{Open document}
    }
    set fileEntry($SaveFile) {
	{ToolBar::SaveFile}
	{Bitmap::get "../gifs/save.gif"}
	{Save activated document}
    }
    set toolEntry($OutRect) {
	{ToolBar::ChangeTool $ToolBar::OutRect}
	{Bitmap::get "../gifs/rect.gif"}
	{Rectangle}
    }
    set toolEntry($OutOval) {
	{ToolBar::ChangeTool $ToolBar::OutOval}
	{Bitmap::get "../gifs/circ.gif"}
	{Oval}
    }
    set toolEntry($FillRect) {
	{ToolBar::ChangeTool $ToolBar::FillRect}
	{Bitmap::get "../gifs/rectf.gif"}
	{Filled rectangle}
    }
    set toolEntry($FillOval) {
	{ToolBar::ChangeTool $ToolBar::FillOval}
	{Bitmap::get "../gifs/circf.gif"}
	{Filled oval}
    }
    set toolEntry($Freehand) {
	{ToolBar::ChangeTool $ToolBar::Freehand}
	{Bitmap::get "../gifs/freeh.gif"}
	{Freehand line}
    }
    set toolEntry($Polyline) {
	{ToolBar::ChangeTool $ToolBar::Polyline}
	{Bitmap::get "../gifs/lines.gif"}
	{Polyline}
    }
    set toolEntry($OutlinePolygon) {
	{ToolBar::ChangeTool $ToolBar::OutlinePolygon}
	{Bitmap::get "../gifs/poly.gif"}
	{Polygon}
    }
    set toolEntry($FilledPolygon) {
	{ToolBar::ChangeTool $ToolBar::FilledPolygon}
	{Bitmap::get "../gifs/polyf.gif"}
	{Filled polygon}
    }
    set toolEntry($Line) {
	{ToolBar::ChangeTool $ToolBar::Line}
	{Bitmap::get "../gifs/line.gif"}
	{Line}
    }
    set toolEntry($Arrow) {
	{ToolBar::ChangeTool $ToolBar::Arrow}
	{Bitmap::get "../gifs/arrow.gif"}
	{Arrow}
    }
    set toolEntry($Text) {
	{ToolBar::ChangeTool $ToolBar::Text}
	{Bitmap::get "../gifs/text1.gif"}
	{Text}
    }
    set toolEntry($OnlinePointer) {
	{ToolBar::ChangeTool $ToolBar::OnlinePointer}
	{Bitmap::get "../gifs/pointer_ro.gif" }
	{Telepointer}
    }

    set actionEntry($Select) {
	{ToolBar::ChangeAction $ToolBar::Select}
	{Bitmap::get "../gifs/select.gif" }
	{Select objects}
    }
    set actionEntry($Move) {
	{ToolBar::ChangeAction $ToolBar::Move}
	{Bitmap::get "../gifs/move.gif" }
	{Move objects}
    }
    set actionEntry($Copy) {
	{ToolBar::ChangeAction $ToolBar::Copy}
	{Bitmap::get "../gifs/copy.gif" }
	{Copy objects}
    }
    set actionEntry($Delete) {
	{ToolBar::ChangeAction $ToolBar::Delete}
	{Bitmap::get "../gifs/erase.gif" }
	{Delete objects}
    }
}

proc ToolBar::CreateFullScreen {parent} {
    variable toolBarFS
    variable showTool
    variable statusBar
    variable toolEntry
    variable actionEntry
    variable fileEntry
    variable fontSelection
    variable color
    variable currentTool

    set toolBarFS [frame $parent.toolBar1 -borderwidth 1 -relief flat]

    foreach entry [array names fileEntry] {
	Button $toolBarFS.file$entry -image [eval [lindex $fileEntry($entry) 1]] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command [lindex $fileEntry($entry) 0]\
	    -helptext [lindex $fileEntry($entry) 2]
	HighlightWidget $toolBarFS.file$entry
	pack $toolBarFS.file$entry -side left
    }

    Separator $toolBarFS.sep0 -bg "grey" -orient vertical -relief ridge
    pack $toolBarFS.sep0 -side left -padx 10 -fill both

    foreach entry [array names actionEntry] {
	Button $toolBarFS.b$entry -image [eval [lindex $actionEntry($entry) 1]] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command [lindex $actionEntry($entry) 0] \
	    -helptext [lindex $actionEntry($entry) 2]
	HighlightWidget $toolBarFS.b$entry
	pack $toolBarFS.b$entry -side left
    }

    Separator $toolBarFS.sep1 -bg "grey" -orient vertical -relief ridge
    pack $toolBarFS.sep1 -side left -padx 10 -fill both

    Button $toolBarFS.undo -image [Bitmap::get "../gifs/undo.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief "flat" -borderwidth 1 -bg $Config::bgColor\
	    -state $undo(state) -justify left -command MLB::Undo \
	    -helptext "Undo"
    HighlightWidget $toolBarFS.undo
    pack $toolBarFS.undo -side left

    Button $toolBarFS.redo -image [Bitmap::get "../gifs/redo.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief "flat" -borderwidth 1 -bg $Config::bgColor\
	    -state $redo(state) -justify left -command MLB::Redo \
	    -helptext "Redo"
    HighlightWidget $toolBarFS.redo
    pack $toolBarFS.redo -side left

    Separator $toolBarFS.sep2 -bg "grey" -orient vertical -relief ridge
    pack $toolBarFS.sep2 -side left -padx 10 -fill both

    foreach entry [array names toolEntry] {
	if {$entry == $currentTool} {
	    Button $toolBarFS.b$entry -image [eval [lindex $toolEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief sunken -borderwidth 1 -bg LightGrey\
		-justify left -command [lindex $toolEntry($entry) 0] \
		-helptext [lindex $toolEntry($entry) 2]
	} else {
	    Button $toolBarFS.b$entry -image [eval [lindex $toolEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
		-justify left -command [lindex $toolEntry($entry) 0] \
		-helptext [lindex $toolEntry($entry) 2]
	}
	HighlightWidget $toolBarFS.b$entry
	pack $toolBarFS.b$entry -side left
    }

    Separator $toolBarFS.sep3 -bg "grey" -orient vertical -relief ridge
    pack $toolBarFS.sep3 -side left -padx 10 -fill both

    Button $toolBarFS.raise -image [Bitmap::get "../gifs/raise.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command MLB::RaiseObjects \
	    -helptext "Raise"
    HighlightWidget $toolBarFS.raise
    pack $toolBarFS.raise -side left

    Button $toolBarFS.lower -image [Bitmap::get "../gifs/lower.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command MLB::LowerObjects \
	    -helptext "Lower"
    HighlightWidget $toolBarFS.lower
    pack $toolBarFS.lower -side left

    for {set i 0} {$i < 2} {incr i} {
	set colorLabel($i) [LabelFrame $toolBarFS.lab$i -text [lindex $Attributes::colorEntry($i) 1]]
	set colorButton($i) [SelectColor [$colorLabel($i) getframe].col -variable Attributes::color($i) -command [lindex $Attributes::colorEntry($i) 0] -type menubutton]
	pack $colorButton($i)
	pack $colorLabel($i) -side left -padx 5
    } 

    set lineWidthButton [SpinBox $toolBarFS.lw -label "Line Width" \
			     -underline 0 -labelwidth 10 -labelanchor w \
			     -range {1 10 1} -textvariable Attributes::lineWidth \
			     -modifycmd [lindex $Attributes::lineAttribute(0) 0]]
    $lineWidthButton setvalue @[expr $Attributes::lineWidth-1]
    $lineWidthButton configure -width 2
    pack $lineWidthButton -side left -padx 10

    pack $toolBarFS -side top -anchor w -expand 0 -padx 2 -pady 2
}


proc ToolBar::create parent {
    variable showTool
    variable toolBar
    variable statusBar
    variable toolEntry
    variable actionEntry
    variable fileEntry
    variable fontSelection
    variable color
    variable sep
    variable currentTool
    variable undo
    variable redo

    set toolBar [frame $parent.toolBar1 -borderwidth 1 -relief flat]

    foreach entry [array names fileEntry] {
	Button $toolBar.file$entry -image [eval [lindex $fileEntry($entry) 1]] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command [lindex $fileEntry($entry) 0]\
	    -helptext [lindex $fileEntry($entry) 2]
	HighlightWidget $toolBar.file$entry
    }

    set sep(0) [Separator $toolBar.sep0 -bg "grey" -orient vertical -relief ridge]

    foreach entry [array names actionEntry] {
	Button $toolBar.b$entry -image [eval [lindex $actionEntry($entry) 1]] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command [lindex $actionEntry($entry) 0] \
	    -helptext [lindex $actionEntry($entry) 2]
	HighlightWidget $toolBar.b$entry
    }

    set sep(1) [Separator $toolBar.sep1 -bg "grey" -orient vertical -relief ridge]

    Button $toolBar.undo -image [Bitmap::get "../gifs/undo.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief "flat" -borderwidth 1 -bg $Config::bgColor\
	    -state $undo(state) -justify left -command MLB::Undo \
	    -helptext "Undo"
    HighlightWidget $toolBar.undo

    Button $toolBar.redo -image [Bitmap::get "../gifs/redo.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief "flat" -borderwidth 1 -bg $Config::bgColor\
	    -state $redo(state) -justify left -command MLB::Redo \
	    -helptext "Redo"
    HighlightWidget $toolBar.redo

    set sep(2) [Separator $toolBar.sep2 -bg "grey" -orient vertical -relief ridge]

    foreach entry [array names toolEntry] {
	if {$entry == $currentTool} {
	    Button $toolBar.b$entry -image [eval [lindex $toolEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief sunken -borderwidth 1 -bg LightGrey\
		-justify left -command [lindex $toolEntry($entry) 0] \
		-helptext [lindex $toolEntry($entry) 2]
	} else {
	    Button $toolBar.b$entry -image [eval [lindex $toolEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
		-justify left -command [lindex $toolEntry($entry) 0] \
		-helptext [lindex $toolEntry($entry) 2]
	}
	HighlightWidget $toolBar.b$entry
    }

    set sep(3) [Separator $toolBar.sep3 -bg "grey" -orient vertical -relief ridge]

    Button $toolBar.raise -image [Bitmap::get "../gifs/raise.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command MLB::RaiseObjects \
	    -helptext "Raise"
    HighlightWidget $toolBar.raise

    Button $toolBar.lower -image [Bitmap::get "../gifs/lower.gif"] \
	    -highlightthickness 0 -takefocus 0 -relief flat -borderwidth 1 -bg $Config::bgColor\
	    -justify left -command MLB::LowerObjects \
	    -helptext "Lower"
    HighlightWidget $toolBar.lower

}


proc ToolBar::Top {} {

    variable toolEntry
    variable actionEntry
    variable fileEntry

    set list [lsort [array names fileEntry]]
    foreach entry $list {
	grid $ToolBar::toolBar.file$entry -row 1 -sticky nsew
	pack $ToolBar::toolBar.file$entry -side left
    }

    $ToolBar::sep(0) configure -orient horizontal
    grid $ToolBar::sep(0) -row 1 -sticky nsew
    pack $ToolBar::sep(0) -side left -padx 10 -fill both


    set i 0
    set list [lsort [array names actionEntry]]
    foreach action $list {
	grid $ToolBar::toolBar.b$action -row 1 -column $i -sticky nsew
	pack $ToolBar::toolBar.b$action -side left
	incr i
    }

    $ToolBar::sep(1) configure -orient horizontal
    grid $ToolBar::sep(1) -row 1 -sticky nsew
    pack $ToolBar::sep(1) -side left -padx 10 -fill both


    grid $ToolBar::toolBar.undo -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.undo -side left
    incr i

    grid $ToolBar::toolBar.redo -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.redo -side left
	incr i

    $ToolBar::sep(2) configure -orient horizontal
    grid $ToolBar::sep(2) -row 1 -sticky nsew
    pack $ToolBar::sep(2) -side left -padx 10 -fill both

    set list [lsort [array names toolEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -row 1 -column $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side left
	incr i
    }

    $ToolBar::sep(3) configure -orient horizontal
    grid $ToolBar::sep(3) -row 1 -sticky nsew
    pack $ToolBar::sep(3) -side left -padx 10 -fill both

    grid $ToolBar::toolBar.raise -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.raise -side left
    incr i

    grid $ToolBar::toolBar.lower -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.lower -side left
    incr i

    if {$Attributes::position == "top"} {
	grid $ToolBar::toolBar -row 1 -column 1
	grid $Toplevel::frame.sep -row 3 -column 1
	grid $WorkingArea::frame -row 4 -column 1
	pack $ToolBar::toolBar -side top -anchor w -expand 0 -padx 2 -pady 2
	if {$Config::showAttributes} {
	    grid $Attributes::toolBar -row 2 -column 1
	    pack $Attributes::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
	}
	pack $Toplevel::frame.sep -fill x -pady 2 -side top
	pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2

    } elseif {$Attributes::position == "bottom"} {
	grid $ToolBar::toolBar -row 1 -column 1
	grid $Toplevel::frame.sep -row 2 -column 1
	grid $WorkingArea::frame -row 3 -column 1
	if {$Config::showAttributes} {
	    grid $Attributes::toolBar -row 4 -column 1
	    pack $Attributes::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
	}
	pack $ToolBar::toolBar -side top -anchor w -expand 0 -padx 2 -pady 2
	pack $Toplevel::frame.sep -fill x -pady 2
	pack $WorkingArea::frame -side bottom -fill both -expand 1 -padx 2 -pady 2
    }
}

proc ToolBar::Bottom {} {
    variable showTool
    variable toolEntry
    variable actionEntry
    variable fileEntry

    set list [lsort [array names fileEntry]]
    foreach entry $list {
	grid $ToolBar::toolBar.file$entry -row 1 -sticky nsew
	pack $ToolBar::toolBar.file$entry -side left
    }

    $ToolBar::sep(0) configure -orient horizontal
    grid $ToolBar::sep(0) -row 1 -sticky nsew
    pack $ToolBar::sep(0) -side left  -padx 10 -fill both

    set i 0
    set list [lsort [array names actionEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -row 1 -column $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side left
	incr i
    }

    $ToolBar::sep(1) configure -orient horizontal
    grid $ToolBar::sep(1) -row 1 -sticky nsew
    pack $ToolBar::sep(1) -side left -padx 10 -fill both

    grid $ToolBar::toolBar.undo -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.undo -side left
    incr i

    grid $ToolBar::toolBar.redo -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.redo -side left
    incr i

    $ToolBar::sep(2) configure -orient horizontal
    grid $ToolBar::sep(2) -row 1 -sticky nsew
    pack $ToolBar::sep(2) -side left -padx 10 -fill both

    set list [lsort [array names toolEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -row 1 -column $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side left
	incr i
    }

    $ToolBar::sep(3) configure -orient horizontal
    grid $ToolBar::sep(3) -row 1 -sticky nsew
    pack $ToolBar::sep(3) -side left -padx 10 -fill both

    grid $ToolBar::toolBar.raise -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.raise -side left
    incr i

    grid $ToolBar::toolBar.lower -row 1 -column $i -sticky nsew
    pack $ToolBar::toolBar.lower -side left
    incr i

    if {$Attributes::position == "top"} {
        grid $ToolBar::toolBar -row 4 -column 1
        grid $Toplevel::frame.sep -row 2 -column 1
        grid $WorkingArea::frame -row 3 -column 1
	if {$Config::showAttributes} {
	    grid $Attributes::toolBar -row 1 -column 1
	    pack $Attributes::toolBar -fill both -side top -anchor w -expand 0 -padx 2 -pady 2
	}
        pack $Toplevel::frame.sep -fill x -pady 2
        pack $ToolBar::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
        pack $WorkingArea::frame -side top -fill both -expand 1 -padx 2 -pady 2

    } elseif {$Attributes::position == "bottom"} {
        grid $Toplevel::frame.sep -row 2 -column 1
        grid $ToolBar::toolBar -row 3 -column 1
        grid $WorkingArea::frame -row 1 -column 1
	if {$Config::showAttributes} {
	    grid $Attributes::toolBar -row 4 -column 1
	    pack $Attributes::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
	}
        pack $ToolBar::toolBar -fill both -side bottom -anchor w -expand 0 -padx 2 -pady 2
        pack $Toplevel::frame.sep -fill x -pady 2 -side bottom
        pack $WorkingArea::frame -side top -fill both -expand 1 -padx 2 -pady 2
    }

}

proc ToolBar::Left {} {
    variable showTool
    variable toolEntry
    variable actionEntry
    variable fileEntry

    set list [lsort [array names fileEntry]]
    foreach entry $list {
	grid $ToolBar::toolBar.file$entry -column 1 -sticky nsew
	pack $ToolBar::toolBar.file$entry -side top
    }

    $ToolBar::sep(0) configure -orient vertical
    grid $ToolBar::sep(0) -column 1 -sticky nsew
    pack $ToolBar::sep(0) -side top -pady 10 -fill both

    set i 0
    set list [lsort [array names actionEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -column 1 -row $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side top
	incr i
    }

    $ToolBar::sep(1) configure -orient vertical
    grid $ToolBar::sep(1) -column 1 -sticky nsew
    pack $ToolBar::sep(1) -side top -pady 10 -fill both

    grid $ToolBar::toolBar.undo -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.undo -side top
    incr i

    grid $ToolBar::toolBar.redo -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.redo -side top
    incr i

    $ToolBar::sep(2) configure -orient vertical
    grid $ToolBar::sep(2) -column 1 -sticky nsew
    pack $ToolBar::sep(2) -side top -pady 10 -fill both

    set list [lsort [array names toolEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -column 1 -row $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side top
	incr i
    }

    $ToolBar::sep(3) configure -orient vertical
    grid $ToolBar::sep(3) -column 1 -sticky nsew
    pack $ToolBar::sep(3) -side top -pady 10 -fill both

    grid $ToolBar::toolBar.raise -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.raise -side top
    incr i

    grid $ToolBar::toolBar.lower -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.lower -side top
    incr i

    grid $ToolBar::toolBar -row 1 -column 1
    grid $WorkingArea::frame -row 1 -column 2
    pack $ToolBar::toolBar -fill both -side left -anchor w -expand 0 -padx 2 -pady 2
    pack $WorkingArea::frame -side right -fill both -expand 1 -padx 2 -pady 2
}

proc ToolBar::Right {} {
    variable showTool
    variable toolEntry
    variable actionEntry
    variable fileEntry

    set list [lsort [array names fileEntry]]
    foreach entry $list {
	grid $ToolBar::toolBar.file$entry -column 1 -sticky nsew
	pack $ToolBar::toolBar.file$entry -side top
    }

    $ToolBar::sep(0) configure -orient vertical
    grid $ToolBar::sep(0) -column 1 -sticky nsew
    pack $ToolBar::sep(0) -side top -fill both -pady 10

    set i 0
    set list [lsort [array names actionEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -column 1 -row $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side top
	incr i
    }

    $ToolBar::sep(1) configure -orient vertical
    grid $ToolBar::sep(1) -column 1 -sticky nsew
    pack $ToolBar::sep(1) -side top -pady 10 -fill both

    grid $ToolBar::toolBar.undo -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.undo -side top
    incr i

    grid $ToolBar::toolBar.redo -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.redo -side top
    incr i

    $ToolBar::sep(2) configure -orient vertical
    grid $ToolBar::sep(2) -column 1 -sticky nsew
    pack $ToolBar::sep(2) -side top -pady 10 -fill both

    set list [lsort [array names toolEntry]]
    foreach tool $list {
	grid $ToolBar::toolBar.b$tool -column 1 -row $i -sticky nsew
	pack $ToolBar::toolBar.b$tool -side top
	incr i
    }

    $ToolBar::sep(3) configure -orient vertical
    grid $ToolBar::sep(3) -column 1 -sticky nsew
    pack $ToolBar::sep(3) -side top -pady 10 -fill both

    grid $ToolBar::toolBar.raise -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.raise -side top
    incr i

    grid $ToolBar::toolBar.lower -column 1 -row $i -sticky nsew
    pack $ToolBar::toolBar.lower -side top
    incr i

    grid $ToolBar::toolBar -row 1 -column 2
    grid $WorkingArea::frame -row 1 -column 1
    pack $ToolBar::toolBar -fill both -side right -anchor w -expand 0 -padx 2 -pady 2
    pack $WorkingArea::frame -side left -fill both -expand 1 -padx 2 -pady 2
}


proc ToolBar::SetUndoButtonState {state} {
    variable toolBar
    variable undo

    if {$state == 0} {
	set undo(state) "disabled"
    } else {
	set undo(state) "normal"
    }

    $toolBar.undo configure -state $undo(state)
}


proc ToolBar::SetRedoButtonState {state} {
    variable toolBar
    variable redo

    if {$state == 0} {
	set redo(state) "disabled"
    } else {
	set redo(state) "normal"
    }

    $toolBar.redo configure -state $redo(state)
}


proc ToolBar::ChangeTool tool {
    variable currentTool
    variable toolBar
    variable toolBarFS

    if {$currentTool == $ToolBar::OnlinePointer} {
	MLB::UnSetWindowPointer
    }
    if {($currentTool == $ToolBar::Polyline)  || ($currentTool == $ToolBar::OutlinePolygon) || ($currentTool == $ToolBar::FilledPolygon)} {
	WorkingArea::AutoEndPolyObjects
    }

    $toolBar.b$currentTool configure -relief flat -bg $Config::bgColor
    if {$WorkingArea::fullscreen(mode)} {
	$toolBarFS.b$currentTool configure -relief flat -bg $Config::bgColor
    }
    if {[info exists ToolBox::toolBox]} {
	$ToolBox::toolBox.b$currentTool configure -relief raised
    }

    set currentTool $tool

    $toolBar.b$currentTool configure -relief sunken -bg LightGrey
    if {$WorkingArea::fullscreen(mode)} {
	$toolBarFS.b$currentTool configure -relief sunken -bg LightGrey
    }
    if {[info exists ToolBox::toolBox]} {
	$ToolBox::toolBox.b$currentTool configure -relief sunken
    }

    if {$currentTool == $ToolBar::OnlinePointer} {
	MLB::SetWindowPointer point black 0
    } else {
	MLB::SetWindowPointer none black 0
    }

#     if {$currentTool == $ToolBar::OnlinePointer} {
# 	MLB::SetWindowPointer point black 0
#     } elseif {$currentTool == $ToolBar::Freehand} {
# 	MLB::SetWindowPointer pencil black 0
#     } elseif {$currentTool == $ToolBar::OutRect} {
# 	MLB::SetWindowPointer dotbox black 0
#     } elseif {$currentTool == $ToolBar::FillRect} {
# 	MLB::SetWindowPointer dotbox black 0
#     } elseif {$currentTool == $ToolBar::OutOval} {
# 	MLB::SetWindowPointer circle black 0
#     } elseif {$currentTool == $ToolBar::FillOval} {
# 	MLB::SetWindowPointer circle black 0
#     } else {
# 	MLB::SetWindowPointer none black 0
#     }


}
proc ToolBar::ChangeAction action {
    variable currentTool
    variable toolBar
    variable toolBarFS


    if {$currentTool == $ToolBar::OnlinePointer} {
	MLB::UnSetWindowPointer
    }
    if {($currentTool == $ToolBar::Polyline)  || ($currentTool == $ToolBar::OutlinePolygon) || ($currentTool == $ToolBar::FilledPolygon)} {
	WorkingArea::AutoEndPolyObjects
    }

    $toolBar.b$currentTool configure -relief flat -bg $Config::bgColor
    if {$WorkingArea::fullscreen(mode)} {
	$toolBarFS.b$currentTool configure -relief flat -bg $Config::bgColor
    }
    if {[info exists ToolBox::toolBox]} {
	$ToolBox::toolBox.b$currentTool configure -relief raised
    }

    set currentTool $action

    $toolBar.b$currentTool configure -relief sunken -bg LightGrey
    if {$WorkingArea::fullscreen(mode)} {
	$toolBarFS.b$currentTool configure -relief sunken -bg LightGrey
    }
    if {[info exists ToolBox::toolBox]} {
	$ToolBox::toolBox.b$currentTool configure -relief sunken
    }

    if {$currentTool == $ToolBar::Delete} {
	#MLB::Delete
	#MLB::SetWindowPointer pirate black 0
    } elseif {$currentTool == $ToolBar::OnlinePointer} {
	MLB::SetWindowPointer point black 0
    } else {
	MLB::SetWindowPointer none black 0
    }

}


proc ToolBar::SetTool tool {
    variable currentTool
    variable toolBar

    if {$currentTool == $ToolBar::Move} {return}
    if {$currentTool == $ToolBar::Copy} {return}
    if {$currentTool == $ToolBar::Select} {return}
    if {$currentTool == $ToolBar::Delete} {return}

    if {$currentTool == $ToolBar::OnlinePointer} {
	MLB::UnSetWindowPointer
    }

    $toolBar.b$currentTool configure -relief flat -bg $Config::bgColor
    if {[info exists ToolBox::toolBox]} {
	$ToolBox::toolBox.b$currentTool configure -relief raised
    }
    if {($currentTool == $ToolBar::Polyline)  || ($currentTool == $ToolBar::OutlinePolygon) || ($currentTool == $ToolBar::FilledPolygon)} {
	WorkingArea::AutoEndPolyObjects
    }

    set currentTool $tool

    $toolBar.b$currentTool configure -relief sunken -bg LightGrey
    if {[info exists ToolBox::toolBox]} {
	$ToolBox::toolBox.b$currentTool configure -relief sunken
    }

    if {$currentTool == $ToolBar::Delete} {
	MLB::Delete
    } elseif {$currentTool == $ToolBar::OnlinePointer} {
	MLB::SetWindowPointer point black 0
    }

}


proc ToolBar::ImportDocument { } {
    variable currentAction

    set filename [Filename::open $ToolBar::imageName { {{Postscript} {.ps .PS}} {{PDF} {.pdf .PDF}} {{All files} * }} {.ps}]

    if {$filename != ""} {
	set ToolBar::imageName $filename
	MLB::ImportDocument $ToolBar::imageName
    }
}


proc ToolBar::LoadImage {tool} {
    variable currentAction

    set filename [Filename::open $ToolBar::imageName {{{GIF} {.gif .GIF}} {{PNG} {.png .PNG}} {{PCX} {.pcx .PCX}} {{TIFF} {.tif .TIFF .tiff .TIFF}} {{JPEG} {.jpg .jpeg .JPG .JPEG}} {{Bitmaps} {.bmp .BMP .dib .DIB .ico .ICO .xbm .XBM}} {{All files} * }} {.gif}]

    if {$filename != ""} {
	set ToolBar::imageName $filename
	set child [winfo children $WorkingArea::canvasPane]
	foreach window $child {
	    if {[winfo ismapped $window]} {
		set x [expr [winfo pointerx $window] - [winfo rootx $window]]
		set y [expr [winfo pointery $window] - [winfo rooty $window]]

		if { "OK" == [MLB::LoadImage $ToolBar::imageName $x $y] } {
		MLB::PlaceImage 0 $x $y
		set currentAction $ToolBar::Image
		set WorkingArea::action 1
		break
		} else {
                  	set currentAction 0
                  	set WorkingArea::action 0
                  	break
	    }

	    }
#		if [MLB::LoadImage $ToolBar::imageName $x $y] {
#                  DebugInfo::info "Ok"
#                  set currentAction 0
#                  set WorkingArea::action 0
#                  break
#		} else {
#                  DebugInfo::info "Nicht ok"
#                  MLB::PlaceImage 0 $x $y
#                  set currentAction $ToolBar::Image
#                  set WorkingArea::action 1
#                  break
#		}

	}
    }
}

proc ToolBar::SaveFile {} {
    set filename [Filename::save $ToolBar::mlbName {{{MLB} {.mlb .MLB}} {{All files} * }} {.mlb}]

    if {$filename != ""} {
	set ToolBar::mlbName $filename
	MLB::SaveDocument $ToolBar::mlbName
    }
}

proc ToolBar::ExportDocument {} {
    set filename [Filename::save $ToolBar::exportName {{{PostScript} {.ps .PS}} \
	    {{HTML} {.htm .html}} \
	    {{GIF} {.gif .GIF}} \
	    {{PNG} {.png .PNG}} \
	    {{PCX} {.pcx .PCX}} \
	    {{TIFF} {.tif .TIFF .tiff .TIFF}} \
	    {{JPEG} {.jpg .jpeg .JPG .JPEG}} \
	    {{Bitmap} {.bmp .BMP}} \
	    {{All files} * }} {.ps}]

    if {$filename != ""} {
	set ToolBar::exportName $filename
	set type [string replace [file extension $ToolBar::exportName] 0 0]
#	if {([string compare $type "ps"] == 0) || ([string compare $type "PS"] == 0)} {
#	    MLB::SaveDocumentAsPS $ToolBar::exportName
#	}
	if {([string compare $type "htm"] == 0) || ([string compare $type "html"] == 0)} {
	    MLB::SaveDocumentAsHtml [string range $ToolBar::exportName 0 [expr [string last ".htm" $ToolBar::exportName] - 1]]
	}
	if {([string compare $type "gif"] == 0) || ([string compare $type "GIF"] == 0) \
		|| ([string compare $type "png"] == 0) || ([string compare $type "PNG"] == 0) \
		|| ([string compare $type "pcx"] == 0) || ([string compare $type "PCX"] == 0) \
		|| ([string compare $type "tif"] == 0) || ([string compare $type "TIF"] == 0) \
		|| ([string compare $type "jpg"] == 0) || ([string compare $type "JPG"] == 0) \
		|| ([string compare $type "jpeg"] == 0) || ([string compare $type "JPEG"] == 0) \
		|| ([string compare $type "ps"] == 0) || ([string compare $type "PS"] == 0) \
		|| ([string compare $type "bmp"] == 0) || ([string compare $type "BMP"] == 0)} {
	    set dir   [file dirname $ToolBar::exportName]
	    set name  [file tail $ToolBar::exportName]
	    set tname [file root $name]
	    set tdir  [file join $dir $tname]
	    file mkdir $tdir
	    MLB::SaveDocumentAsImage [file join $dir $tname $tname] $type
	}
    }
}

proc ToolBar::OpenFile {} {

    set filename [Filename::read]

    if {$filename != ""} {
	set ToolBar::mlbName $filename
	MLB::OpenDocument $ToolBar::mlbName
	$Toplevel::toplevel setmenustate sessionedit normal
    }
}

proc ToolBar::InsertFile {} {

    set filename [Filename::read]

    if {$filename != ""} {
	set ToolBar::mlbName $filename
	MLB::InsertDocument $ToolBar::mlbName
	$Toplevel::toplevel setmenustate sessionedit normal
    }
}

namespace eval ToolOptions {
}

proc ToolOptions::Create {} {

    Dialog .toolOption
    set frame [.toolOption getframe]
    frame $frame.frame

    set i 0
    set j 0
    set list [lsort [array names ToolBar::toolEntry]]
    foreach entry $list {
	if {$i == 4} {
	    incr j
	    set i 0
	}
	checkbutton $frame.frame.$entry -variable ToolBar::showTool($entry) -image [eval [lindex $ToolBar::toolEntry($entry) 1]]
	pack $frame.frame.$entry -side left
	grid $frame.frame.$entry -row $j -column $i -padx 5
	incr i
    }

    incr j
    Separator $frame.frame.sep1 -orient horizontal
    pack $frame.frame.sep1 -fill x
    grid $frame.frame.sep1 -row $j -columnspan 4 -pady 4 -sticky ew

    set i 0
    incr j
    set list [lsort [array names ToolBar::actionEntry]]
    foreach entry $list {
	if {$i == 4} {
	    incr j
	    set i 0
	}
	checkbutton $frame.frame.$entry -variable ToolBar::showTool($entry) -image [eval [lindex $ToolBar::actionEntry($entry) 1]]
	pack $frame.frame.$entry -side left
	grid $frame.frame.$entry -row $j -column $i -padx 5
	incr i
    }

    incr j
    Separator $frame.frame.sep2 -orient horizontal
    pack $frame.frame.sep2 -fill x
    grid $frame.frame.sep2 -row $j -columnspan 4 -pady 4 -sticky ew

    set i 0
    incr j
    set list [lsort [array names ToolBar::fileEntry]]
    foreach entry $list {
	if {$i == 4} {
	    incr j
	    set i 0
	}
	checkbutton $frame.frame.$entry -variable ToolBar::showTool($entry) -image [eval [lindex $ToolBar::fileEntry($entry) 1]]
	pack $frame.frame.$entry -side left
	grid $frame.frame.$entry -row $j -column $i -padx 5
	incr i
    }

    pack $frame.frame

    .toolOption add -name ok -command  ToolOptions::Hide
}

proc ToolOptions::Run {} {
    if {![winfo exists .toolOption]} {
	ToolOptions::Create
	.toolOption draw
    } else {
	wm deiconify .toolOption
    }
}

proc ToolOptions::Hide {} {
    .toolOption withdraw
    ToolBox::Update
}

namespace eval ToolBox {
    variable toolBox
}

proc ToolBox::Create {} {
    variable toolBox

    Dialog .toolBox -modal none
    wm protocol  .toolBox WM_DELETE_WINDOW ToolBox::Hide
    set toolBox [.toolBox getframe]

    set list [lsort [array names ToolBar::toolEntry]]
    foreach entry $list {
	if {$entry == $ToolBar::currentTool} {
	    Button $toolBox.b$entry -image [eval [lindex $ToolBar::toolEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief sunken -borderwidth 1 \
		-justify left -command [lindex $ToolBar::toolEntry($entry) 0] \
		-helptext [lindex $ToolBar::toolEntry($entry) 2]
	} else {
	    Button $toolBox.b$entry -image [eval [lindex $ToolBar::toolEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 \
		-justify left -command [lindex $ToolBar::toolEntry($entry) 0] \
		-helptext [lindex $ToolBar::toolEntry($entry) 2]
	}
    }

    Separator $toolBox.sep0 -orient horizontal

    foreach entry [array names ToolBar::actionEntry] {
	Button $toolBox.b$entry -image [eval [lindex $ToolBar::actionEntry($entry) 1]] \
	    -highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 \
	    -justify left -command [lindex $ToolBar::actionEntry($entry) 0] \
	    -helptext [lindex $ToolBar::actionEntry($entry) 2]
    }

    Separator $toolBox.sep1 -orient horizontal

    set list [lsort [array names ToolBar::fileEntry]]
    foreach entry $list {
	    Button $toolBox.file$entry -image [eval [lindex $ToolBar::fileEntry($entry) 1]] \
		-highlightthickness 0 -takefocus 0 -relief raised -borderwidth 1 \
		-justify left -command [lindex $ToolBar::fileEntry($entry) 0]\
		-helptext [lindex $ToolBar::fileEntry($entry) 2]
    }



    .toolBox add -text "Hide" -command ToolBox::Hide

    ToolBox::Update
}

proc ToolBox::Update {} {
    variable toolBox

    if {![winfo exists .toolBox]} {
	ToolBox::Create
    } else {
	pack $toolBox -side top -expand yes

	set i 0
	set l 0

	set list [lsort [array names ToolBar::toolEntry]]
	foreach entry $list {
	    if {$ToolBar::showTool($entry)} {
		if {$i == 4} {
		    incr l
		    set i 0
		}
		pack $toolBox.b$entry -side left
		grid $toolBox.b$entry -column $i -row $l
		incr i
	    } else {
		pack forget $toolBox.b$entry
		grid forget $toolBox.b$entry
	    }
	}

	incr l
	pack $toolBox.sep0 -side left
	grid $toolBox.sep0 -columnspan 4 -row $l -sticky ew -pady 3

	set j 0
	incr l
	set list [lsort [array names ToolBar::actionEntry]]
	foreach entry $list {
	    if {$ToolBar::showTool($entry)} {
		pack $toolBox.b$entry -side left
		grid $toolBox.b$entry -column $j -row $l
		incr j
	    } else {
		pack forget $toolBox.b$entry
		grid forget $toolBox.b$entry
	    }
	}

	incr l
	pack $toolBox.sep1 -side left
	grid $toolBox.sep1 -columnspan 4 -row $l -sticky ew -pady 3

	set k 0
	incr l
	set list [lsort [array names ToolBar::fileEntry]]
	foreach entry $list {
	    if {$ToolBar::showTool($entry)} {
		pack $toolBox.file$entry -side left
		grid $toolBox.file$entry -column $k -row $l
		incr k
	    } else {
		pack forget $toolBox.file$entry
		grid forget $toolBox.file$entry
	    }
	}

	if {$i == 0 || $j == 0} {
	    pack forget $toolBox.sep0
	    grid forget $toolBox.sep0
	}
 	if {$j == 0 || $k == 0} {
 	    pack forget $toolBox.sep1
 	    grid forget $toolBox.sep1
 	}
    }

    .toolBox draw none
}

proc ToolBox::Show {} {

    if {![winfo exists .toolBox]} {
	ToolBox::Create
	.toolBox draw none
    } else {
	wm deiconify .toolBox
    }

}

proc ToolBox::Hide {} {

    .toolBox withdraw
}



