##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval Voting {
    variable openTool
    variable question
    variable parentWindow
    variable qlist
    variable ui_font_tiny   "-*-helvetica-medium-r-normal-*-8-*-*-*-p-*-iso8859-1"
    variable ui_font_small  "-*-helvetica-medium-r-normal-*-10-*-*-*-p-*-iso8859-1" 
    variable tmp
    variable type
    variable idx

    set openTool 0
    set question(nbr) 0
    set qlist " "
}


proc Voting::Tool {} {
    variable openTool

    if {$openTool} {
	Voting::OpenTool
    } else {
	Voting::CloseTool
    }
} 


proc Voting::OpenTool { } {
    VT::OpenRTPI
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Voting::CloseTool { } {
    VT::CloseRTPI
    set openTool 0
    WorkingArea::create $Toplevel::frame 
    MLB::ReDrawPage
}


proc Voting::Create {parent} {
    variable question
    variable qlist
    variable parentWindow

    frame $parent.top
    label $parent.top.l -text "Voting Tool" -anchor w

    set sw [ScrolledWindow $parent.vtsw]
    set qlist [listbox $sw.l -selectmode single -bg white -height 3 -width 8 -relief sunken]
    $sw setwidget $qlist

    bind $qlist <Double-ButtonPress-1> Voting::DisplayResults

    frame $parent.bottom
    
    Button $parent.bottom.n -image new -anchor w -relief flat -bd 1 -command {Voting::TypeDialog} 
    DynamicHelp::register $parent.bottom.n balloon "Create new question"
    HighlightWidget $parent.bottom.n

    Button $parent.bottom.d -image delete -anchor w -relief flat -bd 1 -command {Voting::Delete}
    DynamicHelp::register $parent.bottom.d balloon "Delete selected question"
    HighlightWidget $parent.bottom.d

    Button $parent.bottom.a -image answer -anchor w -relief flat -bd 1 -command {Voting::Answer}
    DynamicHelp::register $parent.bottom.a balloon "Answer selected question"
    HighlightWidget $parent.bottom.a

    Button $parent.bottom.r -image results -anchor w -relief flat -bd 1 -command {Voting::DisplayResults}
    DynamicHelp::register $parent.bottom.r balloon "Display results of selected question"
    HighlightWidget $parent.bottom.r

    pack $parent.top.l -anchor w -side left
    pack $parent.top -fill x -anchor w		       

    pack $sw  -side top  -expand yes -fill both -anchor w

    pack $parent.bottom.n -side left -padx 4
    pack $parent.bottom.d -side left -padx 4
    pack $parent.bottom.a -side left -padx 4
    pack $parent.bottom.r -side left -padx 4
    pack $parent.bottom -fill x -anchor w		       

    return $qlist
}


proc Voting::RebuildQuestionList {}  { 
    variable question
    variable qlist

    if [winfo exists $qlist] {
	$qlist delete 0 end
	for {set i 0} {$i < $question(nbr)} {incr i} {
	    if {!$question($i,deleted)} {
		$qlist insert $i $question($i,text)
	    }
	}
    }
}


proc Voting::TypeDialog { } {
    variable type

    toplevel .vtype
    wm withdraw .vtype
    focus .vtype
    grab .vtype
    wm title .vtype "Voting"
    catch {wm iconbitmap .vtype "../bin/mlb.ico"}
    wm resizable .vtype 0 0 
    wm geometry .vtype +[winfo pointerx .]+[winfo pointery .]

    frame .vtype.top
    label .vtype.top.l -text "Question Type:" -justify left -anchor w
    set type "mc"
    radiobutton .vtype.top.r1 -text "Multiple Choice" -variable Voting::type -value "mc"
    radiobutton .vtype.top.r2 -text "Text" -variable Voting::type -value "tx"
    pack .vtype.top.l .vtype.top.r1 .vtype.top.r2 -side left -padx 2 -pady 2
    pack .vtype.top

    frame  .vtype.b
    button .vtype.b.b1 -text "OK" -command "Voting::CreateQuestion" -width 5
    button .vtype.b.b2 -text "Cancel" -command "destroy .voting" -width 5
    pack   .vtype.b.b1 .vtype.b.b2 -side left -pady 2 -padx 6 
    pack   .vtype.b -pady 10

    wm deiconify .vtype
}


proc Voting::CreateQuestion { } {
    variable type

    if {[string compare $type "mc"] == 0} {
	Voting::ResponseNbrDialog 
    } else {
	Voting::CreateTextQuestion
    }
}


proc Voting::ResponseNbrDialog {} {
    variable tmp
    
    regexp {\+([0-9]+)\+([0-9]+)} [wm geometry .vtype] m x y
    destroy .vtype

    toplevel .voting
    wm withdraw .voting
    focus .voting
    grab .voting
    wm title .voting "Voting"
    catch {wm iconbitmap .voting "../bin/mlb.ico"}
    wm resizable .voting 0 0 
    wm geometry .voting +$x+$y

    frame   .voting.top
    label   .voting.top.l -text "Number of responses:" -width 17 -justify left -anchor w
    SpinBox .voting.top.s -dragenabled false -dropenabled false -entrybg white -textvariable Voting::tmp \
	    -width 5 -values {2 3 4 5 6 7} -editable 0 -justify left\
	    -helptext "Specify the number of responses\npossible for this question"
    pack .voting.top.l .voting.top.s -side left -padx 2 -pady 2
    pack .voting.top
    
    frame  .voting.b
    button .voting.b.b1 -text "OK" -command "Voting::CreateMCQuestion" -width 5
    button .voting.b.b2 -text "Cancel" -command "destroy .voting" -width 5
    pack   .voting.b.b1 .voting.b.b2 -side left -pady 2 -padx 2
    pack   .voting.b -pady 10

    wm deiconify .voting
}


proc Voting::CreateMCQuestion {} {
    variable tmp

    regexp {\+([0-9]+)\+([0-9]+)} [wm geometry .voting] m x y
    destroy .voting

    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Create Question"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +$x+$y
    
    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background white -justify left -width 20
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    pack   .question.t -pady 4 

    for {set i 1} {$i <= $tmp} {incr i} {
	frame  .question.a$i 
	label  .question.a$i.l -text "$i:" -width 3 -justify left -anchor w
	entry  .question.a$i.e -background white -justify left -width 15
	pack   .question.a$i.l .question.a$i.e -side left -padx 2 -pady 2
	pack   .question.a$i -fill both -expand yes
    }
   
    frame  .question.b
    button .question.b.b1 -text "OK" -command "Voting::CreateMCQuestionOK" -width 5
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Voting::CreateMCQuestionOK {} {
    variable tmp

    for {set i 1} {$i <= $tmp} {incr i} {
	set t [.question.a$i.e get] 
	append t " "
	lappend q $t
    }

    VT::CreateMCQuestion [.question.t.e get] $tmp $q

    destroy .question
}


proc Voting::CreateTextQuestion { } {
    regexp {\+([0-9]+)\+([0-9]+)} [wm geometry .vtype] m x y
    destroy .vtype

    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Create Question"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +$x+$y

    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background white -justify left -width 20
    bind   .question.t.e <Return> {Voting::CreateTextQuestionOK}
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    pack   .question.t -pady 4 

    frame  .question.b
    button .question.b.b1 -text "OK" -command "Voting::CreateTextQuestionOK" -width 5
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Voting::CreateTextQuestionOK { } {
    VT::CreateTextQuestion [.question.t.e get]
    destroy .question
}


proc Voting::GetSelectedIndex { } {
    variable qlist
    variable question

    set selected [$qlist curselection]
    if {[llength $selected] == 0} {
	return -1
    }

    set idx [lindex [$qlist curselection] 0]
    
    set t 0
    for {set i 0} {$i < $question(nbr)} {incr i} {
	if {$question($i,deleted)} {continue}
	if {$t == $idx} {
	    return $i
	}
	incr t
    }

    return -1
}

proc Voting::Delete { } {
    variable qlist
    variable question

    set idx [GetSelectedIndex]
    if {$idx == -1} {return}

    destroy .msgdlg
    if [MessageDlg .msgdlg -parent . -title "Delete" -default 1 -cancel 1\
	    -message "Delete question $question($idx,text)?" -icon question -justify center -type yesno] {
	return
    }

    VT::DeleteQuestion $question($idx,id)

    set question($idx,deleted) 1
    unset question($idx,text)
    unset question($idx,id)
    unset question($idx,myAnswer)
    if {[string compare $question($idx,type) "mc"] == 0} {
	for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	    unset question($idx,result,$i)
	    unset question($idx,resp,$i) 
	}
	unset question($idx,nbrOfResps)
	unset question($idx,nbrOfAnswers)
    } else {
	unset question($idx,result)
    }
    unset question($idx,type)

    DeleteQuestionFromList $idx

    if {[winfo exists .qresult] && ($Voting::idx == $idx)} {
	destroy .qresult
    }
}


proc Voting::RemoteDeleteQuestion {id} {
    variable qlist
    variable question

    for {set idx 0} {$idx < $question(nbr)} {incr idx} {
	if {$question($idx,deleted)} {continue}
	if {$question($idx,id) == $id} {
	    set question($idx,deleted) 1
	    unset question($idx,id)
	    unset question($idx,text)
	    unset question($idx,myAnswer)
	    if {[string compare $question($idx,type) "mc"] == 0} {
		for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
		    unset question($idx,result,$i)
		    unset question($idx,resp,$i) 
		}
		unset question($idx,nbrOfResps)
		unset question($idx,nbrOfAnswers)
	    } else {
		unset question($idx,result)
	    }
	    unset question($idx,type)

	    DeleteQuestionFromList $idx
	    
	    if {[winfo exists .qresult] && ($Voting::idx == $idx)} {
		destroy .qresult
	    }
	}
    }
}


proc Voting::DeleteQuestionFromList {idx} {
    variable qlist
    variable question

    if {![winfo exists $qlist]} {return}

    set i 0
    for {set t 0} {$t <= $idx} {incr t} {
	if {$question($t,deleted)} {continue} 
	incr i
    }

    $qlist delete $i
}


proc Voting::Answer { } {
    variable question
    variable idx

    set idx [GetSelectedIndex]
    if {$idx == -1} {return}

    if {[string compare $question($idx,type) "mc"] == 0} {
	AnswerMC
    } else {
	AnswerText
    }
}


proc Voting::AnswerMC { } {
    variable idx
    variable question
    variable tmp
    
    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Answer Question"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +[winfo pointerx .]+[winfo pointery .]

    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background grey -justify left -width 20
    .question.t.e insert end $question($idx,text)
    .question.t.e configure -state disabled
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    pack   .question.t -pady 4 

    frame .question.r
    set tmp $question($idx,myAnswer)
    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i} {
	radiobutton .question.r.r$i -text "$question($idx,resp,$i)" -variable Voting::tmp -value $i
	pack .question.r.r$i -side top
    }
    pack   .question.r -pady 4 

    frame  .question.b
    button .question.b.b1 -text "OK" -command "Voting::AnswerMCOK" -width 5
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Voting::AnswerMCOK { } {
    variable idx
    variable question
    variable tmp

    set question($idx,myAnswer) $tmp 
    VT::ChangeMCAnswer $question($idx,id) $tmp
    destroy .question
}


proc Voting::AnswerText { } {
    variable idx
    variable question
    
    toplevel .question
    wm withdraw .question
    focus .question
    grab .question
    wm title .question "Answer Question"
    catch {wm iconbitmap .question "../bin/mlb.ico"}
    wm resizable .question 0 0 
    wm geometry .question +[winfo pointerx .]+[winfo pointery .]

    frame  .question.t
    label  .question.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .question.t.e -background grey -justify left -width 20
    .question.t.e insert end $question($idx,text)
    .question.t.e configure -state disabled
    pack   .question.t.l .question.t.e -side left -padx 2 -pady 2
    pack   .question.t -pady 4 

    frame  .question.a
    label  .question.a.l -text "Your Answer" -width 10 -justify left -anchor w
    entry  .question.a.e -background white -justify left -width 20
    .question.a.e insert end $question($idx,myAnswer)
    bind   .question.a.e <Return> {Voting::AnswerTextOK}
    pack   .question.a.l .question.a.e -side left -padx 2 -pady 2
    pack   .question.a -pady 4 

    frame  .question.b
    button .question.b.b1 -text "OK" -command "Voting::AnswerTextOK" -width 5
    pack   .question.b.b1 -pady 2 -padx 2
    pack   .question.b -pady 10

    wm deiconify .question
}


proc Voting::AnswerTextOK { } {
    variable idx
    variable question
    
    set question($idx,myAnswer) [.question.a.e get]
    VT::ChangeTextAnswer $question($idx,id) $question($idx,myAnswer)
    destroy .question
}


proc Voting::DisplayResults { } {
    variable question
    variable idx

    set idx [GetSelectedIndex]
    if {$idx == -1} {return}

    if {[string compare $question($idx,type) "mc"] == 0} {
	DisplayMCResults
    } else {
	DisplayTextResults
    }
    
}


proc Voting::DisplayMCResults { } {
    variable question
    variable idx

    if [winfo exists .qresult] {
	destroy .qresult
    }

    toplevel .qresult
    wm withdraw .qresult
    focus .qresult
    wm title .qresult "Question Results"
    catch {wm iconbitmap .qresult "../bin/mlb.ico"}
    wm resizable .qresult 0 0 
    wm geometry .qresult +[winfo pointerx .]+[winfo pointery .]

    frame  .qresult.t
    label  .qresult.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .qresult.t.e -background grey -justify left -width 20
    .qresult.t.e insert end $question($idx,text)
    .qresult.t.e configure -state disabled
    pack   .qresult.t.l .qresult.t.e -side left -padx 2 -pady 2 -fill x 
    pack   .qresult.t -pady 4  -fill x 

    canvas .qresult.c -height 200 -width 320 -bg white    
    pack .qresult.c 
    
    .qresult.c create text 10 100 -text "%"
    .qresult.c create line 40 20 40 175
    
    for {set i 0} {$i <= 10} {incr i 1} {
	set y [expr 175 -($i*15)]
	.qresult.c create line 40 $y 43 $y -width 2
	.qresult.c create text 39 $y -text [expr $i*10] -anchor e 
    }

    .qresult.c create line 40 175 310 175 -width 2

    set slice [expr 270 / $question($idx,nbrOfResps)]
    set bar [expr $slice * 0.67]
    set space [expr $slice - $bar]
    set width [expr $bar / 2]

    for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i 1} {
	set x [expr 40 + $space + $width + ($i-1)*$slice]
	.qresult.c create text $x 179 \
		-text $question($idx,resp,$i)\
		-anchor n\
		-width $slice\
		-justify left

	.qresult.c create line $x 175 $x 172 -width 1
    }

    if {$question($idx,nbrOfAnswers) != 0} {
	for {set i 1} {$i <= $question($idx,nbrOfResps)} {incr i 1} {
	    set x [expr 40 + $space + $width + ($i-1)*$slice]
	    if {$question($idx,result,$i) != 0} {
		Voting::DrawDataBar $x $question($idx,result,$i) $width
	    }
	}
    }
    
    .qresult.c create text 150 7 \
         -text "Total Responses: $question($idx,nbrOfAnswers)" \
         -tags update

    frame  .qresult.b
    button .qresult.b.b1 -text "Dismiss" -command "destroy .qresult" -width 5
    pack   .qresult.b.b1 -pady 2 -padx 2
    pack   .qresult.b -pady 10

    wm deiconify .qresult
}


proc Voting::DrawDataBar {xpos res width} {
    set ypos 175
    set percent [expr $res*100]
    set topy [expr $ypos - ($percent * 1.5)]
    set texty [expr $topy - 5]
    set leftx [expr $xpos - $width]
    set rightx [expr $xpos + $width]

    .qresult.c create rectangle $leftx $ypos $rightx $topy \
	    -fill blue -tags update
}


proc Voting::DisplayTextResults {} {
    variable question
    variable idx

    if [winfo exists .qresult] {
	destroy .qresult
    }

    toplevel .qresult
    wm withdraw .qresult
    focus .qresult
    wm title .qresult "Question Results"
    catch {wm iconbitmap .qresult "../bin/mlb.ico"}
    wm resizable .qresult 0 0 
    wm geometry .qresult +[winfo pointerx .]+[winfo pointery .]

    frame  .qresult.t
    label  .qresult.t.l -text "Question" -width 10 -justify left -anchor w
    entry  .qresult.t.e -background grey -justify left -width 20
    .qresult.t.e insert end $question($idx,text)
    .qresult.t.e configure -state disabled
    pack   .qresult.t.l .qresult.t.e -side left -padx 2 -pady 2 -fill x 
    pack   .qresult.t -pady 4  -fill x 

    frame .qresult.m
    set sw [ScrolledWindow .qresult.m.sw]
    text $sw.t -relief sunken -bd 2 -setgrid 1 -height 15 -width 50 -bg white
    $sw.t insert end $question($idx,result)
    $sw.t configure -state disabled
    $sw setwidget $sw.t
    pack $sw -side top  -fill both -expand 1
    pack .qresult.m

    frame  .qresult.b
    button .qresult.b.b1 -text "Dismiss" -command "destroy .qresult" -width 5
    pack   .qresult.b.b1 -pady 2 -padx 2
    pack   .qresult.b -pady 10

    wm deiconify .qresult
}


proc Voting::AddMCQuestion {id text nbrOfResps resplist} {
    variable question
    variable qlist

    set idx $question(nbr)
    incr question(nbr)

    set question($idx,id) $id
    set question($idx,type) "mc"
    set question($idx,deleted) 0
    set question($idx,text) $text
    set question($idx,nbrOfResps) $nbrOfResps
    set question($idx,myAnswer) 1
    set question($idx,nbrOfAnswers) 0

    set i 1
    foreach resp $resplist {
	set question($idx,resp,$i) $resp
	set question($idx,result,$i) 1
	incr i
    }

    if [winfo exists $qlist] {
	$qlist insert $idx $question($idx,text)
    }
}


proc Voting::AddTextQuestion {id text} {
    variable question
    variable qlist

    set idx $question(nbr)
    incr question(nbr)
    
    set question($idx,id) $id
    set question($idx,type) "tx"
    set question($idx,deleted) 0
    set question($idx,text) $text
    set question($idx,result) ""
    set question($idx,myAnswer) ""

    if [winfo exists $qlist] {
	$qlist insert $idx $question($idx,text)
    }
}


proc Voting::SetMCResult {id nbrOfAnswers results} {
    variable question

    set idx -1
    for {set i 0} {$i < $question(nbr)} {incr i} {
	if {$question($i,deleted)} {continue}
	if {$id == $question($i,id)} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} {return}
    
    set question($idx,nbrOfAnswers) $nbrOfAnswers

    set i 1
    foreach result $results {
	set question($idx,result,$i) $result
	incr i
    }

    if {[winfo exists .qresult] && ($Voting::idx == $idx)} {
	destroy .qresult
	DisplayMCResults
    }
}


proc Voting::SetTextResult {id result} {
    variable question

    set idx -1
    for {set i 0} {$i < $question(nbr)} {incr i} {
	if {$question($i,deleted)} {continue}
	if {$id == $question($i,id)} {
	    set idx $i
	    break
	}
    }
    if {$idx == -1} {return}

    set question($idx,result) $result

    if {[winfo exists .qresult] && ($Voting::idx == $idx)} {
	.qresult.m.sw.t configure -state normal
	.qresult.m.sw.t delete 0.0 end
	.qresult.m.sw.t insert end $result
	.qresult.m.sw.t configure -state disabled
    }
}


proc Voting::ClearAllQuestions { } {
    variable question

    for {set i 0} {$i < $question(nbr)} {incr i} {
	unset question($i,deleted)
	unset question($i,id)
	unset question($i,text)
	unset question($i,myAnswer)
	if {[string compare $question($i,type) "mc"] == 0} {
	    for {set j 1} {$j <= $question($i,nbrOfResps)} {incr j} {
		unset question($i,result,$j)
		unset question($i,resp,$j) 
	    }
	    unset question($i,nbrOfResps)
	    unset question($i,nbrOfAnswers)
	} else {
	    unset question($i,result)
	}
	unset question($i,type)
    }

    set question(nbr) 0
}