##########################################################################################################
# mlb - multimedia lecture board 
# Copyright 1999 - 2003
# 
# University of Mannheim, Germany
# University of Freiburg, Germany
#
# Development:
# Leader: Juergen Vogel
# Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
# Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
#
# Development was sponsored by DFN (ANETTE project).
#
# License Information:
# The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
# 
# This program is free software; you can redistribute it and/or modify it under the terms of 
# the GNU General Public License as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See the GNU General Public License for more details.
# The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
#
# The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
# modifications of this software made by others.
#
# Contact:
# Juergen Vogel - vogel@informatik.uni-mannheim.de
# www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
#
##########################################################################################################

namespace eval WorkingArea {
    variable pageView
    variable frame
    variable pane
    variable toolPaneW
    variable sw
    variable canvasPane
    variable polyStatus
    variable rebuild
    variable action
    variable fullscreen
    variable weights
    variable animationView

    set rebuild 0

    set polyStatus "end"
    set action 0

    set fullscreen(mode) 0

    set weights(0) 11
    set weights(1) 500
    set animationView 0
}


proc WorkingArea::create {parent} {
    variable frame
    variable pane
    variable toolPane
    variable toolPaneW
    variable toolframe
    variable pageView
    variable sw
    variable canvasPane
    variable weights

    if [winfo exists $parent.frame] {
	destroy $parent.frame
    }
    set frame [frame $parent.frame]
    $frame configure -relief flat

    if {!$Record::openTool && !$Config::showPagetree && !$Config::showParticipants && !$Feedback::openTool && !$Alt::openTool && !$Config::showAnimations} {

	set sw [ScrolledWindow $frame.sw]
	set sf [ScrollableFrame $sw.f]
	$sw setwidget $sf

	set canvasPane [$sf getframe]

	pack $sw  -side top  -expand yes -fill both -anchor w
	pack $frame -side bottom -fill both -expand 1
    } else {

	set pane [PanedWindow $frame.pw -side top]
	set toolPane [$pane add -minsize 130 -weight $weights(0)]

	BuildToolpane

	set p [$pane add -weight $weights(1)]

	set sw [ScrolledWindow $p.sw]
	set sf [ScrollableFrame $sw.f]
	$sw setwidget $sf

	set canvasPane [$sf getframe]

	pack $sw  -side top  -expand yes -fill both -anchor w
	pack $pane  -side bottom -fill both -expand 1

	pack $frame -side bottom -fill both -expand 1
    }
}

proc WorkingArea::FullScreen {{bars 0}} {
    global   mlbVersion
    variable fullscreen

    if {$fullscreen(mode)} {
	set fullscreen(mode) 0
	destroy .full
	wm deiconify .
	MLB::SetZoom $fullscreen(zoom)
	return
    }

    wm withdraw .
    set fullscreen(mode) 1

    toplevel .full
    wm withdraw  .full
    wm title     .full "mlb $mlbVersion"
    catch {wm iconbitmap .full "../bin/mlb.ico"}
    wm resizable .full 0 0
    wm protocol  .full WM_DELETE_WINDOW WorkingArea::FullScreen
    wm geometry  .full [winfo screenwidth .full]x[winfo screenheight .full]+0+0
    focus .full

    if {$bars == 1} {
	ToolBar::CreateFullScreen .full
    }

    frame .full.f -width [winfo screenwidth .full] -height [winfo screenheight .full] -bg $PageOptions::pageBackground
    pack  .full.f -fill both -expand 1

    set fullscreen(zoom) $Config::zoom
    set zoom1 [expr [winfo screenwidth .full] / $PageOptions::pageDimension(x).0 * 100.0]
    set zoom2 [expr [winfo screenheight .full] / $PageOptions::pageDimension(y).0 * 100.0]
    if {$zoom1 < $zoom2} {
	MLB::SetZoom $zoom1
    } else {
	MLB::SetZoom $zoom2
    }

    wm deiconify .full
}

proc WorkingArea::BuildToolpane { } {
    variable toolPane
    variable toolPaneW
    variable toolframe
    variable rebuild
    variable animationView

    catch {
	pack forget $toolframe
	destroy $toolframe
    }

    set toolframe [frame $toolPane.f]
    set toolPaneW [PanedWindow $toolframe.pw -side left]

    if {$Record::openTool} {
	set toolPane0 [$toolPaneW add -weight 0]
	set recView   [Record::Create $toolPane0]
	pack $toolPane0 -fill both -expand 1
    }

    if {$Config::showPagetree} {
	set toolPane1 [$toolPaneW add -minsize 100]
	set pageView  [PageTree::create $toolPane1]
	pack $toolPane1 -fill both -expand 1
	if {$rebuild} {
	    MLB::RebuildPageTree
	}
    }

    if {$Config::showParticipants} {
	set toolPane2 [$toolPaneW add -minsize 100]
	set partView  [Participant::CreateParticipantList $toolPane2]
	pack $toolPane2 -fill both -expand 1
	if {$rebuild} {
	    MLB::RebuildParticipantList
	}
    }

    if {$Feedback::openTool} {
	set toolPane4 [$toolPaneW add -minsize 180]
	set ftView [Feedback::Create $toolPane4]
	pack $toolPane4 -fill both -expand 1
	Feedback::ShowResults
    }

    if {$Voting::openTool} {
	set toolPane7 [$toolPaneW add -minsize 100]
	set vtView [Voting::Create $toolPane7]
	pack $toolPane7 -fill both -expand 1
	Voting::RebuildQuestionList
    }

    if {$Alt::openTool} {
	set toolPane5 [$toolPaneW add -minsize 100]
	set altView [Alt::Create $toolPane5]
	pack $toolPane5 -fill both -expand 1
	if {$rebuild} {
	   Alt::RebuildApplicationList
	}
    }

    if {$Config::showAnimations} {
	set toolPane6 [$toolPaneW add -minsize 150]
	set animationView  [Animations::create $toolPane6]
	pack $toolPane6 -fill both -expand 1
    } else {
        # if animations tool is requested to close
        if { 0 != $animationView  } {
            if { 0 == $Animations::editModeValue } {
                MLB::SetAnimationEditMode "1"
            }
            set animationView 0
        }
    }

    if {!$rebuild} {
	set rebuild 1
    }

    pack $toolPaneW -fill both -expand 1
    pack $toolframe -fill both -expand 1
}

proc WorkingArea::ShowHideToolBar {widget} {
    if {$Config::showToolBar} {
	if {$ToolBar::position == "top"} {
	    ToolBar::Top
	} elseif {$ToolBar::position == "bottom"} {
	    ToolBar::Bottom
	} elseif {$ToolBar::position == "left"} {
	    ToolBar::Left
	} elseif {$ToolBar::position == "right"} {
	    ToolBar::Right
	}
    } else {
	pack forget $widget
    }
}

proc WorkingArea::ShowHideAttributes {widget} {
    if {$Config::showAttributes} {
	if {$Attributes::position == "top"} {
	    Attributes::Top
	} elseif {$Attributes::position == "bottom"} {
	    Attributes::Bottom
	}
    } else {
	pack forget $widget
    }
}


proc WorkingArea::BeginAction {_x _y} {
    variable polyStatus
    variable action

    set action 1

    if {$Grid::snapStatus} {
#	set x int([expr [$Canvas::canvas canvasx $_x $Grid::snapSize] * $Config::zoom / 100])
#	set y int([expr [$Canvas::canvas canvasy $_y $Grid::snapSize] * $Config::zoom / 100])
	set x [$Canvas::canvas canvasx $_x [expr $Grid::snapSize * $Config::zoom / 100]]
	set y [$Canvas::canvas canvasy $_y [expr $Grid::snapSize * $Config::zoom / 100]]
    } else {
	set x $_x
	set y $_y
    }

    if {$ToolBar::currentAction == $ToolBar::Image} {
	MLB::PlaceImage 2 $x $y
	set ToolBar::currentAction 0
    } elseif {$ToolBar::currentTool == $ToolBar::OutRect} {
	MLB::CreateRectangle 0 $x $y $Attributes::color(0) \
	    "none" $Attributes::lineWidth $Attributes::lineStyle
    } elseif {$ToolBar::currentTool == $ToolBar::OutOval} {
	MLB::CreateOval 0 $x $y $Attributes::color(0) \
	    "none" $Attributes::lineWidth $Attributes::lineStyle $Attributes::lineStyle
    } elseif {$ToolBar::currentTool == $ToolBar::FillRect} {
	MLB::CreateRectangle 0 $x  $y $Attributes::color(0) $Attributes::color(1) \
	    $Attributes::lineWidth $Attributes::lineStyle
    } elseif {$ToolBar::currentTool == $ToolBar::FillOval} {
	MLB::CreateOval 0 $x $y $Attributes::color(0) $Attributes::color(1) \
	    $Attributes::lineWidth $Attributes::lineStyle
    } elseif {$ToolBar::currentTool == $ToolBar::Freehand} {
	MLB::CreateFreehand 0 $x $y $Attributes::color(0) \
	    $Attributes::lineWidth " " \
	    "butt" "round" "none" $Attributes::arrowShape\
	    nosmooth
    } elseif {$ToolBar::currentTool == $ToolBar::Polyline} {
	if {$polyStatus == "end"} {
	    MLB::CreatePolyline 0 $x $y $Attributes::color(0) \
		$Attributes::lineWidth $Attributes::lineStyle $Attributes::capStyle $Attributes::joinStyle \
		$Attributes::arrowStyle $Attributes::arrowShape\
		nosmooth
	    set polyStatus "continue"
	} elseif {$polyStatus == "continue"} {
	    MLB::CreatePolyline 1 $x $y
	}
    } elseif {$ToolBar::currentTool == $ToolBar::OutlinePolygon} {
	if {$polyStatus == "end"} {
	    MLB::CreatePolygon 0 $x $y $Attributes::color(0) \
		"none" $Attributes::lineWidth $Attributes::lineStyle
	    set polyStatus "continue"
	} elseif {$polyStatus == "continue"} {
	    MLB::CreatePolygon 1 $x $y
	}
    } elseif {$ToolBar::currentTool == $ToolBar::FilledPolygon} {
	if {$polyStatus == "end"} {
	    MLB::CreatePolygon 0 $x $y $Attributes::color(0) \
		$Attributes::color(1) $Attributes::lineWidth $Attributes::lineStyle
	    set polyStatus "continue"
	} elseif {$polyStatus == "continue"} {
	    MLB::CreatePolygon 1 $x $y
	}
    } elseif {$ToolBar::currentTool == $ToolBar::Line} {
	MLB::CreateLine 0 $x $y $Attributes::color(0) \
	    $Attributes::lineWidth $Attributes::lineStyle \
	    $Attributes::capStyle $Attributes::joinStyle \
	    $Attributes::arrowStyle $Attributes::arrowShape
    } elseif {$ToolBar::currentTool == $ToolBar::Arrow} {
	MLB::CreateLine 0 $x $y $Attributes::color(0) \
	    $Attributes::lineWidth $Attributes::lineStyle \
	    $Attributes::capStyle $Attributes::joinStyle \
	    "last" $Attributes::arrowShape
    } elseif {$ToolBar::currentTool == $ToolBar::Move} {
	MLB::Move 0 $_x $_y  $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Copy} {
	MLB::Copy 0 $_x $_y $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Delete} {
	MLB::SelectObjects 0 $_x $_y
    } elseif {$ToolBar::currentTool == $ToolBar::Select} {
	MLB::SelectObjects 0 $_x $_y
    } elseif {$ToolBar::currentTool == $ToolBar::OnlinePointer} {
	set pointer [lindex $OnlinePointer::ptype($Attributes::pointerType) 0]
	MLB::SetWindowPointer $pointer $Attributes::pointerColor 1
	MLB::OnlinePointer 0 $_x $_y $Attributes::pointerColor
    }
}

proc WorkingArea::ContinueAction {_x _y} {
    variable action

    if {!$action} return

    if {$Grid::snapStatus} {
	#set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	#set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
	set x [$Canvas::canvas canvasx $_x [expr $Grid::snapSize * $Config::zoom / 100]]
	set y [$Canvas::canvas canvasy $_y [expr $Grid::snapSize * $Config::zoom / 100]]
    } else {
	set x $_x
	set y $_y
    }

    if {$ToolBar::currentTool == $ToolBar::OutRect} {
	MLB::CreateRectangle 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::OutOval} {
	MLB::CreateOval 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::FillRect} {
	MLB::CreateRectangle 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::FillOval} {
	MLB::CreateOval 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Freehand} {
	MLB::CreateFreehand 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Polyline} {
	MLB::CreatePolyline 3 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::OutlinePolygon} {
	MLB::CreatePolygon 3 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::FilledPolygon} {
	MLB::CreatePolygon 3 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Line} {
	MLB::CreateLine 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Arrow} {
	MLB::CreateLine 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Move} {
	MLB::Move 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Copy} {
	MLB::Copy 1 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Delete} {
	MLB::SelectObjects 1 $_x $_y
    } elseif {$ToolBar::currentTool == $ToolBar::Select} {
	MLB::SelectObjects 1 $_x $_y
    } elseif {$ToolBar::currentTool == $ToolBar::OnlinePointer} {
	MLB::OnlinePointer 1 $_x $_y
    }
}

proc WorkingArea::EndAction {_x _y} {
    variable action

    if {!$action} return

    set action 0

    if {$Grid::snapStatus} {
	#set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	#set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
	set x [$Canvas::canvas canvasx $_x [expr $Grid::snapSize * $Config::zoom / 100]]
	set y [$Canvas::canvas canvasy $_y [expr $Grid::snapSize * $Config::zoom / 100]]
    } else {
	set x $_x
	set y $_y
    }

    if {$ToolBar::currentTool == $ToolBar::OutRect} {
	MLB::CreateRectangle 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::OutOval} {
	MLB::CreateOval 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::FillRect} {
	MLB::CreateRectangle 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::FillOval} {
	MLB::CreateOval 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Freehand} {
	MLB::CreateFreehand 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Line} {
	MLB::CreateLine 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Arrow} {
	MLB::CreateLine 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Text} {
	MLB::CreateText 0 $x $y $Attributes::color(0) \
	    $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight \
	    $Attributes::fontSlant $Attributes::fontPosition
    } elseif {$ToolBar::currentTool == $ToolBar::Delete} {
	MLB::SelectObjects 2 $_x $_y
	MLB::Delete
    } elseif {$ToolBar::currentTool == $ToolBar::Select} {
	MLB::SelectObjects 2 $_x $_y
    } elseif {$ToolBar::currentTool == $ToolBar::OnlinePointer} {
	MLB::SetWindowPointer point black 0
	MLB::OnlinePointer 2 $_x $_y
    } elseif {$ToolBar::currentTool == $ToolBar::Move} {
	MLB::Move 2 $x $y
    } elseif {$ToolBar::currentTool == $ToolBar::Copy} {
	MLB::Copy 2 $x $y
    }
}

proc WorkingArea::ShowHelpLine {_x _y} {
    variable polyStatus

    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    if {$ToolBar::currentAction == $ToolBar::Image} {
	MLB::PlaceImage 1 $x $y
    }

    if {$polyStatus == "continue"} {
	if {$ToolBar::currentTool == $ToolBar::Polyline} {
	    MLB::CreatePolyline 3 $x $y
	}
	if {$ToolBar::currentTool == $ToolBar::OutlinePolygon} {
	    MLB::CreatePolygon 3 $x $y
	}
	if {$ToolBar::currentTool == $ToolBar::FilledPolygon} {
	    MLB::CreatePolygon 3 $x $y
	}
    }
}


proc WorkingArea::ShowTicks {_x _y} {

    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    if {$Grid::tickStatus} {
	MLB::ShowTicks 1 $x $y
    }

}

proc WorkingArea::EndPolyObjects {_x _y} {
    variable polyStatus

    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    if {$polyStatus != "end"} {
	if {$ToolBar::currentTool == $ToolBar::Polyline} {
	    MLB::CreatePolyline 2 $x $y
	}
	if {$ToolBar::currentTool == $ToolBar::OutlinePolygon} {
	    MLB::CreatePolygon 2 $x $y
	}
	if {$ToolBar::currentTool == $ToolBar::FilledPolygon} {
	    MLB::CreatePolygon 2 $x $y
	}
	set polyStatus "end"
    }
}


proc WorkingArea::AutoEndPolyObjects { } {
    variable polyStatus

    if {$polyStatus != "end"} {
	if {$ToolBar::currentTool == $ToolBar::Polyline} {
	    MLB::CreatePolyline 2 0 0
	}
	if {$ToolBar::currentTool == $ToolBar::OutlinePolygon} {
	    MLB::CreatePolygon 2 0 0
	}
	if {$ToolBar::currentTool == $ToolBar::FilledPolygon} {
	    MLB::CreatePolygon 2 0 0
	}
	set polyStatus "end"
    }
}

proc WorkingArea::BeginAction2 {_x _y} {
    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    MLB::Move 0 $_x $_y $x $y
}

proc WorkingArea::ContinueAction2 {_x _y} {
    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    MLB::Move 1 $x $y
}

proc WorkingArea::EndAction2 {x y} {
    #Grid::snapStatus ???
    MLB::Move 2 $x $y
}

proc WorkingArea::BeginAction3 {x y} {
    #Grid::snapStatus ???
    if {$ToolBar::currentTool == $ToolBar::OnlinePointer} {
	MLB::UnSetWindowPointer
	MLB::SetWindowPointer point black 0
  
        #MB
    	MLB::OnlineBox 0 $x $y $Attributes::pointerColor $Attributes::boxWidth $Attributes::boxStyle
    } else {
	MLB::SelectObjects 0 $x $y
    }
}

#MB
proc WorkingArea::ContinueAction3 {_x _y} {
    #??? vgl. mit anderen ..Action-Methoden !!
    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    if {$ToolBar::currentTool == $ToolBar::OnlinePointer} {
	MLB::OnlineBox 1 $x $y
    } else {
        MLB::SelectObjects 1 $x $y
    }
}

#MB
proc WorkingArea::EndAction3 {_x _y} {
    if {$Grid::snapStatus} {
	set x [$Canvas::canvas canvasx $_x $Grid::snapSize]
	set y [$Canvas::canvas canvasx $_y $Grid::snapSize]
    } else {
	set x $_x
	set y $_y
    }

    if {$ToolBar::currentTool == $ToolBar::OnlinePointer} {
	MLB::OnlineBox 2 $x $y
	return
    }

    MLB::SelectObjects 2 $_x $_y
    if {$ToolBar::currentTool == $ToolBar::Delete} {
	MLB::Delete
    }
}

proc WorkingArea::PressCtrlX {x y} {
    MLB::CutObjects
}

proc WorkingArea::PressCtrlC {x y} {
    MLB::CopyObjects
}

proc WorkingArea::PressCtrlV {x y} {
    MLB::PasteObjects
}

proc WorkingArea::PressKey {x y key ascii} {
    if {$ToolBar::currentTool == $ToolBar::Text} {
	switch $key {
	    F1           { }
	    F2           { }
	    F3           { }
	    F4           { }
	    F5           { }
	    F6           { }
	    F7           { }
	    F8           { }
	    F9           { }
	    F10          { }
	    F11          { }
	    F12          { }
	    Shift_L      { }
	    Shift_R      { }
	    Caps_Lock    { }
	    Control_L    { }
	    Control_R    { }
	    Mode_switch  { }
	    Alt_L        { }
	    Super_L      { }
	    Super_R      { }
	    Menu         { }
	    Tab          { }
	    ISO_Left_Tab { }
	    Num_Lock     { }
	    Print        { }
	    Scroll_Lock  { }
	    Pause        { }
	    Insert       { }
	    Delete       { MLB::GoRight $x $y; MLB::Backspace $x $y}
	    KP_Insert    { }
	    Home         { MLB::GoHome $x $y }
	    KP_Home      { MLB::GoHome $x $y }
	    End          { MLB::GoEnd  $x $y }
	    KP_End       { MLB::GoEnd  $x $y }
	    Prior        { MLB::PreviousPage}
	    KP_Prior     { MLB::PreviousPage}
	    Next         { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    KP_Next      { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    Up           { MLB::GoUp    $x $y }
	    KP_Up        { MLB::GoUp    $x $y; MLB::PreviousPage }
	    Left         { MLB::GoLeft  $x $y }
	    KP_Left      { MLB::GoLeft  $x $y }
	    Right        { MLB::GoRight $x $y }
	    KP_Right     { MLB::GoRight $x $y }
	    Down         { MLB::GoDown  $x $y; if {!$Config::showAnimations} {} else {if {$Animations::editModeValue} {} else {Animations::OnSpace}}}
	    KP_Down      { MLB::GoDown  $x $y; if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    KP_Begin     { }
	    KP_Delete    { MLB::GoRight $x $y; MLB::Backspace $x $y }
	    KP_Divide    { }
	    KP_Multiply  { }
	    KP_Subtract  { }
	    KP_Add       { }
	    BackSpace    { MLB::Backspace $x $y }
	    Return       { MLB::NewLine   $x $y }
	    KP_Enter     { MLB::NewLine   $x $y }
            asciicircum  {
		if {"$ascii" == "^"} {
		    MLB::UpdateText $x $y "^"  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            degree       {
		MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    1            {
		if {"$ascii" == "1"} {
		    MLB::UpdateText $x $y 1  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            exclam       {
		MLB::UpdateText $x $y ! $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    2            {
		if {"$ascii" == "2"} {
		    MLB::UpdateText $x $y 2  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            quotedbl     {
		MLB::UpdateText $x $y "\"" $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    3            {
		if {"$ascii" == "3"} {
		    MLB::UpdateText $x $y 3  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            section      {
		MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    4            {
		if {"$ascii" == "4"} {
		    MLB::UpdateText $x $y 4  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            dollar       {
		MLB::UpdateText $x $y $  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    5            {
		if {"$ascii" == "5"} {
		    MLB::UpdateText $x $y 5  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            percent      {
		MLB::UpdateText $x $y %  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    6            {
		if {"$ascii" == "6"} {
		    MLB::UpdateText $x $y 6  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ""  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            ampersand    {
		MLB::UpdateText $x $y &  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            slash        {
		MLB::UpdateText $x $y /  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            parenleft    {
		MLB::UpdateText $x $y "("  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            parenright   {
		MLB::UpdateText $x $y ")"  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            equal        {
		MLB::UpdateText $x $y =  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            ssharp       {
		if {"$ascii" == "\\"} {
		    MLB::UpdateText $x $y "\\"  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition}
	    }
            question     {
		MLB::UpdateText $x $y ?  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            acute        {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            grave        {
		MLB::UpdateText $x $y `  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            udiaeresis   {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            Udiaeresis   {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            odiaeresis   {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            Odiaeresis   {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            adiaeresis   {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            Adiaeresis   {
		MLB::UpdateText $x $y   $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            plus         {
		if {"$ascii" == "+"} {
		    MLB::UpdateText $x $y +  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y ~  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            asterisk     {
		MLB::UpdateText $x $y *  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            numbersign   {
		MLB::UpdateText $x $y "\#"  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            apostrophe   {
		MLB::UpdateText $x $y '  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            minus        {
		MLB::UpdateText $x $y -  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            underscore   {
		MLB::UpdateText $x $y _  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            period       {
		MLB::UpdateText $x $y .  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            colon        {
		MLB::UpdateText $x $y :  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            comma        {
		MLB::UpdateText $x $y ,  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            semicolon    {
		MLB::UpdateText $x $y "\;"  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
            less         {
		if {"$ascii" == "<"} {
		    MLB::UpdateText $x $y <  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		} else  {
		    MLB::UpdateText $x $y |  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			    $Attributes::fontSlant $Attributes::fontPosition
		}
	    }
            greater      {
		MLB::UpdateText $x $y >  $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	    default      {
		MLB::UpdateText $x $y "$ascii" $Attributes::fontFamily $Attributes::fontSize $Attributes::fontWeight\
			$Attributes::fontSlant $Attributes::fontPosition
	    }
	}
    } else {
	switch $key {
	    KP_Up        { MLB::PreviousPage }
	    Up           { MLB::PreviousPage }
	    KP_Left      { MLB::PreviousPage }
	    Left         { MLB::PreviousPage }
	    Prior        { MLB::PreviousPage }
	    KP_Down      { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    Down         { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    KP_Right     { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    Right        { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    Next         { if {!$Config::showAnimations} {MLB::NextPage} else {if {$Animations::editModeValue} {MLB::NextPage} else {Animations::OnSpace}}}
	    Home         { MLB::FirstPage }
	    space        { Animations::OnSpace }
	    Delete       { MLB::Delete }
	    KP_Delete    { MLB::Delete }
	}
    }
}

proc WorkingArea::SetBindings {canvas} {

    bind $canvas <ButtonPress-1> {
	WorkingArea::BeginAction %x %y
    }
    bind $canvas <Button1-Motion> {
	WorkingArea::ContinueAction %x %y
	WorkingArea::ShowTicks %x %y
    }
    bind $canvas <ButtonRelease-1> {
	WorkingArea::EndAction %x %y
    }
    bind $canvas <Motion> {
	WorkingArea::ShowHelpLine %x %y
	WorkingArea::ShowTicks %x %y
    }
    bind $canvas <Double-ButtonPress-1> {
	WorkingArea::EndPolyObjects %x %y
	MLB::SelectThumbnail %x %y
    }
    bind $canvas <ButtonPress-2> {
	WorkingArea::BeginAction2 %x %y
	WorkingArea::ShowTicks %x %y
    }
    bind $canvas <Button2-Motion> {
	WorkingArea::ContinueAction2 %x %y
    }
    bind $canvas <ButtonRelease-2> {
	WorkingArea::EndAction2 %x %y
    }
    bind $canvas <ButtonPress-3> {
	WorkingArea::BeginAction3 %x %y
    }
    bind $canvas <Button3-Motion> {
	WorkingArea::ContinueAction3 %x %y
	WorkingArea::ShowTicks %x %y
    }
    bind $canvas <ButtonRelease-3> {
	WorkingArea::EndAction3 %x %y
    }
    bind $canvas <Control-x> {
	WorkingArea::PressCtrlX %x %y
    }
    bind $canvas <Control-c> {
	WorkingArea::PressCtrlC %x %y
    }
    bind $canvas <Control-v> {
	WorkingArea::PressCtrlV %x %y
    }
    bind $canvas <Any-Key> {
	WorkingArea::PressKey %x %y %K %A
    }
}


proc WorkingArea::SetBindingsThumbnails {canvas} {
    bind $canvas <ButtonPress-1> {
    	MLB::SelectThumbnail 3 %x %y
    }
    bind $canvas <Double-ButtonPress-1> {
    	MLB::SelectThumbnail 4 %x %y
    }
    bind $canvas <Any-Key> {
      switch %K {
	      KP_Up        { MLB::SelectThumbnail 2  0 -1 }
	      Up           { MLB::SelectThumbnail 2  0 -1 }
	      KP_Left      { MLB::SelectThumbnail 2 -1  0 }
	      Left         { MLB::SelectThumbnail 2 -1  0 }
	      KP_Down      { MLB::SelectThumbnail 2  0  1 }
	      Down         { MLB::SelectThumbnail 2  0  1 }
	      KP_Right     { MLB::SelectThumbnail 2  1  0 }
	      Right        { MLB::SelectThumbnail 2  1  0 }
	      Return       { MLB::SelectThumbnail 5 }
	    }
    }
}


proc WorkingArea::FreezeImages {} {
    MLB::FreezeImages $Config::freezeImages
}


proc WorkingArea::ParticipantHints {} {
    MLB::ParticipantHints $Config::participantHints
}


proc WorkingArea::FitPageToWindow { } {
    variable sw

    set zoom1 [expr round([winfo width $sw] / $PageOptions::pageDimension(x).0 * 100.0) - 1]
    set zoom2 [expr round([winfo height $sw] / $PageOptions::pageDimension(y).0 * 100.0) - 1]
    if {$zoom1 < $zoom2} {
	set Config::zoom $zoom1
    } else {
	set Config::zoom $zoom2
    }

    MLB::SetZoom $Config::zoom
}


namespace eval Canvas {
    variable canvas
    variable tmp

    set tmp 0
}

proc Canvas::Clear {_canvas width height} {
    variable canvas

    set canvas $_canvas

    $canvas delete all

    if {$Grid::gridStatus} {
	Grid::Draw $canvas $width $height
    }
    if {$Grid::tickStatus} {
	Grid::Ticks $canvas $width $height
    }
}


proc Canvas::Create {id width height bg thumbnails} {
    variable canvas

    if {$WorkingArea::fullscreen(mode)} {
	set canvas [canvas .full.f.canvas$id -width $width -height $height -bg $bg -highlightthickness 0]
    } else {
	set canvas [canvas $WorkingArea::canvasPane.canvas$id -width $width -height $height -highlightthickness 0 -bg $bg]
    }
    if {$thumbnails} {
      WorkingArea::SetBindingsThumbnails $canvas
    } else {
      WorkingArea::SetBindings $canvas
    }
    
    return $canvas
}


proc Canvas::Delete {_canvas} {

    if [winfo exists $_canvas] {
	$_canvas delete all
	pack forget $_canvas
	destroy $_canvas
    }
}

proc Canvas::Show {_canvas} {
    variable canvas

    set canvas $_canvas
    pack configure $canvas -anchor center -side left -expand 1 -padx 1
    focus $canvas
    #Animations::update
}

proc Canvas::Hide {_canvas} {
    variable canvas

    set canvas $_canvas
    pack forget $canvas
}

proc Canvas::DisplayConflictHint {canvas x y args} {
    variable tmp

    incr x [winfo rootx $canvas]
    incr y [winfo rooty $canvas]

    toplevel .chint$tmp -relief flat
    wm overrideredirect .chint$tmp 1
    wm transient .chint$tmp
    wm resizable .chint$tmp 0 0
    wm geometry .chint$tmp +$x+$y

    append t "Conflicts with:"

    foreach p $args {
	append t "\n"
	append t $p
    }

    label .chint$tmp.l -text $t -bg lightyellow -justify left
    pack .chint$tmp.l

    after 3000 [list destroy .chint$Canvas::tmp]

    incr tmp
}


proc Canvas::DisplayParticipantHint {canvas x y participant} {
    variable tmp

    incr x [winfo rootx $canvas]
    incr y [winfo rooty $canvas]

    toplevel .chint$tmp -relief flat
    wm overrideredirect .chint$tmp 1
    wm transient .chint$tmp
    wm resizable .chint$tmp 0 0
    wm geometry .chint$tmp +$x+$y

    label .chint$tmp.l -text $participant -bg lightyellow -justify left
    pack .chint$tmp.l

    after 2000 [list destroy .chint$Canvas::tmp]

    incr tmp
}
