/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifndef __WHITEBOARD_H__
#define __WHITEBOARD_H__

#ifdef WIN32
  #include <winsock.h>
  #include <io.h>
#else
  #include <sys/socket.h>
  #include <arpa/inet.h>
  #include <unistd.h>
  #include <netinet/in.h>
#endif


#include "iohelpers.h"
#include <iostream>


#include <tcl.h>
#include <types.h>
#include <string>


extern "C" {
  void linksocket(int fd, int mask, Tcl_FileProc* callback);
  void unlinksocket(int fd);

  #ifdef WIN32
    void clearReadSocketMessageQueue(int fd);
    void restoreReadSocketMessageQueue(int fd);
  #endif
}


namespace UCE {

   // read a fully headerblock 
  class ReceiveBlock {
  public:
    ReceiveBlock(int sock);
    ~ReceiveBlock()        { if (!isEmpty()) delete[] data; }
    
    uint8  getSID()        { return sID; }
    uint32 getMID()        { return mID; }
    uint16 getCode()       { return code; }
    char*  getData()       { return data; }
    uint32 getDataLength() { return datalength; }
 
    bool isReceiveError()  { return recvError; }
    bool isEmpty()         { return (datalength == 0); }
    bool isError()         { return (code > 500); }
    bool isFatal()         { return (code > 900); }
    
    std::string getErrorMsg() {
      if (!isError()) return "(this message is no error message!)";
      uint16* p16 = (uint16*)data;
      uint16 len = ntohs(p16[0]);
      if (len == 0) return "(no error message!)";
      return std::string(data + 2, len);
    }
      
  private:
    uint8  sID;
    uint16 code;
    uint32 mID, datalength;
    char*  data;
    bool   recvError;
    static char recvbuf[1000]; 
  };


  
  class MLB_Interface {
  public:
    MLB_Interface() {}
    ~MLB_Interface() {}

    virtual void setTelePointer(float xRel, float yRel, bool pressed) = 0;
    virtual void setActivePage(uint32 id) = 0;
  };
    

  
  static void failToRead();
  static void UceHandleIncomingData(ClientData clientData, int mask);
  
  class UCE_Interface {
  public:
    UCE_Interface(MLB_Interface* _mlb, std::string _address, int _port, std::string _username, std::string _pass) : mlb(_mlb), address(_address), port(_port), username(_username), pass(_pass) {}
    ~UCE_Interface();
  
    void handleIncomingData(ClientData clientData, int mask);
  
    int cmd_open_connection();
    int cmd_load(uint32 id, std::string name, uint32 order, uint32 size, char* page);
    int cmd_delete(uint32 id);
    int cmd_rename(uint32 id, std::string name);
    int cmd_set_active(uint32 id);
    
    static UCE_Interface* ucei;

  private:
    void process_SET_TELEPOINTER(char* data);
    void process_SET_ACTIVEPAGE(char* data);

    int send(char* data, uint32 size);
    int send(uint8, uint8, char* data, uint32 size);


    MLB_Interface* mlb;
    std::string address;
    int port;
    std::string username;
    std::string pass;
    
    int sock;
    uint8 wbID;
  };

}




#endif
