/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : alt_manager.cpp
 authors : Juergen Vogel
 date    : 28.07.2001
 desc    : 

 *************************************************************&********************************************/


#ifdef WIN32
#include <winsock.h>
#else
#include <arpa/inet.h>
#endif

#include <math.h>

#include "SMP.h"
#include "general.h"

#include <rtpi/rtpiException.h>

#include "mlb.h"
#include "alt_manager.h"
#include "mlb_communication.h"
#include "mlb_participants.h"
#include "mlb_timermanager.h"
#include "mlb_gui.h"

#define DEBUG_RTPI
//#define DEBUG_RTCPI
#define DEBUG

void ALT_handleReceivedRtpi(SMP::Indication ind) {
  if (ALT::AltManager::altm) ALT::AltManager::altm->HandleReceivedRtpi(ind);
}


void ALT_handleReceivedRtcpi(SMP::Indication ind) {
 if (ALT::AltManager::altm)  ALT::AltManager::altm->HandleReceivedRtcpi(ind);
}


ALT::AltManager* ALT::AltManager::altm = NULL;
char* ALT::AltManager::sendBuffer = new char[1000];
const unsigned int ALT::AltManager::T = 500;


ALT::Application::Application(uint32 _id, std::string _name, std::string _exec, std::string _params, std::string _lpath) 
  : id(_id), name(_name), exec(_exec), params(_params) , lpath(_lpath) {
}


int ALT::Application::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (AltManager::VERSION << 6);

  p8[4] = name.length();
  memcpy(buf + 5, name.c_str(), name.length());
  
  int padd = ((name.length() + 1) % 4 == 0) ? 0 : 4 - ((name.length() + 1) % 4);
  int offset = 5 + name.length();
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[offset + i] = 0;
  }

  offset += padd;

  p8[offset] = exec.length();
  ++offset;
  memcpy(buf + offset, exec.c_str(), exec.length());
  offset += exec.length();

  padd = ((exec.length() + 1) % 4 == 0) ? 0 : 4 - ((exec.length() + 1) % 4);
  if (padd != 0) { // padding needed
    for (int j = 0; j < padd; ++j)
      p8[offset + j] = 0;
  }

  offset += padd;

  p8[offset] = params.length();
  ++offset;
  memcpy(buf + offset, params.c_str(), params.length());
  offset += params.length();

  padd = ((params.length() + 1) % 4 == 0) ? 0 : 4 - ((params.length() + 1) % 4);
  if (padd != 0) { // padding needed
    for (int k = 0; k < padd; ++k)
      p8[offset + k] = 0;
  }

  return offset + padd;
}


int ALT::Application::ParseState(char* buf) {
  uint8*  p8  = (uint8*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != AltManager::VERSION) {
    GUI::MessageWindow::Error("Application::ParseState - wrong version of application launch protocol\n");
    return -1;
  }

  int offset = 5;
  int padd;
  int length;
  char* tmp;
 
  length = p8[4];
  tmp = new char[length + 1];
  memcpy(tmp, buf + offset, length);
  tmp[length] = '\0';
  name = tmp;
  delete[] tmp;

  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;

  length = p8[offset]; ++offset;
  tmp = new char[length + 1];
  memcpy(tmp, buf + offset, length);
  tmp[length] = '\0';
  exec = tmp;
  delete[] tmp;

  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;

  length = p8[offset]; ++offset;
  tmp = new char[length + 1];
  memcpy(tmp, buf + offset, length);
  tmp[length] = '\0';
  params = tmp;
  delete[] tmp;

  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);

  return offset + length + padd;
}


int ALT::Application::FlushEdit(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (AltManager::VERSION << 6);
  p8[0] = p8[0] | AltManager::EDIT_APPLICATION;

  p8[4] = name.length();
  memcpy(buf + 5, name.c_str(), name.length());
  
  int padd = ((name.length() + 1) % 4 == 0) ? 0 : 4 - ((name.length() + 1) % 4);
  int offset = 5 + name.length();
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[offset + i] = 0;
  }

  offset += padd;

  p8[offset] = exec.length();
  ++offset;
  memcpy(buf + offset, exec.c_str(), exec.length());
  offset += exec.length();

  padd = ((exec.length() + 1) % 4 == 0) ? 0 : 4 - ((exec.length() + 1) % 4);
  if (padd != 0) { // padding needed
    for (int j = 0; j < padd; ++j)
      p8[offset + j] = 0;
  }

  offset += padd;

  p8[offset] = params.length();
  ++offset;
  memcpy(buf + offset, params.c_str(), params.length());
  offset += params.length();

  padd = ((params.length() + 1) % 4 == 0) ? 0 : 4 - ((params.length() + 1) % 4);
  if (padd != 0) { // padding needed
    for (int k = 0; k < padd; ++k)
      p8[offset + k] = 0;
  }

  return offset + padd;
}


int ALT::Application::ParseEdit(char* buf) {
  uint8*  p8  = (uint8*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != AltManager::VERSION) {
    GUI::MessageWindow::Error("Application::ParseEdit - wrong version of application launch protocol\n");
    return -1;
  }

  if ((p8[0] & ALT_MASK_TYPE) != AltManager::EDIT_APPLICATION) {
    GUI::MessageWindow::Error("Application::ParseEdit - wrong type\n");
    return -1;
  }

  int offset = 5;
  int padd;
  int length;
  char* tmp;
 
  length = p8[4];
  tmp = new char[length + 1];
  memcpy(tmp, buf + offset, length);
  tmp[length] = '\0';
  name = tmp;
  delete[] tmp;

  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;

  length = p8[offset]; ++offset;
  tmp = new char[length + 1];
  memcpy(tmp, buf + offset, length);
  tmp[length] = '\0';
  exec = tmp;
  delete[] tmp;

  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;

  length = p8[offset]; ++offset;
  tmp = new char[length + 1];
  memcpy(tmp, buf + offset, length);
  tmp[length] = '\0';
  params = tmp;
  delete[] tmp;

  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);

  return offset + length + padd;
}


int ALT::Application::FlushStart(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (AltManager::VERSION << 6);
  p8[0] = p8[0] | AltManager::START_APPLICATION;

  return 4;
}


int ALT::Application::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (AltManager::VERSION << 6);
  p8[0] = p8[0] | AltManager::DELETE_APPLICATION;

  return 4;
}

 
ALT::AltManager::AltManager() : openConnection(false) {
}


ALT::AltManager::~AltManager() {
  CloseRTPI();
}


bool ALT::AltManager::OpenRTPI() {
  if (openConnection || !MLB::CommunicationManager::cm->openSMP || !MLB::CommunicationManager::cm->openIDServer) {
    GUI::MessageWindow::Error("AltManager::OpenRTPI - RTP/I Connection cannot be set up %u %u %u\n", 
			      openConnection, MLB::CommunicationManager::cm->openSMP, MLB::CommunicationManager::cm->openIDServer);
    return false;
  }

  SMP::SMP* smp = MLB::CommunicationManager::cm->smp;

  // setup SMP
  if ((rtpiSocket = smp->Open(::ALT_handleReceivedRtpi)) < 0) {
    GUI::MessageWindow::Error("AltManager::Open - could not connect to smp\n");
    return false;
  }

  if ((rtcpiSocket = smp->Open(::ALT_handleReceivedRtcpi)) < 0) {
    GUI::MessageWindow::Error("AltManager::Open - could not connect to smp\n");
    return false;
  }

  // setup RTP/I
  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("AltManager::OpenRTPI - joining %q - %u - %u\n", MLB::CommunicationManager::cm->ip.c_str(), 
		 MLB::CommunicationManager::cm->port+11, MLB::CommunicationManager::cm->ttl);
  #endif

  if (!smp->JoinGroup(rtpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+11, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("AltManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }
  if (!smp->JoinGroup(rtcpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+12, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("AltManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }

  rtpi = new RTPI::Rtpi();
  rtpi->join_group( MLB::CommunicationManager::cm->username, this, this, new RTPI::TransparentReliableTransport());
  
  openConnection = true;

  return true;
}


bool ALT::AltManager::CloseRTPI() {
  if (!openConnection) {
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::CloseRTPI - session already closed\n");
    #endif
    return false;
  }

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("AltManager::LeaveGroup - leaving group\n");
  #endif

  rtpi->leave_group();
  
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtpiSocket);
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtcpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtcpiSocket);

  delete rtpi;
  
  openConnection = false;

  for (std::map<uint32, Application*>::iterator i = applications.begin(); i != applications.end(); ++i) {
    delete i->second;
  }
  applications.clear();

  return true;
}


void ALT::AltManager::HandleReceivedRtpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got data %u\n", ind.buf->getSize());
    #endif

    RefCount<RTPI::RtpiAduVector> rtpiAdus;
    try {
      rtpiAdus = rtpi->create_rtpi_adus(ind.buf);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("AltManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    if (!rtpiAdus) 
      return; // adu not complete, received only fragments

    for (RTPI::RtpiAduVector::iterator i = rtpiAdus->begin(); i != rtpiAdus->end(); ++i) {
      if ((*i)->GetAduHeader()->GetPayloadType() != APPLICATION_TYPE) {
	GUI::MessageWindow::Error("AltManager::HandleReceivedRtpi - wrong payload type %u\n", 
		       (*i)->GetAduHeader()->GetPayloadType());
	continue;
      }

      switch ((*i)->GetType()) {
      case RTPI::RtpiHeader::PKT_EVENT: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & ALT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("AltManager::ReceiveEvent - wrong version of chat protocol\n");
	  continue;
	}

	int type = p8[0] & ALT_MASK_TYPE;
	switch (type) {
	case EDIT_APPLICATION: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - edit application with subid %u\n", (*i)->GetSubcomponentID());
          #endif

	  Application* appl = GetApplication((*i)->GetSubcomponentID());
	  if (!appl) {
            #ifdef DEBUG
	    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - application with subid does not exist%u\n", (*i)->GetSubcomponentID());
            #endif
	    RequestSubcomponent((*i)->GetSubcomponentID());
	    break;
	  }

	  appl->ParseEdit((*i)->GetPayload());
	  GuiEditApplication(appl->id);

	  break;
	}
	case DELETE_APPLICATION: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - delete application with subid %u\n", (*i)->GetSubcomponentID());
          #endif

	  Application* appl = GetApplication((*i)->GetSubcomponentID());
	  if (!appl) {
            #ifdef DEBUG
	    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - application with subid does not exist%u\n", (*i)->GetSubcomponentID());
            #endif
	    break;
	  }

	  UnregisterSubcomponent(appl->id);
	  applications.erase(appl->id);
	  GuiDeleteApplication(appl->id);
	  delete appl;
	  break;
	}
	case START_APPLICATION: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - start application with subid %u\n", (*i)->GetSubcomponentID());
          #endif

	  Application* appl = GetApplication((*i)->GetSubcomponentID());
	  if (!appl) {
            #ifdef DEBUG
	    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - application with subid does not exist%u\n", (*i)->GetSubcomponentID());
            #endif
	    RequestSubcomponent((*i)->GetSubcomponentID());
	    break;
	  }

	  GuiStartApplication(appl->id);
	  break;
	}
	}
	
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE: {

	#ifdef DEBUG
	GUI::MessageWindow::Info("AltManager::ReceiveState - received state for application with subid %u\n", (*i)->GetSubcomponentID());
	#endif

	Application* appl = GetApplication((*i)->GetSubcomponentID());
	if (appl) { 
	  // already here
	  if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_REPLY, (*i)->GetSubcomponentID())) {
	    // we have an own state reply timer -> delete 
	    MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_REPLY, (*i)->GetSubcomponentID());
	  } 
	  break;
	}

	if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID())) {
	  // we have an own state query timer -> delete 
	  MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID());
	} 

	appl = new Application((*i)->GetSubcomponentID());
	appl->ParseState((*i)->GetPayload());

	applications.insert(std::map<uint32, Application*>::value_type(appl->id, appl));

	RegisterSubcomponent(appl->id);

	GuiAddApplication(appl->id);
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE_QUERY: {

	#ifdef DEBUG
	GUI::MessageWindow::Info("AltManager::ReceiveStateQuery - got State Query for %u\n", (*i)->GetSubcomponentID());
	#endif

	Application* appl = GetApplication((*i)->GetSubcomponentID());
	if (!appl) { 
	  // not here
	  if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID())) {
	    // we have an own state query timer -> delete 
	    MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID());
	  } 
	  break;
	}

	AnswerStateQuery(appl->id);
	break;
      }
      default:
	GUI::MessageWindow::Error("AltManager::HandleReceivedRtpi - wrong adu type %u\n", 
		       (*i)->GetType());
      }
    }

    try {
      rtpi->handle_received_rtpi(rtpiAdus, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("AltManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    return;
  }

  
  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    return;
  }
  

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got LJNEEDED\n");
    #endif
    return;
														 

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("AltManager::HandleReceivedRtpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;


  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void ALT::AltManager::HandleReceivedRtcpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got data %u\n", ind.buf->getSize());
    #endif
    return;
  }

  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    try {
      rtpi->handle_received_rtcpi(ind.buf, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("AltManager::HandleReceivedRtcpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }
    return;
  }

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got LJNEEDED\n");
    #endif
    return;

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("AltManager::HandleReceivedRtcpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;

  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("AltManager::HandleReceivedRtcpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void ALT::AltManager::AddApplication(std::string name, std::string exec, std::string params, std::string lpath) {
  if (!openConnection) return;

  uint32 subid = MLB::CommunicationManager::cm->GetObjectID();
  
  #ifdef DEBUG
  GUI::MessageWindow::Info("AltManager::AddApplication - add appl %u with name %q exec %q params %q lpath %q\n", 
			   subid, name.c_str(), exec.c_str(), params.c_str(), lpath.c_str());
  #endif

  Application* appl = new Application(subid, name, exec, params, lpath);
  applications.insert(std::map<uint32, Application*>::value_type(subid, appl));

  RegisterSubcomponent(subid);

  int bytes = appl->FlushState(sendBuffer);
  SendState(subid, 3, sendBuffer, bytes);

  GuiAddApplication(subid);
}


void ALT::AltManager::EditApplication(uint32 subid, std::string name, std::string exec, std::string params, std::string lpath)  {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("AltManager::AddApplication - edit appl %u with name %q exec %q params %q lpath %q\n", 
			   subid, name.c_str(), exec.c_str(), params.c_str(), lpath.c_str());
  #endif

  appl->name = name;
  appl->exec = exec;
  appl->params = params;
  appl->lpath = lpath;

  int bytes = appl->FlushEdit(sendBuffer);
  SendEvent(subid, sendBuffer, bytes);

  GuiEditApplication(subid);
}


void ALT::AltManager::DeleteApplication(uint32 subid) {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("AltManager::DeleteApplication - delete appl %u\n", subid);
  #endif

  int bytes = appl->FlushDelete(sendBuffer);
  SendEvent(subid, sendBuffer, bytes);

  UnregisterSubcomponent(subid);

  applications.erase(subid);
  delete appl;
  
  GuiDeleteApplication(subid);
}


void ALT::AltManager::StartApplication(uint32 subid) {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("AltManager::StartApplication - start appl %u\n", subid);
  #endif

  int bytes = appl->FlushStart(sendBuffer);
  SendEvent(subid, sendBuffer, bytes);
}


void ALT::AltManager::SendState(uint32 subid, int priority, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("AltManager::SendState - sending state for subid %u\n", subid);
  #endif

  RefCount<RTPI::RtpiAdu> state = new RTPI::RtpiState(subid, APPLICATION_TYPE, 0, rtpitime(), priority, payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(state);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("AltManager::SendState - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void ALT::AltManager::SendEvent(uint32 subid, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("AltManager::SendEvent - sending event for subid %u\n", subid);
  #endif

  RefCount<RTPI::RtpiAdu> event = new RTPI::RtpiEvent(subid, APPLICATION_TYPE, 0, rtpitime(), payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(event);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("AltManager::SendEvent - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void ALT::AltManager::SendStateQuery(uint32 subid, int priority) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("AltManager::SendStateQuery - sending state query for subid %u\n", subid);
  #endif

  RefCount<RTPI::RtpiAdu> query = new RTPI::RtpiStateQuery(subid, APPLICATION_TYPE, rtpitime(), priority);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(query);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("AltManager::SendStateQuery - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void ALT::AltManager::send_rtcpi(RefCount<Buffer> buf) {
  if (!openConnection) return;
  MLB::CommunicationManager::cm->smp->SendUnreliable(rtcpiSocket, buf->getCont(), buf->getSize());
}


void ALT::AltManager::subcomponent_changed(RTPI::SubComponentMapElement* subcomp, uint8 reason) {
}


void ALT::AltManager::subcomponent_added(RTPI::SubComponentMapElement* subcomp, int reason) {
  Application* appl = GetApplication(subcomp->first);
  if (appl) return; //already here

  RequestSubcomponent(subcomp->first);
}


void ALT::AltManager::subcomponent_removed(uint32 id) {
}


void ALT::AltManager::participant_added(uint32 pid) {
}


void ALT::AltManager::participant_removed(uint32 pid) {
}


void ALT::AltManager::participant_changed(uint32 pid) {
}


void ALT::AltManager::RegisterSubcomponent(uint32 subid) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("AltManager::RegisterSubcomponent - register sub with id %u\n", subid);
  #endif

  // register subcomponent
  rtpi->add_subcomponent(subid, "appl", true);
}


void ALT::AltManager::UnregisterSubcomponent(uint32 subid) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("AltManager::UnregisterSubcomponent - unregister sub with id %u\n", subid);
  #endif

  rtpi->delete_subcomponent(subid);
}


void ALT::AltManager::RequestSubcomponent(uint32 subid, int priority) {
  double x = drand48();
  int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
  double N = (n == 1) ? 1 : n - 1;
  
  #ifdef DEBUG
  GUI::MessageWindow::Info("AltManager::RequestSubcomponent - x: %f, N: %f\n", x, N);
  #endif
 
  if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
    // 1. send state query immediately
    SendStateQuery(subid, priority);

    #ifdef DEBUG
    GUI::MessageWindow::Info("AltManager::RequestSubcomponent - send state query for %u immediately\n", subid);
    #endif
  } else {
    // 2. set state query timer
    int runningTime = (int)(T * (1 + log(x) / log(N)));
    MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime, MLB::TimerObject::STATE_QUERY, subid, priority));

    #ifdef DEBUG
    GUI::MessageWindow::Info("AltManager::RequestSubcomponent - send state query for %u later, set timer with running time %u\n", 
			     subid ,runningTime);
    #endif
  }
}


void ALT::AltManager::AnswerStateQuery(uint32 subid, int priority) {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  double x = drand48();
  int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
  double N = (n == 1) ? 1 : n - 1;
  
  #ifdef DEBUG
  GUI::MessageWindow::Info("AltManager::AnswerStateQuery - x: %f, N: %f\n", x, N);
  #endif
 
  if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
    // 1. send state query immediately
    int bytes = appl->FlushState(sendBuffer);
    SendState(subid, priority, sendBuffer, bytes);

    #ifdef DEBUG
    GUI::MessageWindow::Info("AltManager::AnswerStateQuery - send state for %u immediately\n", subid);
    #endif
  } else {
    // 2. set state query timer
    int runningTime = (int)(T * (1 + log(x) / log(N)));
    MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime, MLB::TimerObject::STATE_QUERY, subid, priority));

    #ifdef DEBUG
    GUI::MessageWindow::Info("AltManager::AnswerStateQuery - send state for %u later, set timer with running time %u\n", 
			     subid ,runningTime);
    #endif
  }
}


void ALT::AltManager::ExecuteTimer(MLB::TimerObject* timer) {
  switch (timer->type) {
  case MLB::TimerObject::STATE_QUERY: {
    SendStateQuery(timer->id, timer->priority);

    #ifdef DEBUG
    GUI::MessageWindow::Info("AltManager::ExecuteTimer - State Query Timer ran out, sending state query\n");
    #endif
    break;
  }
  case MLB::TimerObject::STATE_REPLY: {
    Application* appl = GetApplication(timer->id);
    if (!appl) return;
    int bytes = appl->FlushState(sendBuffer);
    SendState(appl->id, timer->priority, sendBuffer, bytes);

    #ifdef DEBUG
    GUI::MessageWindow::Info("AltManager::ExecuteTimer - State Reply Timer ran out, sending state\n");
    #endif
    break;
  }
  }

  delete timer;
}


void ALT::AltManager::GuiAddApplication(uint32 subid) {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  Et_GlobalEvalF(Et_Interp, "Alt::AddApplication %u \"%q\" \"%q\" \"%q\" \"%q\"", 
		 appl->id, appl->name.c_str(), appl->exec.c_str(), appl->params.c_str(), appl->lpath.c_str());
}


void ALT::AltManager::GuiEditApplication(uint32 subid) {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  Et_GlobalEvalF(Et_Interp, "Alt::EditApplication %u \"%q\" \"%q\" \"%q\" \"%q\"", 
		 appl->id, appl->name.c_str(), appl->exec.c_str(), appl->params.c_str(), appl->lpath.c_str());
}


void ALT::AltManager::GuiDeleteApplication(uint32 subid) {
  Et_GlobalEvalF(Et_Interp, "Alt::DeleteApplication %u", subid);
}


void ALT::AltManager::GuiStartApplication(uint32 subid) {
  Application* appl = GetApplication(subid);
  if (!appl) return;

  Et_GlobalEvalF(Et_Interp, "Alt::StartApplication %u", subid);
}


ALT::Application* ALT::AltManager::GetApplication(uint32 id) {
  std::map<uint32, Application*>::iterator a = applications.find(id);
  if (a == applications.end()) return NULL;
  return a->second;
}
