/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : chat_manager.h 
 authors : Juergen Vogel
 date    : 
 desc    : 

***********************************************************************************************************/

#ifndef __CHAT_MANAGER_H__
#define __CHAT_MANAGER_H__

#define CHAT_MASK_V    0xC0
#define CHAT_MASK_TYPE 0x3F

#include <rtpi/rtpi.h>
#include <rtpi/rtpiPacket.h>
#include <rtpi/rtcpiPacket.h>
#include <rtpi/rtpihandler.h>
#include <rtpi/rtpiparticipant.h>
#include <rtpi/rtpiInterfaces.h>
#include <rtpi/rtpiTransport.h>
#include <rtpi/rtpiSeqNumber.h>
#include <util/types.h>
#include <util/Buffer.h>

#include <mlb_communication.h>
#include <mlb_timermanager.h>
#include <mlb_gui.h>

#include <tcl.h>

#include <list>
#include <string>

namespace CHAT {
  class Message {
  public:
    Message(string nickname, string message);
    ~Message() {}

    string nickname;
    string message;
  };

  class ChatManager : public RTPI::RtcpiReportInterface, public RTPI::RtpiFeedbackInterface, public MLB::TimerFeedbackInterface {
  public:
    enum types {
      // general
      VERSION = 0,
      APPLICATION_TYPE = 3, // chat
      SUB_ID = 0, // there is only one subcomponent
      HISTORY_SIZE = 10,
      
      // operation Types
      ADD_MESSAGE = 0
    };
    
    ChatManager();
    ~ChatManager();
    
    bool OpenConnection() {return openConnection;}

    bool OpenRTPI();
    bool CloseRTPI();

    // handle incoming data
    void HandleReceivedRtpi(SMP::Indication ind);
    void HandleReceivedRtcpi(SMP::Indication ind);

    // functions called by gui
    void SendMessage(char* nickname, char* message);

    int FlushState(char* buf);
    int FlushAddMessage(char* buf, char* nickname, char* message);

    // outgoing data
    void SendState(int priority, char* payload, int size);
    void SendEvent(char* payload, int size);
    void SendStateQuery(int priority);

    // functions implemented due to  RtcpiReportInterface
    void    send_rtcpi(RefCount<Buffer>);
    u_int32 get_local_address(int i=0)   {return MLB::CommunicationManager::cm->get_local_address(i);}
    int     addr_is_local(u_int32 addr)  {return MLB::CommunicationManager::cm->addr_is_local(addr);}
    u_int32 get_session_bandwith()       {return MLB::CommunicationManager::cm->bandwidth/10;}
    u_int32 get_new_pid()                {return MLB::CommunicationManager::cm->pid;}
    u_int32 get_new_subid()              {return MLB::CommunicationManager::cm->get_new_subid();}
    string  get_session_name(string ="") {return MLB::CommunicationManager::cm->name;}
    string  get_session_email()          {return MLB::CommunicationManager::cm->email;}
    string  get_session_phone()          {return MLB::CommunicationManager::cm->phone;}
    string  get_session_loc()            {return MLB::CommunicationManager::cm->location;}
    string  get_session_note()           {return ""; }
    string  get_session_tool()           {return "chat"; }

    // functions implemented due to RtpiFeedbackInterface
    void subcomponent_added(RTPI::SubComponentMapElement* subcomp, int reason);
    void subcomponent_changed(RTPI::SubComponentMapElement* subcomp, uint8 reason);
    void subcomponent_removed(uint32 id);
    void participant_added(uint32 pid);
    void participant_removed(uint32 pid);
    void participant_changed(uint32 pid);

    static ChatManager* cm;

    static const unsigned int T;
    static char* sendBuffer;

  private:
    GUI::Chat *chatGui;
    
    void RememberMessage(char* nickname, char* message);
    void DisplayHistory();

    // history of messages
    std::list<Message*> history;

    // UDP 
    bool openConnection;

    // SMP 
    int rtpiSocket;
    int rtcpiSocket;

    // RTP/I 
    bool init; // while init, don't answer state queries
    RTPI::Rtpi* rtpi;

    // timer management
    void ExecuteTimer(MLB::TimerObject* obj);
  };
}


#endif
