/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : ft_manager.cpp
 authors : Juergen Vogel
 date    : 09.05.2001
 desc    : 

***********************************************************************************************************/


#ifdef WIN32
#include <winsock.h>
#else
#include <arpa/inet.h>
#endif

#include <math.h>

#include "SMP.h"
#include "general.h"

#include <rtpi/rtpiException.h>

#include "mlb.h"
#include "ft_manager.h"
#include "mlb_communication.h"
#include "mlb_participants.h"
#include "mlb_timermanager.h"
#include "mlb_gui.h"

#define DEBUG_RTPI
//#define DEBUG_RTCPI
#define DEBUG

void FT_handleReceivedRtpi(SMP::Indication ind) {
  if (FT::FtManager::ftm) FT::FtManager::ftm->HandleReceivedRtpi(ind);
}


void FT_handleReceivedRtcpi(SMP::Indication ind) {
  if (FT::FtManager::ftm) FT::FtManager::ftm->HandleReceivedRtcpi(ind);
}


FT::FtManager* FT::FtManager::ftm = NULL;
char* FT::FtManager::sendBuffer = new char[10000];
const unsigned int FT::FtManager::T = 500;


int FT::Answer::FlushCreate(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (FtManager::VERSION << 6);
  p8[0] = p8[0] | (FtManager::CREATE_ANSWER);

  p8[1] = value;

  return 4;
}


int FT::Answer::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (FtManager::VERSION << 6);
  p8[0] = p8[0] | (FtManager::DELETE_ANSWER);

  return 4;
}


FT::Question::Question(uint32 _id, uint32 _ttl, std::string _text, int nbr, std::string _expl)
  : visible(true), id(_id), ttl(_ttl), text(_text) {

  std::string::size_type b,e;

  b = _expl.find('{');
  e = _expl.find('}');

  std::string tmp = _expl.substr(b+1,e-b-2);

  explanations.push_back(_expl.substr(b+1,e-b-2));

  while ((b = _expl.find('{', e)) != std::string::npos) {
    e = _expl.find('}', b);
    tmp =  _expl.substr(b+1,e-b-2);
    explanations.push_back(_expl.substr(b+1,e-b-2));
  }
}


void FT::Question::Update(std::string _text, int nbr, std::string _expl) {
  text = _text;
  
  explanations.clear();

  std::string::size_type b,e;

  b = _expl.find('{');
  e = _expl.find('}');

  std::string tmp = _expl.substr(b+1,e-b-2);

  explanations.push_back(_expl.substr(b+1,e-b-2));

  while ((b = _expl.find('{', e)) != std::string::npos) {
    e = _expl.find('}', b);
    tmp =  _expl.substr(b+1,e-b-2);
    explanations.push_back(_expl.substr(b+1,e-b-2));
  }
}


double FT::Question::CalculateResult() {
  if (answers.size() == 0) return 0.0;

  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;
  double result = 0.0;

  for (std::list<Answer>::iterator a = answers.begin(); a != answers.end(); ++a) {
    // check ttl
    if ((ttl != 0) && (now > a->createTime + ttl)) {
      // answer has expired
      a = answers.erase(a);
      if (a !=  answers.begin()) --a; 
      continue;
    } 

    result += double(a->value);
  }

  result /= answers.size();
  return result;
}


void FT::Question::DisplayUpdate() {
  for (std::vector<std::string>::iterator i = explanations.begin(); i != explanations.end(); ++i) {
    if (Et_GlobalEvalF(Et_Interp,"lappend tmplist \"%q\"", i->c_str()) != TCL_OK)
      GUI::MessageWindow::Error("FT::Question::DisplayUpdate - tcl error %q\n", Et_Interp->result);
  }
  
  if (Et_GlobalEvalF(Et_Interp,"Feedback::UpdateQuestion %u \"%q\" %u %u $tmplist", 
		     id, text.c_str(), visible, explanations.size()) != TCL_OK) 
    GUI::MessageWindow::Error("FT::Question::Display - tcl error %q\n", Et_Interp->result);
  
  if (Et_GlobalEvalF(Et_Interp,"unset tmplist") != TCL_OK) 
    GUI::MessageWindow::Error("FT::Question::Display - tcl error %q\n", Et_Interp->result);
}


void FT::Question::DisplayAdd() {
  for (std::vector<std::string>::iterator i = explanations.begin(); i != explanations.end(); ++i) {
    if (Et_GlobalEvalF(Et_Interp,"lappend tmplist \"%q\"", i->c_str()) != TCL_OK)
      GUI::MessageWindow::Error("FT::Question::DisplayAdd - tcl error %q\n", Et_Interp->result);
  }
  
  if (Et_GlobalEvalF(Et_Interp,"Feedback::AddQuestion %u \"%q\" %u %u $tmplist", 
		     id, text.c_str(), visible, explanations.size()) != TCL_OK) 
    GUI::MessageWindow::Error("FT::Question::DisplayAdd - tcl error %q\n", Et_Interp->result);
  
  if (Et_GlobalEvalF(Et_Interp,"unset tmplist") != TCL_OK) 
    GUI::MessageWindow::Error("FT::Question::DisplayAdd - tcl error %q\n", Et_Interp->result);
}


void FT::Question::DisplayResult() {
  if (Et_GlobalEvalF(Et_Interp,"Feedback::SetResult %u %.2f", id, CalculateResult()) != TCL_OK) 
    GUI::MessageWindow::Error("FT::Question::DisplayResult - tcl error %q\n", Et_Interp->result);
}


int FT::Question::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (FtManager::VERSION << 6);
  p8[0] = p8[0] | (visible);

  p8[1] = explanations.size();

  p16[1] = htons(ttl);

  int offset = 4;
  int padd;
  int length;

  // 1. flush question text
  length = text.length();
  p8[offset] = length;
  ++offset;

  memcpy(buf + offset, text.c_str(), length);
  
  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[offset + length + i] = 0;
  }
  
  offset += (length + padd);
  
  // 2. flush possible answers' explanations
  for (int i = 1; i <= explanations.size(); ++i) {
    length = explanations[i-1].length();
    p8[offset] = length;
    ++offset;

    memcpy(buf + offset, explanations[i-1].c_str(), length);

    padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
    if (padd != 0) { // padding needed
      for (int i = 0; i < padd; ++i)
	p8[offset + length + i] = 0;
    }

    offset += (length + padd);
  }

  p16[offset/2] = htons(answers.size());
  p16[offset/2 + 1] = 0;

  offset += 4;

  // 3. flush all received answers
  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;
  uint32 tl = 0; // time left
  for (std::list<Answer>::iterator a = answers.begin(); a != answers.end(); ++a) {
    p8[offset]    = a->value;
    p8[offset + 1]= 0;
    
    tl = ttl - (now - a->createTime);
    p16[offset/2 + 1] = htons((tl < 0) ? 0 : tl);

    p32[offset/4 + 1] = htonl(a->pid);

    offset += 8;
  }

  return offset;
}


int FT::Question::ParseState(char* buf, uint32 ts) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != FtManager::VERSION) {
    GUI::MessageWindow::Error("Question::ParseState - wrong version of feedback protocol\n");
    return -1;
  }

  visible = p8[0] & FT_MASK_VIS;

  int nbr = p8[1];
  
  ttl = ntohs(p16[1]);

  int offset = 4;
  int padd;
  int length;
  char* tmp;

  // 1. read question text
  length = p8[offset];
  ++offset;
  tmp = new char[length + 1];
  tmp[length] = '\0';
  memcpy(tmp, buf + offset, length);
  text = tmp;
  delete[] tmp;
  
  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;
  
  // 2. read possible answers' explanations
  explanations.clear();
  for (int i = 1; i <= nbr; ++i) {
    length = p8[offset];
    ++offset;
    tmp = new char[length + 1];
    tmp[length] = '\0';
    memcpy(tmp, buf + offset, length);
    explanations.push_back(tmp);
    delete[] tmp;

    padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
    offset += length + padd;
  }

  // 3. read all included answers
  nbr = ntohs(p16[offset/2]);
  offset += 4;

  answers.clear();
  for (int j = 1; j <= nbr; j++) {
    answers.push_back(Answer(ntohl(p32[offset/4 + 1]), p8[offset], ntohs(p16[offset/2 + 1]) + ts - ttl));
    offset += 8;
  }
  
  return offset;
}


int FT::Question::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (FtManager::VERSION << 6);
  p8[0] = p8[0] | (FtManager::DELETE_QUESTION);

  return 4;
}


int FT::Question::FlushChangeVisibility(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (FtManager::VERSION << 6);
  p8[0] = p8[0] | (FtManager::CHANGE_VISIBILITY);

  p8[1] = visible;

  return 4;
}


FT::FtManager::FtManager() : openConnection(false), init(true) {
}


FT::FtManager::~FtManager() {
  CloseRTPI();
}


void FT::FtManager::Init() {
  // create default questions
}


bool FT::FtManager::OpenRTPI() {
  if (openConnection || !MLB::CommunicationManager::cm->openSMP || !MLB::CommunicationManager::cm->openIDServer) {
    GUI::MessageWindow::Error("FtManager::OpenRTPI - RTP/I Connection cannot be set up\n");
    return false;
  }

  SMP::SMP* smp = MLB::CommunicationManager::cm->smp;

  // setup SMP
  if ((rtpiSocket = smp->Open(::FT_handleReceivedRtpi)) < 0) {
    GUI::MessageWindow::Error("FtManager::Open - could not connect to smp\n");
    return false;
  }

  if ((rtcpiSocket = smp->Open(::FT_handleReceivedRtcpi)) < 0) {
    GUI::MessageWindow::Error("FtManager::Open - could not connect to smp\n");
    smp->Close(rtpiSocket);
    return false;
  }

  // setup RTP/I
  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("FtManager::OpenRTPI - joining %q - %u - %u\n", MLB::CommunicationManager::cm->ip.c_str(), 
		 MLB::CommunicationManager::cm->port+9, MLB::CommunicationManager::cm->ttl);
  #endif

  if (!smp->JoinGroup(rtpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+9, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("FtManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }
  if (!smp->JoinGroup(rtcpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+10, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("FtManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }

  rtpi = new RTPI::Rtpi();
  rtpi->join_group( MLB::CommunicationManager::cm->username, this, this, new RTPI::TransparentReliableTransport());
  
  openConnection = true;

  return true;
}


bool FT::FtManager::CloseRTPI() {
  if (!openConnection) {
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::CloseRTPI - session already closed\n");
    #endif
    return false;
  }

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("FtManager::CloseRTPI - leaving group\n");
  #endif
  
  rtpi->leave_group();

  MLB::CommunicationManager::cm->smp->LeaveGroup(rtpiSocket);
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtcpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtcpiSocket);

  delete rtpi;
  
  openConnection = false;
  init = true;

  Et_GlobalEvalF(Et_Interp, "Feedback::ClearAllQuestions");

  for (std::list<Question*>::iterator i = questions.begin(); i != questions.end(); ++i) delete *i;
  questions.clear();

  return true;
}


void FT::FtManager::HandleReceivedRtpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got data %u\n", ind.buf->getSize());
    #endif

    RefCount<RTPI::RtpiAduVector> rtpiAdus;
    try {
      rtpiAdus = rtpi->create_rtpi_adus(ind.buf);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    if (!rtpiAdus) 
      return; // adu not complete, received only fragments

    for (RTPI::RtpiAduVector::iterator i = rtpiAdus->begin(); i != rtpiAdus->end(); ++i) {
      if ((*i)->GetAduHeader()->GetPayloadType() != APPLICATION_TYPE) {
	GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - wrong payload type %u\n", 
		       (*i)->GetAduHeader()->GetPayloadType());
	continue;
      }

      switch ((*i)->GetType()) {
      case RTPI::RtpiHeader::PKT_EVENT: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & FT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("FtManager::ReceiveEvent - wrong version of feedback protocol\n");
	  continue;
	}

	int type = p8[0] & FT_MASK_TYPE;
	switch (type) {
	case CHANGE_VISIBILITY: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - change visibility of question\n");
          #endif
	  
	  Question* q = GetQuestion((*i)->GetSubcomponentID());
	  if (!q) {
	    GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	    break;
	  }
	  
	  q->visible = p8[1];
	  
	  if (q->visible) ChangeSubcomponentActiveness(q->id, true);
	  else ChangeSubcomponentActiveness(q->id, false);

	  // update gui
	  if (Et_GlobalEvalF(Et_Interp,"Feedback::SetVisibility %u %u", q->id, q->visible) != TCL_OK) 
	    GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);

	  break;
	}
	case DELETE_QUESTION: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - delete question\n");
          #endif

	  bool found = false;
	  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
	    if ((*q)->id == (*i)->GetSubcomponentID()) {
	      // update GUI
	      if (Et_GlobalEvalF(Et_Interp,"Feedback::RemoteDeleteQuestion %u", (*q)->id) != TCL_OK) 
		GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);

	      UnregisterSubcomponent((*i)->GetSubcomponentID());
      
	      delete *q;
	      questions.erase(q);
	      found = true;
	      break;
	    }
	  }
	  
	  // error
	  if (!found) GUI::MessageWindow::Error("FtManager::DeleteQuestion - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	  break;
	}
	case CREATE_ANSWER: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - received answer\n");
          #endif

	  Question* q = GetQuestion((*i)->GetSubcomponentID());
	  if (!q) {
	    GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	    continue;
	  }

	  bool found = false;
	  std::list<Answer>::iterator a;
	  for (a = q->answers.begin(); a != q->answers.end(); ++a) {
	    if ((*i)->GetParticipantID() == a->pid) {
	      a->value = p8[1];
	      a->createTime = (*i)->GetAduHeader()->GetTimestamp();
	      found = true;
	      break;
	    }
	  }

	  if (!found) {
	    if (a == q->answers.end()) {
	      // this is the first answer to this question
	      Answer answer;
	      answer.pid        = (*i)->GetParticipantID();
	      answer.value      = p8[1];
	      answer.createTime = (*i)->GetAduHeader()->GetTimestamp();
	      q->answers.push_back(answer);
	    }
	  }

	  // update gui
	  if (q->visible) q->DisplayResult();
	  break;
	}
	case DELETE_ANSWER: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - received delete answer\n");
          #endif

	  break;
	}
	}
	
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & FT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("FtManager::ReceiveState - wrong version of protocol\n");
	  continue;
	}

	Question* q = GetQuestion((*i)->GetSubcomponentID());
	if (q) {
	  //question already exists

	  if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_REPLY, (*i)->GetSubcomponentID())) {
	    // we have an own state reply timer -> delete 
	    MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_REPLY,(*i)->GetSubcomponentID());
	  } 

	  if (((RTPI::RtpiStateHeader*)(*i)->GetAduHeader())->GetPriority() == 3) {
	    // update question
	    q->ParseState((*i)->GetPayload(), (*i)->GetAduHeader()->GetTimestamp());
	    
	    if (q->visible) {
	      q->DisplayUpdate();
	      q->DisplayResult();
	    }
	    break;
	  }
	  // else ignore
	  break;
	}

	// new question
	if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID())) {
	  // we have an own state query timer -> delete 
	  MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID());
	} 

	q = new Question((*i)->GetSubcomponentID());
	q->ParseState((*i)->GetPayload(), (*i)->GetAduHeader()->GetTimestamp());
	questions.push_back(q);

	RegisterSubcomponent(q->id);

	if (q->visible) {
	  q->DisplayAdd();
	  q->DisplayResult();
	}
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE_QUERY: {
	#ifdef DEBUG
	GUI::MessageWindow::Info("FtManager::ReceiveStateQuery - got State Query\n");
	#endif

	if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID())) {
	  // we have an own state query timer -> delete 
	  MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID());
	} 

	Question* q = GetQuestion((*i)->GetSubcomponentID());
	if (!q) {
	  #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::ReceiveStateQuery - did not find question with id %u", (*i)->GetSubcomponentID());
	  #endif
	  continue;
	}

	double x = drand48();
	int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
	double N = (n == 1) ? 1 : n - 1;

	#ifdef DEBUG
	GUI::MessageWindow::Info("FtManager::ReceiveStateQuery - x: %f, N: %f\n", x, N);
	#endif

	if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
	  // send State immediately
  	  #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::ReceiveStateQuery - answer immediately\n");
	  #endif

	  int bytes = q->FlushState(sendBuffer);
	  SendState(q->id, 0, sendBuffer, bytes);

	} else {
	  // set state reply timer
	  int runningTime = (int)(T * (1 + log(x) / log(N)));
	  MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime , MLB::TimerObject::STATE_REPLY, 
								     (*i)->GetSubcomponentID(), 0));
  	  #ifdef DEBUG
	  GUI::MessageWindow::Info("FtManager::ReceiveStateQuery - answer later, set timer with running time %u\n", runningTime);
	  #endif
	}
	continue;
      }
      default:
	GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - wrong adu type %u\n", 
		       (*i)->GetType());
      }
    }

    try {
      rtpi->handle_received_rtpi(rtpiAdus, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    return;
  }

  
  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    return;
  }
  

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got LJNEEDED\n");
    #endif
    return;
														 

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("FtManager::HandleReceivedRtpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;


  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void FT::FtManager::HandleReceivedRtcpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got data %u\n", ind.buf->getSize());
    #endif
    return;
  }

  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    try {
      rtpi->handle_received_rtcpi(ind.buf, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("FtManager::HandleReceivedRtcpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }
    return;
  }

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got LJNEEDED\n");
    #endif
    return;

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("FtManager::HandleReceivedRtcpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;

  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("FtManager::HandleReceivedRtcpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void FT::FtManager::SendState(uint32 subID, int priority, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("FtManager::SendState - sending state\n");
  #endif

  RefCount<RTPI::RtpiAdu> state = new RTPI::RtpiState(subID, APPLICATION_TYPE, 0, rtpitime(), priority, payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(state);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("FtManager::SendState - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void FT::FtManager::SendEvent(uint32 subID, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("FtManager::SendEvent - sending event\n");
  #endif

  RefCount<RTPI::RtpiAdu> event = new RTPI::RtpiEvent(subID, APPLICATION_TYPE, 0, rtpitime(), payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(event);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("FtManager::SendEvent - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }
 
  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void FT::FtManager::SendStateQuery(uint32 subID, int priority) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("FtManager::SendStateQuery - sending state query\n");
  #endif

  RefCount<RTPI::RtpiAdu> query = new RTPI::RtpiStateQuery(subID, APPLICATION_TYPE, rtpitime(), priority);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(query);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("FtManager::SendStateQuery - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void FT::FtManager::CreateQuestion(int argc, const char** argv) {
  uint32 subid = MLB::CommunicationManager::cm->GetObjectID();
  
  Question* q = new Question(subid, 0, argv[1], atoi(argv[2]), argv[3]);;
  questions.push_back(q);
  
  RegisterSubcomponent(subid);

  int bytes = q->FlushState(sendBuffer);
  SendState(subid, 3, sendBuffer, bytes);

  q->DisplayAdd();
}


void FT::FtManager::EditQuestion(int argc,const  char** argv) {
  Question* q = GetQuestion(MLB::StringToUint32((char*)argv[1]));
  if (!q) {
    GUI::MessageWindow::Error("FtManager::EditQuestion - question with id %u does not exist\n", MLB::StringToUint32((char*)argv[1]));
    return;
  }
  
  q->Update(argv[2], atoi(argv[3]), argv[4]);

  int bytes = q->FlushState(sendBuffer);
  SendState(q->id, 3, sendBuffer, bytes);
}


void FT::FtManager::ChangeVisibility(uint32 id, bool visible) {
  Question* q = GetQuestion(id);
  if (!q) {
    GUI::MessageWindow::Error("FtManager::ChangeVisibility - question with id %u does not exist\n", id);
    return;
  }

  q->visible = visible;
  
  int bytes = q->FlushChangeVisibility(sendBuffer);
  SendEvent(q->id, sendBuffer, bytes);
  
  if (visible) ChangeSubcomponentActiveness(q->id, true);
  else ChangeSubcomponentActiveness(q->id, false);
}


void FT::FtManager::DeleteQuestion(uint32 id) {
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
    if ((*q)->id == id) {
      int bytes = (*q)->FlushDelete(sendBuffer);
      SendEvent(id, sendBuffer, bytes);
      
      delete *q;
      questions.erase(q);

      UnregisterSubcomponent(id);

      return;
    }
  }

  // error
  GUI::MessageWindow::Error("FtManager::DeleteQuestion - question with id %u does not exist\n", id);
}


void FT::FtManager::RebuildScores() {
  // compute feedback result for all questions and update gui
}


void FT::FtManager::ChangeAnswer(uint32 id, int value) {
  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;

  bool found = false;
  std::list<Answer>::iterator a;
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
    if ((*q)->id == id) {
      for (a = (*q)->answers.begin(); a != (*q)->answers.end(); ++a) {
	if (MLB::CommunicationManager::cm->pid == a->pid) {
	  a->value = value;
	  a->createTime = now;
	  found = true;
	  break;
	}
      }

      if (!found) {
	if (a == (*q)->answers.end()) {
	  // this is our first answer to this question
	  Answer answer;
	  answer.pid        = MLB::CommunicationManager::cm->pid;
	  answer.value      = value;
	  answer.createTime = now;
	  (*q)->answers.push_back(answer);
	  a = (*q)->answers.end();
	  --a;
	}
      }

      int bytes = a->FlushCreate(sendBuffer);
      SendEvent(id, sendBuffer, bytes);
      
      // update gui
      if ((*q)->visible) (*q)->DisplayResult();
      return;
    }
  }

  // error
  GUI::MessageWindow::Error("FtManager::ChangeAnswer - question with id %u does not exist\n", id);
}


void FT::FtManager::send_rtcpi(RefCount<Buffer> buf) {
  if (!openConnection) return;
  MLB::CommunicationManager::cm->smp->SendUnreliable(rtcpiSocket, buf->getCont(), buf->getSize());
}


void FT::FtManager::RegisterSubcomponent(uint32 subID) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("FtManager::RegisterSubcomponent - register sub with id %u\n", subID);
  #endif

  // register subcomponent
  rtpi->add_subcomponent(subID, "ftq", true);
}


void FT::FtManager::UnregisterSubcomponent(uint32 subID) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("FtManager::UnregisterSubcomponent - unregister sub with id %u\n", subID);
  #endif

  rtpi->delete_subcomponent(subID);
}


void  FT::FtManager::ChangeSubcomponentActiveness(uint32 subID, bool active) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("FtManager::ChangeSubcomponentActiveness - set sub with id %u active = %u\n", subID, active);
  #endif

  if (active) rtpi->activate_subcomponent(subID);
  else rtpi->deactivate_subcomponent(subID);
}


void FT::FtManager::subcomponent_changed(RTPI::SubComponentMapElement* subcomp, uint8 reason) {
}


void FT::FtManager::subcomponent_added(RTPI::SubComponentMapElement* subcomp, int reason) {
  Question* q = GetQuestion(subcomp->first);
  if (q) return; // question already there

  // set state query timer
  double x = drand48();
  int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
  double N = (n == 1) ? 1 : n - 1;
  
  #ifdef DEBUG
  GUI::MessageWindow::Info("FtManager::subcomponent_added - x: %f, N: %f\n", x, N);
  #endif
 
  if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
    // 1. send state query immediately
    SendStateQuery(subcomp->first, 2);

    #ifdef DEBUG
    GUI::MessageWindow::Info("FtManager::subcomponent_added - send state query immediately\n");
    #endif
  } else {
    // 2. set state query timer
    int runningTime = (int)(T * (1 + log(x) / log(N)));
    MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime , MLB::TimerObject::STATE_QUERY, subcomp->first, 2));

    #ifdef DEBUG
    GUI::MessageWindow::Info("FtManager::subcomponent_added - send state query later, set timer with running time %u\n", runningTime);
    #endif
  }
}


void FT::FtManager::subcomponent_removed(uint32 id) {
}


void FT::FtManager::participant_added(uint32 pid) {
}


void FT::FtManager::participant_removed(uint32 pid) {
  // delete all answers
}


void FT::FtManager::participant_changed(uint32 pid) {
}


void FT::FtManager::ExecuteTimer(MLB::TimerObject* timer) {
  switch (timer->type) {
  case MLB::TimerObject::STATE_QUERY: {
    SendStateQuery(timer->id, timer->priority);
    break;
  }
  case MLB::TimerObject::STATE_REPLY: {
    #ifdef DEBUG
    GUI::MessageWindow::Info("FtManager::ExecuteTimer - State Reply Timer ran out, sending state\n");
    #endif

    Question* q = GetQuestion(timer->id);
    if (!q) return; 
    int bytes = q->FlushState(sendBuffer);
    SendState(q->id, timer->priority, sendBuffer, bytes);

    break;
  }
  }

  delete timer;
}
   

FT::Question* FT::FtManager::GetQuestion(uint32 id) {
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q)
    if (id == (*q)->id) return *q;

  return NULL; // not found
}    


