/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : hrt_manager.cpp
 authors : Juergen Vogel
 date    : 18.02.2000
 desc    : 

 *********************************************************************************************************/


#ifdef WIN32
#include <winsock.h>
#else
#include <arpa/inet.h>
#endif

#include <math.h>

#include "SMP.h"
#include "general.h"

#include <rtpi/rtpiException.h>

#include "mlb.h"
#include "hrt_manager.h"
#include "mlb_communication.h"
#include "mlb_participants.h"
#include "mlb_timermanager.h"
#include "mlb_gui.h"

#define DEBUG_RTPI
//#define DEBUG_RTCPI
#define DEBUG

void HRT_handleReceivedRtpi(SMP::Indication ind) {
  if (HRT::HrtManager::hrtm) HRT::HrtManager::hrtm->HandleReceivedRtpi(ind);
}


void HRT_handleReceivedRtcpi(SMP::Indication ind) {
 if (HRT::HrtManager::hrtm)  HRT::HrtManager::hrtm->HandleReceivedRtcpi(ind);
}


HRT::HrtManager* HRT::HrtManager::hrtm = NULL;
char* HRT::HrtManager::sendBuffer = new char[1000];
const unsigned int HRT::HrtManager::T = 500;


HRT::HrtManager::HrtManager() : openConnection(false), indicate(true), init(true) {
}


HRT::HrtManager::~HrtManager() {
  CloseRTPI();
}


bool HRT::HrtManager::OpenRTPI() {
  if (openConnection || !MLB::CommunicationManager::cm->openSMP || !MLB::CommunicationManager::cm->openIDServer) {
    GUI::MessageWindow::Error("HrtManager::OpenRTPI - RTP/I Connection cannot be set up %u %u %u\n", 
			      openConnection, MLB::CommunicationManager::cm->openSMP, MLB::CommunicationManager::cm->openIDServer);
    return false;
  }

  SMP::SMP* smp = MLB::CommunicationManager::cm->smp;

  // setup SMP
  if ((rtpiSocket = smp->Open(::HRT_handleReceivedRtpi)) < 0) {
    GUI::MessageWindow::Error("HrtManager::Open - could not connect to smp\n");
    return false;
  }

  if ((rtcpiSocket = smp->Open(::HRT_handleReceivedRtcpi)) < 0) {
    GUI::MessageWindow::Error("HrtManager::Open - could not connect to smp\n");
    return false;
  }

  // setup RTP/I
  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("HrtManager::OpenRTPI - joining %q - %u - %u\n", MLB::CommunicationManager::cm->ip.c_str(), 
		 MLB::CommunicationManager::cm->port+5, MLB::CommunicationManager::cm->ttl);
  #endif

  if (!smp->JoinGroup(rtpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+5, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("HrtManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }
  if (!smp->JoinGroup(rtcpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+6, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("HrtManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }

  rtpi = new RTPI::Rtpi();
  rtpi->join_group( MLB::CommunicationManager::cm->username, this, this, new RTPI::TransparentReliableTransport());
  
  openConnection = true;

  // register subcomponent (there is only one!)
  rtpi->add_subcomponent(0, "participant list", true);

  // Initialize -> light-weight late join
  SendStateQuery(2);
  MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(2000, MLB::TimerObject::INIT));

  return true;
}


bool HRT::HrtManager::CloseRTPI() {
  if (!openConnection) {
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::CloseRTPI - session already closed\n");
    #endif
    return false;
  }

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("HrtManager::LeaveGroup - leaving group\n");
  #endif

  rtpi->leave_group();
  
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtpiSocket);
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtcpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtcpiSocket);

  delete rtpi;
  
  openConnection = false;
  init = true;

  return true;
}


void HRT::HrtManager::RaiseHand() {
  #ifdef DEBUG
  GUI::MessageWindow::Info("HrtManager::RaiseHand\n");
  #endif

  AddParticipant(MLB::CommunicationManager::cm->pid);
  DisplayParticipantList();

  int bytes = FlushRaiseHand(sendBuffer);
  SendEvent(sendBuffer, bytes);
}


void HRT::HrtManager::LowerHand() {
  #ifdef DEBUG
  GUI::MessageWindow::Info("HrtManager::LowerHand\n");
  #endif

  RemoveParticipant(MLB::CommunicationManager::cm->pid);
  DisplayParticipantList();

  int bytes = FlushLowerHand(sendBuffer);
  SendEvent(sendBuffer, bytes);

  return;
}


void HRT::HrtManager::CallParticipant(uint32 pid) {
  MLB::Participant* p = MLB::ParticipantManager::pm->GetParticipant(pid);
  if (!p) return;

  if (!p->raisedHand) 
    return; // has not raised his hand

  if (p->pid == MLB::CommunicationManager::cm->pid) {
    // call myself
    if (Et_GlobalEvalF(Et_Interp,"Participant::CallDialog yourself:-)") != TCL_OK)
      GUI::MessageWindow::Error("HrtManager::CallParticipant - tcl error %q\n", Et_Interp->result);

    #ifdef DEBUG
    GUI::MessageWindow::Info("HrtManager::CallParticipant - called myself\n");
    #endif
  }
 
  #ifdef DEBUG
  GUI::MessageWindow::Info("HrtManager::CallParticipant - called participant %u\n", pid);
  #endif

  int bytes = FlushCallParticipant(sendBuffer, pid);
  SendEvent(sendBuffer, bytes);

  p->raisedHand = false;

  DisplayParticipantList();
}


void HRT::HrtManager::HandleReceivedRtpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got data %u\n", ind.buf->getSize());
    #endif

    RefCount<RTPI::RtpiAduVector> rtpiAdus;
    try {
      rtpiAdus = rtpi->create_rtpi_adus(ind.buf);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    if (!rtpiAdus) 
      return; // adu not complete, received only fragments

    for (RTPI::RtpiAduVector::iterator i = rtpiAdus->begin(); i != rtpiAdus->end(); ++i) {
      if ((*i)->GetAduHeader()->GetPayloadType() != APPLICATION_TYPE) {
	GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - wrong payload type %u\n", 
		       (*i)->GetAduHeader()->GetPayloadType());
	continue;
      }

      switch ((*i)->GetType()) {
      case RTPI::RtpiHeader::PKT_EVENT: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & HRT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("HrtManager::ReceiveEvent - wrong version of chat protocol\n");
	  continue;
	}

	int type = p8[0] & HRT_MASK_TYPE;
	switch (type) {
	case RAISE_HAND: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - participant %u raised his hand\n", (*i)->GetParticipantID());
          #endif

	  AddParticipant((*i)->GetParticipantID());
	  DisplayParticipantList();
	  if (indicate) {
	    MLB::Participant* participant = MLB::ParticipantManager::pm->GetParticipant((*i)->GetParticipantID());
	    if (participant) {
	      if (!participant->name.empty()) {
		if (Et_GlobalEvalF(Et_Interp,"Participant::RaiseHandDialog \"%q\"", participant->name.c_str()) != TCL_OK)
		  GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);
	      } else {
		if (Et_GlobalEvalF(Et_Interp,"Participant::RaiseHandDialog \"%q\"", participant->cname.c_str()) != TCL_OK)
		  GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);
	      }
	    }
	  }
	  break;
	}
	case LOWER_HAND: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - participant %u lowered his hand\n", (*i)->GetParticipantID());
          #endif

	  RemoveParticipant((*i)->GetParticipantID());
	  DisplayParticipantList();
	  break;
	}
	case CALL_PARTICIPANT: {
	  uint32 called = ntohl(p32[1]);

          #ifdef DEBUG
	  GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - participant %u called %u\n", (*i)->GetParticipantID(), called);
          #endif

	  RemoveParticipant(called);
	  DisplayParticipantList();
	  if (called == MLB::CommunicationManager::cm->pid) {
	    // we have been called
	    MLB::Participant* caller = MLB::ParticipantManager::pm->GetParticipant((*i)->GetParticipantID());
	    if (caller) {
	      if (!caller->name.empty()) {
		if (Et_GlobalEvalF(Et_Interp,"Participant::CallDialog \"%q\"", caller->name.c_str()) != TCL_OK)
		  GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);
	      } else {
		if (Et_GlobalEvalF(Et_Interp,"Participant::CallDialog \"%q\"", caller->cname.c_str()) != TCL_OK)
		  GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);
	      }
	    }
	  }
	  break;
	}
	}
	
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & HRT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("HrtManager::ReceiveState - wrong version of protocol\n");
	  continue;
	}

	if (init) {
	  // we are late joiner -> init our participant list

	  for (std::vector<MLB::Participant*>::iterator i = MLB::ParticipantManager::pm->participants.begin(); 
	       i != MLB::ParticipantManager::pm->participants.end(); ++i) {
	    (*i)->raisedHand = false;
	  }

	  int nbr = ntohs(p16[1]);
	  for (int j = 1; j <= nbr; ++j) {
	    AddParticipant(ntohl(p32[j]));
	  }

	  init = false;
	  DisplayParticipantList();
	  
	  break;
	}

	if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_REPLY)) {
	  // we have an own state reply timer -> delete 
	  MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_REPLY);
	} 

	// we are no late joiner -> do nothing
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE_QUERY: {
	#ifdef DEBUG
	GUI::MessageWindow::Info("HrtManager::ReceiveStateQuery - got State Query\n");
	#endif

	if (init) break; // we don't know yet if we are late joiner or the session is empty

	double x = drand48();
	int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
	double N = (n == 1) ? 1 : n - 1;

	#ifdef DEBUG
	GUI::MessageWindow::Info("HrtManager::ReceiveStateQuery - x: %f, N: %f\n", x, N);
	#endif

	if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
	  // send State immediately
	  int bytes = FlushState(sendBuffer);
	  SendState(0, sendBuffer, bytes);

  	  #ifdef DEBUG
	  GUI::MessageWindow::Info("HrtManager::ReceiveStateQuery - answer immediately\n");
	  #endif
	} else {
	  // set state reply timer
	  int runningTime = (int)(T * (1 + log(x) / log(N)));
	  MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime , MLB::TimerObject::STATE_REPLY, 
								     (*i)->GetSubcomponentID(), 0));

  	  #ifdef DEBUG
	  GUI::MessageWindow::Info("HrtManager::ReceiveStateQuery - answer later, set timer with running time %u\n", runningTime);
	  #endif
	}
	break;
      }
      default:
	GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - wrong adu type %u\n", 
		       (*i)->GetType());
      }
    }

    try {
      rtpi->handle_received_rtpi(rtpiAdus, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    return;
  }

  
  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    return;
  }
  

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got LJNEEDED\n");
    #endif
    return;
														 

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("HrtManager::HandleReceivedRtpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;


  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void HRT::HrtManager::HandleReceivedRtcpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got data %u\n", ind.buf->getSize());
    #endif
    return;
  }

  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    try {
      rtpi->handle_received_rtcpi(ind.buf, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("HrtManager::HandleReceivedRtcpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }
    return;
  }

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got LJNEEDED\n");
    #endif
    return;

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("HrtManager::HandleReceivedRtcpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;

  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("HrtManager::HandleReceivedRtcpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void HRT::HrtManager::SendState(int priority, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("HrtManager::SendState - sending state\n");
  #endif

  RefCount<RTPI::RtpiAdu> state = new RTPI::RtpiState(SUB_ID, APPLICATION_TYPE, 0, rtpitime(), priority, payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(state);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("HrtManager::SendState - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void HRT::HrtManager::SendEvent(char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("HrtManager::SendEvent - sending event\n");
  #endif

  RefCount<RTPI::RtpiAdu> event = new RTPI::RtpiEvent(SUB_ID, APPLICATION_TYPE, 0, rtpitime(), payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(event);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("HrtManager::SendEvent - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }
 
  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void HRT::HrtManager::SendStateQuery(int priority) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("HrtManager::SendStateQuery - sending state query\n");
  #endif

  RefCount<RTPI::RtpiAdu> query = new RTPI::RtpiStateQuery(SUB_ID, APPLICATION_TYPE, rtpitime(), priority);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(query);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("HrtManager::SendStateQuery - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }
 
  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


int HRT::HrtManager::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (HrtManager::VERSION << 6);

  int nbr = 0;
  for (std::vector<MLB::Participant*>::iterator i = MLB::ParticipantManager::pm->participants.begin(); 
       i != MLB::ParticipantManager::pm->participants.end(); ++i) {
    if (!(*i)->raisedHand) continue;

    p32[nbr + 1] = htonl((*i)->pid);
    ++nbr;
  }

  p16[1] = htons(nbr);
    
  return 4 + 4*nbr;
}


int HRT::HrtManager::FlushRaiseHand(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (HrtManager::VERSION << 6);
  p8[0] = p8[0] | HrtManager::RAISE_HAND;

  return 4;
}


int HRT::HrtManager::FlushLowerHand(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (HrtManager::VERSION << 6);
  p8[0] = p8[0] | HrtManager::LOWER_HAND;

  return 4;
}


int HRT::HrtManager::FlushCallParticipant(char* buf, uint32 id) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (HrtManager::VERSION << 6);
  p8[0] = p8[0] | HrtManager::CALL_PARTICIPANT;

  p32[1] = htonl(id);

  return 8;
}


void HRT::HrtManager::send_rtcpi(RefCount<Buffer> buf) {
  if (!openConnection) return;
  MLB::CommunicationManager::cm->smp->SendUnreliable(rtcpiSocket, buf->getCont(), buf->getSize());
}


void HRT::HrtManager::subcomponent_changed(RTPI::SubComponentMapElement* subcomp, uint8 reason) {
}


void HRT::HrtManager::subcomponent_added(RTPI::SubComponentMapElement* subcomp, int reason) {
}


void HRT::HrtManager::subcomponent_removed(uint32 id) {
}


void HRT::HrtManager::participant_added(uint32 pid) {
}


void HRT::HrtManager::participant_removed(uint32 pid) {
}


void HRT::HrtManager::participant_changed(uint32 pid) {
}


void HRT::HrtManager::AddParticipant(uint32 pid) {
  MLB::Participant* p = MLB::ParticipantManager::pm->GetParticipant(pid);
  if (!p) {
    MLB::ParticipantManager::pm->AddParticipant(pid);
    p = MLB::ParticipantManager::pm->GetParticipant(pid);
  }
  
  p->raisedHand = true;
}


void HRT::HrtManager::RemoveParticipant(uint32 pid) {
  MLB::Participant* p = MLB::ParticipantManager::pm->GetParticipant(pid);
  if (!p) return; 
  
  p->raisedHand = false;
}


void HRT::HrtManager::DisplayParticipantList() {
  if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::HR_INDICATE))
    return; // timer is already running

  MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(1000, MLB::TimerObject::HR_INDICATE));
}


void HRT::HrtManager::ExecuteTimer(MLB::TimerObject* timer) {
  switch (timer->type) {
  case MLB::TimerObject::INIT: {
    init = false;

    if (Et_GlobalEvalF(Et_Interp,"Participant::HrEnableGUI") != TCL_OK)
      GUI::MessageWindow::Error("HrtManager::ExecuteTimer - tcl error %q\n", Et_Interp->result);

    #ifdef DEBUG
    GUI::MessageWindow::Info("HrtManager::ExecuteTimer - Init Timer ran out\n");
    #endif
    break;
  }
  case MLB::TimerObject::STATE_QUERY: {
    break;
  }
  case MLB::TimerObject::STATE_REPLY: {
    int bytes = FlushState(sendBuffer);
    SendState(timer->priority, sendBuffer, bytes);

    #ifdef DEBUG
    GUI::MessageWindow::Info("HrtManager::ExecuteTimer - State Reply Timer ran out, sending state\n");
    #endif
    break;
  }
  case MLB::TimerObject::HR_INDICATE: {
    static bool on = true;
    if (on) {
      on = false;
      if (ParticipantsIndicatorOn()) {
	MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(1000, MLB::TimerObject::HR_INDICATE));
      }
    } else {
      on = true;
      ParticipantsIndicatorOff();
      MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(1000, MLB::TimerObject::HR_INDICATE));
    }
    break;
  }
  }

  delete timer;
}

bool HRT::HrtManager::ParticipantsIndicatorOn() {
  bool one = false;
  for (std::vector<MLB::Participant*>::iterator i = MLB::ParticipantManager::pm->participants.begin(); 
       i != MLB::ParticipantManager::pm->participants.end(); ++i) {
    if ((*i)->raisedHand) {
      one = true;
      if (Et_GlobalEvalF(Et_Interp,"Participant::HrIndicatorOn %u", (*i)->pid) != TCL_OK) 
	GUI::MessageWindow::Error("HrtManager::ParticipantsIndicatorOn - tcl error %q\n", Et_Interp->result);
    } else {
      if ((*i)->status != MLB::Participant::EXITED) {
	if (Et_GlobalEvalF(Et_Interp,"Participant::HrIndicatorOff %u", (*i)->pid) != TCL_OK) 
	  GUI::MessageWindow::Error("HrtManager::ParticipantsIndicatorOff - tcl error %q\n", Et_Interp->result);    
      }
    }
  }

  if (one) {
    if (Et_GlobalEvalF(Et_Interp,"Participant::SetCallButtonState normal") != TCL_OK) 
      GUI::MessageWindow::Error("HrtManager::ParticipantsIndicatorOn - tcl error %q\n", Et_Interp->result);
  } else {
    if (Et_GlobalEvalF(Et_Interp,"Participant::SetCallButtonState disabled") != TCL_OK) 
      GUI::MessageWindow::Error("HrtManager::ParticipantsIndicatorOn - tcl error %q\n", Et_Interp->result);
  }

  return one;
}

void HRT::HrtManager::ParticipantsIndicatorOff() {
  for (std::vector<MLB::Participant*>::iterator i = MLB::ParticipantManager::pm->participants.begin(); 
       i != MLB::ParticipantManager::pm->participants.end(); ++i) {
    if ((*i)->status != MLB::Participant::EXITED) {
      if (Et_GlobalEvalF(Et_Interp,"Participant::HrIndicatorOff %u", (*i)->pid) != TCL_OK) 
	GUI::MessageWindow::Error("HrtManager::ParticipantsIndicatorOff - tcl error %q\n", Et_Interp->result);    
    }
  }
}
