/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifdef WIN32
#include <io.h>
#include <process.h>
#include <fcntl.h>
#include <windows.h>
#include <time.h>
#include <winsock.h>
#endif

#include "tcl.h"
#include "tk.h"

#ifdef WIN32
#define WM_WSOCK_READY 	WM_USER+123

extern HINSTANCE Tk_GetHINSTANCE();

#define MAX_FD 1024

static Tcl_FileProc *sockproc[MAX_FD];
static HWND sockwin;

static LRESULT CALLBACK
WSocketHandler(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  Tcl_FileProc *p;
  int fd;

  switch (message) {
  case WM_CREATE:{
    CREATESTRUCT *info = (CREATESTRUCT *) lParam;
    SetWindowLong(hwnd, GWL_USERDATA, (DWORD) info->lpCreateParams);
    return 0;
  }

  case WM_DESTROY:
    return 0;

  case WM_WSOCK_READY:
    fd = (UINT) wParam;
    if (fd >= 0 && fd < MAX_FD && (p = sockproc[fd]) != 0) {
      (*p) ((ClientData) fd, TK_READABLE);
      return 0;
    }
    break;
  }
  return DefWindowProc(hwnd, message, wParam, lParam);
}
#endif

void linksocket(int fd, int mask, Tcl_FileProc * callback)
{
#ifdef WIN32
  int status;
  int flags = 0;


  if (fd < 0 || fd >= MAX_FD || sockproc[fd])
    return;

  if (sockwin == 0) {
    /*
     * Register the Message window class.
     */
    WNDCLASS cl;

    cl.style = CS_HREDRAW | CS_VREDRAW;
    cl.lpfnWndProc = WSocketHandler;
    cl.cbClsExtra = 0;
    cl.cbWndExtra = 0;
    cl.hInstance = Tk_GetHINSTANCE();
    cl.hIcon = NULL;
    cl.hCursor = NULL;
    cl.hbrBackground = NULL;
    cl.lpszMenuName = NULL;
    cl.lpszClassName = "WSocket";
    RegisterClass(&cl);

    sockwin = CreateWindow("WSocket", "",
			   WS_POPUP | WS_CLIPCHILDREN,
			   CW_USEDEFAULT, CW_USEDEFAULT, 1, 1,
			   NULL,
			   NULL, Tk_GetHINSTANCE(), callback);
    ShowWindow(sockwin, SW_HIDE);
  }

  if (TK_READABLE & mask) 
    flags |= FD_READ;
  if (TK_WRITABLE & mask)
    flags |= FD_WRITE;
#ifdef WIN32
  if (FD_ACCEPT & mask)
    flags |= FD_ACCEPT;
  if (FD_CONNECT & mask)
    flags |= FD_CONNECT;
  if (FD_CLOSE & mask)
    flags |= FD_CLOSE;
#endif

  if ((status = WSAAsyncSelect(fd, sockwin, WM_WSOCK_READY, flags)) > 0) {
    fprintf(stderr, "WSAAsyncSelect: %d error %lu\n",
	    status, GetLastError());
    exit(1);
  }

  sockproc[fd] = callback;
#else
  Tcl_CreateFileHandler(fd, mask, callback, (ClientData)fd);
#endif
}

void unlinksocket(int fd)
{
#ifdef WIN32
  if (fd >= 0 && fd < MAX_FD && sockproc[fd] != 0) {
    sockproc[fd] = 0;
    (void) WSAAsyncSelect(fd, sockwin, 0, 0);
  }
#else
  Tcl_DeleteFileHandler(fd);
#endif
}

#ifdef WIN32
void clearReadSocketMessageQueue(int fd) {
  if (fd >= 0 && fd < MAX_FD && sockproc[fd] != 0) {
    WSAAsyncSelect(fd, sockwin, 0, 0);
    //printf("clearReadSocketMessageQueue - %d ok\n", fd);
  }
}

void restoreReadSocketMessageQueue(int fd) {
  int status;

  if (fd < 0 || fd >= MAX_FD || sockproc[fd] == 0)
    return;
	
  if ((status = WSAAsyncSelect(fd, sockwin, WM_WSOCK_READY, FD_READ)) > 0) {
    fprintf(stderr, "WSAAsyncSelect: %d error %lu\n", status, GetLastError());
    exit(1);
  }

  //printf("restoreReadSocketMessageQueue - %d ok\n", fd);
}

#endif
