/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_animations.h
 authors : Andrius Kurtinaitis
 date    : 07.08.2001
 desc    : Presentation animation classes.

 **********************************************************************************************************/

#ifndef __MLB_ANIMATIONS__
#define __MLB_ANIMATIONS__

#include <map>
#include <algorithm>

#include "mlb_objects.h"
#include "estring.h"
#include "mlb_animationstep.h"

namespace MLB
{
  class Page;

  /**    Class for animations on one page.
    *    Contains ordered list of animation groups.
    **/
  class Animations {

    public:
      // types
      typedef std::list<AnimationStep>              step_list;
      typedef std::list<AnimationStep>::iterator    step_iterator;
      typedef std::multimap<string, AnimationStep*> StringStepMap;

    private:

      // groups of animated objects
      step_list _steps;
      Page&     _page;

    public:
      /** constructs animations object for given page **/
      Animations( Page& page ) :_page( page ) {};

      /* Iterator functions */
             step_iterator GetStep   ( const string& name );
      inline step_iterator BeginStep () { return _steps.begin();  };
      inline step_iterator EndStep   () { return _steps.end  ();  };
      inline step_iterator AddNewStep() { return _steps.insert( EndStep(), AnimationStep(_page) ); }
      inline AnimationStep& GetLastStep() { return _steps.back(); }

      inline void RemoveStep  ( const string& stepName ) { _steps.erase( GetStep(stepName) ); }
             void RemoveObject( const string& stepName, const string& objectName );
             void MoveStep    ( const string& stepName, int position );

      /**   Updates the animation tree - removes all deleted objects. **/
      // inline void Update() { for( step_iterator i=_steps.begin(); i!=_steps.end(); i++ ) i->Update(); }
      // delete object from all steps
      void DeleteObject(DrawObject* obj);

      // Functions for getting animation tree structure
      string GetRunStepNames      ();
      string GetStepNames         ();
      string GetObjectNames       ( const string& stepName, bool show );
      string GetObjectTitle       ( const string& stepName, const string& objectName );
      string GetStepNameForObject ( const string& objName  );
      string GetStepTitle         ( const string& stepName );
      bool   IsStepOpen           ( const string& stepName );
      void   SetStepTitle         ( const string& stepName, const string& title  );
      void   SetStepOpen          ( const string& stepName, bool open );
      bool   HasObjectsIn         ( const DrawObject::object_list&  objs         );

      // Animation
      void Reset                  ();
      void RunFirstStep           ( bool run );
      void RunAllSteps            ( bool run );
      bool RunStep                ( const string& stepName, bool run );

      // Selection
      string  AddSelectedToAnimationStep( const std::string& groupName, bool show );
      void    SelectAnimationStepObjects( const std::string& groupName  );
      void    SelectAnimationObject     ( const std::string& tkRef      );

      // File I/O
      void          Save( FILE *_fp, int _level );
      StringStepMap Load( SGMLElement *_root    );

  }; // Animations

} // namespace MLB

#endif

//=========================================================================//

