/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_animationstep.h
 authors : Andrius Kurtinaitis
 date    : 17.09.2001
 desc    : Class definition corresponding one animation step in a presentation.

***********************************************************************************************************/

#ifndef __MLB_ANIMATION_STEP_
#define __MLB_ANIMATION_STEP_

#include <list>
#include <algorithm>

#include "mlb_animationstep.h"
#include "mlb_objects.h"
#include "estring.h"

namespace MLB
{
  class Page;

//-------------------------------------------------------------------------//

  /**    Class representing a group of objects participating in one step
    *    of presentation animation.
    **/
  class AnimationStep {

    private:
      Page&                   _page;
      DrawObject::object_list _showObjects;
      DrawObject::object_list _hideObjects;

      // properties
      static int _count;
      int        _number;
      bool       _run;
      bool       _open;
      string     _title;

    //---------------------------------------------------------------------//

    public:
      // construct
      AnimationStep( Page& page ) :_page( page ), _number( _count++ ), _run(false), _open(false), _title( "Step" ) {}

      // get/set
      inline int    GetNumber() { return _number; }
      inline string GetName  () { return "Step" + estring(_number);    }
      inline string GetTitle () { return _title;                       }
      inline bool   IsOpen   () { return _open && (!_showObjects.empty() || !_hideObjects.empty()); }
      inline void   SetTitle ( const string& title )  { _title = title; }
      inline void   SetOpen  ( const bool    open  )  { _open  = open;  }

      // animation
    private:
      void AppendVisibilityAdus ( RefCount<RTPI::RtpiAduVector>& adus, bool visible, bool direction );
      bool SetLocalObjectVisible( DrawObject*                    obj,  bool visible );

    public:
             bool   Reset    ();
             bool   Run      ( bool run );
      inline bool   IsRun    () { return _run; }

      // Iterator functions
             DrawObject::object_iterator GetObject  ( const string& name );
             DrawObject::object_iterator GetObject  ( const string& name, bool show );
      inline DrawObject::object_iterator BeginObject( bool show ) { if(show) return _showObjects.begin(); return _hideObjects.begin(); }
      inline DrawObject::object_iterator EndObject  ( bool show ) { if(show) return _showObjects.end();   return _hideObjects.end();   }

      inline bool   HasObject       ( DrawObject*   obj     )       { return std::find( _showObjects.begin(), _showObjects.end(), obj )!=_showObjects.end() || std::find( _hideObjects.begin(), _hideObjects.end(), obj )!=_hideObjects.end(); }
      inline void   AddObject       ( DrawObject*   obj, bool show) { if(show) _showObjects.push_back( obj ); else _hideObjects.push_back( obj ); } //obj->SetAnimationStepName( GetName() ); }
             void   RemoveObject    ( const string& objName );
             string GetObjectTitle  ( const string& objName );
      inline string GetObjectTitle  ( DrawObject*   obj     ) { return string( obj->GetTkRef() ); }
      inline string GetObjectName   ( DrawObject*   obj     ) { return string( obj->GetTkRef() ) + "-" + estring(_number); }
             string GetObjectNames  ( bool          show    );

      /**   Updates the animation group - removes all deleted objects. **/
      //void Update();
      void DeleteObject(DrawObject* obj);

      // File I/O
      void Save( FILE *_fp, int _level );

  }; // AnimationStep

} // namespace MLB

#endif

//=========================================================================//

