/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_char.h
 authors : Gabriela Maass, Juergen Vogel
 date    : 26.06.2000
 desc    :

 **********************************************************************************************************/

#ifndef __MLB_CHAR__
#define __MLB_CHAR__

#ifdef WIN32
#  pragma warning (disable: 4786)
#endif

#include <stdlib.h>
#include <stdio.h>
#include <list>
#include <vector>

#ifndef WIN32
#  include <unistd.h>
#else
#  include <windows.h>
#endif

#include "tk.h"
#include "tcl.h"

#include "mlb_generic.h"
#include "mlb_font.h"
#include "mlb_gui.h"

namespace MLB {

  class Char : public GenericObject {
  private:
    static uint32 count;

    TT_BBox bbox;
    int ascent;
    int descent;
    int lbearing;
    int size;
    int x, y;
    int width, height;
    std::string charTag;
    std::string color;
    bool drawed;
    MLB::Font* font;

  public:
    char ascii;

    Char(Font *font,char *_color,char _ascii);
    ~Char();

    virtual void Delete();

    void DeInit();
    void Init(Font *_font);
    void Draw(GUI::Canvas *_canvas, int _x, int _y);
    void ReDraw(GUI::Canvas *_canvas, int _x, int _y);
    void DrawTextCursor(GUI::Canvas *_canvas, uint32 id);
    void Hide(GUI::Canvas *_canvas);
    void Move(GUI::Canvas *_canvas, int _diffX, int _diffY, bool redraw = false);
    int GetWidth() {return width;}
    int GetHeight() {return height;}
    std::string GetColor() {return color;}
    char GetAscii() {return ascii;} 
    Font* GetFont() {return font;}
    int GetX() {return x;}
    int GetY() {return y;}
    int GetType() {return GenericObject::CHAR;};
    bool Overlap(GUI::Canvas *_canvas,int _x, int _y, bool *_smaller);
    bool IsEditablePoint(int _x, int _y, bool *_smaller);
    void ChangeFont(GUI::Canvas *_canvas, Font *_font);
    void ChangeColor(GUI::Canvas *_canvas,char *_color);
  };
}

#endif
