/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifdef WIN32
#  pragma warning (disable: 4786 4181)
#endif

#include "mlb_charfactory.h"


MLB::FontMap* MLB::CharFactory::fontMap = new MLB::FontMap();
int MLB::CharFactory::count = 1;


bool MLB::CharFactory::GetCharBitmap(Font* font, std::string color, char ascii, int* width, int* ascent, int* descent, int* lbearing, TT_BBox* bbox, std::string* charTag) {
  CharMap*  charMap;
  ColorMap* colorMap;
  
  // search for font
  FontMap::iterator fontIter = fontMap->find(font);
  if (fontIter != fontMap->end()) {

    // search for color
    colorMap = fontIter->second;
    ColorMap::iterator colorIter = colorMap->find(color);
    if (colorIter != colorMap->end()) {
    
      // search for char
      charMap = colorIter->second;
      CharMap::iterator charIter = charMap->find(ascii);
      if (charIter != charMap->end()) {
    
        // return character
        *charTag  = charIter->second->charTag;
        *width    = charIter->second->width;
        *ascent   = charIter->second->ascent;
        *descent  = charIter->second->descent;
        *lbearing = charIter->second->lbearing;
        ++charIter->second->refs;
        return true;
      }
    } else {
      
      // insert new charMap
      charMap = new CharMap();
      colorMap->insert(ColorMap::value_type(color, charMap));
    }
  } else {
    
    // insert new colorMap and charMap
    charMap  = new CharMap();
    colorMap = new ColorMap();
    colorMap->insert(ColorMap::value_type(color, charMap));
    fontMap->insert(FontMap::value_type(font, colorMap));
  }
  
  // insert new charObject
  std::string block  = font->GetCharBitmap(ascii, width, ascent, descent, lbearing, bbox);
    
  char buf[100];
  sprintf(buf, "%u", count++);
  *charTag  = "charImg";
  *charTag += buf;

  charMap->insert(CharMap::value_type(ascii, new CharObject(*charTag, *width, *ascent, *descent, *lbearing)));

  Magick::Blob blob;
  try {
    blob.update((char*)block.c_str(), block.length());
    PutTkBlock(*charTag, blob, color);
  } catch (Magick::Exception& exc)    {
    GUI::MessageWindow::Error("Char::Init - Cannot build blob. %q", exc.what() );
    return false;
  }

  return true;
}



bool MLB::CharFactory::DeleteCharBitmap(Font* font, std::string color, char ascii) {
  CharMap*  charMap;
  ColorMap* colorMap;

  // search for font
  FontMap::iterator fontIter = fontMap->find(font);
  if (fontIter != fontMap->end()) {
    
    // search for color
    colorMap = fontIter->second;
    ColorMap::iterator colorIter = colorMap->find(color);
    if (colorIter != colorMap->end()) {
    
      // search for char
      charMap = colorIter->second;
      CharMap::iterator charIter = charMap->find(ascii);
      if (charIter != charMap->end()) {
    
        CharObject* charObj = charIter->second;
        if (--(charObj->refs) == 0) {
        
          // delete charObj
          RemoveTkBlock(charObj->charTag);
          delete charObj;
          charMap->erase(charIter);
          if (charMap->empty()) {
            delete charMap;
            colorMap->erase(colorIter);
            if (colorMap->empty()) {
              delete colorMap;
              fontMap->erase(fontIter);
            }
          }
          return true;
        }
      }
    }
  }
  return false;
}
    
    
  
void MLB::CharFactory::PutTkBlock(std::string charTag, Magick::Blob blob, std::string color) {
  // image
  Magick::Image image(blob);

  // set color
  image.transparent(Magick::ColorRGB("#FFFFFF"));
  image.opaque(Magick::ColorRGB("#000000"), Magick::ColorRGB(color.c_str()));
  //image.transparent(Magick::ColorRGB("#000000"));
  //image.opaque(Magick::ColorRGB("#FFFFFF"), Magick::ColorRGB(color.c_str()));

  // resize
  //Magick::Geometry newSize((SessionManager::sm->GetZoom() * image.columns() / Text::RESOLUTION), 
  //(SessionManager::sm->GetZoom() * image.rows() / Text::RESOLUTION));
  //image.scale(newSize);

  // put
  TkImageBlock block(image);
  block.putBlock(charTag);
}


void MLB::CharFactory::RemoveTkBlock(std::string charTag) {
  if(!charTag.empty()) GUI::Image::Destroy(charTag.c_str()); 
} 



