/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifndef __MLB_CHAR_FACTORY__
#define __MLB_CHAR_FACTORY__

#include <map>

#include "tk.h"
#include "tcl.h"

#include "mlb_generic.h"
#include "mlb_font.h"
#include "mlb_gui.h"
#include "mlb_tkphoto.h"


#ifdef WIN32
#  pragma warning (disable: 4786 4181)
#endif

namespace MLB {

  struct CharObject {
    CharObject(std::string _charTag, int _width, int _ascent, int _descent, int _lbearing) 
      : charTag(_charTag), width(_width), ascent(_ascent), descent(_descent), lbearing(_lbearing), refs(1) {}
    std::string charTag;    
    int refs, width, ascent, descent, lbearing;
  };

  
  typedef std::map<char, CharObject*> CharMap;
  typedef std::map<std::string, CharMap*>  ColorMap;
  typedef std::map<Font*, ColorMap*>  FontMap;
  
  
  class CharFactory {
  public:
    // true: no error, false: else
    static bool GetCharBitmap(Font* font, std::string color, char ascii, int* width, int* ascent, int* descent, int* lbearing, TT_BBox* bbox, std::string* charTag);
    
    // true: char was deleted, false: else
    static bool DeleteCharBitmap(Font* font, std::string color, char ascii);
    
  private:
    CharFactory() {}
    ~CharFactory() {}
    
    static void PutTkBlock(std::string charTag, Magick::Blob blob, std::string color);
    static void RemoveTkBlock(std::string charTag);

    static int count;
    static FontMap* fontMap;
  };
}

#endif
