/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_folder.cpp 
 authors : Juergen Vogel, Gabriela Maass
 date    : 18.02.2000
 desc    : 

***********************************************************************************************************/

#ifdef WIN32
#pragma warning(disable: 4786)
#endif

#ifdef WIN32
#  include <strstrea.h>
#endif


#include "mlb.h"
#include "mlb_folder.h"
#include "mlb_page.h"
#include "mlb_session.h"
#include "mlb_communication.h"
#include "mlb_adu.h"
#include "mlb_gui.h"
#include "mlb_char.h"
#include "mlb_charfactory.h"


MLB::Folder::Folder(uint32 _id, char *_name) 
  : GenericContainer(_id) {
  currentPage = NULL;
  SetName(_name);
}


MLB::Folder::Folder(uint32 _id, SGMLElement *root) 
  : GenericContainer(_id) {
  MLB::Folder
    *folder;
  MLB::Page
    *page;
  SGMLElement 
    *hilf;

  currentPage = NULL;

  if (!root) return;

  SetName(root->GetAttribute("name"));

  if (root->Name == "THUMBNAILS") {
    char* tmp;
    tmp = root->GetAttribute("firstX");
    if (tmp) thumbnailFirstX = atoi(tmp); else return;
    tmp = root->GetAttribute("firstY");
    if (tmp) thumbnailFirstY  = atoi(tmp); else return;
    tmp = root->GetAttribute("incrX");
    if (tmp) thumbnailIncrX  = atoi(tmp); else return;
    tmp = root->GetAttribute("incrY");
    if (tmp) thumbnailIncrY   = atoi(tmp); else return;
    tmp = root->GetAttribute("nbrX");
    if (tmp) thumbnailNbrX   = atoi(tmp); else return;
    tmp = root->GetAttribute("nbrY");
    if (tmp) thumbnailNbrY = atoi(tmp); else return;
  }

  hilf = root->son;
  while (hilf) {
    if (hilf->Name == "FOLDER") {
      folder = new MLB::Folder(CommunicationManager::cm->GetObjectID(),hilf);
      InsertContainer(folder);
    }
    else if (hilf->Name == "PAGE") {
      page = new MLB::Page(CommunicationManager::cm->GetObjectID(), Session::PRIVATE, hilf);
      if (!currentPage) currentPage = page;
      if (!SessionManager::sm->GetActivePage()) SessionManager::sm->SetActivePage(page);
      InsertContainer(page);
    }
    else {
      //cerr << "Folder::Folder - Error: unknown container type " << hilf->Name.c_str() << endl;
    }
    hilf = hilf->next;
  }

  if (!SessionManager::sm->GetActiveFolder()) SessionManager::sm->SetActiveFolder(this);
}


void MLB::Folder::InitTelepointers() {
  for (std::list<MLB::GenericContainer *>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
    if ((*i)->GetType() == GenericObject::PAGE) {
      ((Page*)*i)->InitTelepointer();
    } else {
      ((Folder*)*i)->InitTelepointers();
    }
  }
}


MLB::Folder::~Folder() {
  DeleteAll(NULL);
}

void MLB::Folder::SetName(char *_name) {
  containerName.erase();

  if (!_name) 
    containerName = "Chapter";
  else 
    containerName = _name;
}


MLB::GenericContainer* MLB::Folder::SetCurrentContainer(std::vector<uint32>* idVector, int pos) {

  SessionManager::sm->SetActiveFolder(this);
  OpenFolder();

  GenericContainer* obj = NULL;
  if (!subContainer.empty() && !idVector->empty() && (pos<idVector->size())) {
    uint32 id = (*idVector)[pos];

    for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
      if ((*i)->GetID() == id) {
	current = id;
	obj = (*i)->SetCurrentContainer(idVector, pos+1);
	if (obj && (obj->GetType() == GenericObject::PAGE)) currentPage = (Page*)obj;
	break;
      }
    }
  } else {
    if (!subContainer.empty()) {
      if (currentPage) {
	currentPage->Activate();
      } else {
	obj = GetFirstPage();
	if (obj) ((Page*)obj)->Activate();
      }
    } 
  }

  return this;
}


std::list<uint32>* MLB::Folder::GetObjectIDList() {
  std::list<uint32>* objList = new std::list<uint32>;
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) 
    objList->push_back((*i)->GetID());
  return objList;
}


MLB::GenericContainer* MLB::Folder::GetNextContainer(MLB::GenericContainer* obj) {
  if (subContainer.size() == 1)
    return this; // there is only one element in this container; therefore select the container itself

  std::list<GenericContainer*>::iterator i = subContainer.begin();
  while (i != subContainer.end()) {
    if ((*i)->GetID() == obj->GetID()) break;
    ++i;
  }
  if (i == subContainer.end()) return NULL;

  std::list<GenericContainer*>::iterator next = i;
  ++next;
  
  if (next == subContainer.end()) {
    --next; // = i
    --next; // previous element
  } 

  return *next;
}


void MLB::Folder::CreateThumbnails(Folder* folder, int &count, int &x, int &y, bool flat) {
  Page* page;

  Page* firstPage = (Page*)SessionManager::sm->GetParentSession(this)->GetFirstPage();
  if (!firstPage) return;

  int glueX   = (firstPage->GetWidth() <= firstPage->GetHeight()) ?  20 :  35;
  int glueY   = (firstPage->GetWidth() <= firstPage->GetHeight()) ?  35 :  20;
  int nbrX    = (firstPage->GetWidth() <= firstPage->GetHeight()) ?   4 :   5;
  int nbrY    = (firstPage->GetWidth() <= firstPage->GetHeight()) ?   5 :   4;
  int pWidth  = (firstPage->GetWidth() <= firstPage->GetHeight()) ? 594 : 842;
  int pHeight = (firstPage->GetWidth() <= firstPage->GetHeight()) ? 842 : 594;
  int box     = 130;
  
  int firstX, firstY, incX, incY, width, height;
  double scaleX, scaleY;
  
  if (flat) {
    if (folder->subContainer.empty()) {
      count = 0;
    } else {
      page = (Page*)folder->subContainer.back();
    }
  } 
      
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
    if ((*i)->GetType() == GenericObject::PAGE) {
      if (count++ == 0) {
        page = new MLB::Page(CommunicationManager::cm->GetObjectID(), Session::PRIVATE, "View", pWidth, pHeight);
        folder->InsertContainerEnd(page);

        x = firstX = (pWidth  - nbrX*box - (nbrX-1)*glueX) / 2;
        y = firstY = (pHeight - nbrY*box - (nbrY-1)*glueY) / 2;
  
        DrawObject* marker = new Rectangle(firstX-2, firstY-2, box+4, box+4, "#0000FF", "none" , 3, " ", CommunicationManager::cm->GetObjectID());
        marker->SetOrder(page->GetNewObjectOrder(0)); 
        page->AppendObject(marker); 
      }
      
      int _width  = ((Page*)*i)->GetWidth();
      int _height = ((Page*)*i)->GetHeight();
      if (_width < _height) {
        width  = (int)(_width * (double)box / _height);
        height = box;
        incX   = (box-width) / 2;
        incY   = 0;
      } else {
        height = (int)(_height * (double)box / _width);
        width  = box;
        incX   = 0;
        incY   = (box-height) / 2;
      }
      scaleX = (double)width  / _width;
      scaleY = (double)height / _height;
      
      // thumbnail box
      Rectangle* b1 = new Rectangle(x-1, y-1, box+2, box+2, "#606060", "none" , 1, " ", CommunicationManager::cm->GetObjectID());
      b1->SetOrder(page->GetNewObjectOrder(1)); 
      b1->SetThumbnailID(((Page*)*i)->GetPageNumber());
      SessionManager::sm->GetParentSession(this)->thumbnailRefs[b1->GetID()] = (*i)->GetID();
      page->AppendObject(b1); 
      
      // page label
      string name = ((Page*)*i)->GetName();
      name.resize(13);
      Group* group = new Group(CommunicationManager::cm->GetObjectID());
      Text* text = new Text(0,0,0,0, "#000000", (char*)name.c_str(), "arial", "12", "normal", "roman", "normal", CommunicationManager::cm->GetObjectID());
      text->SetX(x+incX+(width-text->GetWidth()) / 2);
      text->SetY(y+box+15);
      text->SetOrder(page->GetPushBackObjectOrder()); 
      page->AppendObject(text);
      group->AddObject(text);
      group->SetOrder(page->GetPushBackObjectOrder()); 
      page->AppendObject(group);

      // copy objects
      DrawObject::object_list objList = ((Page*)*i)->GetObjects();
      DrawObject* obj;
      for (DrawObject::object_list::iterator j = objList.begin(); j != objList.end(); ++j) {
        if ((*j)->GetType() != MLB::GenericObject::ASCIITEXT) {
          obj = (*j)->Copy();
          switch (obj->GetType()) {
          case MLB::GenericObject::POLYGON: {
            for (std::list<Point*>::iterator k = ((PolyObject*)obj)->points.begin(); k != ((PolyObject*)obj)->points.end(); ++k) {
              (*k)->x = x + incX + (int)((*k)->x * scaleX);
              (*k)->y = y + incY + (int)((*k)->y * scaleY);
            }
            int lineWidth = (int)(((ColoredObject*)obj)->GetLineWidth() * (scaleX < scaleY) ? scaleX : scaleY);
            ((ColoredObject*)obj)->SetLineWidth((lineWidth < 1) ? 1 : lineWidth);
            break;                                 
          }
          case MLB::GenericObject::FREEHAND:
          case MLB::GenericObject::POLYLINE: {
            for (std::list<Point*>::iterator k = ((PolyObject*)obj)->points.begin(); k != ((PolyObject*)obj)->points.end(); ++k) {
              (*k)->x = x + incX + (int)((*k)->x * scaleX);
              (*k)->y = y + incY + (int)((*k)->y * scaleY);
            }
            int lineWidth = (int)(((ColoredObject*)obj)->GetLineWidth() * (scaleX < scaleY) ? scaleX : scaleY);
            ((ColoredObject*)obj)->SetLineWidth((lineWidth < 1) ? 1 : lineWidth);

            static char as[12];
            unsigned a1, a2, a3;
            sscanf(((PolyLine*)obj)->GetArrowShape().c_str(), "%u %u %u", &a1, &a2, &a3);
            a1 *= (scaleX < scaleY) ? scaleX : scaleY;
            a2 *= (scaleX < scaleY) ? scaleX : scaleY;
            a3 *= (scaleX < scaleY) ? scaleX : scaleY;
            sprintf(as, "%u %u %u", (a1 < 0)? 1:a1, (a2 < 0)? 1:a2, (a3 < 0)? 1:a3);
            ((PolyLine*)obj)->SetArrowShape(as);
            break;                                 
          }
          case MLB::GenericObject::LINE: {
            int _x = x + incX + (int)((*j)->GetX() * scaleX);
            int _y = y + incY + (int)((*j)->GetY() * scaleY);
            int _w = (int)((*j)->GetWidth() * scaleX);
            int _h = (int)((*j)->GetHeight() * scaleY);
            obj->ModifyCoords(_x, _y, _w, _h);

            int lineWidth = (int)(((ColoredObject*)obj)->GetLineWidth() * (scaleX < scaleY) ? scaleX : scaleY);
            ((ColoredObject*)obj)->SetLineWidth((lineWidth < 1) ? 1 : lineWidth);

            static char as[12];
            unsigned a1, a2, a3;
            sscanf(((Line*)obj)->GetArrowShape().c_str(), "%u %u %u", &a1, &a2, &a3);
            a1 *= (scaleX < scaleY) ? scaleX : scaleY;
            a2 *= (scaleX < scaleY) ? scaleX : scaleY;
            a3 *= (scaleX < scaleY) ? scaleX : scaleY;
            sprintf(as, "%u %u %u", (a1 < 0)? 1:a1, (a2 < 0)? 1:a2, (a3 < 0)? 1:a3);
            ((Line*)obj)->SetArrowShape(as);
            break;
          }
          case MLB::GenericObject::RECTANGLE:
      	  case MLB::GenericObject::OVAL: {
            int _x = x + incX + (int)(obj->GetX() * scaleX);
            int _y = y + incY + (int)(obj->GetY() * scaleY);
            int _w = (int)(obj->GetWidth() * scaleX);
            int _h = (int)(obj->GetHeight() * scaleY);
            obj->ModifyCoords(_x, _y, _w, _h);

            int lineWidth = (int)(((ColoredObject*)obj)->GetLineWidth() * (scaleX < scaleY) ? scaleX : scaleY);
            ((ColoredObject*)obj)->SetLineWidth((lineWidth < 1) ? 1 : lineWidth);
            break;
          }
          case MLB::GenericObject::IMAGE: { 
            int _x = x + incX + (int)(obj->GetX() * scaleX);
            int _y = y + incY + (int)(obj->GetY() * scaleY);
            int _w = (int)(obj->GetWidth() * scaleX);
            int _h = (int)(obj->GetHeight() * scaleY);
            obj->ModifyCoords(_x, _y, _w, _h);
            break;
          }
          }  
          obj->SetOrder(page->GetPushBackObjectOrder()); 
          page->AppendObject(obj);
        }
      }  
           
      // copy text
      DrawObject::group_list groupList = ((Page*)*i)->GetGroups();
      for (DrawObject::group_list::iterator g = groupList.begin(); g != groupList.end(); ++g) {
        if ((*g)->GetType() == MLB::GenericObject::GROUP) {
          if (((Group*)*g)->IsTextGroup()) {
            obj = ((Group*)*g)->Copy(page);
            
            std::list<DrawObject*> textList = ((Group*)obj)->GetGroupObjects();
            for (std::list<DrawObject*>::iterator k = textList.begin(); k != textList.end(); ++k) {
	            ((Text*)*k)->InitFont((scaleX < scaleY) ? scaleX : scaleY);
              ((Text*)*k)->Update();
	      
              std::list<Char*> charList = ((Text*)*k)->GetText();
	            for (std::list<Char*>::iterator l = charList.begin(); l != charList.end(); ++l) {
                CharFactory::DeleteCharBitmap((*l)->GetFont(), (*l)->GetColor(), (*l)->GetAscii());
                (*l)->Init(((Text*)*k)->GetFont());
              }
            }
            
            int _x = x + incX + (int)(obj->GetX() * scaleX);
            int _y = y + incY + (int)(obj->GetY() * scaleY);
            ((Group*)obj)->Move(page->GetCanvas(), _x - ((Group*)obj)->GetX(), _y - ((Group*)obj)->GetY());
	          ((Group*)obj)->UpdateDimension();
	          obj->SetOrder(page->GetPushBackObjectOrder()); 
            page->AppendObject(obj);
          }
        }	  
      }
            
      // page box
      Rectangle* b2 = new Rectangle(x+incX, y+incY, width, height, "#C0C0C0", "none" , 1, " ", CommunicationManager::cm->GetObjectID());
      b2->SetOrder(page->GetPushBackObjectOrder()); 
      page->AppendObject(b2); 

       // next thumbnail
      x += (box + glueX);
      if ((count % nbrX) == 0) {
        x = (page->GetWidth() - nbrX*box - (nbrX-1)*glueX) / 2;
        y += (box + glueY);
      }
      if (count == (nbrX*nbrY)) {
        count = 0;
      }
      
      SessionManager::sm->progress->IncrementProgress();

    } else {
      if (*i != folder) {
        if (flat) {
         ((Folder*)*i)->CreateThumbnails(folder, count, x, y, flat);
        } else {
          int _count = 0, _x = 0, _y = 0;
          Folder* f = new Folder(CommunicationManager::cm->GetObjectID(), (*i)->GetName());      
          folder->InsertContainerEnd(f);
          ((Folder*)*i)->CreateThumbnails(f, _count, _x, _y, flat);
        }
      }
    }
  }
  
  if (folder == SessionManager::sm->GetParentSession(this)->thumbnailFolder) {
    folder->thumbnailFirstX = firstX-2;
    folder->thumbnailFirstY = firstY-2;
    folder->thumbnailIncrX  = box+glueX;
    folder->thumbnailIncrY  = box+glueY;
    folder->thumbnailNbrX   = nbrX;
    folder->thumbnailNbrY   = nbrY;
  }
}


void MLB::Folder::DeleteAll(GUI::Canvas* _canvas) {
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
    (*i)->DeleteAll(NULL);
    delete *i;
  }

  subContainer.clear();
  current = 0;
  currentPage = NULL;
} 


void MLB::Folder::DeleteObjectsFromRTPI() {
  CommunicationManager::cm->UnregisterSubcomponent(GetID());
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) 
    (*i)->DeleteObjectsFromRTPI();
}

void MLB::Folder::DrawPageTree(string _nodeNumber, string _nodePath, string parentNode, int pos, bool *_open) {
  char buf[200];
  sprintf(buf,".%u",id);
  nodePath = _nodePath + buf;

  if (nodeName.empty()) {
    sprintf(buf,"n:%u",id);
    nodeName = buf;
    bool thumbnailFolder = (SessionManager::sm->GetParentSession(this)->thumbnailFolder == this);
    GUI::PageView::InsertFolderNode(nodeName.c_str(),containerName.c_str(),parentNode.c_str(),nodePath.c_str(),pos,thumbnailFolder);
  }
  else {
    GUI::PageView::DrawFolderNode((char *)nodeName.c_str(), (char *)containerName.c_str(),(char *)nodePath.c_str());
  }

  if (this == SessionManager::sm->GetActiveFolder())
    GUI::PageView::SelectFolderNode(nodeName.c_str());

  if (!subContainer.empty()) {
    std::string nodeNumber;
    uint32 count = 1;
    for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
      sprintf (buf,".%u",count);
      nodeNumber = _nodeNumber + buf;
      (*i)->DrawPageTree(nodeNumber, nodePath, nodeName, count-1, _open);
      ++count;
    }
  }

  if (this == MLB::SessionManager::sm->GetActiveFolder())  *_open = true;
  if (*_open) OpenFolder();
}


void MLB::Folder::OpenFolder() {
  GUI::PageView::OpenFolderNode(nodeName.c_str());
}


void MLB::Folder::ClearPageTree() {
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) 
    (*i)->ClearPageTree();

  GUI::PageView::Delete(GetNodeName().c_str());

  ClearNodeName();
}


int MLB::Folder::GetNbrOfPages() {
  int count = 0;
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i)
    if ((*i)->GetType() == GenericObject::PAGE) ++count;
    else count += (*i)->GetNbrOfPages();
  
  return count;
}


void  MLB::Folder::SaveContainer(FILE *fp, int level)
{
#ifdef _DEBUG1
  cout << "MLB::Folder::SaveContainer" << endl;
#endif

  std::list<GenericContainer *>::iterator 
    temp;
  std::string 
    tab;
  int
    i;

  for (i=0; i<level; i++) 
    tab += "  ";

  if (SessionManager::sm->GetParentSession(this)->thumbnailFolder == this) {
    fprintf(fp,"%s<THUMBNAILS name=\"%s\" firstX=\"%d\" firstY=\"%d\" incrX=\"%d\" incrY=\"%d\" nbrX=\"%d\" nbrY=\"%d\">\n",
            tab.c_str(),containerName.c_str(), thumbnailFirstX, thumbnailFirstY, thumbnailIncrX, thumbnailIncrY, thumbnailNbrX, thumbnailNbrY);
  } else {
    fprintf(fp,"%s<FOLDER name=\"%s\">\n",tab.c_str(),containerName.c_str());
  }
  
  if (!subContainer.empty()) {
    temp = subContainer.begin();
    while (temp!=subContainer.end()) {
      (*temp)->SaveContainer(fp,(level+1));
      temp++;
    }
  }

  if (SessionManager::sm->GetParentSession(this)->thumbnailFolder == this) {
    fprintf(fp,"%s</THUMBNAILS>\n",tab.c_str());
  } else {
    fprintf(fp,"%s</FOLDER>\n",tab.c_str());
  }

  tab.erase();
}

void MLB::Folder::SaveAsPs(FILE* fp) {
  #ifdef _DEBUG1
  cout << "MLB::Folder::SaveAsPs" << endl;
  #endif

  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) 
    (*i)->SaveAsPs(fp);
}


void MLB::Folder::SaveAsHtml(char* directory, char* docname, FILE* indexframe, int& nbr, int last) {
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) 
    (*i)->SaveAsHtml(directory, docname, indexframe, nbr, last);
}


void MLB::Folder::SaveAsImage(char* name, char* type, int& nbr) {
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) 
    (*i)->SaveAsImage(name, type, nbr);
}


void MLB::Folder::ReplaceIDs() {
  uint32 newID;
  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
    newID = CommunicationManager::cm->GetObjectID();
    if ((*i)->GetID() == current) current = newID;
    (*i)->SetID(newID);
    (*i)->ReplaceIDs();
  }
}


void MLB::Folder::SendAllObjects(uint32 timestamp, int priority) {
  int bytes;
  RefCount<Buffer> buf;
  RefCount<RTPI::RtpiAdu> adu;

  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
    bytes = (*i)->GetStateSize();
    buf   = new Buffer(bytes);

    switch ((*i)->GetType()) {
    case GenericObject::FOLDER: {
      ((Folder*)*i)->FlushState(buf->getCont());
      adu = CommunicationManager::cm->CreateState((*i)->GetID(), GetID(), CommunicationManager::FOLDER, priority, buf, timestamp);
      TimeWarp::tw->InsertAdu(adu);
      CommunicationManager::cm->SendAduReliable(adu);
       
      ((Folder*)*i)->SendAllObjects(timestamp, priority);
      break;
    }
    case GenericObject::PAGE: {
      ((Page*)*i)->FlushState(buf->getCont());
      adu = CommunicationManager::cm->CreateState((*i)->GetID(), GetID(), CommunicationManager::PAGE, priority, buf, timestamp);
      TimeWarp::tw->InsertAdu(adu);
      CommunicationManager::cm->SendAduReliable(adu);

      ((Page*)*i)->SendAllObjects(timestamp, priority);
      break;
    }
    }
  }
}


RefCount<RTPI::RtpiAdu> MLB::Folder::GetStateAdu(uint32 timestamp, int priority) {
  GenericContainer* parent = SessionManager::sm->GetParentContainer(GetID());
  if (!parent) return NULL;
  int bytes = GetStateSize();
  RefCount<Buffer> buf = new Buffer(bytes);
  FlushState(buf->getCont());
  return CommunicationManager::cm->CreateState(GetID(), parent->GetID(), CommunicationManager::FOLDER, priority, buf, timestamp);
}


void MLB::Folder::StoreAllObjectsInTWQ(uint32 timestamp) {
  RefCount<Buffer> buf;
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;

  for (std::list<GenericContainer*>::iterator i = subContainer.begin(); i != subContainer.end(); ++i) {
    buf   = new Buffer((*i)->GetStateSize());

    switch ((*i)->GetType()) {
    case GenericObject::FOLDER: {
      ((Folder*)*i)->FlushState(buf->getCont());
      adus->push_back(CommunicationManager::cm->CreateState((*i)->GetID(), GetID(), CommunicationManager::FOLDER, 3, buf, timestamp));
      break;
    }
    case GenericObject::PAGE: {
      ((Page*)*i)->FlushState(buf->getCont());
      adus->push_back(CommunicationManager::cm->CreateState((*i)->GetID(), GetID(), CommunicationManager::PAGE, 3, buf, timestamp));
      break;
    }
    }
  }

  TimeWarp::tw->InsertAdus(adus);
}


int MLB::Folder::FlushState(char* buf, std::string name, uint32 order) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_STATE << 2);

  p8[1] = CommunicationManager::FOLDER;

  p32[1] = htonl(order);

  int nlength = name.length();
  p8[8] = nlength;
  memcpy(buf + 9, name.c_str(), nlength);
  
  int padd = ((nlength + 1) % 4 == 0) ? 0 : 4 - ((nlength + 1) % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[9 + nlength + i] = 0;
  }
  
  return 9 + nlength + padd;
}


int MLB::Folder::GetStateSize(std::string name) {
  int nlength = name.length(); 
  return 9 + nlength + ((nlength % 4 == 0) ? 0 : 4 - (nlength % 4));
}


int MLB::Folder::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_STATE << 2);

  p8[1] = CommunicationManager::FOLDER;

  p32[1] = htonl(order);

  int nlength = containerName.length();
  p8[8] = nlength;
  memcpy(buf + 9, containerName.c_str(), nlength);
  
  int padd = ((nlength + 1) % 4 == 0) ? 0 : 4 - ((nlength + 1) % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[9 + nlength + i] = 0;
  }
  
  return 9 + nlength + padd;
}


int MLB::Folder::GetStateSize() {
  int nlength = containerName.length(); 
  return 9 + nlength + ((nlength % 4 == 0) ? 0 : 4 - (nlength % 4));
}


int MLB::Folder::ParseState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != CommunicationManager::VERSION) {
    SessionManager::sm->message->Error("Folder::ParseCreate - wrong version of mlb protocol");
    return -1;
  }

  // referenced
  if (((p8[0] & MLB_MASK_OPT) >> 2) != CommunicationManager::ADU_STATE) {
    SessionManager::sm->message->Error("Folder::ParseCreate - wrong operation type");
    return -1;
  }

  if (p8[1] != CommunicationManager::FOLDER) {
    SessionManager::sm->message->Error("Folder::ParseCreate - wrong object type %u", (int)p8[1]);
    return -1;
  }

  order = ntohl(p32[1]);

  int nlength = p8[8];
  char* tmp = new char[nlength + 1];
  memcpy(tmp, buf + 9, nlength);
  tmp[nlength] = '\0';
  containerName = string(tmp);
  delete[] tmp;

  int padd = ((nlength + 1) % 4 == 0) ? 0 : 4 - ((nlength + 1) % 4);
  
  return 9 + nlength + padd;
}


int MLB::Folder::FlushSetActive(char* buf) {
  uint8*  p8  = (uint8*)buf;
  
  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::FOLDER;

  p8[2] = CommunicationManager::SET_ACTIVE;
  p8[3] = 0;
  
  return 4;
}


int MLB::Folder::FlushChangeName(char* buf, string name) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::FOLDER;

  p8[2] = CommunicationManager::CHG_NAME;
  
  int nlength = name.length();
  p8[3] = nlength;
  
  memcpy(buf + 4, name.c_str(), nlength);
  
  int padd = (nlength % 4 == 0) ? 0 : 4 - (nlength % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[4 + nlength + i] = 0;
  }
  
  return 4 + nlength + padd;
}


int MLB::Folder::GetChangeNameSize(string name) {
  int nlength = name.length(); 
  return 4 + nlength + ((nlength % 4 == 0) ? 0 : 4 - (nlength % 4));
}


int MLB::Folder::FlushChangeParentObject(char* buf, uint32 order) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;
  
  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::FOLDER;

  p8[2] = CommunicationManager::CHG_PARENT;

  p32[1] = htonl(order);
  
  return 12;
}


int MLB::Folder::FlushDeleteS(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;
 
  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_DELETE << 2);

  p8[1] = CommunicationManager::FOLDER;

  return 4;
}


void MLB::Folder::ResetRecording()
{
#ifdef _DEBUG1
  cout << "MLB::Folder::ClearRecordedObjects" << endl;
#endif

  std::list<GenericContainer *>::iterator 
    temp;

  if (!subContainer.empty()) {
    temp = subContainer.begin();
    while (temp!=subContainer.end()) {
      (*temp)->ResetRecording();
      temp++;
    }
  }
}


MLB::Page* MLB::Folder::GetPrevPage(MLB::Page *active, bool *found) {
  std::list<MLB::GenericContainer *>::iterator 
    temp;
  Page 
    *prevPage,
    *retPage;

  prevPage = NULL;
  temp = subContainer.begin();
  while (temp != subContainer.end() && !*found) {
    if ((*temp) == active) {
      *found = true;
      MLB::SessionManager::sm->GetActiveFolder()->OpenFolder();
      return prevPage;
    }
    if ((*temp)->GetType() == GenericObject::PAGE) {
      MLB::SessionManager::sm->SetActiveFolder(this);
      prevPage = (Page *)(*temp);
    }
    else if ((*temp)->GetType() == GenericObject::FOLDER) {
      retPage = ((Folder *)(*temp))->GetPrevPage(active,found);
      if (retPage) {
	prevPage = retPage;
      }
    }
    ++temp;
  }

  return prevPage;
}

MLB::Page *MLB::Folder::GetNextPage(MLB::Page *active, bool *found) {
  std::list<MLB::GenericContainer *>::iterator 
    temp;
  Page
    *nextPage;

  nextPage = NULL;
  temp = subContainer.begin();
  while (temp != subContainer.end()) {
    if ((*temp)->GetType() == GenericObject::FOLDER) {
      nextPage = ((Folder *)(*temp))->GetNextPage(active,found);
      if (nextPage) {
	return nextPage;
      }
    }
    if (((*temp)->GetType() == GenericObject::PAGE) && *found) {
      MLB::SessionManager::sm->SetActiveFolder(this);
      return (Page *)(*temp);
    }
    if ((*temp) == active) {
      *found = true;
    }
    ++temp;
  }

  return NULL;
}
