/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_folder.h
 authors : Juergen Vogel, Gabriela Maass
 date    : 18.02.2000
 desc    : 

 **********************************************************************************************************/

#ifndef __MLB_FOLDER__
#define __MLB_FOLDER__

#ifdef WIN32
#  pragma warning (disable: 4786)
#endif

#include "sgml.h"
#include "mlb_generic.h"
#include "mlb_page.h"

namespace MLB {

  class Folder : public GenericContainer {
  protected:
      Page* currentPage;
      GUI::PageView* pageView;
  public:    
      int thumbnailFirstX, thumbnailFirstY, thumbnailIncrX, thumbnailIncrY, thumbnailNbrX, thumbnailNbrY;

  public:
      Folder(uint32 id, char *_name=NULL);
      Folder(uint32 id, SGMLElement *root);
      ~Folder();

      void InitTelepointers();

      void Reset();

      virtual void SetName(char *_name);
      virtual int GetType() {return MLB::GenericObject::FOLDER;}

      virtual void DrawPageTree(std::string nodeNumber, std::string _nodePath, std::string parentNode, int pos, bool *open);
      virtual void ClearPageTree();
      void OpenFolder();

      virtual void SaveContainer(FILE *fp, int level);
      virtual void SaveAsPs(FILE *fp); 
      virtual void SaveAsHtml(char* directory, char* docname, FILE* indexframe, int& nbr, int last);
      virtual void SaveAsImage(char* name, char* type, int& nbr);

      virtual void DeleteAll(GUI::Canvas* _canvas);
      virtual void DeleteObjectsFromRTPI();

      Page* GetCurrentPage() {return currentPage;}
      void  SetCurrentPage(Page *page) {currentPage = page;}

      Page* GetPrevPage(Page *active, bool *found);
      Page* GetNextPage(Page *active, bool *found);

      virtual int GetNbrOfPages();
      std::list<uint32>* GetObjectIDList();

      virtual GenericContainer *SetCurrentContainer(std::vector<uint32>* idVector, int pos);
      GenericContainer* GetNextContainer(GenericContainer* obj);

      void CreateThumbnails(Folder* f, int &count, int &x, int &y, bool flat);
      
      void SendAllObjects(uint32 timestamp, int priority = 3);
      void StoreAllObjectsInTWQ(uint32 timestamp);
      RefCount<RTPI::RtpiAdu> GetStateAdu(uint32 timestamp, int priority);
      void ReplaceIDs();

      static int FlushState(char* buf, std::string name, uint32 order); 
      int FlushState(char* buf);
      static int GetStateSize(std::string name);
      int GetStateSize();
      int ParseState(char* buf);

      static int FlushSetActive(char* buf);
      static int GetSetActiveSize() {return 4;}
      int FlushChangeName(char* buf, string name);
      int GetChangeNameSize(string name);
      static int FlushChangeParentObject(char* buf, uint32 order);
      static int GetChangeParentObjectSize() {return 12;}

      static int FlushDeleteS(char* buf);
      int FlushDelete(char* buf) { return FlushDeleteS(buf); }
      
      virtual void ResetRecording();
    };
}

#endif
