/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_font.h
 authors : Gabriela Maass, Juergen Vogel
 date    : 19.06.2000
 desc    : class to create TrueType-fonts

 ********************************************************************************8************************/

#ifndef __MLB_FONT__
#define __MLB_FONT__

#ifdef WIN32
#  pragma warning (disable: 4786)
#endif

#include <stdlib.h>
#include <stdio.h>
#include <list>
#include <string>

#ifndef WIN32
#  include <unistd.h>
#else
#  include <windows.h>
#endif
 
#include "tk.h"
#include "tcl.h"
#include "freetype.h"
#include "extend/ftxpost.h"

#include <list>

namespace MLB { 

  class Font {
  protected:
    unsigned int size;
    TT_Face face;
    TT_Instance instance;  
    TT_Instance_Metrics instance_metrics;
    TT_Face_Properties face_properties;
    TT_Glyph* glyphs;
    TT_UShort code;
    TT_CharMap char_map; 
    char* filename;
    int number_glyphs;
    int character_map;
    int ascent;
    int descent;
    double zoom;

  public:
    Font(char *_filename, TT_Face _face, TT_Face_Properties _face_properties, unsigned int _size, double zoom);
    ~Font();

    int GetSize();
    int GetHeight();
    double GetZoom() {return zoom;}
    int Ascent();
    int Descent();
    char *GetFilename();
    unsigned char *RenderGlyph(TT_Glyph glyph,int x_offset,int y_offset, int width, int height, TT_Glyph_Metrics *glyph_metrics);
    std::string GetCharBitmap(char ascii, int *width, int *ascent, int *descent, int *lbearing, TT_BBox *bbox);
    void *GetCharPixmap(char ascii, int *width, int *height, int *cols, int *ascent, int *descent, int *lbearing, TT_BBox *bbox);
  };


  class FontFamily {
  protected:
    std::list<Font*> fonts;
    std::string name;
    std::string weight;
    std::string slant;
    char* fontFilename;
    TT_Face face;  
    TT_Post ttpost;
    TT_Face_Properties face_properties;
    TT_Error error;

  public:
    FontFamily(TT_Engine engine, char *fontPath, char *_family, char *_weight, char *_slant);
    ~FontFamily();
    
    Font* FindFontInFamily(char *_family, char *_size, char *_weight, char *_slant, double zoom);
    Font* AddFont(int size, double zoom);
  };

  class FontList {
  public:
    static char *Fonts[];

    FontList(char *path);
    ~FontList();

    Font* GetFont(char *_family, char *_size, char *_weight, char *_slant, double zoom);

  protected:
    std::list<FontFamily*>  families;
    TT_Engine engine;
    char* fontPath;
  };
}

#endif
