/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_gui.cpp
 authors : Gabriela Maass, Juergen Vogel
 date    : 22.11.2000
 desc    : Functions to change tcl values and widgets

**********************************************************************************************************/

#include <string>
#include "estring.h"

#include "mlb_gui.h"
#include "pointer.h"

#ifdef _ImageMagick
  namespace MagickLib
    {
#  undef PACKAGE
#  undef VERSION
#  undef class
    }
#include <Magick++.h>
#endif



/*****************************************************/

char *transform(const char *format,va_list ap)
{
static char NeedEsc[] = {
  1,   1,   1,   1,   1,   1,   1,   1, 'b', 't', 'n',   1, 'f', 'r',   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  0,   0, '"',   0, '$',   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, '[','\\', ']',   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   1,   0,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
};
  char
    c, d;
  int
    zw_int;
  double
    zw_double;
  char
    *fmt,
    *ret,
    *zw_string,
    buf[20];
  std::string
    str;
  int i, k;

  fmt = (char *)format;
  str.erase();
  while ((c=(*fmt)) !=0 ) {
    if (c != '%') {
      do {
	str.append(1,c);
      } while ((c=(*++fmt))!='%' && c!=0 );
      if( c==0 ) break;
    }
    if (c=='%') {
      c=(*++fmt);
      switch (c) {
      case '%':
	str.append(1,c);
	break;
      case 'd':
	zw_int = va_arg(ap,int);
	sprintf(buf,"%d",zw_int);
	str += buf;
	break;
      case 'u':
	zw_int = va_arg(ap,int);
	sprintf(buf,"%u",zw_int);
	str += buf;
	break;
	break;
      case 'q':
	zw_string = va_arg(ap,char*);
	if( zw_string==0 ) zw_string = "(NULL)";
	for(i=0; (d=zw_string[i])!=0; i++) {
	  k = NeedEsc[d&0xff];
	  if( k==0 ) {
	    str.append(1,d);
	  }
	  else if( k==1 ) {
	    str.append(1,'\\');
	    str.append(1,(char)(((d>>6) & 3) + '0'));
	    str.append(1,(char)(((d>>3) & 7) + '0'));
	    str.append(1,(char)((d & 7) + '0'));
	  }
	  else {
	    str.append(1,'\\');
	    str.append(1,(char)k);
	  }
	}
	break;
      case 'c':
	c = va_arg(ap,int);
	str.append(1,c);
	break;
      case 'f':
        zw_double = va_arg(ap,double);
	sprintf(buf,"%f",zw_double);
	str += buf;
	break;
      case 's':
        zw_string = va_arg(ap,char*);
	str += zw_string;
	break;
      default:
	str.append(1,c);
      }
    }
    ++fmt;
  }

  ret = strdup((char *)str.c_str());
  str.erase();
  return ret;
}

/*****************************************************/

GUI::Attributes::Attributes()
{
}

GUI::Attributes::~Attributes()
{
}

void GUI::Attributes::SetFont(char *family, char *size, char *weight, char *slant, char *position)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetFont %s %s %s %s %s",
		     family,size,weight,slant,position) != TCL_OK)
    GUI::MessageWindow::Error("TclError in GUI::Attributes::SetFont: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetOutlineColor(char *color)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetOutlineColor %s",color) != TCL_OK)
    GUI::MessageWindow::Error("TclError in GUI::Attributes::SetOutlineColor: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetFillColor(char *color)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetFillColor %s",color) != TCL_OK)
    GUI::MessageWindow::Error("TclError in GUI::Attributes::SetFillColor: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetLineWidth(char *lineWidth)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetLineWidth %s",lineWidth) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetLineWidth: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetLineStyle(char *lineStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetLineStyle {%s}",lineStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetLineStyle: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetCapStyle(char *capStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetCapStyle %s",capStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetCapStyle: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetJoinStyle(char *joinStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetJoinStyle %s",joinStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetJoinStyle: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetArrowStyle(char *arrowStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetArrowStyle %s",arrowStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetArrowStyle: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetArrowShape(char *arrowShape)
{
  if (Et_GlobalEvalF(Et_Interp,"Attributes::SetArrowShape {%s}",arrowShape) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetArrowShape: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetTool(char *tool)
{
  if (Et_GlobalEvalF(Et_Interp,"ToolBar::SetTool $ToolBar::%s",tool) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetTool: %q\n",Et_Interp->result);
}

void GUI::Attributes::ResetAction()
{
  if (Et_GlobalEvalF(Et_Interp,"set ToolBar::currentAction 0") != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::ResetAction: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetPageBackground(const char *color)
{
  if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageBackground %s",color) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageBackground: %q\n",Et_Interp->result);
}

void GUI::Attributes::SetPageDimension(const int width, const int height)
{
  if (width > height) {
    if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageOrientation landscape") != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    if ((width==842) && (height==594)) {
      if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageSize A4") != TCL_OK)
	GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    }
    else if ((width==792) && (height==612)) {
      if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageSize letter") != TCL_OK)
	GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    }
    else {
      if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageSize other") != TCL_OK)
	GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    }
  }
  else {
    if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageOrientation portrait") != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    if ((width==594) && (height==842)) {
      if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageSize A4") != TCL_OK)
	GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    }
    else if ((width==612) && (height==792)) {
      if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageSize letter") != TCL_OK)
	GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    }
    else {
      if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageSize other") != TCL_OK)
	GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
    }
  }
  if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageDimension(x) %u",width) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
  if (Et_GlobalEvalF(Et_Interp,"set PageOptions::pageDimension(y) %u",height) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Attributes::SetPageDimension: %q\n",Et_Interp->result);
}


/***********************************************************
 ***********************************************************
 ***********************************************************/

Tk_Cursor
  GUI::Cursor::watch      = 0,
  GUI::Cursor::point      = 0,
  GUI::Cursor::pointer_ro = 0,
  GUI::Cursor::pointer_ru = 0,
  GUI::Cursor::pointer_lo = 0,
  GUI::Cursor::pointer_lu = 0;

GUI::Cursor::Cursor(char *widget) {
  if (widget) { //J
    tkWin = Tk_NameToWindow(Et_Interp, widget, Tk_MainWindow(Et_Interp));
    tkWin = Tk_Parent(tkWin);
  } else {
    tkWin = Tk_MainWindow(Et_Interp);
  }
}

GUI::Cursor::~Cursor() {
}

void GUI::Cursor::Set(int which) {
  switch (which) {
  case WATCH:
    if (!GUI::Cursor::watch)
      watch = Tk_GetCursor(Et_Interp, tkWin, "watch");
    if (strlen(Et_Interp->result) == 0) Tk_DefineCursor(tkWin, watch); // there was no error
    break;
  }
}

void GUI::Cursor::Set(const char* which, const char* color) {
#ifdef WIN32
  Tcl_Obj
    *obj;
#endif

  if (strcmp(which,"pirate") == 0) {
    Tk_Cursor cursor = Tk_GetCursor(Et_Interp, tkWin, "pirate");
    if (strlen(Et_Interp->result) == 0) Tk_DefineCursor(tkWin, cursor);
  } else if (strcmp(which,"circle") == 0) {
    Tk_Cursor cursor = Tk_GetCursor(Et_Interp, tkWin, "circle");
    if (strlen(Et_Interp->result) == 0) Tk_DefineCursor(tkWin, cursor);
  } else if (strcmp(which,"dotbox") == 0) {
    Tk_Cursor cursor = Tk_GetCursor(Et_Interp, tkWin, "dotbox");
    if (strlen(Et_Interp->result) == 0) Tk_DefineCursor(tkWin, cursor);
  } else if (strcmp(which,"pencil") == 0) {
    Tk_Cursor cursor = Tk_GetCursor(Et_Interp, tkWin, "pencil");
    if (strlen(Et_Interp->result) == 0) Tk_DefineCursor(tkWin, cursor);
  } else if (strcmp(which,"none") == 0) {
    Tk_UndefineCursor(tkWin);
  } else if (strcmp(which,"point") == 0) {
    if (!point) {
#ifdef WIN32
      obj = Tcl_NewStringObj("@point.cur",10);
      point = Tk_AllocCursorFromObj(Et_Interp,tkWin,obj);
#else
      point = Tk_GetCursorFromData(Et_Interp,tkWin,
				   point_bits,point_bits,
				   point_width,point_height,
				   point_x_hot,point_y_hot,
				   Tk_GetUid(color),Tk_GetUid(color));
#endif
    }
    xHot = point_x_hot;
    yHot = point_y_hot;
    if (strlen(Et_Interp->result) == 0) Tk_DefineCursor(tkWin, point);
  }
  else if (strcmp(which,"pointer_ro") == 0) {
    xHot = pointer_ro_x_hot;
    yHot = pointer_ro_y_hot;
  }
  else if (strcmp(which,"pointer_ru") == 0) {
    xHot = pointer_ru_x_hot;
    yHot = pointer_ru_y_hot;
  }
  else if (strcmp(which,"pointer_lo") == 0) {
    xHot = pointer_lo_x_hot;
    yHot = pointer_lo_y_hot;
  }
  else if (strcmp(which,"pointer_lu") == 0) {
    xHot = pointer_lu_x_hot;
    yHot = pointer_lu_y_hot;
  }
}

void GUI::Cursor::Reset() {
  Tk_UndefineCursor(tkWin);
}

int GUI::Cursor::GetXHot()
{
  return xHot;
}

int GUI::Cursor::GetYHot()
{
  return yHot;
}

/***********************************************************
 ***********************************************************
 ***********************************************************/

GUI::Progress::Progress()
{
}

GUI::Progress::~Progress()
{
}

void GUI::Progress::ShowProgress(char *title, int max)
{
  if (Et_GlobalEvalF(Et_Interp,"Toplevel::ShowProgress \"%s\" %d",title,max) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Progress::ShowProgress: %q\n",Et_Interp->result);
}

void GUI::Progress::IncrementProgress()
{
  if (Et_GlobalEvalF(Et_Interp,"Toplevel::IncrementProgress") != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Progress::IncrementProgress: %q\n",Et_Interp->result);
}

void GUI::Progress::HideProgress()
{
  if (Et_GlobalEvalF(Et_Interp,"Toplevel::HideProgress") != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Progress::HideProgress: %q\n",Et_Interp->result);
}


/***********************************************************
 ***********************************************************
 ***********************************************************/

GUI::MessageWindow::MessageWindow()
{
}

GUI::MessageWindow::~MessageWindow()
{
}


void GUI::MessageWindow::Error(char *format, ...)
{
  va_list
    ap;
  char
    *str;

  va_start(ap,format);
  str = transform(format,ap);
  va_end(ap);
  Et_GlobalEvalF(Et_Interp,"DebugInfo::error \"%s\"",str);
  //Et_GlobalEvalF(Et_Interp,"DebugInfo::errorPopup \"%s\"",str); //J
  delete[] str;
}

void GUI::MessageWindow::Info(char *format, ...)
{
  va_list
    ap;
  char
    *str;

  va_start(ap,format);
  str = transform(format,ap);
  va_end(ap);
  Et_GlobalEvalF(Et_Interp,"DebugInfo::info \"%q\"",str);
  delete[] str;
}

void GUI::MessageWindow::InfoBlue(char *format, ...)
{
  va_list
    ap;
  char
    *str;

  va_start(ap,format);
  str = transform(format,ap);
  va_end(ap);
  Et_GlobalEvalF(Et_Interp,"DebugInfo::infoBlue \"%q\"",str);
  delete[] str;
}

void GUI::MessageWindow::Debug(char *format, ...)
{
  va_list
    ap;
  char
    *str;

  va_start(ap,format);
  str = transform(format,ap);
  va_end(ap);
  Et_GlobalEvalF(Et_Interp,"DebugInfo::debug \"%q\"",str);
  delete[] str;
}

/***********************************************************
 ***********************************************************
 ***********************************************************/

void GUI::Toplevel::SetMenuState(char *menustate)
{
    Et_GlobalEvalF(Et_Interp,"$Toplevel::toplevel setmenustate %s",menustate);
}

void GUI::Toplevel::SetStatus(char *status)
{
    Et_GlobalEvalF(Et_Interp,"TopLevel::SetStatus \"%s\"",status);
}

/***********************************************************
 ***********************************************************
 ***********************************************************/

GUI::Canvas::Canvas() {
  widget    = NULL;
}


GUI::Canvas::Canvas(uint32 id, int width, int height, const char *color) {
  widget    = NULL;
  Create(id,width,height,color);
}


GUI::Canvas::~Canvas() {
  Delete();
}


void GUI::Canvas::Delete() {
  if (widget) {
    if (Et_GlobalEvalF(Et_Interp,"Canvas::Delete %s",widget) != TCL_OK) {
      GUI::MessageWindow::Error("TclError in GUI::Canvas::Delete: %q\n",Et_Interp->result);
      delete[] widget;
      delete cursor;
    }
  }
}


int GUI::Canvas::Create(uint32 id, int _width, int _height, const char *color, bool thumbnailPage) {
  width = _width;
  height = _height;

  if (Et_GlobalEvalF(Et_Interp,"Canvas::Create %u %u %u %s %d",id,width,height,color,thumbnailPage) != TCL_OK) {
    GUI::MessageWindow::Error ("TclError in GUI::Canvas::Create: %q\n",Et_Interp->result);
    widget = strdup(" ");
    return 0;
  }

  widget = strdup(Et_Interp->result);
  cursor = new Cursor(widget);

  tkWin = Tk_NameToWindow(Et_Interp, widget, Tk_MainWindow(Et_Interp));
  return 1;
}


char *GUI::Canvas::GetWidget() {
  return widget;
}


void GUI::Canvas::SetDimension(int _width, int _height) {
  width = _width;
  height = _height;

  if (Et_GlobalEvalF(Et_Interp,"%s configure -width %u -height %u",widget,width,height) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Canvas::SetDimension: %q\n",Et_Interp->result);
}

void GUI::Canvas::Hide() {
  if (Et_GlobalEvalF(Et_Interp,"Canvas::Hide %s",widget) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Canvas:Hide: %q\n",Et_Interp->result);
}


void GUI::Canvas::Show() {
  if (Et_GlobalEvalF(Et_Interp,"Canvas::Show %s",widget) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Canvas:Show: %q\n",Et_Interp->result);
}


#ifdef __WIN32__
#  include "tkWinInt.h"
#else
#  include <X11/Xutil.h>


typedef struct ColormapData {
  int
    separated,                           /* Whether to use separate color bands */
    color,                               /* Whether window is color or black/white */
    ncolors;                             /* Number of color values stored */
  XColor
    *colors;                             /* Pixel value -> RGB mappings */
  int
    red_mask, green_mask, blue_mask;     /* Masks and shifts for each */
  int
    red_shift, green_shift, blue_shift;  /* color band */
} ColormapData;

#endif


bool GUI::Canvas::GetAsImage(Magick::Image& image) {
  int
    depth, x, y,
    width, height;
  unsigned char
    *pixels;

#ifndef __WIN32__
    XImage
      *ximage;
    ColormapData
      cdata;
    int
      r, g, b,
      ncolors, i;
    Colormap
      cmap;

#else
#   undef XGetPixel
#   define XGetPixel(P,X,Y) GetPixel(P, X, Y)
    TkWinDCState
      DCi;
    HDC
      ximage;
#endif
    unsigned long
      pixel;
    Visual
      *visual;
    unsigned char
      *p;

    tkWin = Tk_NameToWindow(Et_Interp, widget, Tk_MainWindow(Et_Interp));

    if (Et_GlobalEvalF(Et_Interp,"update") != TCL_OK) {
      GUI::MessageWindow::Error ("Canvas::GetAsImage - %q\n",Et_Interp->result);
    }

    if (!tkWin) {
      Tcl_AppendResult(Et_Interp, "Window \"", widget,"\" doesn't exist", (char *) NULL);
      GUI::MessageWindow::Error("Canvas::GetAsImage - %s\n",Et_Interp->result);
      return false;
    }

    if (!Tk_IsMapped(tkWin)) {
      Tcl_AppendResult(Et_Interp, "Window \"", widget,"\" isn't mapped", (char *) NULL);
      GUI::MessageWindow::Error("Canvas::GetAsImage - %s\n",Et_Interp->result);
      return false;
    }

    if (!Tk_WindowId(tkWin)) {
      Tcl_AppendResult(Et_Interp, "Window \"", widget,"\" there is no id", (char *) NULL);
      GUI::MessageWindow::Error("Canvas::GetAsImage - %s\n",Et_Interp->result);
      return false;
    }

    width = Tk_Width(tkWin);
    height = Tk_Height(tkWin);

    if ((width <= 0) || (height <= 0)) {
	GUI::MessageWindow::Error("Canvas::GetAsImage - (width <= 0) || (height <= 0)\n");
      return false;
    }

    /*
     * If the window is off the screen it will generate an BadMatch/XError
     * We catch any BadMatch errors here
     */

#ifndef __WIN32__
    /*
     * Generate an XImage from the window.  We can then read pixel
     * values out of the XImage.
     */

    ximage = XGetImage(Tk_Display(tkWin), Tk_WindowId(tkWin), 0, 0,
		       width, height, AllPlanes, ZPixmap);


    if (ximage == (XImage*) NULL) {
        Tcl_AppendResult(Et_Interp, "Window \"", widget,
                "\" cannot be transformed into a pixmap (possibly obscured?)",
                (char *) NULL);
	GUI::MessageWindow::Error("Canvas::GetAsImage - %s\n",Et_Interp->result);
      return NULL;
    }
#else
    ximage = TkWinGetDrawableDC(Tk_Display(tkWin), Tk_WindowId(tkWin), &DCi);
#endif

    depth = Tk_Depth(tkWin);
    visual = Tk_Visual(tkWin);
#ifndef __WIN32__
    cmap = Tk_Colormap(tkWin);

    /*
     * Obtain information about the colormap, ie the mapping between
     * pixel values and RGB values.  The code below should work
     * for all Visual types.
     */

    ncolors = visual->map_entries;
    cdata.colors = (XColor *) ckalloc(sizeof(XColor) * ncolors);
    cdata.ncolors = ncolors;
    if (visual->c_class == DirectColor || visual->c_class == TrueColor) {
        cdata.separated = 1;
        cdata.red_mask = visual->red_mask;
        cdata.green_mask = visual->green_mask;
        cdata.blue_mask = visual->blue_mask;
        cdata.red_shift = 0;
        cdata.green_shift = 0;
        cdata.blue_shift = 0;
        while ((0x0001 & (cdata.red_mask >> cdata.red_shift)) == 0)
            cdata.red_shift ++;
        while ((0x0001 & (cdata.green_mask >> cdata.green_shift)) == 0)
            cdata.green_shift ++;
        while ((0x0001 & (cdata.blue_mask >> cdata.blue_shift)) == 0)
            cdata.blue_shift ++;
        for (i = 0; i < ncolors; i ++)
            cdata.colors[i].pixel =
                    ((i << cdata.red_shift) & cdata.red_mask) |
                    ((i << cdata.green_shift) & cdata.green_mask) |
                    ((i << cdata.blue_shift) & cdata.blue_mask);
    } else {
        cdata.separated=0;
        for (i = 0; i < ncolors; i ++) cdata.colors[i].pixel = i;
    }
    cdata.color = !(visual->c_class == StaticGray || visual->c_class == GrayScale);

    XQueryColors(Tk_Display(tkWin), cmap, cdata.colors, ncolors);
#endif

    pixels = new unsigned char[width*height*3];
    p = pixels;
    for (y = 0; y<height; y++) {
      for (x = 0; x<width; x++) {
	pixel = XGetPixel(ximage, x, y);
#ifndef __WIN32__
	if (cdata.separated) {
	  r = (pixel & cdata.red_mask) >> cdata.red_shift;
	  *p = cdata.colors[r].red >> 8;
	  p++;
	  if (cdata.color) {
	    g = (pixel & cdata.green_mask) >> cdata.green_shift;
	    b = (pixel & cdata.blue_mask) >> cdata.blue_shift;
	    *p = cdata.colors[g].green >> 8;
	    p++;
	    *p = cdata.colors[b].blue >> 8;
	    p++;
	  }
	} else {
	  *p = cdata.colors[pixel].red >> 8;
	  p++;
	  if (cdata.color) {
	    *p = cdata.colors[pixel].green >> 8;
	    p++;
	    *p = cdata.colors[pixel].blue >> 8;
	    p++;
	  }
	}
#else
	*p = GetRValue(pixel);
	p++;
	*p = GetGValue(pixel);
	p++;
	*p = GetBValue(pixel);
	p++;
#endif
      }
    }

#ifndef __WIN32__
    XDestroyImage(ximage);
    ckfree((char *) cdata.colors);
#else
#   undef XGetPixel
    TkWinReleaseDrawableDC(Tk_WindowId(tkWin), ximage, &DCi);
#endif

    try {
      //Magick::Image image;
      // bergebener Typ wird hier nicht bercksichtigt
      image.read(width, height, "RGB", (MagickLib::StorageType)0, pixels);
      image.quality(100);
      
      delete[] pixels;
      return true;
    }
    catch ( Magick::Exception& exc )
    {
      delete[] pixels;
      GUI::MessageWindow::Error( "Canvas::GetAsImage - Cannot write image: %q", exc.what() );
      return false;
    }
}


// type ?? 
void GUI::Canvas::SaveAsImage(char* name, char* type) {
  try {
    Magick::Image image;
    if (GetAsImage(image)) image.write(name);
  } catch (Magick::Exception& exc) {
    GUI::MessageWindow::Error( "Canvas::GetAsImage - Cannot write image: %q", exc.what() );
  }
}


void GUI::Canvas::Clear(int width, int height)
{
  if (Et_GlobalEvalF(Et_Interp,"Canvas::Clear %s %u %u",widget,width,height) != TCL_OK) {
    GUI::MessageWindow::Error ("TclError in GUI::Canvas::Clear: %q\n",Et_Interp->result);
  }
}

void GUI::Canvas::SetBackground(const char *color)
{
  if (Et_GlobalEvalF(Et_Interp,"%s configure -background %q",widget,color) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::SetBackgroundColor: %q\n",Et_Interp->result);

}

void GUI::Canvas::BeginTicks(int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"set ticksLineX [%s create line %d %d %d %d %d %d -fill #FF0000 -width 2]",widget,x-3,0,x,5,x+3,0) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::ShowTicks: %q\n",Et_Interp->result);

  if (Et_GlobalEvalF(Et_Interp,"set ticksLineY [%s create line %d %d %d %d %d %d -fill #FF0000 -width 2]",widget,0,y-3,5,y,0,y+3) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::ShowTicks: %q\n",Et_Interp->result);
}

void GUI::Canvas::MoveTicks(int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"%s coords $ticksLineX %d %d %d %d %d %d",widget,x-3,0,x,5,x+3,0) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::ShowTicks: %q\n",Et_Interp->result);

  if (Et_GlobalEvalF(Et_Interp,"%s coords $ticksLineY %d %d %d %d %d %d",widget,0,y-3,5,y,0,y+3) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::ShowTicks: %q\n",Et_Interp->result);

}

void GUI::Canvas::EndTicks(int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"%s delete $ticksLineX",widget) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::ShowTicks: %q\n",Et_Interp->result);

  if (Et_GlobalEvalF(Et_Interp,"%s delete $ticksLineY",widget) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::ShowTicks: %q\n",Et_Interp->result);
}

void GUI::Canvas::BeginSelectRegion(int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"set selRect [%s create rectangle %d %d %d %d -outline #FF0000 -width 1]",
		     widget,x,y,x,y) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Canvas::BeginSelectRegion: %q\n",Et_Interp->result);
}

void GUI::Canvas::ModifySelectRegion(int firstX, int firstY, int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"%s coords $selRect %d %d %d %d",widget,firstX,firstY,x,y) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Canvas::ModifySelectRegion: %q\n",Et_Interp->result);
}

void GUI::Canvas::DeleteSelectRegion()
{
  if (Et_GlobalEvalF(Et_Interp,"if [info exists selRect] {%s delete $selRect}",widget) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Canvas::DeleteSelectRegion: %q\n",Et_Interp->result);
}

void GUI::Canvas::BeginHelpLine(int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"set helpLine [%s create line %d %d %d %d -fill #FF0000 -width 1]",widget,x,y,x,y) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::AppendNewObject: %q\n",Et_Interp->result);
}

void GUI::Canvas::ModifyHelpLine(int firstX, int firstY, int x, int y)
{
    if (Et_GlobalEvalF(Et_Interp,"%s coords $helpLine %d %d %d %d",widget,firstX,firstY,x,y) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in Page::AppendNewObject: %q\n",Et_Interp->result);
}

void GUI::Canvas::DeleteHelpLine()
{
  if (Et_GlobalEvalF(Et_Interp,"%s delete $helpLine",widget) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Page::AppendNewObject: %q\n",Et_Interp->result);
}

void GUI::Canvas::SetCursor(const char* name, const char* color)
{
  cursor->Set(name,color);
}

void GUI::Canvas::ResetCursor()
{
  cursor->Reset();
}

void GUI::Canvas::MoveObject(const char *ref, double diffX, double diffY)
{
  if (Et_GlobalEvalF(Et_Interp,"%s move $%s %f %f",widget,ref,diffX,diffY) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::Canvas::MoveObject: %q\n",Et_Interp->result);
}

bool GUI::Canvas::Overlap(const char *ref, int x, int y)
{
  int
    result;

  if (Et_GlobalEvalF(Et_Interp,"lsearch -exact [%s find overlapping %d %d %d %d] $%s",widget,x,y,x,y,ref) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Rectangle::Overlap: %q\n",Et_Interp->result);

  result = atoi(Et_Interp->result);
  if (result == -1)
    return false;
  else
    return true;
}

void GUI::Canvas::DrawLine(const char *ref, int x, int y, int width, int height, const char *color, int lineWidth,
			   const char *dash, const char *capStyle, const char *joinStyle, const char *arrowStyle, const char *arrowShape)
{
  if (Et_GlobalEvalF(Et_Interp,
		     "set %s [%s create line %d %d %d %d  -fill %s -width %d -dash \"%s\" -capstyle \"%s\" -joinstyle \"%s\" -arrow \"%s\" -arrowshape {%s}]",
		     ref,widget,x,y,x+width,y+height,color,lineWidth,dash,capStyle,joinStyle,arrowStyle,arrowShape) != TCL_OK)
   GUI::MessageWindow::Error("TclError in Canvas::DrawLine: %q",Et_Interp->result);

}

//-------------------------------------------------------------------------//

void GUI::Canvas::DrawRectangle(const char *ref, int x, int y, int width, int height, const char *color, int lineWidth, const char *lineStyle)
{
    std::string cmd = std::string(widget) + " create rectangle " +
                  estring(x) + " " +
                  estring(y) + " " +
                  estring(x+width) + " " +
                  estring(y+height) +
                 // " -dash \"" + lineStyle + "\"" +
                  " -outline " + color +
                  " -width " + estring(lineWidth);

    //Et_GlobalEvalF( Et_Interp, "DebugInfo::info \"Before drawing rectangle: [%s find all]\n\"", widget );

    Et_GlobalEvalF( Et_Interp, "%s find all", widget );

    if (Et_GlobalEvalF(Et_Interp, ( std::string("set ") + ref + " [" + cmd  + "]").c_str()) != TCL_OK)
      GUI::MessageWindow::Error ("TclError: %q\n",Et_Interp->result);

    Et_GlobalEvalF( Et_Interp, "%s find all", widget );

    //Et_GlobalEvalF(Et_Interp, "DebugInfo::info \"After drawing rectangle: [%s find all]\n\"", widget);

    /*
    if (Et_GlobalEvalF(Et_Interp,
		       "set %s [%s create rectangle %d %d %d %d -dash \"%s\" -outline %s -width %d]",
		       ref,widget,x,y,x+width,y+height,lineStyle,color,lineWidth) != TCL_OK)
      GUI::MessageWindow::Error ("TclError: %q\n",Et_Interp->result);
    */
}

//-------------------------------------------------------------------------//

void GUI::Canvas::DrawRectangle(const char *ref, int x, int y, int width, int height, const char *color, const char *fillColor, int lineWidth, const char *lineStyle)
{
  if (Et_GlobalEvalF(Et_Interp,
		     "set %s [%s create rectangle %d %d %d %d -dash \"%s\" -outline %s -fill %s -width %d]",
		     ref,widget,x,y,x+width,y+height,lineStyle,color,fillColor,lineWidth) != TCL_OK)
    GUI::MessageWindow::Error ("TclError: %q\n",Et_Interp->result);
}

//-------------------------------------------------------------------------//

void GUI::Canvas::DrawOval(const char *ref, int x, int y, int width, int height, const char *color, int lineWidth, const char *lineStyle)
{
  std::string cmd = std::string(widget) + " create oval " +
                estring(x) + " " +
                estring(y) + " " +
                estring(x+width) + " " +
                estring(y+height) +
               // " -dash \"" + lineStyle + "\"" +
                " -outline " + color +
                " -width " + estring(lineWidth);

  Et_GlobalEvalF( Et_Interp, "%s find all", widget );

  if (Et_GlobalEvalF(Et_Interp, ( std::string("set ") + ref + " [" + cmd  + "]").c_str()) != TCL_OK)
    GUI::MessageWindow::Error ("TclError: %q\n",Et_Interp->result);

  Et_GlobalEvalF( Et_Interp, "%s find all", widget );

  /*
  if (Et_GlobalEvalF(Et_Interp,"set %s [%s create oval %d %d %d %d -dash \"%s\" -outline %s -width %d]",
		     ref,widget,x,y,x+width,y+height,lineStyle,color,lineWidth) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Oval::Draw: %q\n",Et_Interp->result);
  */
}

//-------------------------------------------------------------------------//

void GUI::Canvas::DrawOval(const char *ref, int x, int y, int width, int height, const char *color, const char *fillColor, int lineWidth, const char *lineStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"set %s [%s create oval %d %d %d %d -dash \"%s\" -outline %s -fill %s -width %d]",
		     ref,widget,x,y,x+width,y+height,lineStyle,color,fillColor,lineWidth) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Oval::Draw: %q\n",Et_Interp->result);

}

void GUI::Canvas::DrawImage(const char *ref, int x, int y, const char *imgTag)
{
  if (Et_GlobalEvalF(Et_Interp,"set %s [%s create image %d %d -anchor nw -image %s]",ref,widget,x,y,imgTag) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::PhotoImage::Draw: %q\n",Et_Interp->result);
}

void GUI::Canvas::DrawPolyLine(const char *ref, std::list<MLB::Point*> points, const char *color, int lineWidth, const char *lineStyle,
			       const char *capStyle, const char *joinStyle, const char *arrowStyle, const char *arrowShape)
{
  std::list<MLB::Point *>::iterator
    i;
  Tcl_DString
    tclString;
  char
    *command;

  Tcl_DStringInit(&tclString);
  if (points.size() > 1) {
    Et_DStringAppendF(&tclString,"set %s [%s create line",ref,widget);
    for (i = points.begin(); i != points.end(); ++i) Et_DStringAppendF(&tclString," %d %d",(*i)->x,(*i)->y);
    command = Et_DStringAppendF(&tclString," -fill %s -width %d -dash \"%s\" -capstyle \"%s\" -joinstyle \"%s\" -arrow \"%s\" -arrowshape {%s}]",
				color,lineWidth,lineStyle,capStyle,joinStyle,arrowStyle,arrowShape);
  }
  else {
    command = Et_DStringAppendF(&tclString,"set %s \"\"",ref);
  }

  if (Et_GlobalEvalF(Et_Interp,"%s",command) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in PolyLine::Draw: %q\n",Et_Interp->result);

  Tcl_DStringFree(&tclString);
}

void GUI::Canvas::DrawPolygon(const char *ref, std::list<MLB::Point *> points, const char *color, int lineWidth,const char *lineStyle)
{
 std::list<MLB::Point *>::iterator
    i;
  Tcl_DString
    tclString;
  char
    *command;

  Tcl_DStringInit(&tclString);
  if (points.size() > 1) {
    Et_DStringAppendF(&tclString,"set %s [%s create line",ref,widget);
    for (i = points.begin(); i != points.end(); ++i) Et_DStringAppendF(&tclString," %d %d",(*i)->x,(*i)->y);
    i = points.begin();
    Et_DStringAppendF(&tclString," %d %d",(*i)->x,(*i)->y);
    command = Et_DStringAppendF(&tclString," -dash \"%s\" -fill %s -width %d]",lineStyle,color,lineWidth);
  }
  else  {
    command = Et_DStringAppendF(&tclString,"set %s \"\"",ref);
  }

  if (Et_GlobalEvalF(Et_Interp,"%s",command) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Polygon::Draw: %q\n",Et_Interp->result);

  Tcl_DStringFree(&tclString);
}

void GUI::Canvas::DrawPolygon(const char *ref, std::list<MLB::Point*> points, const char *color, const char *fillColor, int lineWidth, const char *lineStyle)
{
 std::list<MLB::Point *>::iterator
    i;
  Tcl_DString
    tclString;
  char
    *command;

  Tcl_DStringInit(&tclString);
  if (points.size() > 1) {
    Et_DStringAppendF(&tclString,"set %s [%s create polygon",ref,widget);
    for (i = points.begin(); i != points.end(); ++i) Et_DStringAppendF(&tclString," %d %d",(*i)->x,(*i)->y);
    command = Et_DStringAppendF(&tclString," -dash \"%s\" -outline %s -fill %s -width %d]",lineStyle,color,fillColor,lineWidth);
  }
  else  {
    command = Et_DStringAppendF(&tclString,"set %s \"\"",ref);
  }

  if (Et_GlobalEvalF(Et_Interp,"%s",command) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Polygon::Draw: %q\n",Et_Interp->result);

  Tcl_DStringFree(&tclString);
}

void GUI::Canvas::DrawLineSegment(const char *ref, int x, int y, int width, int height, const char *color, int lineWidth, const char *lineStyle)
{
    if (Et_GlobalEvalF(Et_Interp,"%s create line %d %d %d %d -dash \"%s\" -fill %s -width %d -tags Segment%s",
		       widget,x,y,width,height,lineStyle,color,lineWidth,ref) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in PolyObject::DrawNewSegment: %q\n",Et_Interp->result);

}

void GUI::Canvas::DeleteLineSegments(const char *ref)
{
  if (Et_GlobalEvalF(Et_Interp,"%s delete Segment%s",widget,ref) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in PolyObject::DeleteSegments: %q\n",Et_Interp->result);
}

void GUI::Canvas::DrawSelectionRectangle(const char *ref, int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"%s create rectangle %d %d %d %d -fill red -outline red -tags Select%s",widget,x-3,y-3,x+3,y+3,ref) != TCL_OK)
    GUI::MessageWindow::Error("TclError in Canvas::DrawSelectionRectangle: %q",Et_Interp->result);

}

void GUI::Canvas::DeleteSelection(const char *ref)
{
  if (Et_GlobalEvalF(Et_Interp,"%s delete Select%s",widget,ref) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Canvas::DeleteSelection: %q\n",Et_Interp->result);
}

void GUI::Canvas::DeleteObject(const char *ref)
{
  if (Et_GlobalEvalF(Et_Interp,"catch {%s delete $%s}",widget,ref) != TCL_OK)
    GUI::MessageWindow::Error("TclError in Canvas::DeleteObject: %q",Et_Interp->result);
  if (Et_GlobalEvalF(Et_Interp,"catch {unset %s}",ref) != TCL_OK)
    GUI::MessageWindow::Error("TclError in Canvas::UnsetObject: %q\n",Et_Interp->result);
}

void GUI::Canvas::DeleteImage(const char *tag)
{
  if (Et_GlobalEvalF(Et_Interp,"image delete \"%s\"",tag) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::Char::ChangeFont: %q\n",Et_Interp->result);

}

void GUI::Canvas::DrawTextCursor(uint32 id, int x, int y, int width, int height, const char *color)
{
    if (Et_GlobalEvalF(Et_Interp,"%s create line %d %d %d %d -width 1 -fill \"%s\" -tag TextCursor%u",
		       widget, x, y, x+width, y+height, color,id) != TCL_OK)
      GUI::MessageWindow::Error("TclError in MLB::DrawTextCursor::Draw: %q\n", Et_Interp->result);
}

void GUI::Canvas::DeleteTextCursor(uint32 id)
{
  if (Et_GlobalEvalF(Et_Interp,"%s delete TextCursor%u",widget,id) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::Page::DeleteTextCursor: %q\n",Et_Interp->result);
}

void GUI::Canvas::DrawTelepointer(const char *ref, const char *name, uint32 id, int x, int y, const char *color)
{
  char imageID[13];
  if (strcmp(name,"pointer_ro") == 0) sprintf(imageID, "tp1%012d", id);
  else if (strcmp(name,"pointer_ru") == 0) sprintf(imageID, "tp2%012d", id);
  else if (strcmp(name,"pointer_lo") == 0) sprintf(imageID, "tp3%012d", id);
  else if (strcmp(name,"pointer_lu") == 0) sprintf(imageID, "tp4%012d", id);

  if (Et_GlobalEvalF(Et_Interp,"%s configure -foreground %s",imageID,color) != TCL_OK)
      printf ("TclError in MLB::Telepointer::Draw: %s\n",Et_Interp->result);
  if (Et_GlobalEvalF(Et_Interp,"set %s [%s create image %d %d -anchor nw -image %s]",ref,widget,x,y,imageID) != TCL_OK)
    printf ("TclError in MLB::Telepointer::Draw: %s\n",Et_Interp->result);
}


void GUI::Canvas::ChangeFillColor(const char *ref, const char *color)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -fill %s",widget,ref,color) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Canvas::ChangeFillColor: %q\n",Et_Interp->result);
}

void GUI::Canvas::ChangeLineColor(const char *ref, const char *color)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -outline %s",widget,ref,color) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Canvas::ChangeLineColor: %q\n",Et_Interp->result);
}

void GUI::Canvas::ChangeLineWidth(const char *ref, int lineWidth)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -width %d",widget,ref,lineWidth) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::ColoredObject::ChangeLineWidth: %q\n",Et_Interp->result);
}

void GUI::Canvas::ChangeLineStyle(const char *ref, const char *lineStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -dash \"%s\"",widget,ref,lineStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::ColoredObject::ChangeLineStyle: %q\n",Et_Interp->result);

}

void GUI::Canvas::ChangeCapStyle(const char *ref, const char *capStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -capstyle %s",widget,ref,capStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::PolyLine::ChangeCapStyle: %q\n",Et_Interp->result);
}

void GUI::Canvas::ChangeJoinStyle(const char *ref, const char *joinStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -joinstyle %s",widget,ref,joinStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::PolyLine::ChangeJoinStyle: %q\n",Et_Interp->result);
}

void GUI::Canvas::ChangeArrowStyle(const char *ref, const char *arrowStyle)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -arrow %s",widget,ref,arrowStyle) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::PolyLine::ChangeArrowStyle: %q\n",Et_Interp->result);
}

void GUI::Canvas::ChangeArrowShape(const char *ref, const char *arrowShape)
{
  if (Et_GlobalEvalF(Et_Interp,"%s itemconfigure $%s -arrowshape {%s}",widget,ref,arrowShape) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::PolyLine::ChangeArrowShape: %q\n",Et_Interp->result);
}

void GUI::Canvas::ModifyCoords(const char *ref, int x, int y, int width, int height)
{
  if (Et_GlobalEvalF(Et_Interp,"%s coords $%s %d %d %d %d",widget,ref,x,y,x+width,y+height) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::DrawObject::ModifyCoords: %q\n",Et_Interp->result);
}

void GUI::Canvas::ModifyPosition(const char *ref, int x, int y)
{
  if (Et_GlobalEvalF(Et_Interp,"%s coords $%s %d %d",widget,ref,x,y) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::DrawObject::ModifyCoords: %q\n",Et_Interp->result);
}

void GUI::Canvas::Scale(const char *ref, float scaleFactor)
{
  if (Et_GlobalEvalF(Et_Interp,"%s scale $%s 0 0 %2.2f %2.2f",widget,ref,scaleFactor,scaleFactor) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in MLB::DrawObject::Scale: %q\n",Et_Interp->result);

}


/***********************************************************
 ***********************************************************
 ***********************************************************/

GUI::PageView::PageView()
{
}

GUI::PageView::~PageView()
{
}

void GUI::PageView::Init(const char *nodeName)
{
  if (Et_GlobalEvalF(Et_Interp,"PageTree::init {%s} Documents", nodeName) != TCL_OK)
    GUI::MessageWindow::Error("TclError in MLB::SessionManager::DrawPageTree: %s",Et_Interp->result);
}

void GUI::PageView::Delete(const char *nodeName)
{
  if (Et_GlobalEvalF(Et_Interp,"$PageTree::tree delete %s", nodeName) != TCL_OK)
    GUI::MessageWindow::Error("SessionManager::ClearPageTree - could not delete: %s", Et_Interp->result);
}

void GUI::PageView::InsertPageNode(char *nodeNumber, char *nodeName,char *containerName, char *parentNode, char *nodeData, int pos)
{
  if (Et_GlobalEvalF(Et_Interp,"PageTree::insertPage %s \"%s\" %s %s %s %d",
		     nodeNumber,containerName,parentNode,nodeName,nodeData,pos) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::PageView::InsertPageNode: %q\n",Et_Interp->result);
}

void GUI::PageView::DrawPageNode(char *nodeNumber, char *nodeName,char *containerName, char *nodeData)
{
    if (Et_GlobalEvalF(Et_Interp,"PageTree::updatePage %s \"%s\" %s %s",
		       nodeNumber,containerName,nodeName,nodeData) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in Page::DrawPageTree: %q\n",Et_Interp->result);
}

void GUI::PageView::InsertSession(const char *nodeName,const char *containerName, const char *parentNode, const char *nodePath, int pos)
{
    if (Et_GlobalEvalF(Et_Interp,"PageTree::insertSession \"%s\" %s %s %s %d",
		       containerName,parentNode, nodeName,nodePath,pos) != TCL_OK)
      GUI::MessageWindow::Error("TclError in Session::DrawPageTree: %q",Et_Interp->result);
}

void GUI::PageView::InsertFolderNode(const char *nodeName,const char *containerName,
				     const char *parentNode, const char *nodeData, int pos, bool thumbnailFolder)
{
  if (thumbnailFolder) {
    if (Et_GlobalEvalF(Et_Interp,"PageTree::insertThumbnails \"\" \"%s\" %s %s %s %d",
		       containerName,parentNode,nodeName,nodeData,pos) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::PageView::InsertFolderNode: %q\n",Et_Interp->result);
  } else {
    if (Et_GlobalEvalF(Et_Interp,"PageTree::insertFolder \"\" \"%s\" %s %s %s %d",
		       containerName,parentNode,nodeName,nodeData,pos) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::PageView::InsertFolderNode: %q\n",Et_Interp->result);
  }
}

void GUI::PageView::DrawFolderNode(const char *nodeName,const char *containerName,
				   const char *nodeData)
{
    if (Et_GlobalEvalF(Et_Interp,"PageTree::updateFolder \"\" \"%s\" %s %s",
		       containerName,nodeName,nodeData) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::PageView::DrawFolderNode: %q\n",Et_Interp->result);
}

void GUI::PageView::OpenFolderNode(const char *nodeName)
{
    if (Et_GlobalEvalF(Et_Interp,"PageTree::openFolder \"%s\"",nodeName) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::PageView::OpenFolderNode: %q\n",Et_Interp->result);
}

void GUI::PageView::SelectNode(const char *nodeName)
{
    if (Et_GlobalEvalF(Et_Interp,"$PageTree::tree selection add %s",nodeName) != TCL_OK)
      GUI::MessageWindow::Error ("TclError in GUI::PageView::SelectNode: %q\n",Et_Interp->result);
}

void GUI::PageView::SelectFolderNode(const char *nodeName)
{
    if (Et_GlobalEvalF(Et_Interp,"$PageTree::tree selection set %s",nodeName) != TCL_OK)
      GUI::MessageWindow::Error ("TclError GUI::PageView::SelectFolderNode: %q\n",Et_Interp->result);
}

void GUI::PageView::Redraw()
{
  if (Et_GlobalEvalF(Et_Interp,"PageTree::redraw") != TCL_OK)
    GUI::MessageWindow::Error ("TclError in Session::DrawPageTree: %q\n",Et_Interp->result);
}

void GUI::PageView::ActivateNewPageButton(bool on)
{
  if (Et_GlobalEvalF(Et_Interp,"PageTree::SetNewPageButtonState %d",on) != TCL_OK)
    GUI::MessageWindow::Error("TclError in GUI::PageView::ActivateNewPageButton: %s",Et_Interp->result);
}

void GUI::PageView::ActivateAnnotationButton(bool on)
{
  if (Et_GlobalEvalF(Et_Interp,"PageTree::SetAnnotationButtonState %d",on) != TCL_OK)
    GUI::MessageWindow::Error("TclError in GUI::PageView::ActivateAnnotationButton: %s",Et_Interp->result);
}

void GUI::PageView::ActivateSynchButton(bool on)
{
  if (Et_GlobalEvalF(Et_Interp,"PageTree::SetSynchButtonState %d",on) != TCL_OK)
    GUI::MessageWindow::Error("TclError in GUI::PageView::ActivateSynchButton: %s",Et_Interp->result);
}

/***********************************************************
 ***********************************************************
 ***********************************************************/


void GUI::Record::Stop()
{
  if (Et_GlobalEvalF(Et_Interp,"Record::StopRecording") != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Record::StopRecording: %q\n",Et_Interp->result);
}


/***********************************************************
 ***********************************************************
 ***********************************************************/


GUI::Chat::Chat()
{
}

GUI::Chat::~Chat()
{
}

void GUI::Chat::ClearMessages()
{
  if (Et_GlobalEvalF(Et_Interp,"Chat::ClearMessages") != TCL_OK)
    GUI::MessageWindow::Error("Chat::ClearMessages - tcl error %q", Et_Interp->result);

}

void GUI::Chat::ShowMessages(char *format, ...)
{
  va_list
    ap;
  char
    *str;

  va_start(ap,format);
  str = transform(format,ap);
  va_end(ap);
  if (Et_GlobalEvalF(Et_Interp,"Chat::ShowMessage %s",str) != TCL_OK)
    GUI::MessageWindow::Error("Chat::ShowMessages - tcl error %q", Et_Interp->result);

  delete[] str;
}

void GUI::Chat::EnableMessageEntry()
{
  if (Et_GlobalEvalF(Et_Interp,"Chat::EnableMessageEntry") != TCL_OK)
    GUI::MessageWindow::Error("Chat::EnableMessageEntry - tcl error %q", Et_Interp->result);
}

/***********************************************************
 ***********************************************************
 ***********************************************************/

void GUI::Image::Create(const char *tag)
{
  if (Et_GlobalEvalF(Et_Interp,"image create photo %s",tag) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Image::Create: %q\n",Et_Interp->result);
}

void GUI::Image::Destroy(const char *tag)
{
  if (Et_GlobalEvalF(Et_Interp,"catch { image delete %s }",tag) != TCL_OK)
    GUI::MessageWindow::Error ("TclError in GUI::Image::Destroy: %q\n",Et_Interp->result);
}

bool GUI::Image::Exists(const char *tag)
{
  if( Et_GlobalEvalF(Et_Interp,"image type %s",tag) != TCL_OK ) return false;
  else return true;
}


void GUI::Image::DestroyImages()
{
  Et_GlobalEvalF(Et_Interp,"catch { foreach val [image names] { if { [string match \"imgTag*\" $val] } { image delete $val } } }");
} 

void GUI::Image::DestroyCharImages()
{
  Et_GlobalEvalF(Et_Interp,"catch { foreach val [image names] { if { [string match \"charImg*\" $val] } { image delete $val } } }");
} 


