/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_misc.cpp
 authors : Gabriela Maass
 date    : 29.11.2000
 desc    : Miscelaneous functions 

***********************************************************************************************************/

#include <string.h>
#include <mlb_misc.h>
#include "mlb_gui.h"

char *correctSlashs(char *_path)
{
  int 
    i;
  char
    *p;

  p = _path; 
  for (i=0; i<strlen(_path); ++i) {
    if (*p == '/') *p = '\\';
    p++;
  }

  return _path;
}

char *MLB::GetPath(char *_path)
{
  char 
    *p,
    *ret;

#ifdef WIN32

  ret = strdup(_path);

  ret = correctSlashs(ret);

  GUI::MessageWindow::Info("getPath: %s\n",ret);

  p = strrchr(ret,'\\');

  if (p) {
    ++p;
    *p = '\0';
  }
  else {
    p = strrchr(ret,'/');
    if (p) {
      ++p;
      *p = '\0';
    }
  }

#else

  ret = strdup(_path);
  p = strrchr(ret,'/');

  if (p) {
    ++p;
    *p = '\0';
  }
  else {
    free (ret);
    ret = strdup("./");
  }
    

#endif
  
  return ret;
}

char *MLB::GetFilename(char *path)
{
  char 
    *p,
    *ret;

#ifdef WIN32
  path = correctSlashs(path);
  GUI::MessageWindow::Info("GetFilename: %s\n",path);
  p = strrchr(path,'/');

  if (p) {
    p++;
    ret = strdup(p);
  }
  else {
    p = strrchr(path,'\\');
    if (p) {
      p++;
      ret = strdup(p);
    }
    else {
      ret = strdup(path);
    }
  }
  

#else
  p = strrchr(path,'/');

  if (p) {
    p++;
    ret = strdup(p);
  }
  else 
    ret = strdup(path);
  
#endif
  return ret;
}

char *MLB::GetSuffix(char *filename)
{
  char 
    *p,
    *ret;

  p = strrchr(filename,'.');


  if (p) {
    p++;
    ret = strdup(p);
  }
  
  return ret;
}

char *MLB::GetPraefix(char *filename)
{
  char 
    *p,
    *ret;

  ret = strdup(filename);
  p = strrchr(ret,'.');

  if (p) {
    *p = '\0';
  }
  
  return ret;
}


