/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_ovals.cpp
 authors : Gabriela Maass, Juergen Vogel
 date    : 3.12.1999
 desc    : Functions to draw and manipulate objects on a tcl-canvas

**********************************************************************************************************/


#include "mlb.h"

#include "mlb_session.h"
#include "mlb_page.h"
#include "mlb_objects.h"
#include "mlb_adu.h"
#include "mlb_gui.h"


MLB::Oval::Oval(uint32 id) : ColoredObject(id) {
  char buf[100];
  sprintf(buf,"%u",id);
  tkref = "Oval";
  tkref += buf;
  layer = 1;
  recorded = false;
}

MLB::Oval::Oval(int _x, int _y, int _width, int _height, char *_color,
		char *_fillColor, int _lineWidth, char *_lineStyle, uint32 _id)
  : ColoredObject(_x,_y,_width,_height,_color,_fillColor,_lineWidth,_lineStyle,_id) {
  char buf[100];
  sprintf(buf,"%u",_id);
  tkref = "Oval";
  tkref += buf;
  layer = 1;

  if (width  == 0) width  = 1;
  if (height == 0) height = 1;

}

MLB::Oval::~Oval() {
}

void MLB::Oval::Draw(GUI::Canvas *_canvas)
{
  if (!visible) return;

  if (fillColor == "none")
    /* OutlineOval */
    _canvas->DrawOval(tkref.c_str(),
		      SessionManager::sm->GetZoom()*x,
		      SessionManager::sm->GetZoom()*y,
		      SessionManager::sm->GetZoom()*width,
		      SessionManager::sm->GetZoom()*height,
		      color.c_str(),
		      SessionManager::sm->GetZoom()*lineWidth,
		      lineStyle.c_str());
  else
    /* FilledOval */
    _canvas->DrawOval(tkref.c_str(),
		      SessionManager::sm->GetZoom()*x,
		      SessionManager::sm->GetZoom()*y,
		      SessionManager::sm->GetZoom()*width,
		      SessionManager::sm->GetZoom()*height,
		      color.c_str(),
		      fillColor.c_str(),
		      SessionManager::sm->GetZoom()*lineWidth,
		      lineStyle.c_str());

  if (selected) DrawSelection(_canvas);
}

void MLB::Oval::ReDraw(GUI::Canvas *canvas)
{
  if (!visible) return;

  canvas->DeleteObject(tkref.c_str());
  Draw(canvas);
}

MLB::DrawObject *MLB::Oval::Copy()
{
  MLB::Oval
    *ret;

  ret = new Oval(x,y,width,height,(char *)color.c_str(),(char *)fillColor.c_str(),
		 lineWidth,(char *)lineStyle.c_str(),CommunicationManager::cm->GetObjectID());

  return (DrawObject *)ret;
}

void MLB::Oval::SaveObject(FILE *fp, int level)
{
  int
    i;

  if (!visible) return;

  for (i=0; i<level; i++) fprintf(fp,"  ");

  fprintf(fp,"<OVAL x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" color=\"%s\" fillColor=\"%s\" lineWidth=\"%d\" lineStyle=\"%s\"%s></OVAL>\n",
	  x,y,width,height,color.c_str(),fillColor.c_str(),lineWidth,lineStyle.c_str(), GetAnimationStepSGMLText().c_str() );
}

void MLB::Oval::SaveAsPs(FILE *fp, int pageWidth, int pageHeight, bool rotate)
{
  XColor
    *xColor;
  float
    red, green, blue;
  int
    radX, radY;
  int
    psX, psY;

  if (!visible) return;
  //  if ((width==0) || (height==0)) return;

  if (rotate) {
    radX = height/2;
    if (radX < 1) radX = 1;
    radY = width/2;
    if (radY < 1) radY = 1;
    psX = y+radX;
    psY = x+radY;
  }
  else {
    radX = width/2;
    if (radX < 1) radX = 1;
    radY = height/2;
    if (radY < 1) radY = 1;
    psX = x+radX;
    psY = pageHeight-(y+radY);
  }

  if (fillColor != "none") {   /* FilledOval */
    xColor = Tk_GetColor(Et_Interp,Tk_MainWindow(Et_Interp),(char *)fillColor.c_str());
    red   = (float)xColor->red/65535;
    green = (float)xColor->green/65535;
    blue  = (float)xColor->blue/65535;

    fprintf(fp,"\n");
    fprintf(fp,"gsave\n");
    fprintf(fp,"%d setlinewidth 0 setlinejoin 2 setlinecap\n",lineWidth);
    fprintf(fp,"%2.3f %2.3f %2.3f setrgbcolor\n",red,green,blue);
    fprintf(fp,"newpath\n");
    fprintf(fp,"%d %d translate\n",psX,psY);
    fprintf(fp,"%d %d scale\n",radX,radY);
    fprintf(fp,"0 0 1 0 360 arc\n");
    fprintf(fp,"fill\n");
    fprintf(fp,"%f %f scale\n",1.0/radX,1.0/radY);
    fprintf(fp,"-%d -%d translate\n",psX,psY);
    fprintf(fp,"stroke\n");
    fprintf(fp,"grestore\n");
  }

  xColor = Tk_GetColor(Et_Interp,Tk_MainWindow(Et_Interp),(char *)color.c_str());
  red   = (float)xColor->red/65535;
  green = (float)xColor->green/65535;
  blue  = (float)xColor->blue/65535;

  fprintf(fp,"gsave\n");
  fprintf(fp,"%d setlinewidth 0 setlinejoin 2 setlinecap\n",lineWidth);
  fprintf(fp,"%2.3f %2.3f %2.3f setrgbcolor\n",red,green,blue);
  fprintf(fp,"newpath\n");
  fprintf(fp,"%d %d translate\n",psX,psY);
  fprintf(fp,"%d %d scale\n",radX,radY);
  fprintf(fp,"0 0 1 0 360 arc\n");
  fprintf(fp,"%f %f scale\n",1.0/radX,1.0/radY);
  fprintf(fp,"-%d -%d translate\n",psX,psY);
  fprintf(fp,"stroke\n");
  fprintf(fp,"grestore\n");
}

int MLB::Oval::RecordableObjects()
{
  if (width==0 && height==0) visible = false;

  if (!visible) return 0;

  if (fillColor == "none") /* Outline Rectangle */
    return 1;
  else
    return 2;
}

void MLB::Oval::SaveAsAOF(FILE *fp) {	
  int
    zX, zY, 
    zWidth, zHeight;

  if (!visible || recorded) return;

  recorded = true;

  if (width < 0) {
    zX = x + width;
    zWidth = width*(-1);
  } 
  else {
    zX = x;
    zWidth = width;
  }
  if (height < 0) {
    zY = y + height;
    zHeight = height*(-1);
  } 
  else {
    zY = y;
    zHeight = height;
  }

  recID = MLB::SessionManager::sm->recCount;
  if (fillColor != "none") { // Filled Oval
    fprintf(fp,"<FILLEDCIRC x=%d y=%d width=%d height=%d fcolor=%d layer=%d></FILLEDCIRC>\n",
	    zX,zY,zWidth,zHeight,CalculateColor(fillColor),layer);
    MLB::SessionManager::sm->recCount++;
  }

  fprintf(fp,"<OUTLINECIRC x=%d y=%d width=%d height=%d linewidth=%d linestyle=%d fcolor=%d layer=%d></OUTLINECIRC>\n",
	  zX,zY,zWidth,zHeight,lineWidth,CalculateLineStyle(),CalculateColor(color),layer);
  MLB::SessionManager::sm->recCount++;
}


bool MLB::Oval::Overlap(GUI::Canvas *canvas, int _x, int _y)
{
  int
    x1, y1,
    x2, y2;

  if (!visible) return false;

  GetBBox(&x1,&y1,&x2,&y2);
  if ((_x < x1) || (_y < y1) || (_x > x2) || (_y > y2)) return false;

  return canvas->Overlap(tkref.c_str(),SessionManager::sm->GetZoom()*_x,SessionManager::sm->GetZoom()*_y);
}


void MLB::Oval::AdjustGui()
{

  GUI::Attributes::SetOutlineColor((char *)color.c_str());
  if (fillColor != "none") {
    GUI::Attributes::SetFillColor((char *)fillColor.c_str());
    GUI::Attributes::SetTool("FillOval");
  }
  else {
    GUI::Attributes::SetTool("OutOval");
  }
}


RefCount<RTPI::RtpiAdu> MLB::Oval::GetStateAdu(uint32 timestamp, int priority) {
  GenericContainer* parent = SessionManager::sm->GetParentContainer(GetID());
  if (!parent) return NULL;
  RefCount<Buffer> buf = new Buffer(GetStateSize());
  FlushState(buf->getCont());
  return CommunicationManager::cm->CreateState(GetID(), parent->GetID(), CommunicationManager::OVAL, priority, buf, timestamp);
}


int MLB::Oval::FlushState(char* buf, uint32 order, bool visible, int x, int y, int width, int height,
			   Color linecolor, Color fillcolor, int linewidth, Style linestyle) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_STATE << 2);
  p8[0] = p8[0] | (visible << 1);
  p8[0] = p8[0] | ((fillcolor.compare("none") == 0) ? 1 : 0); // transparent

  p8[1] = CommunicationManager::OVAL;

  if (width  < 0) p8[3] = p8[3] | (1 << 1);
  if (height < 0) p8[3] = p8[3] | 1;

  p32[1] = htonl(order);

  p16[4] = htons(x);
  p16[5] = htons(y);

  p16[6] = htons(abs(width));
  p16[7] = htons(abs(height));

  ColoredObject::FlushColor(buf + 16, linecolor);
  ColoredObject::FlushLineStyle(buf + 19, linestyle);

  if (fillcolor.compare("none") == 0)
    p32[5] = 0;
  else
    ColoredObject::FlushColor(buf + 20, fillcolor);
  p8[23] = linewidth;

  return 24;
}


int MLB::Oval::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_STATE << 2);
  p8[0] = p8[0] | (visible << 1);
  p8[0] = p8[0] | ((fillColor.compare("none") == 0) ? 1 : 0); // transparent

  p8[1] = CommunicationManager::OVAL;

  if (width  < 0) p8[3] = p8[3] | (1 << 1);
  if (height < 0) p8[3] = p8[3] | 1;

  p32[1] = htonl(order);

  p16[4] = htons(x);
  p16[5] = htons(y);

  p16[6] = htons(abs(width));
  p16[7] = htons(abs(height));

  ColoredObject::FlushColor(buf + 16, color);
  ColoredObject::FlushLineStyle(buf + 19, lineStyle);

  if (fillColor.compare("none") == 0)
    p32[5] = 0;
  else
    ColoredObject::FlushColor(buf + 20, fillColor);
  p8[23] = lineWidth;

  return 24;
}


int MLB::Oval::ParseState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != CommunicationManager::VERSION) {
    SessionManager::sm->message->Error("Oval::ParseCreate - wrong version of mlb protocol");
    return -1;
  }

  // referenced
  if (((p8[0] & MLB_MASK_OPT) >> 2) != CommunicationManager::ADU_STATE) {
    SessionManager::sm->message->Error("Oval::ParseCreate - wrong operation type");
    return -1;
  }

  visible = (p8[0] & MLB_MASK_VIS) >> 1;

  if (p8[1] != CommunicationManager::OVAL) {
    SessionManager::sm->message->Error("Oval::ParseCreate - wrong object type %u", (int)p8[1]);
    return -1;
  }

  order = ntohl(p32[1]);

  x = ntohs(p16[4]);
  y = ntohs(p16[5]);

  width  = htons(p16[6]);
  height = htons(p16[7]);
  if (((p8[3] & MLB_MASK_SW) >> 1) == 1) width = -width;
  if ((p8[3] & MLB_MASK_SH) == 1) height = -height;

  color     = ColoredObject::ParseColor(buf + 16);
  lineStyle = ColoredObject::ParseLineStyle(buf + 19);

  if (p8[0] & MLB_MASK_T) // transparent -> no fill color
    fillColor = "none";
  else
    fillColor = ColoredObject::ParseColor(buf + 20);

  lineWidth = p8[23];

  return 24;
}


int MLB::Oval::FlushChangeVisibility(char* buf, bool vis) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);
  p8[0] = p8[0] | (vis << 1);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_VISIBILITY;
  p8[3] = 0;

  return 4;
}


int MLB::Oval::FlushMove(char* buf, int x, int y) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::MOVE;

  p8[3] = 0;

  p16[2] = htons(x);
  p16[3] = htons(y);

  return 8;
}

int MLB::Oval::FlushChangeSize(char* buf, int x, int y, int width, int height) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_SIZE;

  p8[3] = 0;
  if (width  < 0) p8[3] = p8[3] | (1 << 1);
  if (height < 0) p8[3] = p8[3] | 1;

  p16[2] = htons(x);
  p16[3] = htons(y);

  p16[4] = htons(abs(width));
  p16[5] = htons(abs(height));

  return 12;
}


int MLB::Oval::SFlushChangeSize(char* buf, int x, int y, int width, int height) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_SIZE;

  p8[3] = 0;
  if (width  < 0) p8[3] = p8[3] | (1 << 1);
  if (height < 0) p8[3] = p8[3] | 1;

  p16[2] = htons(x);
  p16[3] = htons(y);

  p16[4] = htons(abs(width));
  p16[5] = htons(abs(height));

  return 12;
}


int MLB::Oval::FlushChangeLinewidth(char* buf, int linewidth) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p32[1] = 0;

  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_LINEWIDTH;

  p8[7] = linewidth;

  return 8;
}


int MLB::Oval::FlushChangeLinestyle(char* buf, Style linestyle) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p32[1] = 0;

  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_LINESTYLE;

  ColoredObject::FlushLineStyle(buf + 7, linestyle);

  return 8;
}


int MLB::Oval::FlushChangeLinecolor(char* buf, Color color) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_LINECOLOR;
  p8[3] = 0;

  ColoredObject::FlushColor(buf + 4, color);
  p8[7] = 0;

  return 8;
}


int MLB::Oval::FlushChangeFillcolor(char* buf, Color color) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::CHG_FILLCOLOR;
  p8[3] = 0;

  ColoredObject::FlushColor(buf + 4, color);
  p8[7] = 0;

  return 8;
}


int MLB::Oval::FlushRaise(char* buf, uint32 order) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::RAISE;

  p32[1] = htonl(order);

  return 8;
}


int MLB::Oval::FlushLower(char* buf, uint32 order) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = CommunicationManager::OVAL;

  p8[2] = CommunicationManager::LOWER;

  p32[1] = htonl(order);

  return 8;
}


int MLB::Oval::FlushDeleteS(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_DELETE << 2);

  p8[1] = CommunicationManager::OVAL;

  return 4;
}

MLB::DrawObject *MLB::Oval::Load(SGMLElement *hilf)
{
  Oval
    *obj;
  char
    *tmp;
  int
    x, y,
    width, height,
    lineWidth;
  char
    *color,
    *fillColor,
    *lineStyle;

  tmp = hilf->GetAttribute("x");
  if (tmp) x = atoi(tmp);
  else     x = 0;
  tmp = hilf->GetAttribute("y");
  if (tmp) y = atoi(tmp);
  else     y = 0;
  tmp = hilf->GetAttribute("width");
  if (tmp) width = atoi(tmp);
  else     width = 0;
  tmp = hilf->GetAttribute("height");
  if (tmp) height = atoi(tmp);
  else     height = 0;
  tmp = hilf->GetAttribute("color");
  if (tmp) color = strdup(tmp);
  else     color = strdup("#ff0000");
  tmp = hilf->GetAttribute("fillColor");
  if (tmp) fillColor = strdup(tmp);
  else     fillColor = strdup("none");
  tmp = hilf->GetAttribute("lineWidth");
  if (tmp) lineWidth = atoi(tmp);
  else     lineWidth = 2;
  tmp = hilf->GetAttribute("lineStyle");
  if (tmp) lineStyle = strdup(tmp);
  else     lineStyle = strdup(" ");
  obj = new Oval(x,y,width,height,color,fillColor,lineWidth,lineStyle,
		 CommunicationManager::cm->GetObjectID());
  obj->SetLayer(0);

  LoadAnimationStep( obj, hilf );

  delete[] color;
  delete[] fillColor;
  delete[] lineStyle;

  return obj;
}

