/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_participants.h 
 authors : Juergen Vogel
 date    : 
 desc    : 

***********************************************************************************************************/


#ifndef __MLB_PARTICIPANTS_H__
#define __MLB_PARTICIPANTS_H__

#include <vector>
#include <tcl.h>
#include <util/types.h>

#include "mlb_timermanager.h"

namespace HRT {
  class HrtManager;
}


namespace MLB {

  class Participant {
  public:
    Participant(uint32 _pid) : pid(_pid), cname(""), name(""), email(""), phone(""), location(""), tool(""), delay(0), status(INACTIVE), raisedHand(false) {}
    
    enum {
      FULL_ACTIVE, 
      HALF_ACTIVE, 
      INACTIVE, 
      EXITED
    } status;
    
    uint32 pid;
    std::string cname;
    std::string name;
    std::string email;
    std::string phone;
    std::string location;
    std::string tool;

    uint32 delay;

    bool raisedHand;

    static const uint32 defaultDelay;
    static const double alpha;
  };

#ifdef WIN32
  using std::vector<Participant*>;
#endif  
  
  class ParticipantManager : public TimerFeedbackInterface {
  public:
    ParticipantManager() {}
    ~ParticipantManager();

    void AddParticipant(uint32 pid);
    void UpdateParticipant(uint32 pid, std::string _cname, std::string _name, std::string _email, std::string _phone, std::string _location, std::string _tool);
    void RemoveParticipant(uint32 pid);
    void RemoveExitedParticipant(uint32 pid);
    void RemoveAllParticipants();
    void RemoveAllExitedParticipants();

    Participant* GetParticipant(uint32 pid);
    int          GetIndex(uint32 pid);

    int GetNumberOfParticipants() {return participants.size();}

    void UpdateParticipantActiveness(uint32 pid);

    void DisplayParticipant(uint32 pid);
    void DisplayActiveness(uint32 pid);
    void DisplayParticipantList();

    uint32 GetDelay(uint32 pid);
    void   AddDelay(uint32 pid, uint32 timestamp);
    uint32 GetMaxDelay();
    uint32 GetAverageDelay();

    // function implemented due to TimerFeedbackInterface
    void ExecuteTimer(TimerObject* timer);

    static ParticipantManager* pm;

  private:
    void ParticipantExitIndicatorOn(uint32 pid);
    void ParticipantExitIndicatorOff(uint32 pid);

    friend class HRT::HrtManager;

    std::vector<Participant*> participants;

  };

}


#endif
