/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifndef __MLB_RECORD__
#define __MLB_RECORD__

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <sys/types.h>
#include <sys/resource.h>
#endif

#include <string>
#include "tcl.h"

namespace Record {

#ifdef WIN32
  typedef struct {
    char *audioName;
    HANDLE pipe;
    unsigned int frequency;
    unsigned short channel;
    unsigned short encoding;
  } recordData;
#endif

  class AOF {
  protected:
    static int
      recCount;
    
#ifdef WIN32
    BOOL
      recording;
    HANDLE
      hProcess;
    HANDLE
      readPipe, 
      writePipe;
#else
    pid_t
      recordingPID;
    int
      readPipe,
      writePipe;
#endif

    FILE 
      *objFp,
      *evqFp;
    std::string
      prefix,
      dataPath,
      miscPath,
      apPath,
      appsPath,
      audioName,
      aofName,
      evqName,
      objName;
    unsigned int 
      startTime,
      pauseTime;
    int
      frameRate;
    
  public:
    
    AOF(char *_name, int _frameRate);
    ~AOF();

    int Start();
    int Pause();
    int Continue();
    int Stop();
    void WriteToPipe(char *text);
    void CreateProcess();
    void KillProcess();
    void WriteAofFile(int _time);
    void WritepaaFile(int _time);

    char *GetPrefix()   {return (char *)prefix.c_str();}
    char *GetAofName()  {return (char *)aofName.c_str();}
    char *GetDataPath() {return (char *)dataPath.c_str();}
    char *GetMiscPath() {return (char *)miscPath.c_str();}
    char *GetApPath()   {return (char *)apPath.c_str();}
    char *GetAppsPath() {return (char *)appsPath.c_str();}

    FILE *GetEvqFp() {return evqFp;}
    FILE *GetObjFp() {return objFp;}

    void ResetTime();
#ifdef WIN32
    unsigned int GetRecordTime();
#endif
    unsigned int GetTime();

#ifdef WIN32
    BOOL CreateReadThread();
    BOOL CreateRecordThread();
#else
    void RecordThread (int readPipe);
    int CreateRecordThread(int readPipe);
    void CreatePipe();
#endif
  };

}
#endif

