/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifdef WIN32

#include <windows.h>
#include <io.h> 

#else

#ifdef LINUX
typedef unsigned int socklen_t;
#endif

XtInputId
  receiveId;
#endif

#include "mlb_tcpip.h"
#include "mlb_record.h"
#include "mlb_session.h"
#include "mlb_gui.h"


#ifdef WIN32

LONG ReadFromSocket(LPVOID lpThreadData)
{ 
  int
    recTime;
  int
    n;
  int 
    *zwInt,
    tcpSocket;
  unsigned char 
    type;
  char
    zw[500];
  
  zwInt = (int *)lpThreadData;
  tcpSocket = *zwInt;

  while ((n=recv(tcpSocket, (char *)&type, 1,0)) >= 0) {
    if (n == 0) 
      Sleep(20);
    else {
      switch (type) {
      case '0':
        if (!MLB::SessionManager::sm->recording)
          recTime = -1;
        else
          recTime = MLB::SessionManager::sm->aofRecording->GetRecordTime();
        sprintf(zw,"%d\n",recTime);
        n = send(tcpSocket,zw, strlen(zw),0);
        if (n < 0) {
           GUI::MessageWindow::Error("no socket\n");
        }
        break;
      case '1':
        if (!MLB::SessionManager::sm->recording)
          sprintf(zw,"noname\n");
        else
          sprintf(zw,"%s\n",MLB::SessionManager::sm->aofRecording->GetPrefix());
      
        n = send(tcpSocket,zw, strlen(zw),0);
        if (n < 0) {
           GUI::MessageWindow::Error("no socket\n");
        }
        break; 
      case '2':
        if (!MLB::SessionManager::sm->recording)
          sprintf(zw,"\n");
        else {
          sprintf(zw,"%s\n",MLB::SessionManager::sm->aofRecording->GetDataPath());
        }
        n = send(tcpSocket,zw, strlen(zw),0);
        if (n < 0) {
           GUI::MessageWindow::Error("noSocket\n");
        } 
        break; 
      case '3':
        if (MLB::SessionManager::sm->recording)
           MLB::SessionManager::sm->UseJedas();
        break;
      case '4':
        if (MLB::SessionManager::sm->recording)
           MLB::SessionManager::sm->UseAppPlay();
        break;
      }
    }
  }
  close(tcpSocket);

  return (0L);
}

LONG ReceiveThread(LPVOID lpThreadData)
{
  WSADATA 
    wsdata;
  WORD    
    wVersionRequested;  
  int      
    cliLen,
    rc;
  struct sockaddr_in      
    cliAddr,
    tcpAddr;
  int
    addrLen;
  int     
    receiveTcpSocket,
    tcpSocket;
  HANDLE 
    hThread;            // thread that reads from pipe
  DWORD 
    dwThreadID;         // id of pipe thread


  wVersionRequested = MAKEWORD(1,1);     
  rc = WSAStartup(wVersionRequested, &wsdata);

  cliAddr.sin_family      = PF_INET;
  cliAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  cliAddr.sin_port        = htons(6002);
  cliLen                  = sizeof(cliAddr);

  receiveTcpSocket = socket(PF_INET, SOCK_STREAM, 0);
  if (receiveTcpSocket < 0) {
    GUI::MessageWindow::Error("no socket\n");
    return (0L);
  }

  if (bind(receiveTcpSocket, (struct sockaddr *)&cliAddr, cliLen) < 0) {
    GUI::MessageWindow::Error("noSocket\n");
    close(receiveTcpSocket);
    return (0L);
  }

  if (listen(receiveTcpSocket,5) != 0) {
    return (0L);
  }

  addrLen = sizeof(tcpAddr);
  while ((tcpSocket = accept(receiveTcpSocket, (struct sockaddr *)&tcpAddr, &addrLen)) != INVALID_SOCKET) {
    hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ReadFromSocket, 
                           &tcpSocket, CREATE_SUSPENDED, &dwThreadID);
  
    if(!hThread) {
      GUI::MessageWindow::Error("no Socket\n");
      return (0L);
    } 
    SetThreadPriority( hThread, THREAD_PRIORITY_BELOW_NORMAL );
    ResumeThread( hThread );

  }

  close(receiveTcpSocket);

  return(0L);      
}

#else

void ReceiveTcpMessage(XtPointer xp,int *fd,XtInputId *id) 
{
  extern char recname[80];
  extern int starttime;
  extern unsigned int GetTimeStamp();

  int 
    n, addrLen,
    recTime;
  char
    line[500];
  char
    zw[100];
  unsigned char 
    type;
  struct sockaddr_in
    *tcpAddr;

  addrLen = sizeof(tcpAddr);
  tcpAddr = (struct sockaddr_in *)xp;

  if ((n = read(*fd, &type, 1)) == 0) {
    printf ("Close tcp connection\n",n);
    XtRemoveInput(*id);
    close(*fd);
    return;
  }

  switch (type) {
  case '0':
    if (starttime == 0)
      recTime = -1;
    else
      recTime = GetTimeStamp()-starttime;
    sprintf(zw,"%d\n",recTime);
    n = write(*fd,zw, strlen(zw));
    if (n < 0) {
      printf ("Fehler beim senden\n");
    }
    break;
  case '1':
    if (recname) 
      sprintf(zw,"%s\n",recname);
    else
      sprintf(zw,"noname\n",recname);
      
    n = write(*fd,(void *)zw, strlen(zw));
    if (n < 0) {
      printf ("Fehler beim senden\n");
    }
    break;
  }
}

void TcpConnect(XtPointer xp,int *fd,XtInputId *id) 
{  
  struct sockaddr_in
    tcpAddr;
  int
    tcpSocket,
    addrLen;

  addrLen = sizeof(tcpAddr);
  if ((tcpSocket = accept(*fd, (struct sockaddr *) &tcpAddr,
			  (socklen_t *)&addrLen)) < 0) {
    perror("server: accept error");
    return;
  }
}
#endif

void InitReceiveConnection()
{    
#ifdef WIN32
  HANDLE 
    hThread;            // thread that reads from pipe
  DWORD 
    dwThreadID;         // id of pipe thread
  
  hThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ReceiveThread, 
                         NULL, CREATE_SUSPENDED, &dwThreadID);
  
  if(!hThread) {
    GUI::MessageWindow::Error("no Thread\n");
    return;
  } 
  SetThreadPriority( hThread, THREAD_PRIORITY_BELOW_NORMAL );
  ResumeThread( hThread );
   
  return;

#else
  int      
    cliLen;
  struct sockaddr_in      
    cliAddr;

  bzero(&cliAddr, sizeof(cliAddr));
  cliAddr.sin_family      = AF_INET;
  cliAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  cliAddr.sin_port        = htons(6002);
  cliLen                  = sizeof(cliAddr);

  receiveTcpSocket = socket(AF_INET, SOCK_STREAM, 0);
  if (receiveTcpSocket < 0) {
    perror("socket");
    return;
  }

  if (bind(receiveTcpSocket, (struct sockaddr *)&cliAddr, cliLen) < 0) {
    perror("bind");
    exit(1);
  }

  listen(receiveTcpSocket,5); 

  receiveId = XtAppAddInput(w->app,receiveTcpSocket,(XtPointer)XtInputReadMask,TcpConnect,NULL);
#endif

}

void CloseTcpConnection()
{
#ifdef WIN32
#else
  XtRemoveInput(receiveId);
  close(receiveTcpSocket);
#endif
}

