/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_telepointer.cpp
 authors : Juergen Vogel, Gabriela Maass
 date    :
 desc    :

 **********************************************************************************************************/

#include "pointer.h"

#include "mlb.h"
#include "mlb_objects.h"
#include "mlb_session.h"
#include "mlb_telepointer.h"
#include "mlb_participants.h"


//
// class Telepointer
//

MLB::Telepointer::Telepointer(uint32 id, uint32 _participant) : ColoredObject(id), participant(_participant) {
  char buf[100];
  sprintf(buf,"%u",id);
  tkref = "Pointer";
  tkref += buf;

  name = NULL;
  status = ACTIVE;
  visible = true;

  CreateImages();
}


MLB::Telepointer::Telepointer(int _x, int _y, char *_color, int _status, uint32 _id, uint32 _participant)
  : ColoredObject(_x,_y,0,0,_color,"none",0,"",_id), participant(_participant) {
  char buf[100];
  sprintf(buf,"%u",_id);
  tkref = "Pointer";
  tkref += buf;
  layer = 1;

  name = NULL;
  status = _status;

  if (_status == ACTIVE) visible = true;
  else visible = false;

  CreateImages();
}

MLB::Telepointer::~Telepointer() {
  if (name) delete[] name;
}


void MLB::Telepointer::CreateImages() {
  char imageID[13];

  sprintf(imageID, "tp1%012d", id);
  if (Et_GlobalEvalF(Et_Interp,"image create bitmap %s -data %s",imageID,pointer_ro_data) != TCL_OK)
    printf ("TclError in MLB::Telepointer::CreateImages: %s\n",Et_Interp->result);

  sprintf(imageID, "tp2%012d", id);
  if (Et_GlobalEvalF(Et_Interp,"image create bitmap %s -data %s",imageID,pointer_ru_data) != TCL_OK)
    printf ("TclError in MLB::Telepointer::CreateImages: %s\n",Et_Interp->result);

  sprintf(imageID, "tp3%012d", id);
  if (Et_GlobalEvalF(Et_Interp,"image create bitmap %s -data %s",imageID,pointer_lo_data) != TCL_OK)
    printf ("TclError in MLB::Telepointer::CreateImages: %s\n",Et_Interp->result);

  sprintf(imageID, "tp4%012d", id);
  if (Et_GlobalEvalF(Et_Interp,"image create bitmap %s -data %s",imageID, pointer_lu_data) != TCL_OK)
    printf ("TclError in MLB::Telepointer::CreateImages: %s\n",Et_Interp->result);
}


void MLB::Telepointer::Set(const char *_name, int _xHot, int _yHot) {
  if (name) delete[] name;
  name = strdup(_name);

  xHot = _xHot;
  yHot = _yHot;
}


void MLB::Telepointer::Update(int _x, int _y) {
  x = _x;
  y = _y;
}


void MLB::Telepointer::Update(char *_color) {
  if (!color.empty()) color.erase();
  color = _color;
}


void MLB::Telepointer::Draw(Page* page) {
  #ifdef DEBUG
  Et_GlobalEvalF(Et_Interp,"DebugInfo::info \"Telepointer::Draw - draw Telepointer with id %u and tkref %q\n\"", id, tkref);
  #endif

  if (!visible || !name) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;
  
  page->GetCanvas()->DrawTelepointer(tkref.c_str(),
				     name,
				     id,
				     SessionManager::sm->GetZoom()*x - xHot,
				     SessionManager::sm->GetZoom()*y - yHot,
				     color.c_str());
}


void MLB::Telepointer::ReDraw(Page* page) {
  if (!visible || !name) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;

  page->GetCanvas()->DeleteObject(tkref.c_str());
  Draw(page);
}


void MLB::Telepointer::Hide(Page* page) {
  if (!visible) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Telepointer::Hide - hide Telepointer with id %u and tkref %q\n", id, tkref.c_str());
  #endif

  visible = false;

  page->GetCanvas()->DeleteObject(tkref.c_str());
}

void MLB::Telepointer::Remove(Page* page) {
  if (!visible) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Telepointer::Remove - hide Telepointer with id %u and tkref %q\n", id, tkref.c_str());
  #endif

  page->GetCanvas()->DeleteObject(tkref.c_str());
}


void MLB::Telepointer::SetActive(Page* page) {
  if (visible) Hide(page);

  visible = true;
  Draw(page);
  status = ACTIVE;

  if (Et_GlobalEvalF(Et_Interp,"Participant::TpIndicatorOn %u %s", participant, GetLineColor().c_str()) != TCL_OK) 
    GUI::MessageWindow::Error("Telepointer::SetActive - tcl error %q\n", Et_Interp->result);
  
}

void MLB::Telepointer::SetInactive(Page* page) {
  Hide(page);
  status = INACTIVE;

  if (Et_GlobalEvalF(Et_Interp,"Participant::TpIndicatorOff %u", participant) != TCL_OK) 
    GUI::MessageWindow::Error("Telepointer::SetActive - tcl error %q\n", Et_Interp->result);
}

bool MLB::Telepointer::IsActive() {
  if (status == ACTIVE) return true;
  else return false;
}


void MLB::Telepointer::SaveAsAOF(FILE *fp) {
  if (status!=ACTIVE || recorded) return;

  recorded = true;

  recID = MLB::SessionManager::sm->recCount;
  fprintf(fp,"<POINTER x=%d y=%d status=1 layer=%d></POINTER>\n",x,y,layer);
  MLB::SessionManager::sm->recCount++;
}

void MLB::Telepointer::FlushType(char* buf, std::string type) {
  uint8* p8 = (uint8*)buf;
  if (type.compare("pointer_ro") == 0) p8[0] = 4;
  else if (type.compare("pointer_ru") == 0) p8[0] = 5;
  else if (type.compare("pointer_lo") == 0) p8[0] = 7;
  else if (type.compare("pointer_lu") == 0) p8[0] = 6;
}


std::string MLB::Telepointer::ParseType(char* buf) {
  uint8* p8 = (uint8*)buf;
  switch (p8[0]) {
  case 4: return "pointer_ro";
  case 5: return "pointer_ru";
  case 7: return "pointer_lo";
  case 6: return "pointer_lu";
  }
  return "";
}


int MLB::Telepointer::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (TelepointerManager::ADU_STATE << 2);
  p8[0] = p8[0] | (visible << 1);

  FlushType(buf + 1, name ? name : "");

  p16[1] = 0;

  p16[2] = htons(x);
  p16[3] = htons(y);

  if (color.compare("none") == 0)
    ColoredObject::FlushColor(buf + 8, string("#000000"));
  else
    ColoredObject::FlushColor(buf + 8, color);
  p8[11] = 0;

  p16[6] = htons(xHot);
  p16[7] = htons(yHot);

  return 16;
}


int MLB::Telepointer::ParseState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != TelepointerManager::VERSION) {
    GUI::MessageWindow::Error("Telepointer::ParseState - wrong version of mlb protocol\n");
    return -1;
  }

  if (((p8[0] & MLB_MASK_OPT) >> 2) != TelepointerManager::ADU_STATE) {
    GUI::MessageWindow::Error("Telepointer::ParseState - wrong operation type\n");
    return -1;
  }

  visible = (p8[0] & MLB_MASK_VIS) >> 1;

  if (name) delete[] name;
  name = strdup(ParseType(buf + 1).c_str());

  x = ntohs(p16[2]);
  y = ntohs(p16[3]);

  color = ColoredObject::ParseColor(buf + 8);
  //if (color.compare("#000000") == 0)
  //  color = "none";

  xHot = ntohs(p16[6]);
  yHot = ntohs(p16[7]);

  return 16;
}


int MLB::Telepointer::FlushChangeVisibility(char* buf) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);
  p8[0] = p8[0] | (visible << 1);

  FlushType(buf + 1, name ? name : "");

  p8[2] = TelepointerManager::CHG_VISIBILITY;
  p8[3] = 0;

  return 4;
}


int MLB::Telepointer::FlushChangeVisibility(char* buf, bool v) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);
  p8[0] = p8[0] | (v << 1);

  FlushType(buf + 1, name ? name : "");

  p8[2] = TelepointerManager::CHG_VISIBILITY;
  p8[3] = 0;

  return 4;
}


int MLB::Telepointer::FlushMove(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);

  FlushType(buf + 1, name ? name : "");

  p8[2] = TelepointerManager::MOVE;
  p8[3] = 0;

  p16[2] = htons(x);
  p16[3] = htons(y);

  p16[4] = htons(xHot);
  p16[5] = htons(yHot);

  return 12;
}


int MLB::Telepointer::FlushChangeType(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);
  p8[0] = p8[0] | (visible << 1);

  FlushType(buf + 1, name ? name : "");

  p8[2] = TelepointerManager::CHG_TYPE;

  return 4;
}


int MLB::Telepointer::FlushChangeLineColor(char* buf) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);

  FlushType(buf + 1, name ? name : "");

  p8[2] = TelepointerManager::CHG_LINECOLOR;
  p8[3] = 0;

  ColoredObject::FlushColor(buf + 4, color);
  p8[7] = 0;

  return 8;
}


int MLB::Telepointer::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_DELETE << 2);
  p8[0] = p8[0] | (visible << 1);

  return 4;
}


//
// class Telebox
//

MLB::Telebox::Telebox(uint32 id, uint32 _participant) : ColoredObject(id), participant(_participant) {
  char buf[100];
  sprintf(buf,"%u",id);
  tkref = "Box";
  tkref += buf;
  visible = true;
}


MLB::Telebox::Telebox(int _x, int _y, int _width, int _height, char *_color, int _lineWidth, char *_lineStyle, int _status, uint32 _id, uint32 _participant)
  : ColoredObject(_x,_y,_width,_height,_color,"none",_lineWidth,_lineStyle,_id), participant(_participant) {
  char buf[100];
  sprintf(buf,"%u",_id);
  tkref = "Box";
  tkref += buf;
  layer = 1;

  if (_status == ACTIVE) visible = true;
  else visible = false;
}

MLB::Telebox::~Telebox() {}


void MLB::Telebox::Update(int _x, int _y, int _width, int _height) {
  x      = _x;
  y      = _y;
  width  = _width;
  height = _height;
}


void MLB::Telebox::Update(char *_color, int _lineWidth, char *_lineStyle) {
  if (!color.empty()) color.erase();
  if (!lineStyle.empty()) lineStyle.erase();
  color = _color;
  lineStyle = _lineStyle;
  lineWidth = _lineWidth;
}


void MLB::Telebox::Draw(Page* page) {
  #ifdef DEBUG
  Et_GlobalEvalF(Et_Interp,"DebugInfo::info \"Telebox::Draw - draw Telebox with id %u and tkref %q\n\"", id, tkref);
  #endif

  if (!visible) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;
  
  page->GetCanvas()->DrawRectangle(tkref.c_str(),
					   SessionManager::sm->GetZoom()*x,
		    	   SessionManager::sm->GetZoom()*y,
			       SessionManager::sm->GetZoom()*width,
			       SessionManager::sm->GetZoom()*height,
			       color.c_str(),
			       SessionManager::sm->GetZoom()*lineWidth,
			       lineStyle.c_str());
}


void MLB::Telebox::ReDraw(Page* page) {
  if (!visible) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;

  page->GetCanvas()->DeleteObject(tkref.c_str());
  Draw(page);
}


void MLB::Telebox::Hide(Page* page) {
  if (!visible) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Telebox::Hide - hide Telebox with id %u and tkref %q\n", id, tkref.c_str());
  #endif

  visible = false;

  page->GetCanvas()->DeleteObject(tkref.c_str());
}


void MLB::Telebox::Remove(Page* page) {
  if (!visible) return;
  if (!page || (page != SessionManager::sm->GetActivePage())) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Telebox::Remove - hide Telebox with id %u and tkref %q\n", id, tkref.c_str());
  #endif

  page->GetCanvas()->DeleteObject(tkref.c_str());
}


void MLB::Telebox::SetActive(Page* page) {
  if (visible) Hide(page);

  visible = true;
  Draw(page);
  status = ACTIVE;
}


void MLB::Telebox::SetInactive(Page* page) {
  Hide(page);
  status = INACTIVE;
}


bool MLB::Telebox::IsActive() {
  if (status == ACTIVE) return true;
  else return false;
}


void MLB::Telebox::SaveAsAOF(FILE *fp) {
/*  if (status!=ACTIVE || recorded) return;

  recorded = true;

  recID = MLB::SessionManager::sm->recCount;
  fprintf(fp,"<POINTER x=%d y=%d status=1 layer=%d></POINTER>\n",x,y,layer);
  MLB::SessionManager::sm->recCount++;*/
}


int MLB::Telebox::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (TelepointerManager::ADU_STATE << 2);
  p8[0] = p8[0] | (visible << 1);
  
  p8[1] = GenericObject::TELEBOX;

  p16[1] = 0;
  if (width  < 0) p8[3] = p8[3] | (1 << 1);
  if (height < 0) p8[3] = p8[3] | 1;

  p16[2] = htons(x);
  p16[3] = htons(y);
  p16[4] = htons(abs(width));
  p16[5] = htons(abs(height));
  
  if (color.compare("none") == 0)
    ColoredObject::FlushColor(buf + 12, string("#000000"));
  else
    ColoredObject::FlushColor(buf + 12, color);
  ColoredObject::FlushLineStyle(buf + 15, lineStyle);

  p8[20] = lineWidth;

  return 21;
}


int MLB::Telebox::ParseState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  if (((p8[0] & MLB_MASK_V) >> 6) != TelepointerManager::VERSION) {
    GUI::MessageWindow::Error("Telebox::ParseState - wrong version of mlb protocol\n");
    return -1;
  }

  if (((p8[0] & MLB_MASK_OPT) >> 2) != TelepointerManager::ADU_STATE) {
    GUI::MessageWindow::Error("Telebox::ParseState - wrong operation type\n");
    return -1;
  }

  visible = (p8[0] & MLB_MASK_VIS) >> 1;

  if (p8[1] != GenericObject::TELEBOX) {
    GUI::MessageWindow::Error("Telebox::ParseState - wrong object type %u\n", (int)p8[1]);
    return -1;
  }

  x      = ntohs(p16[2]);
  y      = ntohs(p16[3]);
  width  = htons(p16[4]);
  height = htons(p16[5]);

  if (((p8[3] & MLB_MASK_SW) >> 1) == 1) width = -width;
  if ((p8[3] & MLB_MASK_SH) == 1) height = -height;

  color     = ColoredObject::ParseColor(buf + 12);
  lineStyle = ColoredObject::ParseLineStyle(buf + 15);
  fillColor = "none"; // transparent -> no fill color
  lineWidth = p8[20];

  return 21;
}


int MLB::Telebox::FlushChangeVisibility(char* buf) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);
  p8[0] = p8[0] | (visible << 1);

  p8[1] = GenericObject::TELEBOX;

  p8[2] = TelepointerManager::CHG_VISIBILITY;
  p8[3] = 0;

  return 4;
}


int MLB::Telebox::FlushChangeVisibility(char* buf, bool v) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);
  p8[0] = p8[0] | (v << 1);

  p8[1] = GenericObject::TELEBOX;

  p8[2] = TelepointerManager::CHG_VISIBILITY;
  p8[3] = 0;

  return 4;
}


int MLB::Telebox::FlushChangeSize(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);

  p8[1] = GenericObject::TELEBOX;

  p8[2] = TelepointerManager::CHG_SIZE;

  p8[3] = 0;
  if (width  < 0) p8[3] = p8[3] | (1 << 1);
  if (height < 0) p8[3] = p8[3] | 1;

  p16[2] = htons(x);
  p16[3] = htons(y);

  p16[4] = htons(abs(width));
  p16[5] = htons(abs(height));

  return 12;
}


int MLB::Telebox::FlushChangeLinewidth(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p32[1] = 0;

  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = GenericObject::TELEBOX;

  p8[2] = CommunicationManager::CHG_LINEWIDTH;

  p8[7] = lineWidth;

  return 8;
}


int MLB::Telebox::FlushChangeLinestyle(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p32[1] = 0;

  p8[0] = p8[0] | (CommunicationManager::VERSION << 6);
  // referenced is 0
  p8[0] = p8[0] | (CommunicationManager::ADU_CHANGE << 2);

  p8[1] = GenericObject::TELEBOX;

  p8[2] = CommunicationManager::CHG_LINESTYLE;

  ColoredObject::FlushLineStyle(buf + 7, lineStyle);

  return 8;
}


int MLB::Telebox::FlushChangeLineColor(char* buf) {
  uint8*  p8  = (uint8*)buf;

  p8[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_CHANGE << 2);

  p8[1] = GenericObject::TELEBOX;

  p8[2] = TelepointerManager::CHG_LINECOLOR;
  p8[3] = 0;

  ColoredObject::FlushColor(buf + 4, color);
  p8[7] = 0;

  return 8;
}


int MLB::Telebox::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (TelepointerManager::VERSION << 6);
  p8[0] = p8[0] | (TelepointerManager::ADU_DELETE << 2);
  p8[0] = p8[0] | (visible << 1);

  p8[1] = GenericObject::TELEBOX;

  return 4;
}


//
// class TelepointerManager
//

void TP_handleReceivedRtpi(SMP::Indication ind) {
  if (MLB::TelepointerManager::tm) MLB::TelepointerManager::tm->HandleReceivedRtpi(ind);
}


void TP_handleReceivedRtcpi(SMP::Indication ind) {
  if (MLB::TelepointerManager::tm) MLB::TelepointerManager::tm->HandleReceivedRtcpi(ind);
}


char* MLB::TelepointerManager::sendBuffer = new char[1000];
MLB::TelepointerManager* MLB::TelepointerManager::tm = NULL;


MLB::TelepointerManager::TelepointerManager() : openConnection(false) {
}


MLB::TelepointerManager::~TelepointerManager() {
  CloseRTPI();
}


bool MLB::TelepointerManager::OpenRTPI() {
  if (openConnection || !MLB::CommunicationManager::cm->openSMP || !MLB::CommunicationManager::cm->openIDServer) {
    GUI::MessageWindow::Error("TelepointerManager::OpenRTPI - RTP/I Connection cannot be set up\n");
    return false;
  }

  SMP::SMP* smp = MLB::CommunicationManager::cm->smp;

  // setup SMP
  if ((rtpiSocket = smp->Open(::TP_handleReceivedRtpi)) < 0) {
    GUI::MessageWindow::Error("TelepointerManager::Open - could not connect to smp\n");
    return false;
  }

  if ((rtcpiSocket = smp->Open(::TP_handleReceivedRtcpi)) < 0) {
    GUI::MessageWindow::Error("TelepointerManager::Open - could not connect to smp\n");
    return false;
  }

  // setup RTP/I
  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("TelepointerManager::JoinGroup - joining %q - %u - %u\n", MLB::CommunicationManager::cm->ip.c_str(),
		 MLB::CommunicationManager::cm->port+3, MLB::CommunicationManager::cm->ttl);
  #endif

  if (!smp->JoinGroup(rtpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+3,
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("TelepointerManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }
  if (!smp->JoinGroup(rtcpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+4,
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("TelepointerManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }

  rtpi = new RTPI::Rtpi();
  rtpi->join_group( MLB::CommunicationManager::cm->username, this, this, new RTPI::TransparentReliableTransport());

  openConnection = true;
  return true;
}


bool MLB::TelepointerManager::CloseRTPI() {
  if (!openConnection) {
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::CloseRTPI - close not possible\n");
    #endif
    return false;
  }

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("TelepointerManager::LeaveGroup - leaving group\n");
  #endif

  rtpi->leave_group();

  MLB::CommunicationManager::cm->smp->LeaveGroup(rtpiSocket);
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtcpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtcpiSocket);

  delete rtpi;

  openConnection = false;
  return true;
}


void MLB::TelepointerManager::HandleReceivedRtpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {
    Session* session = SessionManager::sm->GetSharedSession();

    RefCount<RTPI::RtpiAduVector> rtpiAdus;
    try {
      rtpiAdus = rtpi->create_rtpi_adus(ind.buf);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n",
		     e.errCode, e.errMsg.c_str());
      return;
    }

    if (!rtpiAdus)
      return; // adu not complete, received only fragments

    for (RTPI::RtpiAduVector::iterator i = rtpiAdus->begin(); i != rtpiAdus->end(); ++i) {
      if ((*i)->GetAduHeader()->GetPayloadType() != APPLICATION_TYPE) {
	      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - wrong payload type %u\n",
		             (*i)->GetAduHeader()->GetPayloadType());
	      return;
      }

      switch ((*i)->GetType()) {
      //
      // Event
      //
      case RTPI::RtpiHeader::PKT_EVENT: {
	      #ifdef DEBUG_RTPI
	      GUI::MessageWindow::Info("TelepointerManager::ReceiveEvent - got Event\n");
	      #endif

	      RTPI::RtpiEvent* event = (RTPI::RtpiEvent*)&(**i);

	      char* buf   = event->GetPayload();
	      uint8*  p8  = (uint8*)buf;
	      uint16* p16 = (uint16*)buf;
	      uint32* p32 = (uint32*)buf;

	      if (((p8[0] & MLB_MASK_V) >> 6) != VERSION) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveEvent - wrong version of telepoointer protocol\n");
	        return;
	      }

	      GenericObject* tmp = session->GetObject(event->GetSubcomponentID());
	      if (!tmp) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveEvent - object with %u id does not exist\n",
	        	 event->GetSubcomponentID());
	        return;
	      }

	      Page* page = (Page*)session->GetContainer(tmp->GetID());
	      if (!page) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveEvent - did not find container for object with id %u\n",
			       tmp->GetID());
	        return;
	      }

	      // register participant activity 
	      ParticipantManager::pm->UpdateParticipantActiveness(event->GetParticipantID());

	      int opType  = (p8[0] & MLB_MASK_OPT) >> 2;
	      int objType = p8[1];
        int subtype = p8[2];

        if (opType == ADU_CHANGE) {
          switch (objType) {
          case GenericObject::TELEBOX: {
            Telebox* obj = (Telebox*)tmp;
            switch (subtype) {
	          case CHG_VISIBILITY: {
	            int visible = (p8[0] & MLB_MASK_VIS) >> 1;
	            if (visible) obj->SetActive(page);
	            else obj->SetInactive(page);
	            ChangeSubcomponentActiveness(obj->GetID(), visible);
	            break;
	          }
	          case CHG_SIZE: {
 	            int width  = ntohs(p16[4]);
	            int height = ntohs(p16[5]);
	            if (((p8[3] & MLB_MASK_SW) >> 1) == 1) width = -width;
	            if ((p8[3] & MLB_MASK_SH) == 1) height = -height;
              obj->Update(ntohs(p16[2]), ntohs(p16[3]), width, height);
	            obj->ReDraw(page);
	            break;
	          }
	          case CHG_LINECOLOR: {
	            obj->Update((char*)ColoredObject::ParseColor((char*)(p8 + 4)).c_str(), obj->GetLineWidth(), (char*)obj->GetLineStyle().c_str());
	            obj->ReDraw(page);
	            break;
	          }
            case CHG_LINEWIDTH: 
            case CHG_LINESTYLE: {
              // not used
              break;
            }
	          }
            break;
          } 
          default: {
            Telepointer* obj = (Telepointer*)tmp;
            switch (subtype) {
	          case CHG_VISIBILITY: {
	            int visible = (p8[0] & MLB_MASK_VIS) >> 1;
	            if (visible) obj->SetActive(page);
	            else obj->SetInactive(page);
	            ChangeSubcomponentActiveness(obj->GetID(), visible);
	            break;
	          }
	          case MOVE: {
	            obj->Update(ntohs(p16[2]), ntohs(p16[3]));
	            obj->ReDraw(page);
	            break;
	          }
	          case CHG_TYPE: {
	            // not used
	            break;
	          }
	          case CHG_LINECOLOR: {
	            obj->Update((char*)ColoredObject::ParseColor((char*)(p8 + 4)).c_str());
	            obj->ReDraw(page);
	            break;
	          }
	          }
            break;
          }
          }
        }

	      if (opType == ADU_DELETE) {
	        switch (objType) {
          case GenericObject::TELEBOX: {
            Telebox* obj = (Telebox*)tmp;
            obj->SetInactive(page);
	          UnregisterSubcomponent(obj->GetID());
	          obj->Hide(page);
	          page->DeleteTelebox(obj->GetID());
	          delete obj;
            break;
          }
          default: {
            Telepointer* obj = (Telepointer*)tmp;
            obj->SetInactive(page);
	          UnregisterSubcomponent(obj->GetID());
	          obj->Hide(page);
	          page->DeleteTelepointer(obj->GetID());
	          delete obj;
  	        break;
          }
          }
        }
      }
      //
      // Cue
      //
      case RTPI::RtpiHeader::PKT_CUE: {
	      #ifdef DEBUG_RTPI
	      GUI::MessageWindow::Info("TelepointerManager::ReceiveCue - got Cue\n");
	      #endif

	      RTPI::RtpiCue* cue = (RTPI::RtpiCue*)&(**i);
	      uint8*  p8   = (uint8*)(cue->GetPayload());
	      uint16* p16  = (uint16*)(cue->GetPayload());
	      uint32* p32  = (uint32*)(cue->GetPayload());

	      if (((p8[0] & MLB_MASK_V) >> 6) != VERSION) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue- wrong version of telepointer protocol\n");
	        return;
	      }

	      Session* session = SessionManager::sm->GetSharedSession();

        GenericObject* tmp = session->GetObject(cue->GetSubcomponentID());
	      if (!tmp) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue - object with %u id does not exist\n",
			       cue->GetSubcomponentID());
	        return;
	      } else {
                #ifdef DEBUG
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue - received cue %u for object %u\n",
			       opType, cue->GetSubcomponentID());
                #endif
	      }

	      Page* page = (Page*)session->GetContainer(tmp->GetID());
	      if (!page) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue - did not find container for object with id %u\n",
			       tmp->GetID());
	        return;
	      }

	      // register participant activity 
	      ParticipantManager::pm->UpdateParticipantActiveness(cue->GetParticipantID());

	      int opType  = (p8[0] & MLB_MASK_OPT) >> 2;
	      int objType = p8[1]; 
        int subtype = p8[2];

        if (opType == ADU_CHANGE) {
          switch (objType) {
          case GenericObject::TELEBOX: {
            switch (subtype) {
            case CHG_SIZE: {
              Telebox* obj = (Telebox*)tmp;
              int width  = ntohs(p16[4]);
	            int height = ntohs(p16[5]);
	            if (((p8[3] & MLB_MASK_SW) >> 1) == 1) width = -width;
	            if ((p8[3] & MLB_MASK_SH) == 1) height = -height;
              obj->Update(ntohs(p16[2]), ntohs(p16[3]), width, height);
	            obj->ReDraw(page);
	            break;
            }
            }
            break;
          }
          default: {
            switch (subtype) {
	          case MOVE: {
	            Telepointer* obj = (Telepointer*)tmp;
              obj->Update(ntohs(p16[2]), ntohs(p16[3]));
	            obj->ReDraw(page);
	            break;
	          }
	          }
	          break;
          }
          }
        }
      }

      //
      // State
      //
      case RTPI::RtpiHeader::PKT_STATE: {
	      #ifdef DEBUG_RTPI
	      GUI::MessageWindow::Info("TelepointerManager::ReceiveState - got State\n");
	      #endif

	      RTPI::RtpiState* state = (RTPI::RtpiState*)&(**i);
	      char* buf   = state->GetPayload();
	      uint8*  p8  = (uint8*)buf;
	      uint16* p16 = (uint16*)buf;
	      uint32* p32 = (uint32*)buf;

	      if (((p8[0] & MLB_MASK_V) >> 6) != VERSION) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveState - wrong version of telepointer protocol\n");
	        return;
	      }

	      Page* page = (Page*)session->GetObject(state->GetObjectID());
	      if (!page) {
	        if (!SessionManager::sm->SynchMode()) page = SessionManager::sm->GetGlobalPage();
	        else page = SessionManager::sm->GetActivePage();
	        if (!page) {
	          GUI::MessageWindow::Error("TelepointerManager::ReceiveState - cannot create, parent object with %u id does not exist\n",
				            state->GetObjectID());
	          return;
	        }
	      }

        int opType  = (p8[0] & MLB_MASK_OPT) >> 2;
	      int objType = p8[1];

        if (opType == ADU_STATE) {
          switch(objType) {
          case GenericObject::TELEBOX: {
            Telebox* obj = page->GetTelebox(state->GetSubcomponentID());
            if (!obj) { // first state of telebox
	            obj = new Telebox(state->GetSubcomponentID(), state->GetParticipantID());
	            page->AddTelebox(obj);
	            obj->ParseState(state->GetPayload());
	            obj->SetVisible();
	            obj->Draw(page);
	            RegisterSubcomponent(obj->GetID());
	          } else {
              if (obj->GetType() != GenericObject::TELEBOX) {
                GUI::MessageWindow::Error("TelepointerManager::ReceiveState - cannot create, object with %u id does already exist\n",
	                                		     state->GetSubcomponentID());
	              return;
              }
              obj->ParseState(state->GetPayload());
	            obj->SetActive(page);
	            ChangeSubcomponentActiveness(obj->GetID(), true);
	          }
            break;
          }
          default: {
            Telepointer* obj = page->GetTelepointer(state->GetSubcomponentID());
	          if (!obj) { // first state of telepointer
	            obj = new Telepointer(state->GetSubcomponentID(), state->GetParticipantID());
	            page->AddTelepointer(obj);
	            obj->ParseState(state->GetPayload());
	            obj->SetVisible();
	            obj->Draw(page);
	            RegisterSubcomponent(obj->GetID());
	          } else {
              if (obj->GetType() != GenericObject::TELEPOINTER) {
                GUI::MessageWindow::Error("TelepointerManager::ReceiveState - cannot create, object with %u id does already exist\n",
	                                  		   state->GetSubcomponentID());
	              return;
              }
	            obj->ParseState(state->GetPayload());
	            obj->SetActive(page);
	            ChangeSubcomponentActiveness(obj->GetID(), true);
	          }
            break;   
	        }
          }
	        
          // register participant activity 
	        ParticipantManager::pm->UpdateParticipantActiveness(state->GetParticipantID());
	      }
      	break;
      }

      //
      // StateQuery
      //
      case RTPI::RtpiHeader::PKT_STATE_QUERY: {
	      #ifdef DEBUG_RTPI
	      GUI::MessageWindow::Info("TelepointerManager::ReceiveStateQuery - got State Query\n");
	      #endif

	      RTPI::RtpiStateQuery* query = (RTPI::RtpiStateQuery*)&(**i);

	      GenericObject* obj = session->GetObject(query->GetSubcomponentID());
	      if (!obj)
	        return; // we don't have the requested object -> do nothing

        Page* page = (Page*)session->GetContainer(query->GetSubcomponentID());
	      if (!page) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveStateQuery - cannot create, parent object with %u id does not exist\n",
			       query->GetSubcomponentID());
	        return;
	      }

	      uint32 pid;
        if (obj->GetType() == GenericObject::TELEPOINTER) {
          pid = ((Telepointer*)obj)->GetParticipantID(); 
        } else {
          pid = ((Telebox*)obj)->GetParticipantID();
        }

        if (pid != CommunicationManager::cm->GetParticipantID())
	        break; // we are not owner of the telepointer / telebox -> do nothing

	      int bytes = obj->FlushState(sendBuffer);
	      SendState(obj->GetID(), page->GetID(), 0, sendBuffer, bytes);  // priority is always 0 -> very volatile information

	      break;
      }
      default:
	      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - wrong adu type %u\n",
		             (*i)->GetType());
	      return;
      }
    }

    try {
      rtpi->handle_received_rtpi(rtpiAdus, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n",
		     e.errCode, e.errMsg.c_str());
      return;
    }

    return;
  }

  case SMP::Indication::UNRLDATAIND: {
    RefCount<RTPI::RtpiAduVector> rtpiAdus;
    try {
      rtpiAdus = rtpi->create_rtpi_adus(ind.buf);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n",
		     e.errCode, e.errMsg.c_str());
      return;
    }

    if (!rtpiAdus)
      return; // adu not complete, received only fragments


    for (RTPI::RtpiAduVector::iterator i = rtpiAdus->begin(); i != rtpiAdus->end(); ++i) {
      if ((*i)->GetAduHeader()->GetPayloadType() != APPLICATION_TYPE) {
	      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - wrong payload type %u\n",
		             (*i)->GetAduHeader()->GetPayloadType());
	      return;
      }

      switch ((*i)->GetType()) {
      case RTPI::RtpiHeader::PKT_CUE: {
	      #ifdef DEBUG_RTPI
	      GUI::MessageWindow::Info("TelepointerManager::ReceiveCue - got Cue\n");
	      #endif

	      RTPI::RtpiCue* cue = (RTPI::RtpiCue*)&(**i);
	      uint8*  p8   = (uint8*)(cue->GetPayload());
	      uint16* p16  = (uint16*)(cue->GetPayload());
	      uint32* p32  = (uint32*)(cue->GetPayload());

	      if (((p8[0] & MLB_MASK_V) >> 6) != VERSION) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue- wrong version of telepoointer protocol\n");
	        return;
	      }

	      Session* session = SessionManager::sm->GetSharedSession();

        GenericObject* tmp = session->GetObject(cue->GetSubcomponentID());
	      if (!tmp) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue - object with %u id does not exist\n",
			       cue->GetSubcomponentID());
	        return;
	      } else {
                #ifdef DEBUG
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue - received cue %u for object %u\n",
			       opType, cue->GetSubcomponentID());
                #endif
	      }

	      Page* page = (Page*)session->GetContainer(tmp->GetID());
	      if (!page) {
	        GUI::MessageWindow::Error("TelepointerManager::ReceiveCue - did not find container for object with id %u\n",
			       tmp->GetID());
	        return;
	      }

        int opType  = (p8[0] & MLB_MASK_OPT) >> 2;
	      int objType = p8[1];
        int subtype = p8[2];

        if (opType == ADU_CHANGE) {
          switch (objType) {
          case GenericObject::TELEBOX: {
            switch (subtype) {
            case CHG_SIZE: {
              Telebox* obj = (Telebox*)tmp;
              int width  = ntohs(p16[4]);
	            int height = ntohs(p16[5]);
	            if (((p8[3] & MLB_MASK_SW) >> 1) == 1) width = -width;
	            if ((p8[3] & MLB_MASK_SH) == 1) height = -height;
              obj->Update(ntohs(p16[2]), ntohs(p16[3]), width, height);
	            obj->ReDraw(page);
	            break;
            }
            }
            break;
          }
          default: {
            switch (subtype) {
	          case MOVE: {
	            Telepointer* obj = (Telepointer*)tmp;
              obj->Update(ntohs(p16[2]), ntohs(p16[3]));
	            obj->ReDraw(page);
	            break;
	          }
	          }
	          break;
          }
          }
        }
      }                                
      }
    }

    try {
      rtpi->handle_received_rtpi(rtpiAdus, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n",
		     e.errCode, e.errMsg.c_str());
      return;
    }

    return;
  }


  case SMP::Indication::LJNEEDED:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got LJNEEDED\n");
    #endif
    return;


  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;


  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void MLB::TelepointerManager::HandleReceivedRtcpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got data %u\n", ind.buf->getSize());
    #endif
    return;
  }

  case SMP::Indication::UNRLDATAIND: {
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    try {
      rtpi->handle_received_rtcpi(ind.buf, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtcpi - RtpiException occured: %u / %q\n",
		     e.errCode, e.errMsg.c_str());
      return;
    }
    return;
  }

  case SMP::Indication::LJNEEDED:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got LJNEEDED\n");
    #endif
    return;

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("TelepointerManager::HandleReceivedRtcpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;

  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("TelepointerManager::HandleReceivedRtcpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void MLB::TelepointerManager::SendState(uint32 subID, uint32 parentID, int priority, char* payload, int size) {
  if (!openConnection) return;

  if (!rtpi->subcomponent_exists(subID)) {
    // new subcomponent -> register
    RegisterSubcomponent(subID); //
  }

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("TelepointerManager::SendState - sending state for subcomponent with id %u and parent %u\n",
		 subID, parentID);
  #endif

  RefCount<RTPI::RtpiAdu> state = new RTPI::RtpiState(subID, APPLICATION_TYPE, 0, rtpitime(), priority, payload, size);
  ((RTPI::RtpiState*)&(*state))->SetObjectID(parentID);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(state);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RtpiException e) {
    GUI::MessageWindow::Error("TelepointerManager::SendState - RtpiException occured: %u / %q\n",
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void MLB::TelepointerManager::SendEvent(uint32 subID, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("TelepointerManager::SendEvent - sending event for subcomponent with id %u\n", subID);
  #endif

  RefCount<RTPI::RtpiAdu> event = new RTPI::RtpiEvent(subID, APPLICATION_TYPE, 0, rtpitime(), payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(event);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RtpiException e) {
    GUI::MessageWindow::Error("TelepointerManager::SendEvent - RtpiException occured: %u / %q\n",
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void MLB::TelepointerManager::SendDelete(uint32 subID, char* payload, int size) {
  if (!openConnection) return;

  rtpi->delete_subcomponent(subID);

  SendEvent(subID, payload, size);
}


void MLB::TelepointerManager::SendCue(uint32 subID, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("TelepointerManager::SendCue - sending cue for subcomponent with id %u\n", subID);
  #endif

  RefCount<RTPI::RtpiAdu> cue = new RTPI::RtpiCue(subID, APPLICATION_TYPE, 0, rtpitime(), payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(cue);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RtpiException e) {
    GUI::MessageWindow::Error("TelepointerManager::SendCue - RtpiException occured: %u / %q\n",
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->SendUnreliable(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void MLB::TelepointerManager::SendStateQuery(uint32 subID, int priority) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("TelepointerManager::SendStateQuery - sending state query for subcomponent with id %u\n", subID);
  #endif

  RefCount<RTPI::RtpiAdu> query = new RTPI::RtpiStateQuery(subID, APPLICATION_TYPE, rtpitime(), priority);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(query);

  RefCount<RTPI::BufferVector> bufVector = NULL;

  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RtpiException e) {
    GUI::MessageWindow::Error("TelepointerManager::SendStateQuery - RtpiException occured: %u / %q\n",
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void MLB::TelepointerManager::send_rtcpi(RefCount<Buffer> buf) {
  if (!openConnection) return;
  MLB::CommunicationManager::cm->smp->SendUnreliable(rtcpiSocket, buf->getCont(), buf->getSize());
}


void MLB::TelepointerManager::subcomponent_changed(RTPI::SubComponentMapElement* subcomp, uint8 reason) {
}


void MLB::TelepointerManager::subcomponent_added(RTPI::SubComponentMapElement* subcomp, int reason) {
   // send state query
  if (!SessionManager::sm->GetSharedSession()->GetObject(subcomp->first)) {
    SendStateQuery(subcomp->first, 2);
  }
}


void MLB::TelepointerManager::subcomponent_removed(uint32 id) {
}


void MLB::TelepointerManager::participant_added(uint32 pid) {
}


void MLB::TelepointerManager::participant_removed(uint32 pid) {
  // look for a stray telepointer of the exiting participant
  SessionManager::sm->GetSharedSession()->DeleteTelepointerOfParticipant(pid);
}


void MLB::TelepointerManager::participant_changed(uint32 pid) {
}


void MLB::TelepointerManager::RegisterSubcomponent(uint32 subID) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("TelepointerManager::RegisterSubcomponent - register sub with id %u\n", subID);
  #endif

  // register subcomponent
  rtpi->add_subcomponent(subID, "tp", true);
}


void MLB::TelepointerManager::UnregisterSubcomponent(uint32 subID) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("TelepointerManager::UnregisterSubcomponent - unregister sub with id %u\n", subID);
  #endif

  rtpi->delete_subcomponent(subID);
}


void MLB::TelepointerManager::ChangeSubcomponentActiveness(uint32 subID, bool active) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("TelepointerManager::ChangeSubcomponentActiveness - set sub with id %u active = %u\n", subID, active);
  #endif

  if (active) rtpi->activate_subcomponent(subID);
  else rtpi->deactivate_subcomponent(subID);
}


