/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_text.cpp
 authors : Gabriela Maass, Juergen Vogel
 date    : 3.12.1999 
 desc    : 

***********************************************************************************************************/

#include "freetype.h"

#include "mlb.h"

#include "mlb_objects.h"
#include "mlb_page.h"
#include "mlb_session.h"
#include "mlb_gui.h"

//#define DEBUG

MLB::Text::Text(uint32 _id)
  : ColoredObject(_id), font(NULL), fontFamily(""), fontSize(""), fontWeight(""), fontSlant(""), textPosition(""),
    ascent(0), descent(0), offY(0) {
  char buf[100];
  sprintf(buf,"%u",_id);
  tkref = "Text";
  tkref += buf;
  activeChar = 0;
  firstPos = true;
  layer = 1;
  recorded = false;
//  recID = -3;
}


MLB::Text::Text(int _x, int _y, int _width, int _height, char *_color,
		char *_textString, char *_family, char *_size, char *_weight, char *_slant, char *_position, uint32 _id)
  : ColoredObject(_x,_y,_width,_height,_color,"none",0,"none", _id) {

  MLB::Char* ascii;
  char zw[20];
  int i, len, size;

  fontFamily   = _family;
  fontSize     = _size;
  fontWeight   = _weight;
  fontSlant    = _slant;
  textPosition = _position;
  layer = 1;
//    recorded = false;
//  recID = -4;

  offY = 0;
  if (textPosition == "up") {
    size = atoi(_size);
    offY += -size/2;
    sprintf(zw,"%d",(2*size)/3);
  }
  else if (textPosition == "down") {
    size = atoi(_size);
    offY += size/3;
    sprintf(zw,"%d",(2*size)/3);
  }
  else  {
    size = atoi(_size);
    sprintf(zw,"%d",size);
  }

  lastZoom = SessionManager::sm->GetZoom();
  font = MLB::SessionManager::sm->fontList->GetFont((char*)fontFamily.c_str(),
						    zw, (char*)fontWeight.c_str(),
						    (char*)fontSlant.c_str(),
						    SessionManager::sm->GetZoom());

  char buf[100];
  sprintf(buf,"%u",_id);
  tkref = "Text";
  tkref += buf;

  height = GetHeight();
  width = 0;
  if (_textString) {
    len = strlen(_textString);
    for (i = 0; i < len; i++) {
      ascii = new MLB::Char(font, (char*)color.c_str(), _textString[i]);
      text.insert(text.end(), ascii);
      width += ascii->GetWidth();
    }
  }

  ascent  = font->Ascent() / SessionManager::sm->GetZoom();
  descent = font->Descent() / SessionManager::sm->GetZoom();

  activeChar = 0;
  if (text.empty())
    firstPos = true;
  else
    firstPos = false;
}


MLB::Text::Text(int _x, int _y, int _width, int _height, Color _color,
		char* _textString, string _family, string _size, string _weight, string _slant, string _position, uint32 _id)
  : ColoredObject(_x,_y,_width,_height,(char*)_color.c_str(),"none",0,"none", _id) {

  Char* ascii;
  char zw[20];
  int i, len, size;

  fontFamily   = _family;
  fontSize     = _size;
  fontWeight   = _weight;
  fontSlant    = _slant;
  textPosition = _position;
  layer = 1;
//  recID=-1;

  offY = 0;
  size = atoi(_size.c_str());
  if (textPosition == "up") {
    offY += -size/2;
    sprintf(zw,"%d",(2*size)/3);
  }
  else if (textPosition == "down") {
    offY += size/3;
    sprintf(zw,"%d",(2*size)/3);
  }
  else  {
    sprintf(zw,"%d",size);
  }

  lastZoom = SessionManager::sm->GetZoom();
  font = SessionManager::sm->fontList->GetFont((char *)fontFamily.c_str(),
					       zw,
					       (char *)fontWeight.c_str(),
					       (char *)fontSlant.c_str(),
					       SessionManager::sm->GetZoom());

  char buf[100];
  sprintf(buf,"%u",_id);
  tkref = "Text";
  tkref += buf;

  height = GetHeight();
  width = 0;
  if (_textString) {
    len = strlen(_textString);
    for (i=0; i<len; i++) {
      ascii = new MLB::Char(font,(char *)color.c_str(),_textString[i]);
      text.insert(text.end(),ascii);
      width += ascii->GetWidth();
    }
  }

  ascent  = font->Ascent() / SessionManager::sm->GetZoom();
  descent = font->Descent() / SessionManager::sm->GetZoom();

  activeChar = 0;
  if (text.empty()) firstPos = true;
  else firstPos = false;
}


int MLB::Text::GetHeight() {
  char size[5];
  sprintf(size, "%d", atoi((char*)fontSize.c_str()));
  Font* f = MLB::SessionManager::sm->fontList->GetFont((char*)fontFamily.c_str(),
						       size,
						       (char*)fontWeight.c_str(),
						       (char*)fontSlant.c_str(),
						       SessionManager::sm->GetZoom());
  return (f->GetHeight() / SessionManager::sm->GetZoom());
}


bool MLB::Text::IsVisible() {
  if (visible && font)
    return true;
  return false;
}


MLB::Char* MLB::Text::GetActiveChar() {
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    if ((*i)->GetID() == activeChar) return *i;
  return NULL;
}


std::list<MLB::Char*>::iterator MLB::Text::GetActiveCharIterator() {
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    if ((*i)->GetID() == activeChar) return i;
  return text.end();
}


void MLB::Text::Update() {
  if (!font) return;

  width = 0;
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i) {
    width += (*i)->GetWidth();
  }

  height = GetHeight();
}

void MLB::Text::InitFont(double scale) {
  char zw[20];
  int  size = atoi(fontSize.c_str());

  offY = 0;
  if (textPosition == "up") {
    offY += -size/2;
    sprintf(zw,"%d",(2*size)/3);
  }
  else if (textPosition == "down") {
    offY += size/3;
    sprintf(zw,"%d",(2*size)/3);
  }
  else  {
    sprintf(zw,"%d",size);
  }

  font = MLB::SessionManager::sm->fontList->GetFont((char*)fontFamily.c_str(),
						    zw,
						    (char*)fontWeight.c_str(),
						    (char*)fontSlant.c_str(),
						    scale);

  height = GetHeight();
  width = 0;

  ascent  = font->Ascent()  / scale;
  descent = font->Descent() / scale;
}


void MLB::Text::ZoomFont(GUI::Canvas* canvas) {
  lastZoom = SessionManager::sm->GetZoom();
  InitFont(lastZoom);
  Update();
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    (*i)->ChangeFont(canvas, font);
}


MLB::Text::~Text() {
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    delete *i;
}


char* MLB::Text::GetString() {
  if (!font) return NULL;

  char* ret = new char[text.size() + 1];
  char* p = ret;

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i) {
    *p = (*i)->ascii; ++p;
  }
  *p = '\0';

  return ret;
}


char MLB::Text::GetChar(int idx) {
  if ((idx < 0) || (idx >= text.size())) return '\0';

  std::list<Char*>::iterator c = text.begin();
  for (int i = 0; i < idx; ++i) ++c;

  return (*c)->ascii;
}


int MLB::Text::EndPos() {
  if (!font) return 0;
  return text.size();
}


char* MLB::Text::GetColor() {
  return (char *)color.c_str();
}


char* MLB::Text::GetFamily() {
  if (!font) return NULL;
  return (char *)fontFamily.c_str();
}


char* MLB::Text::GetSize() {
  if (!font) return NULL;
  return (char *)fontSize.c_str();
}


char* MLB::Text::GetWeight() {
  if (!font) return NULL;
  return (char *)fontWeight.c_str();
}


char* MLB::Text::GetSlant() {
  if (!font) return NULL;
  return (char *)fontSlant.c_str();
}


char* MLB::Text::GetPosition() {
  if (!font) return NULL;
  return (char *)textPosition.c_str();
}


void MLB::Text::ChangeString(GUI::Canvas *_canvas, char *_textString) {

  Char* ascii;
  char* p;
  int i, length;

  i = 0;
  for (std::list<Char*>::iterator t = text.begin(); t != text.end(); ++t) {
    (*t)->Hide(_canvas);
    delete *t;
    ++i;
  }
  text.clear();

  p = _textString;
  length = strlen(_textString);
  width = 0;
  for (i=0; i<length; i++) {
    ascii = new MLB::Char(font,(char *)color.c_str(),*p);
    text.push_back(ascii);
    width += ascii->GetWidth();
    ++p;
  }

  activeChar = 0;
  if (text.empty())
    firstPos = true;
  else
    firstPos = false;
}


void MLB::Text::Change(Page* page, int argc, char *argv[], int ttype) {
  char
    *newKey,
    *family,
    *size,
    *weight,
    *slant,
    *position;

  if (!font) return;

  recorded = false;

  switch (ttype) {
  case INSERT : {
    newKey   = argv[3];
    family   = argv[4];
    size     = argv[5];
    weight   = argv[6];
    slant    = argv[7];
    position = argv[8];
    InsertNewChars(page->GetCanvas(),newKey,family,size,weight,slant,position);
    break;
  }
  case MLB::Text::BACKSPACE :
    DeleteChar(page->GetCanvas());
    break;
  case MLB::Text::LEFT :
    GoLeft();
    break;
  case MLB::Text::RIGHT :
    GoRight();
    break;
  case MLB::Text::UP :
    break;
  case MLB::Text::DOWN :
    break;
  case MLB::Text::END :
    GoEnd();
    break;
  case MLB::Text::HOME :
    GoHome();
    break;
  }
  //ReDraw(page->GetCanvas());
}

void MLB::Text::InsertNewChars(GUI::Canvas *_canvas, char* _string, char* family, char* size, char* weight, char* slant, char* position) {
  if (!font) return;

  Char* ascii;
  int len = strlen(_string);
  char* p = _string;

  std::list<Char*>::iterator actPos = GetActiveCharIterator();

  for (int i = 0; i < len; ++i) {
    ascii = new Char(font, (char*)color.c_str(), *p);
    width += ascii->GetWidth();

    if (actPos != text.end()) ++actPos; 

    if (firstPos) {
      text.insert(text.begin(), ascii);
      actPos = text.begin();
      activeChar = ascii->GetID();
      firstPos = false;
    } else {
      actPos = text.insert(actPos, ascii);
      activeChar = ascii->GetID();
    }
    ++p;
  }

  if (text.empty()) firstPos = true;
  else firstPos = false;
}


void MLB::Text::RemoteInsertNewChars(char* newChars) {

  #ifdef DEBUG
  GUI::MessageWindow::Info("Text::RemoteInsertChar -  insert text %q\n", newChars);
  #endif

  Char* ascii;
  int length = strlen(newChars);

  std::list<Char*>::iterator actPos = GetActiveCharIterator();

  for (int i = 0; i < length; ++i) {
    ascii = new Char(font, (char*)color.c_str(), newChars[i]);
    width += ascii->GetWidth();

    if (actPos != text.end()) ++actPos;

    if (firstPos) {
      text.insert(text.begin(),ascii);
      actPos = text.begin();
      activeChar = ascii->GetID();
      firstPos = false;
    } else {
      actPos = text.insert(actPos,ascii);
      activeChar = ascii->GetID();
    }
  }
}


bool MLB::Text::DeleteChar(GUI::Canvas *_canvas) {
  if (!visible) return false;
  if (text.empty()) return false;

  std::list<Char*>::iterator temp;
  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos == text.end()) {
    GUI::MessageWindow::Error("Text::DeleteChar - active char is invalid\n");
    return true;
  }

  bool begin = (actPos == text.begin()) ? true : false;

  if (actPos != text.end()) {
    temp = actPos;
    --actPos;
    width -= (*temp)->GetWidth();
    (*temp)->Hide(_canvas);
    delete(*temp);
    text.erase(temp);
  }

  firstPos = false;
  if (begin) {
    actPos = text.begin();
    firstPos = true;
  }
  if (text.empty())
    firstPos = true;

  if (actPos != text.end()) activeChar = (*actPos)->GetID();
  else activeChar = 0;

  return true;
}


/*
void MLB::Text::RemoteDeleteChars(GUI::Canvas *_canvas, int nbr) {
  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos == text.end()) {
    GUI::MessageWindow::Error("Text::InsertNewChars - active char is invalid\n");
    return;
  }

  if (deleted || text.empty() || (actPos == text.end())) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Text::RemoteDeleteChars - delete %u chars\n", nbr);
  #endif

  std::list<Char*>::iterator pos = actPos;
  ++pos;

  for (int i = 0; i < nbr; ++i) {
    if (pos == text.end()) break;
    (*pos)->Hide(_canvas);
    width -= (*pos)->GetWidth();
    delete *pos;
    pos = text.erase(pos);
  }

  if (text.empty()) {
    firstPos = true;
    actPos = text.begin();
  }

   activeChar = (*actPos)->GetID();
}


void MLB::Text::RemoteDeleteActive(GUI::Canvas *_canvas) {
  if (deleted || firstPos || text.empty()) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Text::RemoteDeleteActive - delete %s\n", (*actPos)->ascii);
  #endif

  width -= (*actPos)->GetWidth();
  (*actPos)->Hide(_canvas);
  delete *actPos;
  actPos = text.erase(actPos);

  if (text.empty()) {
    firstPos = true;
    actPos = text.begin();
  }
}


void MLB::Text::RemoteDeleteAllChars(GUI::Canvas *_canvas) {
  if (deleted) return;

  #ifdef DEBUG
  GUI::MessageWindow::Info("Text::RemoteDeleteAllChars - delete all chars\n");
  #endif

  if (text.empty()) return;

  for (std::list<MLB::Char *>::iterator pos = text.begin(); pos != text.end(); ++pos) {
    (*pos)->Hide(_canvas);
    width -= (*pos)->GetWidth();
    delete *pos;
    pos = text.erase(pos);
  }

  firstPos = true;

  actPos = text.begin();
}
*/

void MLB::Text::Draw(GUI::Canvas* canvas) {
  if (!visible || !font) return;

  if (lastZoom != SessionManager::sm->GetZoom())
    ZoomFont(canvas);

  int diffX = 0;
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i) {
    (*i)->Draw(canvas, x+diffX, y+offY-ascent);
    diffX += (*i)->GetWidth();
  }

  if (selected) DrawSelection(canvas);
}

void MLB::Text::ReDraw(GUI::Canvas* canvas) {
  if (!visible || text.empty() || !font) return;

  if (lastZoom != SessionManager::sm->GetZoom())
    ZoomFont(canvas);

  int diffX = 0;
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i) {
    (*i)->ReDraw(canvas, x+diffX, y+offY-ascent);
    diffX += (*i)->GetWidth();
  }

  if (((Group*)group)->GetActiveItem() == this) {
    DeleteTextCursor(canvas);
    DrawTextCursor(canvas);
  }

  if (selected) DrawSelection(canvas);
}


void MLB::Text::Move(GUI::Canvas *_canvas, int diffX, int diffY, bool redraw) {
  if (!visible || !font) return;

  recorded = false;

  x += diffX;
  y += diffY;

  if (text.empty()) return;

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    (*i)->Move(_canvas,diffX,diffY,redraw);

  if (redraw && selected) DrawSelection(_canvas);
}

void MLB::Text::ChangeFont(GUI::Canvas *_canvas, int argc, char *argv[]) {
  ChangeFont(_canvas, argv[1], argv[2], argv[3], argv[4]);
}

void MLB::Text::ChangeFont(GUI::Canvas *_canvas, string _family, string _size, string _weight, string _slant) {

  char
    zw[20];
  int
    size;

  if (!visible) return;
  recorded = false;

  if ((fontFamily   == _family) &&
      (fontSize     == _size) &&
      (fontWeight   == _weight) &&
      (fontSlant    == _slant)) return;

  fontFamily   = _family;
  fontSize     = _size;
  fontWeight   = _weight;
  fontSlant    = _slant;

  offY = 0;
  if (textPosition == "up") {
    size = atoi(_size.c_str());
    offY += -size/2;
    sprintf(zw,"%d",(2*size)/3);
  }
  else if (textPosition == "down") {
    size = atoi(_size.c_str());
    offY += size/3;
    sprintf(zw,"%d",(2*size)/3);
  }
  else  {
    size = atoi(_size.c_str());
    sprintf(zw,"%d",size);
  }

  lastZoom = SessionManager::sm->GetZoom();
  font = MLB::SessionManager::sm->fontList->GetFont((char *)fontFamily.c_str(),
						    zw,
						    (char *)fontWeight.c_str(),
						    (char *)fontSlant.c_str(),
						    SessionManager::sm->GetZoom());

  if (!font) return;

  ascent  = font->Ascent() / SessionManager::sm->GetZoom();
  descent = font->Descent() / SessionManager::sm->GetZoom();

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i) (*i)->ChangeFont(_canvas,font);

  Update();
  ReDraw(_canvas);
}


char *MLB::Text::GetTextRef() {
  return (char *)(tkref.c_str());
}


int MLB::Text::GetPos() {
  if (firstPos) return 0;
  int count = 1;
  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  for (std::list<Char*>::iterator i = text.begin(); i != actPos; ++i) ++count;
  return count;
}


int MLB::Text::GetCurrentX() {
  if (text.empty() || firstPos) return 0;
  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  int x = (*actPos)->GetWidth();
  for (std::list<Char*>::iterator i = text.begin(); i != actPos; ++i) x += (*i)->GetWidth();
  return x;
}


int MLB::Text::GetCurrentIndex() {
  if (text.empty() || firstPos) return 0;
  int idx = 0;
  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  for (std::list<Char*>::iterator i = text.begin(); i != actPos; ++i) ++idx;
  return idx;
}


int MLB::Text::GetEndIndex() {
  return (text.size() - 1);
}


bool MLB::Text::GoRight() {
  if (!visible || text.empty()) return false;

  if (firstPos) {
    firstPos = false;
    activeChar = (*text.begin())->GetID();
    return true;
  }

  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos == text.end()) {
    return false;
  }
  if (*actPos != text.back()) {
    firstPos = false;
    ++actPos;
    activeChar = (*actPos)->GetID();
    return true;
  }

  return false;
}


bool MLB::Text::GoLeft() {
  if (!visible || firstPos) return false;

  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos == text.end()) {
    return false;
  }
  if (actPos != text.begin()) {
    --actPos;
    activeChar = (*actPos)->GetID();
    return true;
  }

  firstPos = true;
  activeChar = 0;
  return true;
}


bool MLB::Text::GoEnd() {
  if (!visible) return false;

  if (!text.empty()) {
  std::list<Char*>::iterator actPos  = text.end();
  --actPos;
  activeChar = (*actPos)->GetID();
    firstPos = false;
  return true;
  } else {
    firstPos = true;
    activeChar = 0;
    return false;
  }
}


bool MLB::Text::GoHome() {
  if (!visible) return false;

  firstPos = true;
  activeChar = 0;
  return !text.empty();
}


void MLB::Text::GetBBox(int *minX, int *minY, int *maxX, int *maxY) {
  *minX = x;
  *minY = y-font->Ascent() / SessionManager::sm->GetZoom();
  *maxX = x;
  *maxY = y+font->Descent() / SessionManager::sm->GetZoom();

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    *maxX += (*i)->GetWidth();
}


void MLB::Text::DrawSelection(GUI::Canvas *_canvas) {
  if (!visible || !font) return;

  if (!group) {
    if (selected) DeleteSelection(_canvas);

    selected = true;

    int x1, y1, x2, y2;
    GetBBox(&x1,&y1,&x2,&y2);

    selectPoints[0] = new Point(x1,y1);
    selectPoints[1] = new Point(x2,y1);
    selectPoints[2] = new Point(x2,y2);
    selectPoints[3] = new Point(x1,y2);

    for (int i= 0; i < 4; i++) _canvas->DrawSelectionRectangle(tkref.c_str(),
							   SessionManager::sm->GetZoom() * selectPoints[i]->x,
							   SessionManager::sm->GetZoom() * selectPoints[i]->y);
  } else {
    group->DrawSelection(_canvas);
  }
}


bool MLB::Text::Overlap(GUI::Canvas *_canvas,int _x, int _y) {
  if (!visible) return false;

  int x1, y1, x2, y2;
  GetBBox(&x1,&y1,&x2,&y2);

  if ((_x < x1) || (_y < y1) || (_x > x2) || (_y > y2)) return false;
  return true;
}


void MLB::Text::DeleteSelection(GUI::Canvas *_canvas) {
  if (!visible || !selected) return;

  selected = false;
  _canvas->DeleteSelection(tkref.c_str());
}


bool MLB::Text::IsEditablePoint(int _x, int _y) {
  return false;
}


MLB::DrawObject* MLB::Text::PointIsInText(int _x, int _y) {
  if (!visible || !font) return NULL;

  bool smaller;

  std::list<Char*>::iterator actPos;

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i) {
    if ((*i)->IsEditablePoint(_x,_y,&smaller)) {
      actPos = i;
      firstPos = false;
      if (smaller) {
	if (actPos != text.begin())
	  actPos--;
	else
	  firstPos = true;
      }
      activeChar = (*actPos)->GetID();
      return this;
    }
  }

  return NULL;
}


void MLB::Text::Hide(GUI::Canvas *_canvas) {
  if (!visible || !font) return;

  if (selected) DeleteSelection(_canvas);

  visible = false;

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    (*i)->Hide(_canvas);

  DeleteTextCursor(_canvas);
}


MLB::DrawObject* MLB::Text::Copy(){
  if (!font) return NULL;

  char* textString = GetString();
  Text* ret = new Text(x,y,width,height,(char *)color.c_str(),textString,(char *)fontFamily.c_str(),
		 (char *)fontSize.c_str(),(char *)fontWeight.c_str(),(char *)fontSlant.c_str(),(char *)textPosition.c_str(),
		 CommunicationManager::cm->GetObjectID());

  ret->SetAnnotation(SessionManager::sm->AnnotationMode());
  free(textString);

  return (DrawObject *)ret;
}


int MLB::Text::SelectObject(int _x, int _y)
{
//   std::list<MLB::Char *>::iterator
//     temp;
//   int
//     x1, y1,
//     x2, y2;

//   if (!visible) return 0;

//   GetBBox(&x1,&y1,&x2,&y2);
//   if ((_x < x1) || (_y < y1) || (_x > x2) || (_y > y2)) return 0;

//   temp = text.begin();
//   while (temp != text.end()) {
//     if ((*temp)->SelectObject(length,selectList) > 0) {
//       actPos = temp;
//       if ((*temp)->SelectObject(_x,_y)) {
// 	return true;
//       }
//       selected = true;
//       return 1;
//     }
//     temp++;
//   }

  return 0;
}

void MLB::Text::DrawTextCursor(GUI::Canvas *_canvas) {
  if (!visible || !font) return;

  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos != text.end() || firstPos) {
    if (*actPos) {
      if (!firstPos)
	(*actPos)->DrawTextCursor(_canvas, group->GetID());
      else
	_canvas->DrawTextCursor(group->GetID(),
				SessionManager::sm->GetZoom() * x,
				SessionManager::sm->GetZoom() * (y-ascent),
				0,
				SessionManager::sm->GetZoom() * (ascent+descent),
				color.c_str());
    }
  }
  else
    _canvas->DrawTextCursor(group->GetID(),
			    SessionManager::sm->GetZoom() * (x+width),
			    SessionManager::sm->GetZoom() * (y-ascent),
			    0,
			    SessionManager::sm->GetZoom() * (ascent+descent),
			    color.c_str());
}


void MLB::Text::DeleteTextCursor(GUI::Canvas *_canvas) {
  if (!visible || !font) return;

  _canvas->DeleteTextCursor(group->GetID());
}


void MLB::Text::SaveObject(FILE *fp, int level) {
  std::list<MLB::Char *>::iterator
    temp;
  int
    i;
  char
    *tab;

  if (!visible || !font) return;
  if (text.empty()) return;

  tab = new char[level*2+1];
  *tab = '\0';
  for (i=0; i<level; i++)
    strcat(tab,"  ");

  fprintf(fp,"%s<TEXT x=\"%d\" y=\"%d\" color=\"%s\" size=\"%s\" family=\"%s\" weight=\"%s\" slant=\"%s\" position=\"%s\" length=\"%d\"%s>\n",
	  tab,x,y,color.c_str(),fontSize.c_str(),fontFamily.c_str(),fontWeight.c_str(),fontSlant.c_str(),textPosition.c_str(),text.size(), GetAnimationStepSGMLText().c_str() );
  fprintf(fp,"%s  \"",tab);

  temp = text.begin();
  while (temp != text.end()) {
    fprintf (fp,"%c",(*temp)->ascii);
    temp++;
  }

  fprintf(fp,"\"\n");
  fprintf(fp,"%s</TEXT>\n",tab);

  delete tab;
}

char *MLB::Text::CalculatePSFontname() {
  char
    *ret;

  if (fontFamily == "arial") {
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica-Oblique");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-BoldOblique");
      }
    }
  }
  else if (fontFamily == "times") {
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("Times-Roman");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Times-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("Times-Italic");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Times-BoldItalic");
      }
    }
  }
  else if (fontFamily == "arialn") {
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica-Narrow");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-Narrow-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica-Narrow-Oblique");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-Narrow-BoldOblique");
      }
    }
  }
  else if (fontFamily == "lucidatypewriter") {
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("LucidaSans-Roman");
      }
      else if (fontWeight == "bold") {
	ret = strdup("LucidaSans-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("LucidaSans-Italic");
      }
      else if (fontWeight == "bold") {
	ret = strdup("LucidaSans-BoldItalic");
      }
    }
  }
  else if (fontFamily == "symbol") {
    ret = strdup("Symbol");
  }

  return ret;
}

void MLB::Text::SaveAsPs(FILE *fp, int pageWidth, int pageHeight, bool rotate) {

  std::list<MLB::Char *>::iterator
    temp;
  XColor
    *xColor;
  float
    red, green, blue;
  int
    psX, psY;
  char
    *aofFont;

  if (!visible || text.empty() || !font) return;

  xColor = Tk_GetColor(Et_Interp,Tk_MainWindow(Et_Interp),(char *)color.c_str());
  red   = (float)xColor->red/65535;
  green = (float)xColor->green/65535;
  blue  = (float)xColor->blue/65535;

  if (rotate) {
    psX = y+offY;
    psY = x;
  }
  else {
    psX = x;
    psY = pageHeight-(y+offY);
  }

  fprintf(fp,"\n");
  fprintf(fp,"gsave\n");
  fprintf(fp,"%2.3f %2.3f %2.3f setrgbcolor\n",red,green,blue);
  aofFont = CalculatePSFontname();
  if (fontFamily == "symbol")
    fprintf(fp,"/%s findfont %d scalefont setfont\n",aofFont,(font->GetSize() / SessionManager::sm->GetZoom()-4));
  else
    fprintf(fp,"/%s findfont %d scalefont ISOEncode setfont\n",aofFont,(font->GetSize() / SessionManager::sm->GetZoom()-4));
  delete[] aofFont;
  fprintf(fp,"%d %d moveto\n",psX,psY);
  if (rotate)
    fprintf(fp,"90 rotate\n");
  fprintf(fp,"(");
  temp = text.begin();
  while (temp != text.end()) {
    if (((*temp)->ascii == '(') || ((*temp)->ascii == ')')) {
      fprintf(fp,"\\");
      fprintf(fp,"%c",(*temp)->ascii);
    }
    if ((*temp)->ascii == '')
      fprintf(fp,"");
    else if ((*temp)->ascii == '')
      fprintf(fp,"");
    else if ((*temp)->ascii == '')
      fprintf(fp,"");
    else
      fprintf(fp,"%c",(*temp)->ascii);
    temp++;
  }
  fprintf(fp,") ");
  fprintf(fp,"[");
  temp = text.begin();
  while (temp != text.end()) {
    fprintf(fp,"%d ",(*temp)->GetWidth());
    temp++;
  }
  fprintf(fp,"] ");
  fprintf(fp,"xshow\n");
  fprintf(fp,"grestore\n");

}


char *MLB::Text::CalculateFontname(int &size) {
  char
    *ret;

  if (fontFamily == "arial") {
    size -= 4;
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica-Oblique");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-BoldOblique");
      }
    }
  }
  else if (fontFamily == "times") {
    size -= 4;
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("Times-Roman");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Times-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("Times-Italic");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Times-BoldItalic");
      }
    }
  }
  else if (fontFamily == "arialn") {
    size -= 5;
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("Helvetica-Oblique");
      }
      else if (fontWeight == "bold") {
	ret = strdup("Helvetica-BoldOblique");
      }
    }
  }
  else if (fontFamily == "lucidatypewriter") {
    size -= 6;
    if (fontSlant == "roman") {
      if (fontWeight == "normal") {
	ret = strdup("LucidaSans-Roman");
      }
      else if (fontWeight == "bold") {
	ret = strdup("LucidaSans-Bold");
      }
    }
    else if (fontSlant == "italic") {
      if (fontWeight == "normal") {
	ret = strdup("LucidaSans-Italic");
      }
      else if (fontWeight == "bold") {
	ret = strdup("LucidaSans-BoldItalic");
      }
    }
  }
  else if (fontFamily == "symbol") {
    size -= 6;
    ret = strdup("Symbol");
  }

  return ret;
}


void MLB::Text::SaveAsAOF(FILE *_fp)
{
  std::list<MLB::Char *>::iterator
    temp;
  char
    *aofFont;
  int
    offY,
    size;

   GUI::MessageWindow::Debug("1 Text::SaveAsAOF font %d visible %d und recorded %d\n",font,visible,recorded);

  if (!visible || !font || recorded) return;

  recorded = true;

  offY = 0;
  if (textPosition == "up") {
    size = atoi(fontSize.c_str());
    offY += -size/2;
    size = (2*size)/3;
  }
  else if (textPosition == "down") {
    size = atoi(fontSize.c_str());
    offY += size/3;
    size = (2*size)/3;
  }
  else  {
    size = atoi(fontSize.c_str());
  }

  recID = MLB::SessionManager::sm->recCount;
  MLB::SessionManager::sm->recCount++;

  aofFont = CalculateFontname(size);

  fprintf(_fp,"<TEXT x=%d y=%d width=%d height=%d>\n",x,y,width,height);
  // notwendig, damit der aofJSync die AOF-Recordings abspielen kann.
  fprintf(_fp,"<TEXTITEM x=%d y=%d font=%s size=%d ccolor=%d weight=%s slant=%s ttf=%s layer=%d>\n",x,y+offY,aofFont,size,CalculateColor(color),fontWeight.c_str(),fontSlant.c_str(),font->GetFilename(),layer);

  delete[] aofFont;

  temp = text.begin();
  while (temp != text.end()) {
    if ((*temp)->ascii == '<') {
      fprintf (_fp,"%c",'\\');
      fprintf (_fp,"%c",(*temp)->ascii);
    }
    else if ((*temp)->ascii == '>') {
      fprintf (_fp,"%c",'\\');
      fprintf (_fp,"%c",(*temp)->ascii);
    }
    else if ((*temp)->ascii == '\\') {
      fprintf (_fp,"%c",'\\');
      fprintf (_fp,"%c",(*temp)->ascii);
    }
    else if ((*temp)->ascii == '')
      fprintf(_fp,"");
    else if ((*temp)->ascii == '')
      fprintf(_fp,"");
    else if ((*temp)->ascii == '')
      fprintf(_fp,"");
    else if ((*temp)->ascii == '')
      fprintf(_fp,"");
    else
      fprintf (_fp,"%c",(*temp)->ascii);
    temp++;
  }
  fprintf(_fp,"\n");
  fprintf(_fp,"</TEXTITEM>\n");
  // notwendig, damit der aofJSync die AOF-Recordings abspielen kann.
  fprintf(_fp,"</TEXT>\n");
}

int MLB::Text::RecordableObjects()
{
  if (!font) visible = false;
  if (!visible) return 0;

  return 1;
}

void MLB::Text::SaveAsAOF(FILE *_fp, int _line, int _offX, int _offY)
{
  std::list<MLB::Char *>::iterator
    temp;
  char
    *aofFont;
  int
    offY,
    size;

     GUI::MessageWindow::Debug("2 Text::SaveAsAOF font %d visible %d und recorded %d\n",font,visible,recorded);


  if (!visible || !font || recorded) return;

  recorded = true;

  offY = 0;
  if (textPosition == "up") {
    size = atoi(fontSize.c_str());
    offY += -size/2;
    size = (2*size)/3;
  }
  else if (textPosition == "down") {
    size = atoi(fontSize.c_str());
    offY += size/3;
    size = (2*size)/3;
  }
  else  {
    size = atoi(fontSize.c_str());
  }

  aofFont = CalculateFontname(size);

  fprintf(_fp,"<TEXTITEM x=%d y=%d offx=%d offy=%d line=%d font=%s size=%d ccolor=%d weight=%s slant=%s ttf=%s layer=%d>\n",x,y,_offX,_offY+offY,_line,aofFont,size,CalculateColor(color),fontWeight.c_str(),fontSlant.c_str(),font->GetFilename(),layer);

  delete[] aofFont;

  temp = text.begin();
  while (temp != text.end()) {
    if ((*temp)->ascii == '<') {
      fprintf (_fp,"%c",'\\');
      fprintf (_fp,"%c",(*temp)->ascii);
    }
    else if ((*temp)->ascii == '>') {
      fprintf (_fp,"%c",'\\');
      fprintf (_fp,"%c",(*temp)->ascii);
    }
    else if ((*temp)->ascii == '\\') {
      fprintf (_fp,"%c",'\\');
      fprintf (_fp,"%c",(*temp)->ascii);
    }
    else if ((*temp)->ascii == '')  {
      fprintf(_fp,"");
    }
    else if ((*temp)->ascii == '')  {
      fprintf(_fp,"");
    }
    else if ((*temp)->ascii == '')  {
      fprintf(_fp,"|");
    }
    else
      fprintf (_fp,"%c",(*temp)->ascii);
    temp++;
  }
  fprintf(_fp,"\n");
  fprintf(_fp,"</TEXTITEM>\n");

}


void MLB::Text::ChangeLineColor(GUI::Canvas *_canvas, char *_color) {
  if (!visible || text.empty()) return;

  recorded = false;

  color = _color;
  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    (*i)->ChangeColor(_canvas,_color);
}

void MLB::Text::AdjustGui() {
  GUI::Attributes::SetOutlineColor((char *)color.c_str());
  GUI::Attributes::SetFont(GetFamily(),GetSize(),GetWeight(),GetSlant(),GetPosition());
  GUI::Attributes::SetTool("Text");
}


MLB::Text* MLB::Text::Split(GUI::Canvas *_canvas) {
  if (!visible || !font || firstPos) return NULL;

  char *p, *q, zw;
  int newX, newY;

  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos == text.end()) {
    return NULL;
  }

  p = strdup(GetString());
  q = p+GetPos();

  if (strlen(q) == 0) {
    free(p);
    return NULL;
  }

  zw = *q;
  *q = '\0';
  ChangeString(_canvas,p);
  *q = zw;
  newX = x+GetWidth();
  newY = y;
  Text* ret = new MLB::Text(newX,newY,0,0,GetColor(),q,GetFamily(),GetSize(),GetWeight(),
		      GetSlant(),GetPosition(),CommunicationManager::cm->GetObjectID());

  ret->firstPos = true;
  ret->SetAnnotation(SessionManager::sm->AnnotationMode());
  free(p);


  return ret;
}


void MLB::Text::Split(char** first, char** last) {
  char* q, *p, zw;

  std::list<Char*>::iterator actPos = GetActiveCharIterator();
  if (actPos == text.end() || firstPos) {
    return;
  }
  
  p = strdup(GetString());
  q = p + GetPos();
  
  if (strlen(q) == 0) {
    *first = p;
    return; // first is identical to original string; last is NULL
  }
  
  zw = *q;
  *q = '\0';
  *first = strdup(p);
  *q = zw;
  *last  = strdup(q);
  
  delete[] p;
}


void MLB::Text::SetActive(int idx) {
  if ((idx < 0) || (idx >= text.size())) return;

  std::list<Char*>::iterator actPos = text.begin();
  for (int i = 0; i < idx; ++i) ++actPos;
  activeChar = (*actPos)->GetID();
}


void MLB::Text::FlushFontWeight(char* buf, std::string& fontWeight) {
  uint8*  p8  = (uint8*)buf;
  if (fontWeight.compare("normal") == 0)
    p8[0] = p8[0] | (0 << 6);
  else if (fontWeight.compare("bold") == 0)
    p8[0] = p8[0] | (1 << 6);
}


std::string MLB::Text::ParseFontWeight(char* buf) {
  uint8*  p8  = (uint8*)buf;
  switch ((p8[0] & MLB_MASK_FWG) >> 6) {
  case 0: return "normal";
  case 1: return "bold";
  default: return "";
  }
}


void MLB::Text::FlushFontSlant(char* buf, std::string& fontSlant) {
  uint8*  p8  = (uint8*)buf;
  if (fontSlant.compare("roman") == 0)
    p8[0] = p8[0] | (0 << 4);
  else if (fontSlant.compare("italic") == 0)
    p8[0] = p8[0] | (1 << 4);
}


std::string MLB::Text::ParseFontSlant(char* buf) {
  uint8*  p8  = (uint8*)buf;
  switch ((p8[0] & MLB_MASK_FSL) >> 4) {
  case 0: return "roman";
  case 1: return "italic";
  default: return "";
  }
}


void MLB::Text::FlushFontPosition(char* buf, std::string& fontPosition) {
  uint8*  p8  = (uint8*)buf;
  if (fontPosition.compare("normal") == 0)
    p8[0] = p8[0] | (0 << 2);
  else if (fontPosition.compare("up") == 0)
    p8[0] = p8[0] | (1 << 2);
  else if (fontPosition.compare("down") == 0)
    p8[0] = p8[0] | (2 << 2);
}


std::string MLB::Text::ParseFontPosition(char* buf) {
  uint8*  p8  = (uint8*)buf;
  switch ((p8[0] & MLB_MASK_FPS) >> 2) {
  case 0: return "normal";
  case 1: return "up";
  case 2: return "down";
  default: return "";
  }
}


MLB::DrawObject *MLB::Text::Load(SGMLElement *hilf) {
  Text
    *obj;
  char
    *tmp;
  int
    x, y,
    width, height;
  char
    *color,
    *size,
    *family,
    *weight,
    *slant,
    *position;

  tmp = hilf->GetAttribute("x");
  if (tmp) x = atoi(tmp);
  else     x = 0;
  tmp = hilf->GetAttribute("y");
  if (tmp) y = atoi(tmp);
  else     y = 0;
  tmp = hilf->GetAttribute("width");
  if (tmp) width = atoi(tmp);
  else     width = 0;
  tmp = hilf->GetAttribute("height");
  if (tmp) height = atoi(tmp);
  else     height = 0;
  tmp = hilf->GetAttribute("color");
  if (tmp) color = strdup(tmp);
  else     color = strdup("#ff0000");
  tmp = hilf->GetAttribute("size");
  if (tmp) size = strdup(tmp);
  else     size = strdup("20");
  tmp = hilf->GetAttribute("family");
  if (tmp) family = strdup(tmp);
  else     family = strdup("times");
  tmp = hilf->GetAttribute("weight");
  if (tmp) weight = strdup(tmp);
  else     weight = strdup("normal");
  tmp = hilf->GetAttribute("slant");
  if (tmp) slant = strdup(tmp);
  else     slant = strdup("roman");
  tmp = hilf->GetAttribute("position");
  if (tmp) position = strdup(tmp);
  else     position = strdup("normal");

  obj = new Text(x,y,width,height,color,(char *)hilf->Parameter.c_str(),family,size,weight,slant,position,
		 CommunicationManager::cm->GetObjectID());
  obj->SetLayer(0);
//  GUI::MessageWindow::Error("Set Layer = 0\n");

  delete[] color;
  delete[] size;
  delete[] family;
  delete[] weight;
  delete[] slant;
  delete[] position;

  return obj;
}

/*
void MLB::Text::SetVisible()
{
  ColoredObject::SetVisible();

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    (*i)->SetVisible();
}


void MLB::Text::SetInvisible()
{
  ColoredObject::SetInvisible();

  for (std::list<Char*>::iterator i = text.begin(); i != text.end(); ++i)
    (*i)->SetInvisible();
}
*/
