/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_timermanager.cpp
 authors : Juergen Vogel
 date    : 
 desc    : 

**********************************************************************************************************/

#ifdef WIN32
#  pragma warning (disable: 4786 4181)
#endif

#include <mlb.h>
#include <mlb_gui.h>
#include <mlb_timermanager.h>
#include <mlb_communication.h>

//#define DEBUG

//
// Functions from class TimerManager
//
MLB::TimerManager* MLB::TimerManager::tm = NULL;


void MLB::ExecuteTimer(ClientData data) {
  TimerManager::tm->ExecuteTimer((TimerObject*)data);
}


void MLB::TimerManager::ExecuteTimer(TimerObject* expiredTimer) {
  // delete expired timer from timerList
  for (std::list<TimerObject*>::iterator i = timerList.begin(); i != timerList.end(); ++i) 
    if (((*i)->type == expiredTimer->type) && ((*i)->id == expiredTimer->id)) {
      timerList.erase(i);
      break;
    }

 expiredTimer->toCall->ExecuteTimer(expiredTimer);
}


bool MLB::TimerManager::SetTimer(TimerFeedbackInterface* toCall, TimerObject* timer) {
  if (timer->runningTime < 5) {
    // execute timer immediately
    #ifdef DEBUG
    GUI::MessageWindow::Info("TimerManager::SetTimer - Deliver Timer at once %u\n", timer->runningTime);
    #endif
    toCall->ExecuteTimer(timer);
    return true;
  }

  #ifdef DEBUG
  GUI::MessageWindow::Info("TimerManager::SetTimer - Set Timer %u\n", timer->runningTime);
  #endif
  timer->toCall = toCall;
  timer->token = Tcl_CreateTimerHandler(timer->runningTime, MLB::ExecuteTimer, timer);
  timerList.push_back(timer);
  return true;
}


bool MLB::TimerManager::TimerRunning(TimerFeedbackInterface* toCall, int type) {
  for (std::list<TimerObject*>::iterator i = timerList.begin(); i != timerList.end(); ++i)
    if (((*i)->toCall == toCall) && ((*i)->type == type))
      return true;

  return false;
}


bool MLB::TimerManager::TimerRunning(TimerFeedbackInterface* toCall, int type, uint32 id) {
  for (std::list<TimerObject*>::iterator i = timerList.begin(); i != timerList.end(); ++i)
    if (((*i)->toCall == toCall) && ((*i)->type == type) && ((*i)->id == id))
      return true;

  return false;
}


bool MLB::TimerManager::DeleteTimer(TimerFeedbackInterface* toCall, int type) {
  for (std::list<TimerObject*>::iterator i = timerList.begin(); i != timerList.end(); ++i)
    if (((*i)->toCall == toCall) && ((*i)->type == type)) {
      Tcl_DeleteTimerHandler((*i)->token);
      delete *i;
      timerList.erase(i);
      return true;
    }

  return false; // timer not found
}


bool MLB::TimerManager::DeleteTimer(TimerFeedbackInterface* toCall, int type, uint32 id) {
  for (std::list<TimerObject*>::iterator i = timerList.begin(); i != timerList.end(); ++i)
    if (((*i)->toCall == toCall) && ((*i)->type == type) && ((*i)->id == id)) {
      Tcl_DeleteTimerHandler((*i)->token);
      delete *i;
      timerList.erase(i);
      return true;
    }

  return false; // timer not found
}


void MLB::TimerManager::DeleteAllTimer() {
  for (std::list<TimerObject*>::iterator i = timerList.begin(); i != timerList.end(); ++i) {
    Tcl_DeleteTimerHandler((*i)->token);
    delete *i;
  }

  timerList.clear();
}
