/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_timermanager.h
 authors : Juergen Vogel
 date    : 
 desc    : 

***********************************************************************************************************/

#ifndef __MLB_TIMERMANAGER_H__
#define __MLB_TIMERMANAGER_H__

#include <util/types.h>


#include <tcl.h>

#include <list>

#ifdef WIN32
#  pragma warning (disable: 4786 4181)
#endif

namespace MLB {

  class TimerObject;
  class TimerManager;

  class TimerFeedbackInterface {
  public:
    virtual void ExecuteTimer(TimerObject* obj) = 0;
  };


  class TimerObject {
  public:
    enum types {
      INIT,
      STATE_QUERY,
      STATE_REPLY,
      ADU_DUE,
      TW_CLEAN,
      FULL_ACTIVENESS,
      HALF_ACTIVENESS,
      UNDO,
      HR_INDICATE
    };
    
    TimerObject(uint32 _runningTime, int _type) 
      : runningTime(_runningTime), type(_type), id(0), priority(0) {}
    TimerObject(uint32 _runningTime, int _type, uint32 _id, int _priority) 
      : runningTime(_runningTime), type(_type), id(_id), priority(_priority) {}
    
    Tcl_TimerToken token;
    uint32 runningTime;
    int type;
    uint32 id;
    int priority;

  protected:
    friend class TimerManager;
    TimerFeedbackInterface* toCall; 
  };
  

  class TimerManager {
  public:
    TimerManager() {}
    ~TimerManager() {DeleteAllTimer();}

    bool SetTimer(TimerFeedbackInterface* toCall, TimerObject* timer);

    bool TimerRunning(TimerFeedbackInterface* toCall, int type);
    bool TimerRunning(TimerFeedbackInterface* toCall, int type, uint32 id);

    bool DeleteTimer(TimerFeedbackInterface* toCall, int type);
    bool DeleteTimer(TimerFeedbackInterface* toCall, int type, uint32 id);

    void ExecuteTimer(TimerObject* timer);

    static TimerManager* tm;
  private:
    void DeleteAllTimer();

    std::list<TimerObject*> timerList;
  };

  void ExecuteTimer(ClientData data); 

#ifdef WIN32
  using std::list<TimerObject*>;
#endif
}



#endif
