/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : mlb_tkphoto.h
 authors : Andrius Kurtinaitis
 date    : 13.07.2001
 desc    : An OO wrapper for Tk_PhotoImageBlock struct

***********************************************************************************************************/

#include <Magick++.h>

#include "mlb_session.h"
#include "mlb_objects.h"
#include "mlb_adu.h"
#include "mlb_misc.h"

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/

/**   Tk image block with de- and ini-tializers and some useful functions.
  *
  **/
struct TkImageBlock : public Tk_PhotoImageBlock
{
  bool pixelOwner;

  /**   Creates block using image object. Allocates all needed structures.
    *   Inverts the transparency for use with Tk.
    **/
  TkImageBlock( Magick::Image& image )
    :pixelOwner( true )
  {
		Magick::Geometry	size( image.size() );

		// attributes
    width      = size.width();
    height     = size.height();
    pixelSize  = 4;
    pitch      = width * pixelSize;

    // color offsets
    for( int i=0; i<pixelSize; i++ ) offset[i] = i;

    // pixels
    pixelPtr = new unsigned char[ width*height*pixelSize ];
    image.write( 0, 0, width, height, "RGBA", (MagickLib::StorageType)0, pixelPtr );
    invertTransparency();

  } // TkImageBlock

  //---------------------------------------------------------------------//

  /**   Initializes block from Tk.
    *   Inverts the transparency for use with ImageMagick.
    **/
  TkImageBlock( string& imgTag )
    :pixelOwner( false )
  {
     Tk_PhotoHandle handle = Tk_FindPhoto( Et_Interp, (char *)imgTag.c_str() );
     if( !handle ) throw string("No photo object could be found by following tag: ") + imgTag;
     Tk_PhotoGetImage( handle, this );
     invertTransparency();

  } // TkImageBlock()

  //---------------------------------------------------------------------//

  /**   Passes the block to Tk.
    **/
  void putBlock( string& imgTag )
  {
    GUI::Image::Create( imgTag.c_str() );
    Tk_PhotoHandle handle = Tk_FindPhoto( Et_Interp, (char*)imgTag.c_str() );
    if( !handle ) throw string("No photo object could be found by following tag: ") + imgTag;
    Tk_PhotoPutBlock( handle, this, 0, 0, width, height, TK_PHOTO_COMPOSITE_SET);

  } // putBlock()

  //---------------------------------------------------------------------//

  /**   Inverts transparency values of this block (255-val).
    *
    *   NOTE: This is a dirty implementation useful only with "RGBARGBA..."
    *   storage scheme. All the read and write mathods use also this
    *   method...
    **/
  void invertTransparency()
  {
    if( 4==pixelSize )
    {
      for( int i=0; i<width*height; i++ )
      {
        unsigned char* off = pixelPtr + i*4 + offset[3];
        *(off) = 255 - *(off);
      } // for
    } // if

  } // invertTransparency()

  //---------------------------------------------------------------------//

  virtual ~TkImageBlock() { if(pixelOwner) delete[] pixelPtr; } // ~TkImageBlock()

}; // struct TkImageBlock

//=========================================================================//
