/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#include "mlb_session.h"
#include "mlb_page.h"
#include "mlb_communication.h"
#include "mlb_objects.h"
#include "mlb_uce.h"

MLB::UceManager* MLB::UceManager::ucem = NULL;
bool MLB::UceManager::connected = false;

void MLB::UceManager::setTelePointer(float xRel, float yRel, bool pressed) {
  if (SessionManager::sm->GetActivePage()) {
    PhotoImage* image = SessionManager::sm->GetActivePage()->GetFirstImage();
    if (!image) return;

    
    const char ** argv0 = new const char*[3];
    argv0[0] = "";
    argv0[1] = "pointer_ro";
    argv0[2] = "#ff0000";
    SessionManager::sm->GetActivePage()->SetWindowPointer(3, argv0);
    delete[] argv0;

    const char** argv = new const char*[5];
    argv[0] = "";
    char one[] = "1", ooh[] = "0";
    argv[1] = (pressed) ? one : ooh;
    char tmp1[5];
    float x = (image->GetWidth() * xRel + image->GetX()) * SessionManager::sm->GetZoom();
    sprintf(tmp1, "%u", (uint32)x); 
    argv[2] = tmp1;
    char tmp2[5];
    float y = (image->GetHeight() * yRel + image->GetY()) * SessionManager::sm->GetZoom();
    sprintf(tmp2, "%u",  (uint32)y); 
    argv[3] = tmp2;
    argv[4] = "#ff0000";
    SessionManager::sm->GetActivePage()->ActivateOnlinePointer(5, argv);
    delete[] argv;
  }
}

void MLB::UceManager::setActivePage(uint32 id) {
  if (id == 1) return;
  Page* page = (Page*)SessionManager::sm->GetObject(id);
  if (!page) return;

  if (page == SessionManager::sm->GetActivePage()) return; // page already active

  if (SessionManager::sm->GetActivePage()) {
    if (SessionManager::sm->GetParentSession(SessionManager::sm->GetActivePage())->GetSessionType() == Session::SHARED) {
      CommunicationManager::cm->DeactivateAllSubcomponents();
    }
    SessionManager::sm->GetActivePage()->DeActivate();
  }

  SessionManager::sm->SetGlobalPage(page->GetID());
  ((Folder*)SessionManager::sm->GetParentContainer(page->GetID()))->SetCurrentPage(page);
  SessionManager::sm->DrawPageTree();

  SessionManager::sm->RemoteSelectContainer(page->GetNodePath());
  SessionManager::sm->ClearPageTree();
  SessionManager::sm->DrawPageTree();

  CommunicationManager::cm->GetRtpi()->send_rtcpi_note();
}

