/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifndef __MLB_UNDOMANAGER__
#define __MLB_UNDOMANAGER__


#include <list>
#include <set>

#include "tk.h"
#include "tcl.h"

#include "mlb_generic.h"
#include "mlb_adu.h"
#include <util/types.h>
#include <util/Buffer.h>


#ifdef WIN32
#  pragma warning (disable: 4786)
#endif


namespace MLB {

  typedef std::list<RefCount<RTPI::RtpiAduList> > UndoAduList;
  typedef std::set<uint32> AnnotationSet;

  class UndoManager : public TimerFeedbackInterface {
  public:
    UndoManager() : undoList(new UndoAduList()), redoList(new UndoAduList()), annotationSet(new AnnotationSet()) {}
    
    ~UndoManager() { 
      ClearList(undoList); 
      ClearList(redoList); 
      annotationSet->clear();
    }

    void InsertAdu(RefCount<RTPI::RtpiAdu> adu); // insert end undolist, delete redolist
    void InsertAduList(RefCount<RTPI::RtpiAduList> aduList);

    void DeleteAdu(uint32 id);
    void DeleteAdus(uint32 parentID);

    void SetAnnotation(uint32 id) { annotationSet->insert(id); }
    bool GetAnnotation(uint32 id) { 
      bool find = (annotationSet->find(id) != annotationSet->end()); 
      if (find) annotationSet->erase(id); 
      return find;
    }

    RefCount<RTPI::RtpiAdu> CreateDelete(RefCount<RTPI::RtpiAdu> adu); 
    RefCount<RTPI::RtpiAdu> CreateInverse(RefCount<RTPI::RtpiAdu> adu); 
    
    RefCount<RTPI::RtpiAdu> CreateStateInverse(RefCount<RTPI::RtpiAdu> adu); 
    RefCount<RTPI::RtpiAdu> CreateEventInverse(RefCount<RTPI::RtpiAdu> adu); 
    
    void Undo(); // get end undolist, insert end redolist
    void Redo(); // get end redolist, insert end undolist

    void ExecuteTimer(TimerObject* timer);

    static UndoManager* um;

  private:
    void ClearList(RefCount<UndoAduList> aduList);
    void SetUndoState(bool active);
    void SetRedoState(bool active);

    void ChangeAduID(RefCount<RTPI::RtpiAdu> adu, uint32 id, uint32 newID); 
    void ChangeAduID(uint32 id, uint32 newID); 
    
    RefCount<UndoAduList>   undoList;
    RefCount<UndoAduList>   redoList;
    RefCount<AnnotationSet> annotationSet;
  };
}

#endif
