/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#include <string>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "sgml.h"
#include "mlb_gui.h"

#define BLANK 32

Attribute::Attribute() 
{
}

Attribute::~Attribute() 
{
  if (!Name.empty()) Name.erase();
  if (!Value.empty()) Value.erase();
}

SGMLElement::SGMLElement()
{
  closed     = false; 

  parent     = NULL;
  son        = NULL;
  prev       = NULL;
  next       = NULL;
}

SGMLElement::~SGMLElement()
{
  std::list<Attribute *>::iterator 
    temp;

  if (!Name.empty()) Name.erase();
  if (!Parameter.empty()) Parameter.erase();

  if (!attributes.empty()) {
    temp = attributes.begin();
    while (temp != attributes.end()) {
      delete(*temp);
      temp++;
    }
    attributes.clear();
  }
}


void SGMLElement::ParseString(std::string strn)
{
  Attribute
    *att;
  size_t
    pos1,
    pos2;

  pos1 = 0;
  pos2 = strn.find(" ",pos1,1);
  if (pos2 > strn.size()) {
    pos2 = strn.size();
  }
  Name = strn.substr(pos1,(pos2-pos1));
  //printf("TagName :%s:\n",(char *)Name.c_str());

  while ((pos2!=strn.size()) && (pos2!=std::string::npos)) {
    att = new Attribute();
    attributes.insert(attributes.end(),att);
    pos1 = strn.find(" ",pos2,1);
    if (pos1 == std::string::npos) break;
    pos2 = strn.find("=",pos1,1);
    pos1++;
    att->Name = strn.substr(pos1,(pos2-pos1));
    //printf("Name :%s:\n",(char *)att->Name.c_str());
    pos2++;
    pos1 = strn.find("\"",pos2,1);
    pos1++;
    pos2 = strn.find("\"",pos1,1);
    att->Value = strn.substr(pos1,(pos2-pos1));
    if (att->Name=="length") {
      parameterLength = atoi((char *)att->Value.c_str());
    }
    //printf("Value :%s:\n",(char *)att->Value.c_str());
  }

}

char *SGMLElement::GetAttribute(char *att) 
{
  std::list<Attribute *>::iterator 
    temp;

  temp = attributes.begin();
  while (temp != attributes.end()) {
    if ((*temp)->Name == att)
      return (char *)(*temp)->Value.c_str();
    temp++;
  }
  
  return NULL;
}

int SGMLElement::verify(char *strn)
{
  char
    *p;

  p = strn;
  if (*p == '/') p++;

  if (Name != p) {
    printf ("Fehler3: %s != %s\n",p,Name.c_str());
    return 0;
  }

  return 1;
}

SGMLTree::SGMLTree(char *sfilename)
{
  root = NULL;
  filename = strdup(sfilename);
  Buildtree();
}

SGMLTree::~SGMLTree()
{
  SGMLElement
    *tmp;

  free (filename);

  tmp = root;

  while (tmp->next) 
    tmp = tmp->next;

  while (tmp) {
    if (tmp->son) {
      tmp = tmp->son; 
      while (tmp->next) 
	tmp = tmp->next;
    }
    else {
      if (tmp->prev) {
	tmp = tmp->prev;
	delete tmp->next;
	tmp->next = NULL;
      }
      else if (tmp->parent) {
	tmp = tmp->parent;
	delete tmp->son;
	tmp->son = NULL;
      }
      else {
	delete tmp;
	tmp = NULL;
	root = NULL;
      }
    }
  }
}

SGMLElement *SGMLTree::EvaluateTag(SGMLElement *hilf, char *str)
{
  std::string
    zw;

  zw = str;

  if (hilf) {
    if (zw[0]=='/') {
      if (hilf->closed) {
	while (hilf->prev) hilf = hilf->prev;
	if (hilf->parent) {
	  hilf = hilf->parent;
	}
	else printf ("FEHLER parent\n");
      }
      hilf->verify((char *)zw.c_str());
      hilf->closed = true;
    }
    else {
      if (hilf->closed) {
	hilf->next = new SGMLElement();
	hilf->next->prev = hilf;
	hilf->next->parent = hilf->parent;
	hilf = hilf->next;
      }
      else {
	hilf->son = new SGMLElement();
	hilf->son->parent = hilf;
	hilf = hilf->son;
      }
      hilf->ParseString(zw);
    }
  }
  else {
    hilf = new SGMLElement();
    root = hilf;
    hilf->ParseString(zw);
  }

  zw.erase();

  return hilf;
}
	
void SGMLTree::Buildtree()
{
  SGMLElement
    *hilf,
    *hilf2;
  FILE
    *datei;
  long
    abs_count, last_value;
  int
    i, j,
    ident;
  std::string
    str;
  char 
    head[10],
    chr[2];

  datei = fopen(filename,"rb");
  if (datei == NULL) { 
    perror("can't read File");
    fprintf(stderr,"could not open %s\n",filename);
    return;
  }

  fread(head,1,4,datei);
  if (strncmp(head,"<MLB",4) != 0)  { 
    GUI::MessageWindow::Error("no legal Document\n");
    return;
  }

  fseek(datei,0,SEEK_END);  
  abs_count = ftell(datei);
  fseek(datei,0,SEEK_SET);
  last_value = 0;

  root = NULL;
  hilf = NULL;
  
  i = 0;
  str.erase();
  ident = 1;
  i = 0;
  while (fread(chr,1,1,datei)) {
    if ((*chr =='\\') && (ident==2)) {
      fread(chr,1,1,datei);
      str.append(1,*chr);
      i++;
    }
    else if ((*chr=='<') && (ident==1)) {
      ident = 0;
      str.erase();
      i = 0;
    }
    else if ((*chr=='>') && (ident==0)) {
      ident = 1;
      hilf2 = hilf;
      hilf = EvaluateTag(hilf2,(char *)str.c_str());
    }
    else if ((*chr=='"') && (ident==1)) {
      ident = 2;
      str.erase();
      for (j=0; j<hilf->parameterLength; j++) {
	fread(chr,1,1,datei);
	hilf->Parameter.append(1,*chr);
      }
      fread(chr,1,1,datei);
      if (*chr!='"') 
	printf ("FEHLER %c\n",*chr);
      ident = 1;
      i = 0;
    }
    else if ((ident==0) || (ident==2)){
      str.append(1,*chr);
      i++;
    }
  }
  
  str.erase();
  fclose(datei);
}            


