/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************/

#ifndef __SGML__
#define __SGML__

#ifndef WIN32
#  pragma warning (disable: 4786)
#  include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <list>
#include <string>

class Attribute
{
 public: 
  std::string
    Name,
    Value;
  
  Attribute();
  ~Attribute();
};

class SGMLElement
{
 public:
  std::string
    Name,
    Parameter;
  int 
    parameterLength;
  std::list<Attribute *>
    attributes;
  std::list<Attribute *>::iterator
    current;
  bool
    closed;
  SGMLElement
    *parent, *son,
    *prev, *next;

  SGMLElement();
  ~SGMLElement();
  void ParseString(std::string strn);
  char *GetAttribute(char *att);
  void AddParameter(char *str, int length);
  int verify(char *strn);
};

class SGMLTree
{
 public:
  SGMLElement
    *root;
  char 
    *filename;

  SGMLTree(char *sfilename);
  ~SGMLTree();
  void Buildtree();
  SGMLElement *EvaluateTag(SGMLElement *hilf, char *str);
  
};


#endif
