/***********************************************************************************************************
 * mlb - multimedia lecture board 
 * Copyright 1999 - 2003
 * 
 * University of Mannheim, Germany
 * University of Freiburg, Germany
 *
 * Development:
 * Leader: Juergen Vogel
 * Team Mannheim: Juergen Vogel, Marcel Busse, Andrius Kurtinaitis, and Werner Geyer.
 * Team Freiburg: Gabriela Maass, Jochen Lienhard, and Tobias Lauer.
 *
 * Development was sponsored by DFN (ANETTE project).
 *
 * License Information:
 * The mlb uses the following libraries: Tcl/Tk, BWidgets, ImageMagick, FreeType, RTP/I and SMP
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of 
 * the GNU General Public License as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * The GNU General Public License can be obtained under http://www.gnu.org/licenses/gpl.html
 *
 * The Universities of Mannheim and Freiburg will appreciate receiving any enhancements and 
 * modifications of this software made by others.
 *
 * Contact:
 * Juergen Vogel - vogel@informatik.uni-mannheim.de
 * www.informatik.uni-mannheim.de/informatik/pi4/projects/mlb
 *
 **********************************************************************************************************

 program : vt_manager.cpp
 authors : Juergen Vogel
 date    : 25.08.2001
 desc    : 

 **********************************************************************************************************/


#ifdef WIN32
#include <winsock.h>
#else
#include <arpa/inet.h>
#endif

#include <math.h>

#include "SMP.h"
#include "general.h"

#include <rtpi/rtpiException.h>

#include "mlb.h"
#include "vt_manager.h"
#include "mlb_communication.h"
#include "mlb_participants.h"
#include "mlb_timermanager.h"
#include "mlb_gui.h"

#define DEBUG_RTPI
//#define DEBUG_RTCPI
#define DEBUG

void VT_handleReceivedRtpi(SMP::Indication ind) {
  if (VT::VtManager::vtm) VT::VtManager::vtm->HandleReceivedRtpi(ind);
}


void VT_handleReceivedRtcpi(SMP::Indication ind) {
  if (VT::VtManager::vtm) VT::VtManager::vtm->HandleReceivedRtcpi(ind);
}


VT::VtManager* VT::VtManager::vtm = NULL;
char* VT::VtManager::sendBuffer = new char[10000];
const unsigned int VT::VtManager::T = 500;


int VT::TextAnswer::FlushCreate(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | (VtManager::CREATE_TX_ANSWER);

  p8[1] = text.length();
  memcpy(buf + 2, text.c_str(), text.length());
  
  int padd = ((text.length() + 2) % 4 == 0) ? 0 : 4 - ((text.length() + 2) % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[2 + text.length() + i] = 0;
  }

  return 2 + text.length() + padd;
}


int VT::TextAnswer::ParseCreate(char* buf) {
  uint8*  p8  = (uint8*)buf;

  if (((p8[0] & VT_MASK_V) >> 6) != VtManager::VERSION) {
    GUI::MessageWindow::Error("TextAnswer::ParseState - wrong version of voting protocol\n");
    return -1;
  }
  
  if ((p8[0] & VT_MASK_TYPE) != VtManager::CREATE_TX_ANSWER) {
    GUI::MessageWindow::Error("TextAnswer::ParseState - wrong version packet type\n");
    return -1;
  }
  
  int length = p8[1];
  char* tmp  = new char[length + 1];
  tmp[length] = '\0';
  memcpy(tmp, buf + 2, length);
  text = tmp;
  delete[] tmp;
  
  int padd = ((length + 2) % 4 == 0) ? 0 : 4 - ((length + 2) % 4);
  
  return 2 + length + padd;
}


int VT::TextAnswer::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | (VtManager::DELETE_TX_ANSWER);

  return 4;
}


int VT::MultipleChoiceAnswer::FlushCreate(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | (VtManager::CREATE_MC_ANSWER);

  p8[1] = value;

  return 4;
}


int VT::MultipleChoiceAnswer::ParseCreate(char* buf) {
  uint8*  p8  = (uint8*)buf;

  if (((p8[0] & VT_MASK_V) >> 6) != VtManager::VERSION) {
    GUI::MessageWindow::Error("MultipleChoiceAnswer::ParseState - wrong version of voting protocol\n");
    return -1;
  }
  
  if ((p8[0] & VT_MASK_TYPE) != VtManager::CREATE_MC_ANSWER) {
    GUI::MessageWindow::Error("MultipleChoiceAnswer::ParseState - wrong packet type\n");
    return -1;
  }
  
  value = p8[1];
  
  return 4;
}


int VT::MultipleChoiceAnswer::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | (VtManager::DELETE_MC_ANSWER);
  
  return 4;
}


VT::Question::~Question() {
  DeleteAllAnswers();
}


void VT::Question::DeleteAllAnswers() {
  for (std::list<Answer*>::iterator a = answers.begin(); a != answers.end(); ++a) {
    delete *a;
  }
  answers.clear();
}


void VT::TextQuestion::DisplayAdd() {
  if (Et_GlobalEvalF(Et_Interp,"Voting::AddTextQuestion %u \"%q\"", id, text.c_str()) != TCL_OK) 
    GUI::MessageWindow::Error("VT::TextQuestion::DisplayAdd - tcl error %q\n", Et_Interp->result);
}


void VT::TextQuestion::DisplayUpdate() {
  if (Et_GlobalEvalF(Et_Interp,"Voting::UpdateTextQuestion %u \"%q\"", id, text.c_str()) != TCL_OK) 
    GUI::MessageWindow::Error("VT::TextQuestion::DisplayUpdate - tcl error %q\n", Et_Interp->result);
}


void VT::TextQuestion::DisplayResult() {
  if (Et_GlobalEvalF(Et_Interp,"Voting::SetTextResult %u \"%q\"", id, CalculateResult().c_str()) != TCL_OK) 
    GUI::MessageWindow::Error("VT::TextQuestion::DisplayResult - tcl error %q\n", Et_Interp->result);
}


std::string VT::TextQuestion::CalculateResult() {
  std::string result;
  for (std::list<Answer*>::iterator a = answers.begin(); a != answers.end(); ++a) {
    MLB::Participant* p = MLB::ParticipantManager::pm->GetParticipant((*a)->pid);
    if (!p) continue;
    if (!(p->name.empty())) result += p->name;
    else result += p->cname;
    result += ": ";
    result += ((TextAnswer*)*a)->text;
    result += "\n----\n";
  }
  return result;
}


int VT::TextQuestion::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | VtManager::TX_QUESTION;

  p16[1] = htons(ttl);
  
  int offset = 4;
  int padd;
  int length;
  
  // 1. flush question text
  length = text.length();
  p8[offset] = length;
  ++offset;
  
  memcpy(buf + offset, text.c_str(), length);
 
  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[offset + length + i] = 0;
  }
  
  offset += (length + padd);
  
  // 2. flush all received answers
  p16[offset/2] = htons(answers.size());
  p16[offset/2 + 1] = 0;

  offset += 4;

  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;
  uint32 tl = 0; // time left
  for (std::list<Answer*>::iterator a = answers.begin(); a != answers.end(); ++a) {
    p32[offset/4] = htonl((*a)->pid);

    tl = ttl - (now - (*a)->createTime);
    p16[offset/2 + 2] = htons((tl < 0) ? 0 : tl);

    p8[offset + 6] = ((TextAnswer*)*a)->text.length(); 
    memcpy(buf + offset + 7, ((TextAnswer*)*a)->text.c_str(), ((TextAnswer*)*a)->text.length());

    padd = ((((TextAnswer*)*a)->text.length() + 3) % 4 == 0) ? 0 : 4 - ((((TextAnswer*)*a)->text.length() + 3) % 4);
    if (padd != 0) { // padding needed
      for (int i = 0; i < padd; ++i)
	p8[offset + 7 + ((TextAnswer*)*a)->text.length() + i] = 0;
    }
  
    offset += (7 + ((TextAnswer*)*a)->text.length() + padd);
  }

  return offset;
}


int VT::TextQuestion::ParseState(char* buf, uint32 ts) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  if (((p8[0] & VT_MASK_V) >> 6) != VtManager::VERSION) {
    GUI::MessageWindow::Error("TextQuestion::ParseState - wrong version of voting protocol\n");
    return -1;
  }

  if ((p8[0] & VT_MASK_TYPE) != VtManager::TX_QUESTION) {
    GUI::MessageWindow::Error("TextQuestion::ParseState - wrong packet type\n");
    return -1;
  }
  
  ttl = ntohs(p16[1]);

  int offset = 4;
  int padd;
  int length;
  char* tmp;

  // 1. read question text
  length = p8[offset];
  ++offset;
  tmp = new char[length + 1];
  tmp[length] = '\0';
  memcpy(tmp, buf + offset, length);
  text = tmp;
  delete[] tmp;
  
  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;
  
  // 2. read all included answers
  int nbr = ntohs(p16[offset/2]);
  offset += 4;

  DeleteAllAnswers();
  for (int j = 1; j <= nbr; j++) {
    length = p8[offset + 6];
    tmp = new char[length + 1];
    tmp[length] = '\0';
    memcpy(tmp, buf + 7 + offset, length);
    padd = ((length + 3) % 4 == 0) ? 0 : 4 - ((length + 3) % 4);
    answers.push_back(new TextAnswer(ntohl(p32[offset/4]), ntohs(p16[offset/2 + 1]) + ts - ttl, tmp));
    delete[] tmp;
    offset += 7 + length + padd;
  }
  
  return offset;
}


int VT::TextQuestion::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | (VtManager::DELETE_TX_QUESTION);

  return 4;
}


VT::MultipleChoiceQuestion::MultipleChoiceQuestion(uint32 _id, uint32 _ttl, std::string _text, int nbr, std::string _expl)
  : Question(_id, _ttl, _text) {

  std::string::size_type b,e;

  b = _expl.find('{');
  e = _expl.find('}');

  std::string tmp = _expl.substr(b+1,e-b-2);

  explanations.push_back(_expl.substr(b+1,e-b-2));

  while ((b = _expl.find('{', e)) != std::string::npos) {
    e = _expl.find('}', b);
    tmp =  _expl.substr(b+1,e-b-2);
    explanations.push_back(_expl.substr(b+1,e-b-2));
  }
}


void VT::MultipleChoiceQuestion::DisplayAdd() {
  for (std::vector<std::string>::iterator i = explanations.begin(); i != explanations.end(); ++i) {
    if (Et_GlobalEvalF(Et_Interp,"lappend tmplist \"%q\"", i->c_str()) != TCL_OK)
      GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DispalyAdd - tcl error %q\n", Et_Interp->result);
  }

  if (Et_GlobalEvalF(Et_Interp,"Voting::AddMCQuestion %u \"%q\" %u $tmplist", id, text.c_str(), explanations.size()) != TCL_OK) 
    GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DispalyAdd - tcl error %q\n", Et_Interp->result);

  if (Et_GlobalEvalF(Et_Interp,"unset tmplist") != TCL_OK) 
      GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DispalyAdd - tcl error %q\n", Et_Interp->result);
}


void VT::MultipleChoiceQuestion::DisplayUpdate() {
  for (std::vector<std::string>::iterator i = explanations.begin(); i != explanations.end(); ++i) {
    if (Et_GlobalEvalF(Et_Interp,"lappend tmplist \"%q\"", i->c_str()) != TCL_OK)
      GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DisplayUpdate - tcl error %q\n", Et_Interp->result);
  }
  
  if (Et_GlobalEvalF(Et_Interp,"Feedback::UpdateMCQuestion %u \"%q\" %u $tmplist", 
		     id, text.c_str(), explanations.size()) != TCL_OK) 
    GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DisplayUpdate - tcl error %q\n", Et_Interp->result);
  
  if (Et_GlobalEvalF(Et_Interp,"unset tmplist") != TCL_OK) 
    GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DisplayUpdate - tcl error %q\n", Et_Interp->result);
}


void VT::MultipleChoiceQuestion::DisplayResult() {
  if (Et_GlobalEvalF(Et_Interp,"Voting::SetMCResult %u %u \"%q\"", id, answers.size(), CalculateResult().c_str()) != TCL_OK) 
    GUI::MessageWindow::Error("VT::MultipleChoiceQuestion::DisplayResult - tcl error %q\n", Et_Interp->result);
}


void VT::MultipleChoiceQuestion::Update(std::string _text, int nbr, std::string _expl) {
  text = _text;
  
  explanations.clear();

  std::string::size_type b,e;
  
  b = _expl.find('{');
  e = _expl.find('}');

  std::string tmp = _expl.substr(b+1,e-b-2);

  explanations.push_back(_expl.substr(b+1,e-b-2));

  while ((b = _expl.find('{', e)) != std::string::npos) {
    e = _expl.find('}', b);
    tmp =  _expl.substr(b+1,e-b-2);
    explanations.push_back(_expl.substr(b+1,e-b-2));
  }
}


std::string VT::MultipleChoiceQuestion::CalculateResult() {
  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;

  int nbr = explanations.size();
  double* result = new double[nbr];
  for (int i = 0; i < nbr; ++i) result[i] = 0.0;
    
  for (std::list<Answer*>::iterator a = answers.begin(); a != answers.end(); ++a) {
    // check ttl
    if ((ttl != 0) && (now > (*a)->createTime + ttl)) {
      // answer has expired
      delete *a;
      a = answers.erase(a);
      if (a !=  answers.begin()) --a; 
      continue;
    } 

    int idx = (((MultipleChoiceAnswer*)*a)->value) - 1;
    result[idx] += 1;
  }

  char tmp[20];
  std::string ret;
  int size = answers.size();
  for (int j = 0; j < nbr; ++j) {
    result[j] /= size;
    sprintf(tmp, "{%.3f} ", result[j]);
    ret += tmp;
  }

  return ret;
}


int VT::MultipleChoiceQuestion::FlushState(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | VtManager::MC_QUESTION;

  p8[1] = explanations.size();

  p16[1] = htons(ttl);
  
  int offset = 4;
  int padd;
  int length;
  
  // 1. flush question text
  length = text.length();
  p8[offset] = length;
  ++offset;
  
  memcpy(buf + offset, text.c_str(), length);
  
  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  if (padd != 0) { // padding needed
    for (int i = 0; i < padd; ++i)
      p8[offset + length + i] = 0;
  }
  
  offset += (length + padd);
  
  // 2. flush possible answers' explanations
  for (int i = 1; i <= explanations.size(); ++i) {
    length = explanations[i-1].length();
    p8[offset] = length;
    ++offset;

    memcpy(buf + offset, explanations[i-1].c_str(), length);

    padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
    if (padd != 0) { // padding needed
      for (int i = 0; i < padd; ++i)
	p8[offset + length + i] = 0;
    }

    offset += (length + padd);
  }


  // 3. flush all received answers
  p16[offset/2] = htons(answers.size());
  p16[offset/2 + 1] = 0;

  offset += 4;

  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;
  uint32 tl = 0; // time left
  for (std::list<Answer*>::iterator a = answers.begin(); a != answers.end(); ++a) {
    p32[offset/4] = htonl((*a)->pid);

    tl = ttl - (now - (*a)->createTime);
    p16[offset/2 + 2] = htons((tl < 0) ? 0 : tl);

    p8[offset + 6]    = ((MultipleChoiceAnswer*)*a)->value;
    p8[offset + 7]= 0;

    offset += 8;
  }

  return offset;
}


int VT::MultipleChoiceQuestion::ParseState(char* buf, uint32 ts) {
  uint8*  p8  = (uint8*)buf;
  uint16* p16 = (uint16*)buf;
  uint32* p32 = (uint32*)buf;

  if (((p8[0] & VT_MASK_V) >> 6) != VtManager::VERSION) {
    GUI::MessageWindow::Error("MultipleChoiceQuestion::ParseState - wrong version of voting protocol\n");
    return -1;
  }

  if ((p8[0] & VT_MASK_TYPE) != VtManager::MC_QUESTION) {
    GUI::MessageWindow::Error("MultipleChoicQuestion::ParseState - wrong packet type\n");
    return -1;
  }
  
  int nbr = p8[1];
  
  ttl = ntohs(p16[1]);

  int offset = 4;
  int padd;
  int length;
  char* tmp;

  // 1. read question text
  length = p8[offset];
  ++offset;
  tmp = new char[length + 1];
  tmp[length] = '\0';
  memcpy(tmp, buf + offset, length);
  text = tmp;
  delete[] tmp;
  
  padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
  offset += length + padd;
  
  // 2. read possible answers' explanations
  explanations.clear();
  for (int i = 1; i <= nbr; ++i) {
    length = p8[offset];
    ++offset;
    tmp = new char[length + 1];
    tmp[length] = '\0';
    memcpy(tmp, buf + offset, length);
    explanations.push_back(tmp);
    delete[] tmp;

    padd = ((length + 1) % 4 == 0) ? 0 : 4 - ((length + 1) % 4);
    offset += length + padd;
  }

  // 3. read all included answers
  nbr = ntohs(p16[offset/2]);
  offset += 4;

  DeleteAllAnswers();
  for (int j = 1; j <= nbr; j++) {
    answers.push_back(new MultipleChoiceAnswer(ntohl(p32[offset/4]), ntohs(p16[offset/2 + 2]) + ts - ttl, p8[offset + 7]));
    offset += 8;
  }
  
  return offset;
}


int VT::MultipleChoiceQuestion::FlushDelete(char* buf) {
  uint8*  p8  = (uint8*)buf;
  uint32* p32 = (uint32*)buf;

  p32[0] = 0;
  p8[0] = p8[0] | (VtManager::VERSION << 6);
  p8[0] = p8[0] | (VtManager::DELETE_MC_QUESTION);

  return 4;
}


VT::VtManager::VtManager() : openConnection(false), init(true) {
}


VT::VtManager::~VtManager() {
  CloseRTPI();
}


void VT::VtManager::Init() {
  // create default questions
}


bool VT::VtManager::OpenRTPI() {
  if (openConnection || !MLB::CommunicationManager::cm->openSMP || !MLB::CommunicationManager::cm->openIDServer) {
    GUI::MessageWindow::Error("VtManager::OpenRTPI - RTP/I Connection cannot be set up\n");
    return false;
  }

  SMP::SMP* smp = MLB::CommunicationManager::cm->smp;

  // setup SMP
  if ((rtpiSocket = smp->Open(::VT_handleReceivedRtpi)) < 0) {
    GUI::MessageWindow::Error("VtManager::Open - could not connect to smp\n");
    return false;
  }

  if ((rtcpiSocket = smp->Open(::VT_handleReceivedRtcpi)) < 0) {
    GUI::MessageWindow::Error("VtManager::Open - could not connect to smp\n");
    smp->Close(rtpiSocket);
    return false;
  }

  // setup RTP/I
  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("VtManager::OpenRTPI - joining %q - %u - %u\n", MLB::CommunicationManager::cm->ip.c_str(), 
		 MLB::CommunicationManager::cm->port+13, MLB::CommunicationManager::cm->ttl);
  #endif

  if (!smp->JoinGroup(rtpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+13, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("VtManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }
  if (!smp->JoinGroup(rtcpiSocket, MLB::CommunicationManager::cm->ip.c_str(), MLB::CommunicationManager::cm->port+14, 
		      MLB::CommunicationManager::cm->ttl, MLB::CommunicationManager::cm->bandwidth/20, 'A', false, false, false)) {
    GUI::MessageWindow::Error("VtManager::Open - smp JoinGroup failed\n");
    smp->Close(rtpiSocket);
    smp->Close(rtcpiSocket);
    return false;
  }

  rtpi = new RTPI::Rtpi();
  rtpi->join_group( MLB::CommunicationManager::cm->username, this, this, new RTPI::TransparentReliableTransport());

  openConnection = true;
  return true;
}


bool VT::VtManager::CloseRTPI() {
  if (!openConnection) {
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::CloseRTPI - session already closed\n");
    #endif
    return false;
  }

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("VtManager::CloseRTPI - leaving group\n");
  #endif
  
  rtpi->leave_group();

  MLB::CommunicationManager::cm->smp->LeaveGroup(rtpiSocket);
  MLB::CommunicationManager::cm->smp->LeaveGroup(rtcpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtpiSocket);
  MLB::CommunicationManager::cm->smp->Close(rtcpiSocket);

  delete rtpi;
  
  openConnection = false;
  init = true;

  Et_GlobalEvalF(Et_Interp, "Voting::ClearAllQuestions");

  for (std::list<Question*>::iterator i = questions.begin(); i != questions.end(); ++i) delete *i;
  questions.clear();

  return true;
}


void VT::VtManager::HandleReceivedRtpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got data %u\n", ind.buf->getSize());
    #endif

    RefCount<RTPI::RtpiAduVector> rtpiAdus;
    try {
      rtpiAdus = rtpi->create_rtpi_adus(ind.buf);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    if (!rtpiAdus) 
      return; // adu not complete, received only fragments

    for (RTPI::RtpiAduVector::iterator i = rtpiAdus->begin(); i != rtpiAdus->end(); ++i) {
      if ((*i)->GetAduHeader()->GetPayloadType() != APPLICATION_TYPE) {
	GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - wrong payload type %u\n", 
		       (*i)->GetAduHeader()->GetPayloadType());
	continue;
      }

      switch ((*i)->GetType()) {
      case RTPI::RtpiHeader::PKT_EVENT: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & VT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("VtManager::ReceiveEvent - wrong version of voting protocol\n");
	  continue;
	}

	int type = p8[0] & VT_MASK_TYPE;
	switch (type) {
	case DELETE_MC_QUESTION:
	case DELETE_TX_QUESTION: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - delete question %u\n", (*i)->GetSubcomponentID());
          #endif

	  bool found = false;
	  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
	    if ((*q)->id == (*i)->GetSubcomponentID()) {
	      // update GUI
	      if (Et_GlobalEvalF(Et_Interp,"Voting::RemoteDeleteQuestion %u", (*q)->id) != TCL_OK) 
		GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - tcl error %q\n", Et_Interp->result);

	      delete *q;
	      questions.erase(q);
	      found = true;
	      break;
	    }
	  }
	  
	  // error
	  if (!found) GUI::MessageWindow::Error("VtManager::DeleteQuestion - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	  break;
	}
	case CREATE_MC_ANSWER: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - received multiple choice answer\n");
          #endif

	  MultipleChoiceQuestion* q = (MultipleChoiceQuestion*)GetQuestion((*i)->GetSubcomponentID());
	  if (!q) {
	    GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	    continue;
	  }

	  bool found = false;
	  std::list<Answer*>::iterator a;
	  for (a = q->answers.begin(); a != q->answers.end(); ++a) {
	    if ((*i)->GetParticipantID() == (*a)->pid) {
	      ((MultipleChoiceAnswer*)*a)->value = p8[1];
	      ((MultipleChoiceAnswer*)*a)->createTime = (*i)->GetAduHeader()->GetTimestamp();
	      found = true;
	      break;
	    }
	  }

	  if (!found) {
	    if (a == q->answers.end()) {
	      // this is the first answer to this question
	      MultipleChoiceAnswer* answer = new MultipleChoiceAnswer();
	      answer->pid        = (*i)->GetParticipantID();
	      answer->createTime = (*i)->GetAduHeader()->GetTimestamp();
	      answer->ParseCreate((*i)->GetPayload());
	      q->answers.push_back(answer);
	    }
	  }

	  // update gui
	  q->DisplayResult();
	  break;
	}
	case CREATE_TX_ANSWER : {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - received text answer\n");
          #endif

	  TextQuestion* q = (TextQuestion*)GetQuestion((*i)->GetSubcomponentID());
	  if (!q) {
	    GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	    continue;
	  }

	  bool found = false;
	  std::list<Answer*>::iterator a;
	  for (a = q->answers.begin(); a != q->answers.end(); ++a) {
	    if ((*i)->GetParticipantID() == (*a)->pid) {
	      ((TextAnswer*)*a)->createTime = (*i)->GetAduHeader()->GetTimestamp();
	      (*a)->ParseCreate((*i)->GetPayload());
	      found = true;
	      break;
	    }
	  }

	  if (!found) {
	    if (a == q->answers.end()) {
	      // this is the first answer to this question
	      TextAnswer* answer = new TextAnswer();
	      answer->pid        = (*i)->GetParticipantID();
	      answer->createTime = (*i)->GetAduHeader()->GetTimestamp();
	      answer->ParseCreate((*i)->GetPayload());
	      q->answers.push_back(answer);
	      a = q->answers.end(); --a;
	    }
	  }

	  // update gui
	  q->DisplayResult();
	  break;
	}	
	case DELETE_MC_ANSWER: 
	case DELETE_TX_ANSWER: {
          #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - received delete answer\n");
          #endif

	  Question* q = GetQuestion((*i)->GetSubcomponentID());
	  if (!q) {
	    GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - question with id %u does not exist\n", (*i)->GetSubcomponentID());
	    continue;
	  }

	  std::list<Answer*>::iterator a;
	  for (a = q->answers.begin(); a != q->answers.end(); ++a) {
	    if ((*i)->GetParticipantID() == (*a)->pid) {
	      delete *a;
	      q->answers.erase(a);
	      q->DisplayResult();
	      break;
	    }
	  }

	  break;
	}
	}
	break;
      }
      case RTPI::RtpiHeader::PKT_STATE: {
	uint8*  p8  = (uint8*)(*i)->GetPayload();
	uint16* p16 = (uint16*)(*i)->GetPayload();
	uint32* p32 = (uint32*)(*i)->GetPayload();

	if (((p8[0] & VT_MASK_V) >> 6) != VERSION) {
	  GUI::MessageWindow::Error("VtManager::ReceiveState - wrong version of protocol\n");
	  continue;
	}

	Question* q = GetQuestion((*i)->GetSubcomponentID());
	if (q) {
	  //question already exists
	  
	  if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_REPLY, (*i)->GetSubcomponentID())) {
	    // we have an own state reply timer -> delete 
	    MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_REPLY, (*i)->GetSubcomponentID());
	  } 

	  if (((RTPI::RtpiStateHeader*)(*i)->GetAduHeader())->GetPriority() == 3) {
	    // update question
	    q->ParseState((*i)->GetPayload(), (*i)->GetAduHeader()->GetTimestamp());
	    q->DisplayUpdate();
	    q->DisplayResult();
	    break;
	  }
	  // else ignore
	  break;
	}

	if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID())) {
	  // we have an own state query timer -> delete 
	  MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID());
	} 

	// new question
	if ((p8[0] & VT_MASK_TYPE) == MC_QUESTION) {
	  q = new MultipleChoiceQuestion((*i)->GetSubcomponentID());
	} else if ((p8[0] & VT_MASK_TYPE) == TX_QUESTION) {
	  q = new TextQuestion((*i)->GetSubcomponentID());
	} else {
	  break; // wrong type
	}

	q->ParseState((*i)->GetPayload(), (*i)->GetAduHeader()->GetTimestamp());
	questions.push_back(q);

	RegisterSubcomponent(q->id);

	q->DisplayAdd();
	q->DisplayResult();

	break;
      }
      case RTPI::RtpiHeader::PKT_STATE_QUERY: {
	#ifdef DEBUG
	GUI::MessageWindow::Info("VtManager::ReceiveStateQuery - got State Query\n");
	#endif

	if (MLB::TimerManager::tm->TimerRunning(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID())) {
	  // we have an own state query timer -> delete 
	  MLB::TimerManager::tm->DeleteTimer(this, MLB::TimerObject::STATE_QUERY, (*i)->GetSubcomponentID());
	} 

	Question* q = GetQuestion((*i)->GetSubcomponentID());
	if (!q) {
	  #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::ReceiveStateQuery - did not find question with id %u", (*i)->GetSubcomponentID());
	  #endif
	  break;
	}

	double x = drand48();
	int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
	double N = (n == 1) ? 1 : n - 1;

	#ifdef DEBUG
	GUI::MessageWindow::Info("VtManager::ReceiveStateQuery - x: %f, N: %f\n", x, N);
	#endif

	if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
	  // send State immediately
  	  #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::ReceiveStateQuery - answer immediately\n");
	  #endif

	  int bytes = q->FlushState(sendBuffer);
	  SendState(q->id, 0, sendBuffer, bytes);

	} else {
	  // set state reply timer
	  int runningTime = (int)(T * (1 + log(x) / log(N)));
	  MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime , MLB::TimerObject::STATE_REPLY, 
								     (*i)->GetSubcomponentID(), 0));
  	  #ifdef DEBUG
	  GUI::MessageWindow::Info("VtManager::ReceiveStateQuery - answer later, set timer with running time %u\n", runningTime);
	  #endif
	}
	break;
      }

      default:
	GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - wrong adu type %u\n", (*i)->GetType());
      }
    }

    try {
      rtpi->handle_received_rtpi(rtpiAdus, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }

    return;
  }

  
  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    return;
  }
  

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got LJNEEDED\n");
    #endif
    return;
														 

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("VtManager::HandleReceivedRtpi - got ERR / probably lost connection to smp\n");
    return;


  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void VT::VtManager::HandleReceivedRtcpi(SMP::Indication ind) {
  if (!openConnection) return;

  switch(ind.ServiceType) {
  case SMP::Indication::DATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got data %u\n", ind.buf->getSize());
    #endif
    return;
  }

  case SMP::Indication::UNRLDATAIND: {	
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got unreliable data %u\n", ind.buf->getSize());
    #endif
    try {
      rtpi->handle_received_rtcpi(ind.buf, ind.senderIPAddr);
    } catch (RTPI::RtpiException e) {
      GUI::MessageWindow::Error("VtManager::HandleReceivedRtcpi - RtpiException occured: %u / %q\n", 
		     e.errCode, e.errMsg.c_str());
      return;
    }
    return;
  }

  case SMP::Indication::LJNEEDED: 
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got LJNEEDED\n");
    #endif
    return;

  case SMP::Indication::DADUIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got DADUIND\n");
    #endif
    return;

  case SMP::Indication::DADUFAIL:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got DADUFAIL\n");
    #endif
    return;

  case SMP::Indication::ERR:
    GUI::MessageWindow::Error("VtManager::HandleReceivedRtcpi - got ERR / probably lost connection to smp\n");
    //unlinksocket(smp_socket);
    return;

  case SMP::Indication::NOSERVICE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got NOSERVICE\n");
    #endif
    return;

  case SMP::Indication::JOININD:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got JOININD\n");
    #endif
    return;

  case SMP::Indication::LEAVIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got LEAVIND\n");
    #endif
    return;

  case SMP::Indication::ACKIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got ACKIND\n");
    #endif
    return;

  case SMP::Indication::DADUCONF:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got DADUCONF\n");
    #endif
    return;

  case SMP::Indication::CLOSEIND:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got CLOSEIND\n");
    #endif
    return;

  case SMP::Indication::ALIVE:
    #ifdef DEBUG_RTCPI
    GUI::MessageWindow::Info("VtManager::HandleReceivedRtcpi - got ALIVE\n");
    #endif
    return;

  default:
    return;
  }
}


void VT::VtManager::SendState(uint32 subID, int priority, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("VtManager::SendState - sending state\n");
  #endif

  RefCount<RTPI::RtpiAdu> state = new RTPI::RtpiState(subID, APPLICATION_TYPE, 0, rtpitime(), priority, payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(state);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("VtManager::SendState - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void VT::VtManager::SendEvent(uint32 subID, char* payload, int size) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("VtManager::SendEvent - sending event\n");
  #endif

  RefCount<RTPI::RtpiAdu> event = new RTPI::RtpiEvent(subID, APPLICATION_TYPE, 0, rtpitime(), payload, size);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(event);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("VtManager::SendEvent - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }
 
  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void VT::VtManager::SendStateQuery(uint32 subID, int priority) {
  if (!openConnection) return;

  #ifdef DEBUG_RTPI
  GUI::MessageWindow::Info("VtManager::SendStateQuery - sending state query\n");
  #endif

  RefCount<RTPI::RtpiAdu> query = new RTPI::RtpiStateQuery(subID, APPLICATION_TYPE, rtpitime(), priority);
  RefCount<RTPI::RtpiAduVector> adus = new RTPI::RtpiAduVector;
  adus->push_back(query);

  RefCount<RTPI::BufferVector> bufVector = NULL;
  try {
    bufVector = rtpi->create_rtpi_packets(adus);
  } catch (RTPI::RtpiException e) {
    GUI::MessageWindow::Error("VtManager::SendStateQuery - RtpiException occured: %u / %q\n", 
		   e.errCode, e.errMsg.c_str());
    return;
  }

  for (RTPI::BufferVector::iterator i = bufVector->begin(); i != bufVector->end(); ++i)
    MLB::CommunicationManager::cm->smp->Send(rtpiSocket, (*i)->getCont(), (*i)->getSize());
}


void VT::VtManager::CreateMCQuestion(int argc, const char** argv) {
  uint32 subid = MLB::CommunicationManager::cm->GetObjectID();
  
  Question* q = new MultipleChoiceQuestion(subid, 0, argv[1], atoi(argv[2]), argv[3]);;
  questions.push_back(q);
  
  RegisterSubcomponent(subid);

  int bytes = q->FlushState(sendBuffer);
  SendState(subid, 3, sendBuffer, bytes);
  
  q->DisplayAdd();
}


void VT::VtManager::CreateTextQuestion(int argc, const char** argv) {
  uint32 subid = MLB::CommunicationManager::cm->GetObjectID();
  
  Question* q = new TextQuestion(subid, 0, argv[1]);
  questions.push_back(q);
  
  RegisterSubcomponent(subid);

  int bytes = q->FlushState(sendBuffer);
  SendState(subid, 3, sendBuffer, bytes);

  q->DisplayAdd();
}


void VT::VtManager::EditMCQuestion(int argc, const char** argv) {
  MultipleChoiceQuestion* q = (MultipleChoiceQuestion*)GetQuestion(MLB::StringToUint32((char*)argv[1]));
  if (!q) {
    GUI::MessageWindow::Error("VtManager::EditMCQuestion - question with id %u does not exist\n", MLB::StringToUint32((char*)argv[1]));
    return;
  }
  
  q->Update(argv[2], atoi(argv[3]), argv[4]);

  int bytes = q->FlushState(sendBuffer);
  SendState(q->id, 3, sendBuffer, bytes);
}


void VT::VtManager::EditTextQuestion(int argc, const char** argv) {
  TextQuestion* q = (TextQuestion*)GetQuestion(MLB::StringToUint32((char*)argv[1]));
  if (!q) {
    GUI::MessageWindow::Error("VtManager::EditTextQuestion - question with id %u does not exist\n", MLB::StringToUint32((char*)argv[1]));
    return;
  }
  
  q->text = argv[2];

  int bytes = q->FlushState(sendBuffer);
  SendState(q->id, 3, sendBuffer, bytes);
}


void VT::VtManager::DeleteQuestion(uint32 id) {
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
    if ((*q)->id == id) {
      int bytes = (*q)->FlushDelete(sendBuffer);
      SendEvent(id, sendBuffer, bytes);
      
      delete *q;
      questions.erase(q);

      UnregisterSubcomponent(id);

      return;
    }
  }

  // error
  GUI::MessageWindow::Error("VtManager::DeleteQuestion - question with id %u does not exist\n", id);
}


void VT::VtManager::ChangeMCAnswer(uint32 id, int value) {
  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;

  bool found = false;
  std::list<Answer*>::iterator a;
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
    if ((*q)->id == id) {
      for (a = (*q)->answers.begin(); a != (*q)->answers.end(); ++a) {
	if (MLB::CommunicationManager::cm->pid == (*a)->pid) {
	  ((MultipleChoiceAnswer*)*a)->value = value;
	  ((MultipleChoiceAnswer*)*a)->createTime = now;
	  found = true;
	  break;
	}
      }

      if (!found) {
	if (a == (*q)->answers.end()) {
	  // this is our first answer to this question
	  MultipleChoiceAnswer* answer = new MultipleChoiceAnswer();
	  answer->pid        = MLB::CommunicationManager::cm->pid;
	  answer->value      = value;
	  answer->createTime = now;
	  (*q)->answers.push_back(answer);
	  a = (*q)->answers.end();
	  --a;
	}
      }

      int bytes = (*a)->FlushCreate(sendBuffer);
      SendEvent(id, sendBuffer, bytes);
      
      // update gui
      (*q)->DisplayResult();

      return;
    }
  }

  // error
  GUI::MessageWindow::Error("VtManager::ChangeMCAnswer - question with id %u does not exist\n", id);
}


void VT::VtManager::ChangeTextAnswer(uint32 id, std::string text) {
  struct timeval time;
  gettimeofday(&time, NULL);
  uint32 now = time.tv_sec;

  bool found = false;
  std::list<Answer*>::iterator a;
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q) {
    if ((*q)->id == id) {
      for (a = (*q)->answers.begin(); a != (*q)->answers.end(); ++a) {
	if (MLB::CommunicationManager::cm->pid == (*a)->pid) {
	  ((TextAnswer*)*a)->text = text;
	  ((TextAnswer*)*a)->createTime = now;
	  found = true;
	  break;
	}
      }

      if (!found) {
	if (a == (*q)->answers.end()) {
	  // this is our first answer to this question
	  TextAnswer* answer = new TextAnswer();
	  answer->pid        = MLB::CommunicationManager::cm->pid;
	  answer->text       = text;
	  answer->createTime = now;
	  (*q)->answers.push_back(answer);
	  a = (*q)->answers.end();
	  --a;
	}
      }

      int bytes = (*a)->FlushCreate(sendBuffer);
      SendEvent(id, sendBuffer, bytes);
      
      // update gui
      (*q)->DisplayResult();

      return;
    }
  }

  // error
  GUI::MessageWindow::Error("VtManager::ChangeTextAnswer - question with id %u does not exist\n", id);
}


void VT::VtManager::send_rtcpi(RefCount<Buffer> buf) {
  if (!openConnection) return;
  MLB::CommunicationManager::cm->smp->SendUnreliable(rtcpiSocket, buf->getCont(), buf->getSize());
}


void VT::VtManager::RegisterSubcomponent(uint32 subID) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("VtManager::RegisterSubcomponent - register sub with id %u\n", subID);
  #endif

  // register subcomponent
  rtpi->add_subcomponent(subID, "vtq", true);
}


void VT::VtManager::UnregisterSubcomponent(uint32 subID) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("VtManager::UnregisterSubcomponent - unregister sub with id %u\n", subID);
  #endif

  rtpi->delete_subcomponent(subID);
}


void  VT::VtManager::ChangeSubcomponentActiveness(uint32 subID, bool active) {
  #ifdef DEBUG_RTCPI
  GUI::MessageWindow::Info("VtManager::ChangeSubcomponentActiveness - set sub with id %u active = %u\n", subID, active);
  #endif

  if (active) rtpi->activate_subcomponent(subID);
  else rtpi->deactivate_subcomponent(subID);
}


void VT::VtManager::subcomponent_changed(RTPI::SubComponentMapElement* subcomp, uint8 reason) {
}


void VT::VtManager::subcomponent_added(RTPI::SubComponentMapElement* subcomp, int reason) {
  Question* q = GetQuestion(subcomp->first);
  if (q) return; // question already there

  // set state query timer
  double x = drand48();
  int n    = MLB::ParticipantManager::pm->GetNumberOfParticipants();
  double N = (n == 1) ? 1 : n - 1;
  
  #ifdef DEBUG
  GUI::MessageWindow::Info("VtManager::subcomponent_added - x: %f, N: %f\n", x, N);
  #endif
 
  if ((n == 1) || (N == 1) || (x < (1.0 / N))) {
    // 1. send state query immediately
    SendStateQuery(subcomp->first, 2);

    #ifdef DEBUG
    GUI::MessageWindow::Info("VtManager::subcomponent_added - send state query immediately\n");
    #endif
  } else {
    // 2. set state query timer
    int runningTime = (int)(T * (1 + log(x) / log(N)));
    MLB::TimerManager::tm->SetTimer(this, new MLB::TimerObject(runningTime , MLB::TimerObject::STATE_QUERY, subcomp->first, 2));

    #ifdef DEBUG
    GUI::MessageWindow::Info("VtManager::subcomponent_added - send state query later, set timer with running time %u\n", runningTime);
    #endif
  }
}


void VT::VtManager::subcomponent_removed(uint32 id) {
}


void VT::VtManager::participant_added(uint32 pid) {
}


void VT::VtManager::participant_removed(uint32 pid) {
  // delete all answers
}


void VT::VtManager::participant_changed(uint32 pid) {
}


void VT::VtManager::ExecuteTimer(MLB::TimerObject* timer) {
  switch (timer->type) {
  case MLB::TimerObject::STATE_QUERY: {
    SendStateQuery(timer->id, timer->priority);
    break;
  }
  case MLB::TimerObject::STATE_REPLY: {
    #ifdef DEBUG
    GUI::MessageWindow::Info("VtManager::ExecuteTimer - State Reply Timer ran out, sending state\n");
    #endif

    Question* q = GetQuestion(timer->id);
    if (!q) return; 
    int bytes = q->FlushState(sendBuffer);
    SendState(q->id, timer->priority, sendBuffer, bytes);

    break;
  }
  }

  delete timer;
}
   

VT::Question* VT::VtManager::GetQuestion(uint32 id) {
  for (std::list<Question*>::iterator q = questions.begin(); q != questions.end(); ++q)
    if (id == (*q)->id) return *q;

  return NULL; // not found
}    


